package pps.core.system.service;

import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.base.service.BaseBaseStationService;
import pps.cloud.base.service.data.CreateBaseStationInput;
import pps.cloud.primbpm.service.BpsWorkFlowProcessService;
import pps.cloud.primbpm.service.data.*;
import pps.core.common.session.PpsUserSession;
import pps.core.system.constant.OperationMaintenanceManagementConstants;
import pps.core.system.constant.OperationManagement;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.operation_maintenance_adjustment_function_request.TerminateWorkItemPpsInput;
import pps.core.system.service.data.operation_maintenance_station_application.*;
import pps.core.system.service.data.operation_maintenance_station_application.ApprovalRejectInput;
import pps.core.system.service.data.operation_maintenance_user.QueryOperationMaintenanceUserOutput;
import pps.core.system.service.data.sys_user.CreateSysUserInput;
import pps.core.system.utils.OperationManagementWorkflowBehaviour;
import xstartup.annotation.XService;
import xstartup.data.XPageResult;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;

@XText("运维管理-场站申请")
@XService
public class OperationMaintenanceStationApplicationService {

    //流程编码
    private final String processCode = "PPS_OPERATION_MAINTENANCE_STATION";

    @XText("新增场站申请")
    @XApiPost
    public XServiceResult createOperationMaintenanceStationApplication(XContext context, CreateOperationMaintenanceStationApplicationInput input) {
        OperationMaintenanceStationApplicationMapper mapper = context.getBean(OperationMaintenanceStationApplicationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String uuid = UUID.randomUUID().toString().trim().replaceAll("-", "");
        OperationMaintenanceStationApplicationEnt entity = new OperationMaintenanceStationApplicationEnt();
        input.setId(uuid);
        input.setCreateTime(new Date());
        input.setmOuId(session.getOuId());
        input.setmOuName(session.getOuName());
        input.setCreateById(session.getId());
        input.setCreateByName(session.getUserName());
        input.setDataDate(new Date());
        if (input.getAlternateFieldsTwo().equals("1")) {
            input.setAuditstatus(OperationManagement.UNDER_REVIEW);
        }
        if (input.getAlternateFieldsTwo().equals("0")) {
            input.setAuditstatus(OperationManagement.SAVED);
        }
        if (null != input.getFileKeyId()) {
            StringBuffer buffer = new StringBuffer();
            for (int i = 0; i < input.getFileKeyId().size(); i++) {
                buffer.append(input.getFileKeyId().get(i) + ",");
            }
            input.setFileKey(String.valueOf(buffer));
        }
        XCopyUtils.copyObject(input, entity);
        if (input.getAlternateFieldsTwo().equals("1")) {
            StartProcessInput startProcessInput = new StartProcessInput();
            startProcessInput.setProcessCode(this.processCode);
            startProcessInput.setBsnsDetailUrl(input.getAlternateFieldsThree());
            startProcessInput.setBsnsId(input.getId());
            //启动相应的工作流转填报
            bpmService.startProcess(context, startProcessInput);
        }
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改场站申请")
    @XApiPost
    public XServiceResult updateOperationMaintenanceStationApplication(XContext context, UpdateOperationMaintenanceStationApplicationInput input) {
        OperationMaintenanceStationApplicationMapper mapper = context.getBean(OperationMaintenanceStationApplicationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceStationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceStationApplicationEnt::getId, input.getId());
        OperationMaintenanceStationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        input.setModifyTime(new Date());
        input.setModifyById(session.getId());
        input.setModifyByName(session.getCreateByName());
        if (null != input.getFileKeyId()) {
            StringBuffer buffer = new StringBuffer();
            for (int i = 0; i < input.getFileKeyId().size(); i++) {
                buffer.append(input.getFileKeyId().get(i) + ",");
            }
            input.setFileKey(String.valueOf(buffer));
        }
        XCopyUtils.copyObject(input, entity);
        if (entity.getAuditstatus() == 4 && input.getAlternateFieldsTwo().equals("1")) {
            entity.setAuditstatus(OperationManagement.UNDER_REVIEW);
            //启动相应的工作流转填报
            ApprovalPassInput approvalPassPpsSouthChinaInput = new ApprovalPassInput();
            approvalPassPpsSouthChinaInput.setId(input.getId());
            approvalPassStation(context, approvalPassPpsSouthChinaInput);
        } else {
            if (input.getAlternateFieldsTwo().equals("1")) {
                entity.setAuditstatus(OperationManagement.UNDER_REVIEW);
                StartProcessInput startProcessInput = new StartProcessInput();
                startProcessInput.setProcessCode(this.processCode);
                startProcessInput.setBsnsDetailUrl(input.getAlternateFieldsThree());
                startProcessInput.setBsnsId(input.getId());
                //启动相应的工作流转填报
                bpmService.startProcess(context, startProcessInput);
            }
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("审核通过场站申请")
    @XApiPost
    public XServiceResult approvalPassStation(XContext context, ApprovalPassInput input) {
        OperationMaintenanceStationApplicationMapper mapper = context.getBean(OperationMaintenanceStationApplicationMapper.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceStationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceStationApplicationEnt::getId, input.getId());
        OperationMaintenanceStationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        //调用工作流通过接口
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        FinishWorkItemDirectlyInput finishWorkItemDirectlyInput = new FinishWorkItemDirectlyInput();
        //流程编号
        finishWorkItemDirectlyInput.setWorkItemId(input.getWorkItemId());
        finishWorkItemDirectlyInput.setProcessCode(processCode);
        finishWorkItemDirectlyInput.setRemark(input.getRemark());
        finishWorkItemDirectlyInput.setBsnsId(entity.getId());
        XServiceResult xServiceResult = bpmService.approvalPass(context, finishWorkItemDirectlyInput);
        xServiceResult.throwIfFail();
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        GetProcessInstStateInput getProcessInstStateInput = new GetProcessInstStateInput();
        getProcessInstStateInput.setBsnsId(input.getId());
        getProcessInstStateInput.setProcessCode(processCode);
        XSingleResult<GetProcessInstStateOutput> getProcessInstStateOutputXSingleResult = bpmService.queryProcessStatus(context, getProcessInstStateInput);
        if (getProcessInstStateOutputXSingleResult.getResult().getState() == 2) {
            entity.setAuditstatus(OperationManagement.UNDER_REVIEW);
        } else if (getProcessInstStateOutputXSingleResult.getResult().getState() == 7) {
            entity.setAuditstatus(OperationManagement.PASSED);
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("审核驳回场站申请")
    @XApiPost
    public XServiceResult approvalRejectStation(XContext context, ApprovalRejectInput input) {
        OperationMaintenanceStationApplicationMapper mapper = context.getBean(OperationMaintenanceStationApplicationMapper.class);
        QueryWrapper<OperationMaintenanceStationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceStationApplicationEnt::getId, input.getId());
        OperationMaintenanceStationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        //调用工作流通过接口
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        pps.cloud.primbpm.service.data.ApprovalRejectInput approvalRejectInput = new pps.cloud.primbpm.service.data.ApprovalRejectInput();
        //获取参数
        if (input.getBackReason() == null) {
            return XServiceResult.error(989, "驳回原因为空");
        }
        approvalRejectInput.setBackReason(input.getBackReason());
        approvalRejectInput.setBsnsId(entity.getId());
        approvalRejectInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalReject(context, approvalRejectInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationManagement.REJECT);
        entity.setAlternateFieldsTwo("0");
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("中止场站申请")
    @XApiPost
    public XServiceResult discontinueOperationMaintenanceStationApplication(XContext context, TerminateWorkItemPpsInput input) {
        OperationMaintenanceStationApplicationMapper mapper = context.getBean(OperationMaintenanceStationApplicationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceStationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceStationApplicationEnt::getId, input.getId());
        OperationMaintenanceStationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        pps.cloud.primbpm.service.data.TerminateWorkItemInput workItemInput = new pps.cloud.primbpm.service.data.TerminateWorkItemInput();
        workItemInput.setRemark(input.getRemark());
        workItemInput.setBsnsId(entity.getId());
        workItemInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalStop(context, workItemInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationManagement.TERMINATION);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("接受需求场站申请")
    @XApiPost
    public XServiceResult acceptDemandOperationMaintenanceStationApplication(XContext context, TerminateWorkItemPpsInput input) {
        OperationMaintenanceStationApplicationMapper mapper = context.getBean(OperationMaintenanceStationApplicationMapper.class);
        BaseBaseStationService bean = context.getBean(BaseBaseStationService.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceStationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceStationApplicationEnt::getId, input.getId());
        OperationMaintenanceStationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        pps.cloud.primbpm.service.data.TerminateWorkItemInput workItemInput = new pps.cloud.primbpm.service.data.TerminateWorkItemInput();
        workItemInput.setRemark(input.getRemark());
        workItemInput.setBsnsId(entity.getId());
        workItemInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalAcceptDemand(context, workItemInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationManagement.ACCEPTDEMAND);
        try {
            CreateBaseStationInput createBaseStationInput = new CreateBaseStationInput();
            createBaseStationInput.setStationName(entity.getStationName());
            createBaseStationInput.setShortName(entity.getStationReferredToAs());
            createBaseStationInput.setStationType(entity.getStationTypeId());
            createBaseStationInput.setProvince(entity.getProvinceName());
            createBaseStationInput.setOuId(entity.getAffiliatedUnitId());
            createBaseStationInput.setPipelineId(entity.getPipelineId());
            bean.createBaseStation(context, createBaseStationInput);
        } catch (Exception e) {
            return XServiceResult.error(e);
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id查询所有信息")
    @XApiPost
    public XSingleResult<OperationMaintenanceStationApplicationOutput> queryOperationMaintenanceStationOutput(XContext context, OperationMaintenanceStationApplicationInput input) {
        OperationMaintenanceStationApplicationMapper mapper = context.getBean(OperationMaintenanceStationApplicationMapper.class);
        OperationMaintenanceStationApplicationOutput output = new OperationMaintenanceStationApplicationOutput();
        QueryWrapper<OperationMaintenanceStationApplicationEnt> queryWrapper = new QueryWrapper<>();
        //如果id为空就根据管线跟时间查询
        queryWrapper.lambda().eq(OperationMaintenanceStationApplicationEnt::getId, input.getId());
        OperationMaintenanceStationApplicationEnt batchMonthlyPlanEnt = mapper.selectOne(queryWrapper);
        XCopyUtils.copyObject(batchMonthlyPlanEnt, output);
        output.setProvinceIdType(Integer.valueOf(output.getProvinceId()));
        if (null != batchMonthlyPlanEnt.getFileKey() && !batchMonthlyPlanEnt.getFileKey().equals("")) {
            for (int i = 0; i < batchMonthlyPlanEnt.getFileKey().split(",").length; i++) {
                String[] split = batchMonthlyPlanEnt.getFileKey().split(",");
                output.setFileKeyId(Arrays.asList(split));
            }
        }
        CheckHasTodoInput checkHasTodoInput = new CheckHasTodoInput();
        checkHasTodoInput.setBsnsId(batchMonthlyPlanEnt.getId());
        XSingleResult<QueryOperationMaintenanceStationApplicationOutput> queryBatchMonthlyPlanTypeOutputXSingleResult = queryOperationMaintenanceStationType(context, checkHasTodoInput);
        int type = queryBatchMonthlyPlanTypeOutputXSingleResult.getResult().getType();
        output.setType(type);
        return XSingleResult.success(output);
    }

    /*
    *   填报+审核：审核通过+审核驳回（3）
        填报+审核：中止+审核通过+审核驳回（8）
        填报+审核：接受需求+中止+审核通过+审核驳回（9）
        填报+审核：接受需求+中止+审核驳回（10）

        填报：提交（0）

        审核：审核通过+审核驳回（1）
        审核：中止+审核通过+审核驳回（5）
        审核：接受需求+中止+审核通过+审核驳回（6）
        审核：接受需求+中止+审核驳回（7）

        没有角色：（2）
    *
    * */
    @XText("查询当前登陆人角色状态")
    @XApiPost
    public XSingleResult<QueryOperationMaintenanceStationApplicationOutput> queryOperationMaintenanceStationType(XContext context, CheckHasTodoInput input) {
        //查询流程下的所有角色
        BpsWorkFlowProcessService bean = context.getBean(BpsWorkFlowProcessService.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryOperationMaintenanceStationApplicationOutput output = new QueryOperationMaintenanceStationApplicationOutput();
        QueryProcessRolesInput queryProcessRolesInput = new QueryProcessRolesInput();
        queryProcessRolesInput.setProcessCode(processCode);
        //获取流程第一个节点的参与者信息（角色信息）  processCode    session.getId() 获取当前登陆人的角色判断 是否一样
        OperationManagementWorkflowBehaviour behaviour = new OperationManagementWorkflowBehaviour();
        //判断当前用户是否为当前流程的 第一个节点角色
        boolean b = behaviour.checkInRoleOrUserId(context, processCode, session.getId());
        //当前用户同时拥有 提报跟审批角色 判断当前用户是否有代办
        XSingleResult<CheckHasTodoOutput> checkHasTodoOutputXSingleResult = bean.checkHasTodo(context, input);
        int roles = queryUsersRoles(context);
        if (checkHasTodoOutputXSingleResult.getResult().getHasTodo() && b) {
            if (roles == 4) {
                output.setType(3);
            } else if (roles == 5) {
                output.setType(8);
            } else if (roles == 6) {
                output.setType(9);
            } else if (roles == 7) {
                output.setType(10);
            }
        } else if (b) {
            //当前用户拥有 提报角色
            output.setType(0);
        } else if (checkHasTodoOutputXSingleResult.getResult().getHasTodo()) {
            //审核角色
            if (roles == 4) {
                output.setType(1);
            } else if (roles == 5) {
                output.setType(5);
            } else if (roles == 6) {
                output.setType(6);
            } else if (roles == 7) {
                output.setType(7);
            }
        } else {
            //没有角色
            output.setType(2);
        }
        return XSingleResult.success(output);
    }

    @XText("查询当前登陆人角色状态")
    public int queryUsersRoles(XContext context) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysRoleViewMapper sysRoleViewMapper = context.getBean(SysRoleViewMapper.class);
        List<SysRoleExtendView> roleList = sysRoleViewMapper.getRoleListUserId(session.getId());
        if (!CollectionUtils.isEmpty(roleList)) {
            for (int i = 0; i < roleList.size(); i++) {
                SysRoleExtendView view = roleList.get(i);
                if (view.getRoleName().equals("在线运维管理_三线运维人员")) {
                    return OperationMaintenanceManagementConstants.DISCONTINUE;
                } else if (view.getRoleName().equals("PPS运维组")) {
                    return OperationMaintenanceManagementConstants.SUSPENSIONACCEPTANCE;
                } else if (view.getRoleName().equals("PPS技术组")) {
                    return OperationMaintenanceManagementConstants.SUSPENDACCEPTSUBMISSIONS;
                }
            }
        }
        return OperationMaintenanceManagementConstants.NOOPERATION;
    }

    @XText("查询当前登陆人角色状态")
    @XApiPost
    public XSingleResult<QueryOperationMaintenanceStationApplicationOutput> queryOperationMaintenanceStationType1(XContext context) {
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryOperationMaintenanceStationApplicationOutput output = new QueryOperationMaintenanceStationApplicationOutput();
        //获取流程第一个节点的参与者信息（角色信息）  processCode    session.getId() 获取当前登陆人的角色判断 是否一样
        OperationManagementWorkflowBehaviour behaviour = new OperationManagementWorkflowBehaviour();
        //判断当前用户是否为当前流程的 第一个节点角色
        boolean b = behaviour.checkInRoleOrUserId(context, processCode, session.getId());
        if (b) {
            //当前用户拥有 提报角色
            output.setType(0);
        } else {
            //没有角色
            output.setType(2);
        }
        return XSingleResult.success(output);
    }

    @XText("分页查询场站申请")
    @XApiPost
    public XPageResult<QueryOperationMaintenanceStationApplicationOutput> queryOperationMaintenanceStationApplication(XContext context, QueryOperationMaintenanceStationApplicationInput input) {
        OperationMaintenanceStationApplicationMapper mapper = context.getBean(OperationMaintenanceStationApplicationMapper.class);
        List<QueryOperationMaintenanceStationApplicationOutput> arrayList = new ArrayList();
        QueryWrapper<OperationMaintenanceStationApplicationEnt> queryWrapper = new QueryWrapper<>();
        if (null != input.getAuditstatus()) {
            queryWrapper.lambda().eq(OperationMaintenanceStationApplicationEnt::getAuditstatus, input.getAuditstatus());
        }
        queryWrapper.lambda().orderByAsc(OperationMaintenanceStationApplicationEnt::getCreateTime);
        List<OperationMaintenanceStationApplicationEnt> outputlist = mapper.selectList(queryWrapper);
        PageHelper.startPage(input.getPage(), input.getLimit());
        if (outputlist.size() > 0) {
            for (OperationMaintenanceStationApplicationEnt stationApplicationEnt : outputlist) {
                QueryOperationMaintenanceStationApplicationOutput userOutput = new QueryOperationMaintenanceStationApplicationOutput();
                XCopyUtils.copyObject(stationApplicationEnt, userOutput);
                if (null != stationApplicationEnt.getFileKey() && !stationApplicationEnt.getFileKey().equals("")) {
                    for (int i = 0; i < stationApplicationEnt.getFileKey().split(",").length; i++) {
                        String[] split = stationApplicationEnt.getFileKey().split(",");
                        userOutput.setFileKeyId(Arrays.asList(split));
                    }
                }
                CheckHasTodoInput checkHasTodoInput = new CheckHasTodoInput();
                checkHasTodoInput.setBsnsId(userOutput.getId());
                XSingleResult<QueryOperationMaintenanceStationApplicationOutput> queryBatchMonthlyPlanTypeOutputXSingleResult = queryOperationMaintenanceStationType(context, checkHasTodoInput);
                userOutput.setType(queryBatchMonthlyPlanTypeOutputXSingleResult.getResult().getType());
                arrayList.add(userOutput);
            }
        } /*else {
            XSingleResult<QueryOperationMaintenanceStationApplicationOutput> queryBatchMonthlyPlanTypeOutputXSingleResult = queryOperationMaintenanceStationType1(context);
            QueryOperationMaintenanceStationApplicationOutput output = new QueryOperationMaintenanceStationApplicationOutput();
            output.setType(queryBatchMonthlyPlanTypeOutputXSingleResult.getResult().getType());
            arrayList.add(output);
        }*/
        PageInfo<QueryOperationMaintenanceStationApplicationOutput> pageInfo = new PageInfo<>(arrayList);
        List<QueryOperationMaintenanceStationApplicationOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryOperationMaintenanceStationApplicationOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
