package pps.core.system.service;

import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.primbpm.service.BpsWorkFlowProcessService;
import pps.cloud.primbpm.service.data.*;
import pps.cloud.system.service.SystemUserRoleRelService;
import pps.cloud.system.service.data.GetSysUserRoleRelInput;
import pps.cloud.system.service.data.GetSysUserRoleRelOutput;
import pps.core.common.session.PpsUserSession;
import pps.core.system.constant.OperationMaintenanceManagementConstants;
import pps.core.system.constant.OperationManagement;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.operation_maintenance_adjustment_function_request.TerminateWorkItemPpsInput;
import pps.core.system.service.data.operation_maintenance_user.*;
import pps.core.system.service.data.operation_maintenance_user.ApprovalRejectInput;
import pps.core.system.service.data.sys_user.*;
import pps.core.system.service.data.sys_user_role_rel.CreateSysUserRoleRelInput;
import pps.core.system.service.data.sys_user_role_rel.CreateSysUserRoleRelOutput;
import pps.core.system.utils.OperationManagementWorkflowBehaviour;
import xstartup.annotation.XService;
import xstartup.data.XPageResult;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;

@XText("运维管理-账号申请")
@XService
public class OperationMaintenanceUserService {

    //流程编码
    private final String processCode = "PPS_OPERATION_MAINTENANCE_USER";

    @XText("新增账号申请")
    @XApiPost
    public XServiceResult createOperationMaintenanceUser(XContext context, CreateOperationMaintenanceUserInput input) {
        OperationMaintenanceUserMapper mapper = context.getBean(OperationMaintenanceUserMapper.class);
        OperationMaintenanceUserInformationMapper mapper1 = context.getBean(OperationMaintenanceUserInformationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String uuid = UUID.randomUUID().toString().trim().replaceAll("-", "");
        OperationMaintenanceUserEnt entity = new OperationMaintenanceUserEnt();
        input.setId(uuid);
        input.setCreateTime(new Date());
        input.setmOuId(session.getOuId());
        input.setmOuName(session.getOuName());
        input.setCreateById(session.getId());
        input.setCreateByName(session.getUserName());
        input.setDataDate(new Date());
        if (input.getAlternateFieldsTwo().equals("0")) {
            input.setAuditstatus(OperationManagement.SAVED);
        }
        if (input.getAlternateFieldsTwo().equals("1")) {
            input.setAuditstatus(OperationManagement.UNDER_REVIEW);
        }
        if (null != input.getFileKeyId()) {
            StringBuffer buffer = new StringBuffer();
            for (int i = 0; i < input.getFileKeyId().size(); i++) {
                buffer.append(input.getFileKeyId().get(i) + ",");
            }
            input.setFileKey(String.valueOf(buffer));
        }
        XCopyUtils.copyObject(input, entity);
        for (int i = 0; i < input.getUserList().size(); i++) {
            OperationMaintenanceUserInformationEnt userInformationEnt = new OperationMaintenanceUserInformationEnt();
            String uuid1 = UUID.randomUUID().toString().trim().replaceAll("-", "");
            XCopyUtils.copyObject(input.getUserList().get(i), userInformationEnt);
            if (!userInformationEnt.getLoginAccount().equals(userInformationEnt.getLoginAccountIam())) {
                return XServiceResult.error(989, "登陆账户与IAM账户不一致");
            }
            userInformationEnt.setId(uuid1);
            userInformationEnt.setAccountNumberId(uuid);
            userInformationEnt.setCreateTime(new Date());
            userInformationEnt.setmOuId(session.getOuId());
            userInformationEnt.setmOuName(session.getOuName());
            userInformationEnt.setCreateById(session.getId());
            userInformationEnt.setCreateByName(session.getUserName());
            userInformationEnt.setDataDate(new Date());
            mapper1.insert(userInformationEnt);
        }
        if (input.getAlternateFieldsTwo().equals("1")) {
            StartProcessInput startProcessInput = new StartProcessInput();
            startProcessInput.setProcessCode(this.processCode);
            startProcessInput.setBsnsDetailUrl(input.getAlternateFieldsThree());
            startProcessInput.setBsnsId(input.getId());
            //启动相应的工作流转填报
            bpmService.startProcess(context, startProcessInput);
        }
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("查询用户列表")
    @XApiPost
    public XListResult<UpdateSysUserOutput> selectSysUserViewPage(XContext context, UpdateSysUserInput input) {
        SysUserService userService = context.getBean(SysUserService.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        input.setOuId(session.getOuId());
        List<UpdateSysUserOutput> outputs = new ArrayList<>();
        QuerySysUserViewInput querySysUserViewInput = new QuerySysUserViewInput();
        querySysUserViewInput.setPage(1);
        querySysUserViewInput.setLimit(999);
        XCopyUtils.copyObject(input, querySysUserViewInput);
        XPageResult<QuerySysUserViewOutput> querySysUserViewOutputXPageResult = userService.selectSysUserViewPage(context, querySysUserViewInput);
        for (QuerySysUserViewOutput item : querySysUserViewOutputXPageResult.getResult().getItems()) {
            UpdateSysUserOutput updateSysUserOutput = new UpdateSysUserOutput();
            XCopyUtils.copyObject(item, updateSysUserOutput);
            outputs.add(updateSysUserOutput);
        }
        return XListResult.success(outputs);
    }

    @XText("修改账号申请")
    @XApiPost
    public XServiceResult updateOperationMaintenanceUser(XContext context, UpDateOperationMaintenanceUserInput input) {
        OperationMaintenanceUserMapper mapper = context.getBean(OperationMaintenanceUserMapper.class);
        OperationMaintenanceUserInformationMapper mapper1 = context.getBean(OperationMaintenanceUserInformationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceUserEnt::getId, input.getId());
        OperationMaintenanceUserEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        input.setModifyTime(new Date());
        input.setModifyById(session.getId());
        input.setModifyByName(session.getCreateByName());
        if (null != input.getFileKeyId()) {
            StringBuffer buffer = new StringBuffer();
            for (int i = 0; i < input.getFileKeyId().size(); i++) {
                buffer.append(input.getFileKeyId().get(i) + ",");
            }
            input.setFileKey(String.valueOf(buffer));
        }
        XCopyUtils.copyObject(input, entity);
        if (input.getUserList().size() > 0) {
            for (int i = 0; i < input.getUserList().size(); i++) {
                String id = input.getUserList().get(i).getId();
                QueryWrapper<OperationMaintenanceUserInformationEnt> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.lambda().eq(OperationMaintenanceUserInformationEnt::getId, id);
                OperationMaintenanceUserInformationEnt entity1 = mapper1.selectOne(queryWrapper1);
                if (entity1 == null) {
                    return XServiceResult.error(context, XError.NotFound);
                }
                XCopyUtils.copyObject(input.getUserList().get(i), entity1);
                mapper1.updateById(entity1);
            }
        }
        if (entity.getAuditstatus() == 4 && input.getAlternateFieldsTwo().equals("1")) {
            entity.setAuditstatus(OperationManagement.UNDER_REVIEW);
            //启动相应的工作流转填报
            ApprovalPassInput approvalPassPpsSouthChinaInput = new ApprovalPassInput();
            approvalPassPpsSouthChinaInput.setId(input.getId());
            approvalPassUser(context, approvalPassPpsSouthChinaInput);
        } else {
            if (input.getAlternateFieldsTwo().equals("1")) {
                entity.setAuditstatus(OperationManagement.UNDER_REVIEW);
                StartProcessInput startProcessInput = new StartProcessInput();
                startProcessInput.setProcessCode(this.processCode);
                startProcessInput.setBsnsDetailUrl(input.getAlternateFieldsThree());
                startProcessInput.setBsnsId(input.getId());
                //启动相应的工作流转填报
                bpmService.startProcess(context, startProcessInput);
            }
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("审核通过账号申请")
    @XApiPost
    public XServiceResult approvalPassUser(XContext context, ApprovalPassInput input) {
        OperationMaintenanceUserMapper mapper = context.getBean(OperationMaintenanceUserMapper.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceUserEnt::getId, input.getId());
        OperationMaintenanceUserEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        //调用工作流通过接口
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        FinishWorkItemDirectlyInput finishWorkItemDirectlyInput = new FinishWorkItemDirectlyInput();
        //流程编号
        finishWorkItemDirectlyInput.setWorkItemId(input.getWorkItemId());
        finishWorkItemDirectlyInput.setProcessCode(processCode);
        finishWorkItemDirectlyInput.setRemark(input.getRemark());
        finishWorkItemDirectlyInput.setBsnsId(entity.getId());
        XServiceResult xServiceResult = bpmService.approvalPass(context, finishWorkItemDirectlyInput);
        xServiceResult.throwIfFail();
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        GetProcessInstStateInput getProcessInstStateInput = new GetProcessInstStateInput();
        getProcessInstStateInput.setBsnsId(input.getId());
        getProcessInstStateInput.setProcessCode(processCode);
        XSingleResult<GetProcessInstStateOutput> getProcessInstStateOutputXSingleResult = bpmService.queryProcessStatus(context, getProcessInstStateInput);
        if (getProcessInstStateOutputXSingleResult.getResult().getState() == 2) {
            entity.setAuditstatus(OperationManagement.UNDER_REVIEW);
        } else if (getProcessInstStateOutputXSingleResult.getResult().getState() == 7) {
            entity.setAuditstatus(OperationManagement.PASSED);
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("审核驳回账号申请")
    @XApiPost
    public XServiceResult approvalReject(XContext context, ApprovalRejectInput input) {
        OperationMaintenanceUserMapper mapper = context.getBean(OperationMaintenanceUserMapper.class);
        QueryWrapper<OperationMaintenanceUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceUserEnt::getId, input.getId());
        OperationMaintenanceUserEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        //调用工作流通过接口
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        pps.cloud.primbpm.service.data.ApprovalRejectInput approvalRejectInput = new pps.cloud.primbpm.service.data.ApprovalRejectInput();
        //获取参数
        if (input.getBackReason() == null) {
            return XServiceResult.error(989, "驳回原因为空");
        }
        approvalRejectInput.setBackReason(input.getBackReason());
        approvalRejectInput.setBsnsId(entity.getId());
        approvalRejectInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalReject(context, approvalRejectInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationManagement.REJECT);
        entity.setAlternateFieldsTwo("0");
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("中止账户申请")
    @XApiPost
    public XServiceResult discontinueOperationMaintenanceUser(XContext context, TerminateWorkItemPpsInput input) {
        OperationMaintenanceUserMapper mapper = context.getBean(OperationMaintenanceUserMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceUserEnt::getId, input.getId());
        OperationMaintenanceUserEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        pps.cloud.primbpm.service.data.TerminateWorkItemInput workItemInput = new pps.cloud.primbpm.service.data.TerminateWorkItemInput();
        workItemInput.setRemark(input.getRemark());
        workItemInput.setBsnsId(entity.getId());
        workItemInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalStop(context, workItemInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationManagement.TERMINATION);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("接受需求账户申请")
    @XApiPost
    public XServiceResult acceptDemandOperationMaintenanceUser(XContext context, TerminateWorkItemPpsInput input) {
        OperationMaintenanceUserMapper mapper = context.getBean(OperationMaintenanceUserMapper.class);
        OperationMaintenanceUserInformationMapper mapper1 = context.getBean(OperationMaintenanceUserInformationMapper.class);
        SysUserService bean = context.getBean(SysUserService.class);
        SysUserRoleRelService roleRelService = context.getBean(SysUserRoleRelService.class);
        SystemUserRoleRelService userRoleRelService = context.getBean(SystemUserRoleRelService.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceUserEnt::getId, input.getId());
        OperationMaintenanceUserEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        pps.cloud.primbpm.service.data.TerminateWorkItemInput workItemInput = new pps.cloud.primbpm.service.data.TerminateWorkItemInput();
        workItemInput.setRemark(input.getRemark());
        workItemInput.setBsnsId(entity.getId());
        workItemInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalAcceptDemand(context, workItemInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationManagement.ACCEPTDEMAND);
        QueryWrapper<OperationMaintenanceUserInformationEnt> userEntQueryWrapper = new QueryWrapper<>();
        userEntQueryWrapper.lambda().eq(OperationMaintenanceUserInformationEnt::getAccountNumberId, entity.getId());
        List<OperationMaintenanceUserInformationEnt> selectList = mapper1.selectList(userEntQueryWrapper);
        if (selectList.size() > 0) {
            for (int i = 0; i < selectList.size(); i++) {
                CreateSysUserInput createSysUserInput = new CreateSysUserInput();
                try {
                    createSysUserInput.setOuId(selectList.get(i).getAffiliatedUnitId());
                    createSysUserInput.setLoginName(selectList.get(i).getLoginAccount());
                    createSysUserInput.setIamLoginName(selectList.get(i).getLoginAccountIam());
                    createSysUserInput.setUserCode(selectList.get(i).getEmployeeNumber());
                    createSysUserInput.setUserName(selectList.get(i).getUserName());
                    createSysUserInput.setPassword(selectList.get(i).getUserPassword());
                    createSysUserInput.setIsInner(selectList.get(i).getIntranet());
                    createSysUserInput.setIsOuter(selectList.get(i).getExternalNetwork());
                    createSysUserInput.setDuty(selectList.get(i).getDutiesName());
                    createSysUserInput.setPostalCode(selectList.get(i).getZipCode());
                    createSysUserInput.setPhone(selectList.get(i).getMobilePhone());
                    createSysUserInput.setEmail(selectList.get(i).getMailbox());
                    createSysUserInput.setAddress(selectList.get(i).getMailAddress());
                    bean.createSysUser(context, createSysUserInput);
                    GetSysUserRoleRelInput getSysUserRoleRelInput = new GetSysUserRoleRelInput();
                    getSysUserRoleRelInput.setUserId(entity.getAccountConsistentId());
                    //根据用户id查询所有角色
                    XListResult<GetSysUserRoleRelOutput> sysRoleByUserId = userRoleRelService.getSysRoleByUserId(context, getSysUserRoleRelInput);
                    if (sysRoleByUserId.getResult().size() > 0) {
                        List<String> roleList = new ArrayList<>();
                        for (GetSysUserRoleRelOutput getSysUserRoleRelOutput : sysRoleByUserId.getResult()) {
                            roleList.add(getSysUserRoleRelOutput.getRoleId());
                        }
                        CreateSysUserRoleRelInput createSysUserRoleRelInput = new CreateSysUserRoleRelInput();
                        createSysUserRoleRelInput.setRoleIdList(roleList);
                        QuerySysUserViewInput querySysUserViewInput = new QuerySysUserViewInput();
                        querySysUserViewInput.setPage(1);
                        querySysUserViewInput.setLimit(20);
                        querySysUserViewInput.setLoginName(createSysUserInput.getLoginName());
                        //根据登陆账号查询用户的id
                        XPageResult<QuerySysUserViewOutput> pageResult = bean.selectSysUserViewPage(context, querySysUserViewInput);
                        for (QuerySysUserViewOutput item : pageResult.getResult().getItems()) {
                            createSysUserRoleRelInput.setUserId(item.getId());
                            //用户绑定多角色关系
                            roleRelService.createSysUserRoleRel(context, createSysUserRoleRelInput);
                        }
                    }
                } catch (Exception e) {
                    return XServiceResult.error(e);
                }
            }
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id查询所有信息")
    @XApiPost
    public XSingleResult<OperationMaintenanceUserOutput> queryOperationMaintenanceUserOutput(XContext context, OperationMaintenanceUserInput input) {
        OperationMaintenanceUserMapper mapper = context.getBean(OperationMaintenanceUserMapper.class);
        OperationMaintenanceUserInformationMapper mapper1 = context.getBean(OperationMaintenanceUserInformationMapper.class);
        QueryWrapper<OperationMaintenanceUserEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceUserEnt::getId, input.getId());
        OperationMaintenanceUserEnt userEnt = mapper.selectOne(queryWrapper);
        QueryWrapper<OperationMaintenanceUserInformationEnt> queryWrapper1 = new QueryWrapper<>();
        OperationMaintenanceUserInformationEnt ent = new OperationMaintenanceUserInformationEnt();
        ent.setAccountNumberId(userEnt.getId());
        queryWrapper1.lambda().eq(OperationMaintenanceUserInformationEnt::getAccountNumberId, ent.getAccountNumberId());
        List<OperationMaintenanceUserInformationEnt> entList = mapper1.selectList(queryWrapper1);
        List<OperationMaintenanceUserInformationEnt> arrayList1 = new ArrayList();
        if (entList.size() > 0) {
            for (OperationMaintenanceUserInformationEnt operationMaintenanceUserInformationEnt : entList) {
                arrayList1.add(operationMaintenanceUserInformationEnt);
            }
        }
        OperationMaintenanceUserOutput userOutput = new OperationMaintenanceUserOutput();
        XCopyUtils.copyObject(userEnt, userOutput);
        userOutput.setUserList(arrayList1);
        if (null != userOutput.getFileKey() && !userOutput.getFileKey().equals("")) {
            for (int i = 0; i < userOutput.getFileKey().split(",").length; i++) {
                String[] split = userOutput.getFileKey().split(",");
                userOutput.setFileKeyId(Arrays.asList(split));
            }
        }
        CheckHasTodoInput checkHasTodoInput = new CheckHasTodoInput();
        checkHasTodoInput.setBsnsId(userOutput.getId());
        XSingleResult<QueryOperationMaintenanceUserOutput> queryBatchMonthlyPlanTypeOutputXSingleResult = queryOperationMaintenanceUserType(context, checkHasTodoInput);
        int type = queryBatchMonthlyPlanTypeOutputXSingleResult.getResult().getType();
        userOutput.setType(type);
        return XSingleResult.success(userOutput);
    }

    @XText("查询当前登陆人角色状态")
    public int queryUsersRoles(XContext context) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysRoleViewMapper sysRoleViewMapper = context.getBean(SysRoleViewMapper.class);
        List<SysRoleExtendView> roleList = sysRoleViewMapper.getRoleListUserId(session.getId());
        if (!CollectionUtils.isEmpty(roleList)) {
            for (int i = 0; i < roleList.size(); i++) {
                SysRoleExtendView view = roleList.get(i);
                if (view.getRoleName().equals("在线运维管理_三线运维人员")) {
                    return OperationMaintenanceManagementConstants.DISCONTINUE;
                } else if (view.getRoleName().equals("PPS运维组")) {
                    return OperationMaintenanceManagementConstants.SUSPENSIONACCEPTANCE;
                } else if (view.getRoleName().equals("PPS技术组")) {
                    return OperationMaintenanceManagementConstants.SUSPENDACCEPTSUBMISSIONS;
                }
            }
        }
        return OperationMaintenanceManagementConstants.NOOPERATION;
    }

    /*
    *   填报+审核：审核通过+审核驳回（3）
        填报+审核：中止+审核通过+审核驳回（8）
        填报+审核：接受需求+中止+审核通过+审核驳回（9）
        填报+审核：接受需求+中止+审核驳回（10）

        填报：提交（0）

        审核：审核通过+审核驳回（1）
        审核：中止+审核通过+审核驳回（5）
        审核：接受需求+中止+审核通过+审核驳回（6）
        审核：接受需求+中止+审核驳回（7）

        没有角色：（2）
    *
    * */
    @XText("查询当前登陆人角色状态")
    @XApiPost
    public XSingleResult<QueryOperationMaintenanceUserOutput> queryOperationMaintenanceUserType(XContext context, CheckHasTodoInput input) {
        //查询流程下的所有角色
        BpsWorkFlowProcessService bean = context.getBean(BpsWorkFlowProcessService.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryOperationMaintenanceUserOutput output = new QueryOperationMaintenanceUserOutput();
        QueryProcessRolesInput queryProcessRolesInput = new QueryProcessRolesInput();
        queryProcessRolesInput.setProcessCode(processCode);
        //获取流程第一个节点的参与者信息（角色信息）  processCode    session.getId() 获取当前登陆人的角色判断 是否一样
        OperationManagementWorkflowBehaviour behaviour = new OperationManagementWorkflowBehaviour();
        //判断当前用户是否为当前流程的 第一个节点角色
        boolean b = behaviour.checkInRoleOrUserId(context, processCode, session.getId());
        //当前用户同时拥有 提报跟审批角色 判断当前用户是否有代办
        XSingleResult<CheckHasTodoOutput> checkHasTodoOutputXSingleResult = bean.checkHasTodo(context, input);
        int roles = queryUsersRoles(context);
        if (checkHasTodoOutputXSingleResult.getResult().getHasTodo() && b) {
            if (roles == 4) {
                output.setType(3);
            } else if (roles == 5) {
                output.setType(8);
            } else if (roles == 6) {
                output.setType(9);
            } else if (roles == 7) {
                output.setType(10);
            }
        } else if (b) {
            //当前用户拥有 提报角色
            output.setType(0);
        } else if (checkHasTodoOutputXSingleResult.getResult().getHasTodo()) {
            //审核角色
            if (roles == 4) {
                output.setType(1);
            } else if (roles == 5) {
                output.setType(5);
            } else if (roles == 6) {
                output.setType(6);
            } else if (roles == 7) {
                output.setType(7);
            }
        } else {
            //没有角色
            output.setType(2);
        }
        return XSingleResult.success(output);
    }

    @XText("查询当前登陆人角色状态")
    @XApiPost
    public XSingleResult<QueryOperationMaintenanceUserOutput> queryOperationMaintenanceUserType1(XContext context) {
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryOperationMaintenanceUserOutput output = new QueryOperationMaintenanceUserOutput();
        //获取流程第一个节点的参与者信息（角色信息）  processCode    session.getId() 获取当前登陆人的角色判断 是否一样
        OperationManagementWorkflowBehaviour behaviour = new OperationManagementWorkflowBehaviour();
        //判断当前用户是否为当前流程的 第一个节点角色
        boolean b = behaviour.checkInRoleOrUserId(context, processCode, session.getId());
        if (b) {
            //当前用户拥有 提报角色
            output.setType(0);
        } else {
            //没有角色
            output.setType(2);
        }
        return XSingleResult.success(output);
    }

    @XText("分页查询账号申请")
    @XApiPost
    public XPageResult<QueryOperationMaintenanceUserOutput> queryOperationMaintenanceUser(XContext context, QueryOperationMaintenanceUserInput input) {
        OperationMaintenanceUserInformationMapper mapper1 = context.getBean(OperationMaintenanceUserInformationMapper.class);
        OperationMaintenanceUserMapper mapper = context.getBean(OperationMaintenanceUserMapper.class);
        List<QueryOperationMaintenanceUserOutput> arrayList = new ArrayList();
        QueryWrapper<OperationMaintenanceUserEnt> queryWrapper = new QueryWrapper<>();
        QueryWrapper<OperationMaintenanceUserInformationEnt> queryWrapper1 = new QueryWrapper<>();
        if (null != input.getAuditstatus()) {
            queryWrapper.lambda().eq(OperationMaintenanceUserEnt::getAuditstatus, input.getAuditstatus());
        }
        queryWrapper.lambda().orderByAsc(OperationMaintenanceUserEnt::getCreateTime);
        List<OperationMaintenanceUserEnt> outputlist = mapper.selectList(queryWrapper);
        PageHelper.startPage(input.getPage(), input.getLimit());
        if (outputlist.size() > 0) {
            for (OperationMaintenanceUserEnt operationMaintenanceUserEnt : outputlist) {
                OperationMaintenanceUserInformationEnt ent = new OperationMaintenanceUserInformationEnt();
                ent.setAccountNumberId(operationMaintenanceUserEnt.getId());
                queryWrapper1.lambda().eq(OperationMaintenanceUserInformationEnt::getAccountNumberId, ent.getAccountNumberId());
                List<OperationMaintenanceUserInformationEnt> entList = mapper1.selectList(queryWrapper1);
                List<OperationMaintenanceUserInformationEnt> arrayList1 = new ArrayList();
                if (entList.size() > 0) {
                    for (OperationMaintenanceUserInformationEnt operationMaintenanceUserInformationEnt : entList) {
                        arrayList1.add(operationMaintenanceUserInformationEnt);
                    }
                }
                QueryOperationMaintenanceUserOutput userOutput = new QueryOperationMaintenanceUserOutput();
                XCopyUtils.copyObject(operationMaintenanceUserEnt, userOutput);
                userOutput.setUserList(arrayList1);
                queryWrapper1.clear();
                if (null != operationMaintenanceUserEnt.getFileKey() && !operationMaintenanceUserEnt.getFileKey().equals("")) {
                    for (int i = 0; i < operationMaintenanceUserEnt.getFileKey().split(",").length; i++) {
                        String[] split = operationMaintenanceUserEnt.getFileKey().split(",");
                        userOutput.setFileKeyId(Arrays.asList(split));
                    }
                }
                CheckHasTodoInput checkHasTodoInput = new CheckHasTodoInput();
                checkHasTodoInput.setBsnsId(userOutput.getId());
                XSingleResult<QueryOperationMaintenanceUserOutput> queryBatchMonthlyPlanTypeOutputXSingleResult = queryOperationMaintenanceUserType(context, checkHasTodoInput);
                userOutput.setType(queryBatchMonthlyPlanTypeOutputXSingleResult.getResult().getType());
                arrayList.add(userOutput);
            }
        } /*else {
            XSingleResult<QueryOperationMaintenanceUserOutput> queryBatchMonthlyPlanTypeOutputXSingleResult = queryOperationMaintenanceUserType1(context);
            QueryOperationMaintenanceUserOutput output = new QueryOperationMaintenanceUserOutput();
            output.setType(queryBatchMonthlyPlanTypeOutputXSingleResult.getResult().getType());
            arrayList.add(output);
        }*/
        PageInfo<QueryOperationMaintenanceUserOutput> pageInfo = new PageInfo<>(arrayList);
        List<QueryOperationMaintenanceUserOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryOperationMaintenanceUserOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
