package pps.core.system.service;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.util.CollectionUtils;
import pps.cloud.dispatch.service.DispReceiveGroupReceiverCloudService;
import pps.cloud.dispatch.service.data.GetDispReceiveGroupReceiverInput;
import pps.cloud.dispatch.service.data.GetDispReceiveGroupReceiverOutput;
import pps.cloud.sms.service.SmsService;
import pps.cloud.sms.service.data.MessageParam;
import pps.core.system.constant.SysInformReceiveTypeConstant;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_inform.SendSysInformInput;
import pps.core.system.utils.UserInfoUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;

import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@XService
@XText("消息通知-发送")
public class SysInformSendService {
    public XServiceResult sysInformSend(XContext context, SendSysInformInput input) {
        //获取消息主体
        SysInformMapper mapper = context.getBean(SysInformMapper.class);
        QueryWrapper<SysInformEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(SysInformEnt::getId, input.getId());
        SysInformEnt ent = mapper.selectOne(wrapper);
        if (ent == null)
            return XServiceResult.OK;
        //判断接收方式，根据不同的接收方式，获取需要发送的人员信息
        SysInformReceiveTypeMapper receiveTypeMapper = context.getBean(SysInformReceiveTypeMapper.class);
        QueryWrapper<SysInformReceiveTypeEnt> receiveTypeEntWrapper = new QueryWrapper<>();
        receiveTypeEntWrapper.lambda().eq(SysInformReceiveTypeEnt::getInformId, input.getId());
        List<SysInformReceiveTypeEnt> receiveTypeList = receiveTypeMapper.selectList(receiveTypeEntWrapper);
        SysInformReceiveTypeDetailMapper receiveTypeDetailMapper = context.getBean(SysInformReceiveTypeDetailMapper.class);
        QueryWrapper<SysInformReceiveTypeDetailEnt> receiveTypeDetailEntWrapper = new QueryWrapper<>();
        receiveTypeDetailEntWrapper.lambda().eq(SysInformReceiveTypeDetailEnt::getInformId, input.getId());
        List<SysInformReceiveTypeDetailEnt> receiveTypeDetailList = receiveTypeDetailMapper.selectList(receiveTypeDetailEntWrapper);
        List<SysInformReceiverView> receiverViews = new ArrayList<>();
        List<SysInformReceiverView> smsViews = new ArrayList<>();
        Date date = new Date();
        for (SysInformReceiveTypeEnt receiveTypeEnt : receiveTypeList) {
            //根据类型，获取类型下所有的子数据
            List<SysInformReceiveTypeDetailEnt> groupDetailList = receiveTypeDetailList.stream().filter(detail -> receiveTypeEnt.getId().equals(detail.getInformRecieveTypeId())).collect(Collectors.toList());
            //根据类型，获取子数据具体的发送对象
            if (null == receiveTypeEnt.getRecieveType())
                receiveTypeEnt.setRecieveType(SysInformReceiveTypeConstant.USER.getValue());
            List<SysUserEnt> userEntList = new ArrayList<>();
            switch (receiveTypeEnt.getRecieveType()) {
                case 1:
                    //人员，直接从人员表中获取详细数据
                    List<String> userIds = groupDetailList.stream().map(SysInformReceiveTypeDetailEnt::getObjId).collect(Collectors.toList());
                    SysUserMapper userMapper = context.getBean(SysUserMapper.class);
                    QueryWrapper<SysUserEnt> userWrapper = new QueryWrapper<>();
                    userWrapper.lambda().in(SysUserEnt::getId, userIds);
                    userEntList = userMapper.selectList(userWrapper);
                    addReceiver(receiverViews, smsViews, ent, userEntList, date);
                    break;
                case 2:
                    //收件组
                    List<String> groupIds = groupDetailList.stream().map(SysInformReceiveTypeDetailEnt::getObjId).collect(Collectors.toList());
                    DispReceiveGroupReceiverCloudService groupReceiverCloudService = context.getBean(DispReceiveGroupReceiverCloudService.class);
                    GetDispReceiveGroupReceiverInput receiverInput = new GetDispReceiveGroupReceiverInput();
                    receiverInput.setGroupIds(groupIds);
                    List<GetDispReceiveGroupReceiverOutput> receiverOutputXListResult = groupReceiverCloudService.getDispReceiveGroupList(context, receiverInput).getResult();
                    if (!CollectionUtils.isEmpty(receiverOutputXListResult)) {
                        for (GetDispReceiveGroupReceiverOutput receiverOutput : receiverOutputXListResult) {
                            if (receiverOutput.getReceiveType() == 1) {
                                SysUserEnt userEnt = new SysUserEnt();
                                userEnt.setPhone(receiverOutput.getUserPhone());
                                userEnt.setId(receiverOutput.getUserId());
                                userEnt.setUserName(receiverOutput.getUserName());
                                userEntList.add(userEnt);
                            }
                        }
                        //取角色
                        List<String> roleIds = receiverOutputXListResult.stream().filter(out -> out.getReceiveType() == 2).map(GetDispReceiveGroupReceiverOutput::getRoleId).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(roleIds)) {
                            SysUserRoleRelViewMapper userRoleRelViewMapper = context.getBean(SysUserRoleRelViewMapper.class);
                            SysUserRoleRelView record = new SysUserRoleRelView();
                            record.setRoleIds(roleIds);
                            List<SysUserRoleRelView> roleRelViews = userRoleRelViewMapper.selectListByRoleIds(record);
                            List<SysUserEnt> roleUserEntList = XCopyUtils.copyNewList(roleRelViews, SysUserEnt.class);
                            userEntList.addAll(roleUserEntList);
                        }
                        addReceiver(receiverViews, smsViews, ent, userEntList, date);
                    }
                    break;
                case 3:
                    //角色，获取角色下面的所有用户
                    SysUserRoleRelViewMapper userRoleRelViewMapper = context.getBean(SysUserRoleRelViewMapper.class);
                    SysUserRoleRelView record = new SysUserRoleRelView();
                    List<String> roleIds = groupDetailList.stream().map(SysInformReceiveTypeDetailEnt::getObjId).collect(Collectors.toList());
                    record.setRoleIds(roleIds);
                    List<SysUserRoleRelView> roleRelViews = userRoleRelViewMapper.selectListByRoleIds(record);
                    userEntList = XCopyUtils.copyNewList(roleRelViews, SysUserEnt.class);
                    addReceiver(receiverViews, smsViews, ent, userEntList, date);
                    break;
                case 4:
                    //机构
                    List<String> orgIds = groupDetailList.stream().map(SysInformReceiveTypeDetailEnt::getObjId).collect(Collectors.toList());
                    SysUserViewMapper sysUserViewMapper = context.getBean(SysUserViewMapper.class);
                    SysUserView sysUserView = new SysUserView();
                    sysUserView.setOuIds(orgIds);
                    List<SysUserView> userViewList = sysUserViewMapper.getSysUserListByOuIds(sysUserView);
                    userEntList = XCopyUtils.copyNewList(userViewList, SysUserEnt.class);
                    addReceiver(receiverViews, smsViews, ent, userEntList, date);
                    break;
                case 5:
                    //角色-提交人本级单位

                case 6:
                    //角色-提交人二级单位

                case 7:
                    //角色-提交人所属企业
                    //角色+组织组织机构
                    SysUserRoleRelViewMapper roleRelViewMapper = context.getBean(SysUserRoleRelViewMapper.class);
                    List<SysUserRoleRelView> relViews = new ArrayList<>();
                    for (SysInformReceiveTypeDetailEnt detailEnt : groupDetailList) {
                        SysUserRoleRelView roleRelView = new SysUserRoleRelView();
                        roleRelView.setRoleId(detailEnt.getObjId());
                        roleRelView.setOuId(detailEnt.getOuId());
                        relViews.addAll(roleRelViewMapper.selectListByRoleIdAndOuId(roleRelView));
                    }
                    List<SysUserRoleRelView> distinctViews = relViews.stream().distinct().collect(Collectors.toList());
                    userEntList = XCopyUtils.copyNewList(distinctViews, SysUserEnt.class);
                    addReceiver(receiverViews, smsViews, ent, userEntList, date);
                    break;
                case 8:
                    //存储过程
                    SysInformReceiverViewMapper receiverViewMapper = context.getBean(SysInformReceiverViewMapper.class);
                    List<SysUserEnt> userEntFunctionList = new ArrayList<>();
                    for (SysInformReceiveTypeDetailEnt detailEnt : groupDetailList) {
                        List<LinkedHashMap<String, Object>> userMapList = receiverViewMapper.selectCommon(detailEnt.getObjId());
                        for (LinkedHashMap<String, Object> userMap : userMapList) {
                            SysUserEnt sysUserEnt = new SysUserEnt();
                            for (Map.Entry<String, Object> entry : userMap.entrySet()) {
                                //4.1.1将取到的字段与userEnt进行匹配
                                String column = toCamelCase(entry.getKey());
                                Object columnValue = entry.getValue();
                                setCommonTranferKeyValue(sysUserEnt ,column ,  columnValue);
                            }
                            userEntFunctionList.add(sysUserEnt);
                        }
                    }
                    addReceiver(receiverViews, smsViews, ent, userEntFunctionList, date);
                    break;
                case 9:
                    //全部用户
                    SysUserMapper sysUserMapper = context.getBean(SysUserMapper.class);
                    List<SysUserEnt> userEntAllList = sysUserMapper.selectList(new QueryWrapper<>());
                    addReceiver(receiverViews, smsViews, ent, userEntAllList, date);
                    break;
                case 10:
                    //机构（含子级）
                    List<String> orgIdList = groupDetailList.stream().map(SysInformReceiveTypeDetailEnt::getObjId).collect(Collectors.toList());
                    SysOrganizationViewMapper organizationViewMapper = context.getBean(SysOrganizationViewMapper.class);
                    List<String> orgId = new ArrayList<>();
                    for(String org : orgIdList){
                        SysOrganizationView view = new SysOrganizationView();
                        view.setIsDeleted(1);
                        view.setId(org);
                        List<SysOrganizationView> orgList = organizationViewMapper.selectAllListByOuId(view);
                        if(!CollectionUtils.isEmpty(orgList))
                            orgId.addAll(orgList.stream().map(SysOrganizationView::getId).distinct().collect(Collectors.toList()));
                    }
                    orgId = orgId.stream().distinct().collect(Collectors.toList());
                    SysUserViewMapper userViewMapper = context.getBean(SysUserViewMapper.class);
                    SysUserView userView = new SysUserView();
                    userView.setOuIds(orgId);
                    List<SysUserView> userList = userViewMapper.getSysUserListByOuIds(userView);
                    userEntList = XCopyUtils.copyNewList(userList, SysUserEnt.class);
                    addReceiver(receiverViews, smsViews, ent, userEntList, date);
                    break;
            }
        }
        //根据发送方式，发送系统消息或者短信
        if (!CollectionUtils.isEmpty(receiverViews)) {
            //插入前，先删除历史数据
            SysInformReceiverMapper receiverMapper = context.getBean(SysInformReceiverMapper.class);
            HashMap<String, Object> deleteMapper = new HashMap();
            deleteMapper.put("inform_id", ent.getId());
            receiverMapper.deleteByMap(deleteMapper);
            //批量插入数据，并判断数据是否需要发送短信
            List<SysInformReceiverView> distinct = receiverViews.stream()
                    .collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
                            new TreeSet<>(Comparator.comparing(o -> o.getUserId() + ";" + o.getMsgType()))), ArrayList::new));
            SysInformReceiverViewMapper receiverViewMapper = context.getBean(SysInformReceiverViewMapper.class);
            List<List<SysInformReceiverView>> splitList =  Stream.iterate(0, n -> n + 1)
                    .limit(distinct.size())
                    .parallel()
                    .map(a -> distinct.parallelStream().skip(a * 500).limit(500).collect(Collectors.toList()))
                    .filter(b -> !b.isEmpty())
                    .collect(Collectors.toList());
            splitList.stream().forEach(item->{
                receiverViewMapper.batchInsert(item);
            });
            if (ent.getMobileMsg() == 1 && !CollectionUtils.isEmpty(smsViews)) {
                try {
                    List<String> phones = smsViews.stream().map(SysInformReceiverView::getMobile).collect(Collectors.toList());
                    SmsService smsService = context.getBean(SmsService.class);
                    MessageParam messageParam = new MessageParam();
                    messageParam.setVariable(ent.getMsg());
                    messageParam.setPhoneNum(String.join(",", phones));
                    XServiceResult result = smsService.sendMessage(context, messageParam);
                    result.throwIfFail();
                    smsViews.stream().forEach(sms -> {
                        SysInformReceiverEnt receiverEnt = XCopyUtils.copyNewObject(sms, SysInformReceiverEnt.class);
                        receiverEnt.setStatus(1);
                        receiverMapper.updateById(receiverEnt);
                    });
                } catch (Exception e) {
                    context.getLogger().error(e.getMessage());
                }
            }
        }

        return XServiceResult.OK;
    }

    private void addReceiver(List<SysInformReceiverView> receiverViews, List<SysInformReceiverView> smsViews, SysInformEnt ent, List<SysUserEnt> userEntList, Date date) {
        userEntList.stream().forEach(i -> {
            if (!XStringUtils.isEmpty(i.getId())) {
                SysInformReceiverView receiverView = new SysInformReceiverView();
                receiverView.setInformId(ent.getId());
//            receiverView.setOuId(i.getOuId());
//            receiverView.setOuName(i.getOuName());
                receiverView.setUserId(i.getId());
                receiverView.setUserName(i.getUserName());
                if(XStringUtils.isNotEmpty(i.getPhone()) && i.getPhone().length() > 15)
                    receiverView.setMobile(UserInfoUtil.decrypt(i.getPhone()));
                else
                    receiverView.setMobile(i.getPhone());
                receiverView.setSendOuId(ent.getOuId());
                receiverView.setSendOuName(ent.getOuName());
                receiverView.setSendUserId(ent.getUserId());
                receiverView.setSendUserName(ent.getUserName());
                receiverView.setSendTime(date);
                receiverView.setTitle(ent.getTitle());
                receiverView.setMsg(ent.getMsg());
                if(null != ent.getEffectiveDays())
                    receiverView.setInvalidTime(XDateUtils.addDays(new Date() , ent.getEffectiveDays()));
                receiverView.setStatus(0);
                //判断系统消息和短信
                if (ent.getMobileMsg() == 1) {
                    SysInformReceiverView mobleMsg = XCopyUtils.copyNewObject(receiverView, SysInformReceiverView.class);
                    mobleMsg.setMsgType(1);
                    mobleMsg.setId(UUID.randomUUID().toString());
                    mobleMsg.setStatus(1);
                    receiverViews.add(mobleMsg);
                    if (XStringUtils.isNotEmpty(i.getPhone()))
                        smsViews.add(receiverView);
                }
                if (ent.getSystemMsg() == 1) {
                    SysInformReceiverView systemMsg = XCopyUtils.copyNewObject(receiverView, SysInformReceiverView.class);
                    systemMsg.setMsgType(0);
                    systemMsg.setId(UUID.randomUUID().toString());
                    receiverViews.add(systemMsg);
                }
            }
        });
    }

    @XText("发送消息")
    @XApiAnonymous
    @XApiPost
    public XServiceResult sendSysInform(XContext context, SendSysInformInput input) {
        sysInformSend(context, input);
        return XServiceResult.OK;
    }

    public void setCommonTranferKeyValue(Object object, String column, Object fieldValue) {
        try {
            if (XStringUtils.isNotEmpty(column)) {
                Field field = object.getClass().getDeclaredField(column);
                field.setAccessible(true);
                field.set(object, fieldValue);
            }
        } catch (Exception e) {

        }
    }

    /**
     * 驼峰式命名法
     */
    public static String toCamelCase(String s) {
        if (s == null) {
            return null;
        }
        s = s.toLowerCase();
        StringBuilder sb = new StringBuilder(s.length());
        boolean upperCase = false;
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);

            if (c == '_') {
                upperCase = true;
            } else if (upperCase) {
                sb.append(Character.toUpperCase(c));
                upperCase = false;
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }


}

