package pps.core.system.service;

import java.util.*;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import pps.cloud.primbpm.service.BpsWorkFlowProcessService;
import pps.cloud.primbpm.service.data.DeleteOrgRoleInput;
import pps.cloud.primbpm.service.data.UpdateOrgRoleInput;
import pps.cloud.primbpm.service.data.UpdateOrgRoleUserInput;
import pps.cloud.primbpm.service.data.UpdateOrgRoleUserOutput;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.common.constant.UserConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.system.config.LoginConfig;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.sys_role.*;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import xstartup.annotation.XService;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XStringUtils;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import xstartup.feature.mybatis.helper.XMapperHelper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.helper.XTransactionHelper;

@XText("角色管理")
@XService
public class SysRoleService {

    @XText("查询角色列表")
    @XApiPost
    public XPageResult<QuerySysRoleViewOutput> selectSysRoleViewPage(XContext context, QuerySysRoleViewInput input){
        SysRoleViewMapper viewMapper = context.getBean(SysRoleViewMapper.class);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        SysRoleView record = new SysRoleView();
        XCopyUtils.copyObject(input, record);

        List<String> mediaTypeList = new ArrayList<>();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        //如果不是超级管理员只查询当前用户的组织机构下的角色以及上级可继承的角色
        if (UserConstant.ISSUPERADMIN_FALSE.equals(session.getIsSuperAdmin())){
            if (XStringUtils.isEmpty(input.getOuId())){
                record.setOuId(session.getOuId());
                input.setOuId(session.getOuId());
            }

            if (XStringUtils.isEmpty(input.getMediaType())){
                mediaTypeList.add("ALL");
                //普通角色只查询所属企业的介质类型
                SysOrganizationPropertyViewMapper mapper = context.getBean(SysOrganizationPropertyViewMapper.class);
                SysOrganizationPropertyView propertyView = new SysOrganizationPropertyView();
                propertyView.setOuId(session.getOuId());
                propertyView.setEndTime(new Date());
                List<SysOrganizationPropertyView> propertyViews = mapper.selectList(propertyView);

                if (!CollectionUtils.isEmpty(propertyViews)){
                    for (SysOrganizationPropertyView propertyEnt : propertyViews) {
                        mediaTypeList.add(propertyEnt.getPropertyValue());
                    }
                    if (!CollectionUtils.isEmpty(mediaTypeList)){
                        record.setMediaTypes(mediaTypeList);
                    }
                }

            }
        }
        List<SysRoleView> list = viewMapper.selectSysRoleListByOuId(record);
        long total=0;
        //查询上级可继承的角色
        if (XStringUtils.isNotEmpty(input.getOuId())){
            Date date = new Date();
            SysOrganizationRelMapper organizationRelMapper = context.getBean(SysOrganizationRelMapper.class);
            QueryWrapper<SysOrganizationRelEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysOrganizationRelEnt::getOuId,input.getOuId())
            .le(true,SysOrganizationRelEnt::getEffectTime,date)
            .ge(true,SysOrganizationRelEnt::getEndTime,date);
            List<SysOrganizationRelEnt> ouRelEnts = organizationRelMapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(ouRelEnts)){
                SysOrganizationRelEnt relEnt = ouRelEnts.get(0);
                String ouIdPath = relEnt.getOuIdPath();
                if (XStringUtils.isNotEmpty(ouIdPath)){
                    String[] split = ouIdPath.split("/");
                    List<String> ouIds = Arrays.asList(split);
                    ouIds=ouIds.stream().filter(id->!id.equals(input.getOuId())).collect(Collectors.toList());
                    record.setOuIds(ouIds);
                    //下级单位是否可继承:0->否；1->是；
                    if (ouIds.size()>0){
                        if (XStringUtils.isNotEmpty(input.getMediaType())){
                            record.setMediaType(input.getMediaType());
                        }else{
                            if (UserConstant.ISSUPERADMIN_FALSE.equals(session.getIsSuperAdmin())){
                                if (!CollectionUtils.isEmpty(mediaTypeList)){
                                    record.setMediaTypes(mediaTypeList);
                                }
                            }
                        }
                        record.setIsInheritable(1);
                        List<SysRoleView> sysRoleViews = viewMapper.selectExtendsSysRoleListByOuIds(record);
                        list.addAll(sysRoleViews);


                    }
                }
            }
        }
        List<String> ouIds = new ArrayList<>();
        for (SysRoleView roleView : list) {
            if (XStringUtils.isNotEmpty(roleView.getOuId())){
                ouIds.add(roleView.getOuId());
            }
        }

        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);
        Date date = new Date();
        List<SysOrganizationRelEnt> ouRelEnts= new ArrayList<>();
        if (!CollectionUtils.isEmpty(ouIds)){
            SysOrganizationRelMapper organizationRelMapper = context.getBean(SysOrganizationRelMapper.class);
            QueryWrapper<SysOrganizationRelEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().in(SysOrganizationRelEnt::getOuId,ouIds)
                    .le(true,SysOrganizationRelEnt::getEffectTime,date)
                    .ge(true,SysOrganizationRelEnt::getEndTime,date);
             ouRelEnts = organizationRelMapper.selectList(wrapper);
        }
        total=list.size();
        Integer page = input.getPage() - 1;
        if (list.size() > input.getPage() * input.getLimit()) {
            list = list.subList(page * input.getLimit(), input.getPage() * input.getLimit());
        } else {
            list = list.subList(page * input.getLimit(), list.size());
        }

        List<QuerySysRoleViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysRoleViewOutput.class);
        for (QuerySysRoleViewOutput view : outputs) {

            if (!CollectionUtils.isEmpty(ouRelEnts)){
                for (SysOrganizationRelEnt ouRelEnt : ouRelEnts) {
                    if (ouRelEnt.getOuId().equals(view.getOuId())){
                        view.setParentOuId(ouRelEnt.getParentOuId());
                    }
                }

            }
            //查询系统管理员角色，如果true的话不可以删除
            String value = LoginConfig.adminRoles.findValue(context);
            if (StringUtils.isNotBlank(value) && value.contains(view.getId())){
                view.setSysAdmin(true);
            }else{
                view.setSysAdmin(false);
            }
            //介质类型
            if (view.getMediaType() != null){
                if (mediaTypes.getResult() != null){
                    List<QuerySysDictionaryViewOutput> items = mediaTypes.getResult();

                    String[] split = view.getMediaType().split(",");
                    StringBuilder buffer = new StringBuilder();
                    for (String mediaType : split) {
                        for (QuerySysDictionaryViewOutput item : items) {
                            if (mediaType.equals(item.getDicKey())){
                                buffer.append(",");
                                buffer.append(item.getDicName());
                            }
                        }
                    }
                    if (buffer.length()>0){
                        String mediaTypeName = buffer.substring(1);
                        view.setMediaTypeName(mediaTypeName);
                    }

                }
                if (view.getMediaTypeName()==null){
                    view.setMediaTypeName(view.getMediaType());
                }
            }
        }

        return XPageResult.success(outputs, input, total);
    }


    @XText("同步角色到工作流")
    @XApiPost
    public XServiceResult syncSysRoleSendWorkflow(XContext context, GetSysRoleInput input) throws XServiceException{

        return XTransactionHelper.begin(context, () -> {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            Integer isSuperAdmin = session.getIsSuperAdmin();
            SysRoleMapper mapper = context.getBean(SysRoleMapper.class);
            SysRoleViewMapper viewMapper = context.getBean(SysRoleViewMapper.class);
            QueryWrapper<SysRoleEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysRoleEnt::getSyncWorkFlow,true);
            if (UserConstant.ISSUPERADMIN_FALSE.equals(isSuperAdmin)){
                //普通角色可以同步所属企业下面所有子级的角色
                SysOrganizationViewMapper sysOrganizationViewMapper = context.getBean(SysOrganizationViewMapper.class);
                SysOrganizationView sysOrganizationView = new SysOrganizationView();
                sysOrganizationView.setEndTime(new Date());
                sysOrganizationView.setIsDeleted(SysOrganizationCode.NORMAL);
                sysOrganizationView.setParentOuId(session.getOuId());
                List<SysOrganizationView> viewList = sysOrganizationViewMapper.selectOuSonByParentOuId(sysOrganizationView);
                List<String> ouIds = new ArrayList<>();
                if (!CollectionUtils.isEmpty(viewList)){
                    for (SysOrganizationView organizationView : viewList) {
                        ouIds.add(organizationView.getId());
                    }
                }
                ouIds.add(session.getOuId());

                wrapper.lambda().in(SysRoleEnt::getOuId,ouIds);
            }
            List<SysRoleEnt> sysRoleEnts = mapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(sysRoleEnts)){
                //调用工作流发起接口
                BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
                UpdateOrgRoleInput param = new UpdateOrgRoleInput();
                List<UpdateOrgRoleInput.RoleInfo> list = new ArrayList<>();
                for (SysRoleEnt sysRoleEnt : sysRoleEnts) {
                    if (sysRoleEnt.getSyncWorkFlow() != null && sysRoleEnt.getSyncWorkFlow()){
                        UpdateOrgRoleInput.RoleInfo roleInfo = new UpdateOrgRoleInput.RoleInfo();
                        roleInfo.setCode(sysRoleEnt.getId());
                        roleInfo.setName(sysRoleEnt.getRoleName());
                        list.add(roleInfo);
                    }
                }
                param.setList(list);
                XListResult<UpdateOrgRoleInput.RoleInfo> orgRoles = bpmService.updateOrgRoles(context, param);
                orgRoles.throwIfFail();

                for (SysRoleEnt sysRoleEnt : sysRoleEnts) {
                    if (sysRoleEnt.getSyncWorkFlow() != null && sysRoleEnt.getSyncWorkFlow()) {
                        if (orgRoles.getResult() != null){
                            for (UpdateOrgRoleInput.RoleInfo roleInfo : orgRoles.getResult()) {
                                if (sysRoleEnt.getId().equals(roleInfo.getCode())) {
                                    //todo  保存工作流角色id
                                    sysRoleEnt.setBpsRoleId(roleInfo.getId());
                                }
                            }
                        }

                    }
                }
                List<SysRoleView> sysRoleViews = XCopyUtils.copyNewList(sysRoleEnts, SysRoleView.class);
                viewMapper.batchUpdateSysRoleSetBpsRoleId(sysRoleViews);
            }



            //返回执行成功的结果，事务提交
            return XServiceResult.OK;

        });
    }


    @XText("新增角色")
    @XApiPost
    public XServiceResult createSysRole(XContext context, CreateSysRoleInput input) throws XServiceException{

        return XTransactionHelper.begin(context, () -> {
            SysRoleEnt entity = new SysRoleEnt();
            SysRoleMapper mapper = context.getBean(SysRoleMapper.class);
            QueryWrapper<SysRoleEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysRoleEnt::getRoleName,input.getRoleName());
            if (XStringUtils.isNotEmpty(input.getOuId())){
                wrapper.lambda().eq(SysRoleEnt::getOuId,input.getOuId());
            }
            List<SysRoleEnt> sysRoleEnts = mapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(sysRoleEnts)){
                return XServiceResult.error(200,"角色名称已存在!");
            }
            XCopyUtils.copyObject(input, entity);
            String roleId = UUID.randomUUID().toString();
            entity.setId(roleId);

            entity.setCreateTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setCreateByName(session.getUserName());
            entity.setCreateById(session.getId());
            //用户不选的话需要绑定为当前操作员的组织机构
            if (XStringUtils.isEmpty(entity.getOuId())){
                entity.setOuId(session.getOuId());
            }
            BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
            if (entity.getSyncWorkFlow() != null && entity.getSyncWorkFlow()){
                //调用工作流发起接口
                UpdateOrgRoleInput param = new UpdateOrgRoleInput();
                List<UpdateOrgRoleInput.RoleInfo> list = new ArrayList<>();
                UpdateOrgRoleInput.RoleInfo roleInfo = new UpdateOrgRoleInput.RoleInfo();
                roleInfo.setCode(roleId);
                roleInfo.setName(entity.getRoleName());
                list.add(roleInfo);
                param.setList(list);
                XListResult<UpdateOrgRoleInput.RoleInfo> orgRoles = bpmService.updateOrgRoles(context, param);
                orgRoles.throwIfFail();
                //todo  保存工作流角色id
                entity.setBpsRoleId(orgRoles.getResult().get(0).getId());
            }

            try {
                mapper.insert(entity);
            }catch (Exception e){
                //调用工作流发起接口
                DeleteOrgRoleInput deleteOrgRoleInput = new DeleteOrgRoleInput();
                List<Integer> list = new ArrayList<>();
                list.add(entity.getBpsRoleId());
                deleteOrgRoleInput.setList(list);
                XServiceResult xServiceResult = bpmService.deleteOrgRoles(context, deleteOrgRoleInput);
                xServiceResult.throwIfFail();
                throw new XServiceException(e);
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;

        });
    }

    @XText("修改角色信息")
    @XApiPost
    public XServiceResult updateSysRole(XContext context, UpdateSysRoleInput input)throws XServiceException{
        return XTransactionHelper.begin(context, () -> {
            SysRoleMapper mapper = context.getBean(SysRoleMapper.class);
            QueryWrapper<SysRoleEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysRoleEnt::getId, input.getId());
            SysRoleEnt entity = mapper.selectOne(queryWrapper);
            if(entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            if (!entity.getRoleName().equals(input.getRoleName())){
                QueryWrapper<SysRoleEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(SysRoleEnt::getRoleName,input.getRoleName());
                if (XStringUtils.isNotEmpty(input.getOuId())){
                    wrapper.lambda().eq(SysRoleEnt::getOuId,input.getOuId());
                }
                List<SysRoleEnt> sysRoleEnts = mapper.selectList(wrapper);
                if (!CollectionUtils.isEmpty(sysRoleEnts)){
                    return XServiceResult.error(200,"角色名称已存在!");
                }
            }
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setModifyTime(new Date());
            entity.setModifyByName(session.getUserName());
            entity.setModifyById(session.getId());
            mapper.updateById(entity);

            BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
            if (entity.getSyncWorkFlow() != null && entity.getSyncWorkFlow()){
                //调用工作流发起接口
                UpdateOrgRoleInput param = new UpdateOrgRoleInput();
                List<UpdateOrgRoleInput.RoleInfo> list = new ArrayList<>();
                UpdateOrgRoleInput.RoleInfo roleInfo = new UpdateOrgRoleInput.RoleInfo();
                roleInfo.setCode(input.getId());
                roleInfo.setName(entity.getRoleName());
                list.add(roleInfo);
                param.setList(list);
                XListResult<UpdateOrgRoleInput.RoleInfo> orgRoles = bpmService.updateOrgRoles(context, param);
                orgRoles.throwIfFail();
                //todo 保存工作流角色id
            }else{
                //调用工作流发起接口
                DeleteOrgRoleInput deleteOrgRoleInput = new DeleteOrgRoleInput();
                List<Integer> list = new ArrayList<>();
                list.add(entity.getBpsRoleId());
                deleteOrgRoleInput.setList(list);
                XServiceResult xServiceResult = bpmService.deleteOrgRoles(context, deleteOrgRoleInput);
                xServiceResult.throwIfFail();
            }

            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });

    }

    @XText("删除角色")
    @XApiPost
    public XServiceResult deleteSysRole(XContext context, DeleteSysRoleInput input)throws XServiceException{

        return XTransactionHelper.begin(context, () -> {
            SysRoleMapper mapper = context.getBean(SysRoleMapper.class);
            SysUserRoleRelMapper relMapper = context.getBean(SysUserRoleRelMapper.class);
            QueryWrapper<SysRoleEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysRoleEnt::getId, input.getId());
            SysRoleEnt entity = mapper.selectOne(queryWrapper);
            if(entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            String value = LoginConfig.adminRoles.findValue(context);
            if (StringUtils.isNotBlank(value) && value.contains(entity.getId())){
                return XServiceResult.error(200, "此角色是管理员角色,请勿删除!");
            }

            QueryWrapper<SysUserRoleRelEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysUserRoleRelEnt::getRoleId,entity.getId());
            List<SysUserRoleRelEnt> relEnts = relMapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(relEnts)){
                List<String> collect = relEnts.stream().map(SysUserRoleRelEnt::getId).collect(Collectors.toList());
                relMapper.deleteBatchIds(collect);
            }

            mapper.deleteById(entity);

            if (entity.getSyncWorkFlow() != null && entity.getSyncWorkFlow()){
                //调用工作流发起接口
                BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
                DeleteOrgRoleInput deleteOrgRoleInput = new DeleteOrgRoleInput();
                List<Integer> list = new ArrayList<>();
                list.add(entity.getBpsRoleId());
                deleteOrgRoleInput.setList(list);
                XServiceResult xServiceResult = bpmService.deleteOrgRoles(context, deleteOrgRoleInput);
                xServiceResult.throwIfFail();
            }

            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });

    }

    @XText("根据组织机构id获取角色列表")
    @XApiPost
    public XPageResult<QuerySysRoleViewOutput> getRoleListByOuIdPage(XContext context, GetRoleListByOuIdInput input){
        SysRoleViewMapper viewMapper = context.getBean(SysRoleViewMapper.class);
        SysRoleView record = new SysRoleView();
        XCopyUtils.copyObject(input, record);
        if(input.getShowAll() != null && input.getShowAll()){
            record.setOuId(null);
        }
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysRoleView> list = viewMapper.selectSysRoleListByOuId(record);
        PageInfo<SysRoleView> pageInfo = new PageInfo<>(list);
        List<QuerySysRoleViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysRoleViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }


}
