package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.BaseBaseEquipmentCloudService;
import pps.cloud.base.service.data.BaseEquipmentPropertyView;
import pps.cloud.base.service.data.base_equipment.GetBaseEquipmentInput;
import pps.cloud.base.service.data.base_equipment.GetBaseEquipmentOutput;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.entity.BaseEquipmentEnt;
import pps.core.base.entity.BaseEquipmentPropertyEnt;
import pps.core.base.entity.BaseEquipmentView;
import pps.core.base.mapper.BaseEquipmentMapper;
import pps.core.base.mapper.BaseEquipmentPropertyMapper;
import pps.core.base.mapper.BaseEquipmentViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;

import java.util.List;
import java.util.stream.Collectors;


@XService
public class BaseBaseEquipmentCloudServiceImpl implements BaseBaseEquipmentCloudService {

    @Override
    public XListResult<GetBaseEquipmentOutput> selectEquimentByOuIdandStationId(XContext context, GetBaseEquipmentInput input) {
        QuerySysDictionaryViewOutput dict = getDictByAlias(context , "equipment_type_cyg");
        List<QuerySysDictionaryViewOutput> dictList = queryChildByDicPathIncludeSelf(context , dict);
        List<String> equipmentTypeList = dictList.stream().map(QuerySysDictionaryViewOutput::getDicKey).collect(Collectors.toList());
        BaseEquipmentViewMapper mapper = context.getBean(BaseEquipmentViewMapper.class);
        BaseEquipmentView record = new BaseEquipmentView();
        XCopyUtils.copyObject(input, record);
        record.setEquipmentTypeList(equipmentTypeList);
        List<BaseEquipmentView> list = mapper.selectList(record);
        List<GetBaseEquipmentOutput> outputs = XCopyUtils.copyNewList(list, GetBaseEquipmentOutput.class);
        outputs.stream().forEach(item->{
            item.setEquipmentTypeName(getDictName(item.getEquipmentType() + "" , dictList));
        });
        return XListResult.success(outputs);
    }
    private QuerySysDictionaryViewOutput getDictByAlias(XContext context , String alias){
        SystemDictionaryService systemDictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictionaryViewInput = new QuerySysDictionaryViewInput();
        dictionaryViewInput.setPage(1);
        dictionaryViewInput.setLimit(1000);
        dictionaryViewInput.setAlias(alias);
        QuerySysDictionaryViewOutput dict = systemDictionaryService.queryByAlias(context , dictionaryViewInput).getResult();
        return dict;
    }
    private List<QuerySysDictionaryViewOutput> queryChildByDicPathIncludeSelf(XContext context , QuerySysDictionaryViewOutput dict){
        SystemDictionaryService systemDictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictionaryViewInput = new QuerySysDictionaryViewInput();
        dictionaryViewInput.setDicPath(dict.getId());
        dictionaryViewInput.setPage(1);
        dictionaryViewInput.setLimit(1000);
        List<QuerySysDictionaryViewOutput> dictList = systemDictionaryService.queryChildByDicPathIncludeSelf(context , dictionaryViewInput).getResult();
        return dictList;
    }

    public String getDictName(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return " ";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict.getDicName();
            }
        }
        return dictKey;
    }

    @Override
    public XSingleResult<GetBaseEquipmentOutput> selectEquipmentById(XContext context, GetBaseEquipmentInput input) {
        BaseEquipmentMapper mapper = context.getBean(BaseEquipmentMapper.class);
        BaseEquipmentEnt equipmentEnt = mapper.selectById(input.getId());
        if( equipmentEnt == null){
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEquipmentOutput output = new GetBaseEquipmentOutput();
        XCopyUtils.copyObject(equipmentEnt, output);
        return XSingleResult.success(output);
    }

    @Override
    public XListResult<GetBaseEquipmentOutput> selectEquipmentExtendPropertyByStationId(XContext context, GetBaseEquipmentInput input) {
        BaseEquipmentViewMapper viewMapper = context.getBean(BaseEquipmentViewMapper.class);
        QuerySysDictionaryViewOutput dict = getDictByAlias(context , "equipment_type_cyg");
        List<QuerySysDictionaryViewOutput> dictList = queryChildByDicPathIncludeSelf(context , dict);
        BaseEquipmentView recordView = XCopyUtils.copyNewObject(input, BaseEquipmentView.class);
        List<BaseEquipmentView> propertyList = viewMapper.selectEquipmentExtendPropertyByStationId(recordView);
        List<GetBaseEquipmentOutput> propertyOutputs = XCopyUtils.copyNewList(propertyList, GetBaseEquipmentOutput.class);
        propertyOutputs.stream().forEach(item->{
            item.setEquipmentTypeName(getDictName(item.getEquipmentType() + "" , dictList));
        });
        return XListResult.success(propertyOutputs);
    }

    @Override
    public XSingleResult<GetBaseEquipmentOutput> selectEquipmentByStationIdAndEquipmentName(XContext context, GetBaseEquipmentInput input) {
        BaseEquipmentMapper mapper = context.getBean(BaseEquipmentMapper.class);
        QueryWrapper<BaseEquipmentEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(BaseEquipmentEnt::getStationId,input.getStationId()).
                eq(BaseEquipmentEnt::getEquipmentName,input.getEquipmentName());
        BaseEquipmentEnt Ent = mapper.selectOne(queryWrapper);
        GetBaseEquipmentOutput view = XCopyUtils.copyNewObject(Ent, GetBaseEquipmentOutput.class);
        return XSingleResult.success(view);
    }
}
