package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.BaseBasePipelineService;
import pps.cloud.base.service.data.BaseNameInput;
import pps.cloud.base.service.data.CommonIDInput;
import pps.cloud.base.service.data.base_pipeline.*;
import pps.core.base.entity.BasePipelineEnt;
import pps.core.base.entity.BasePipelineView;
import pps.core.base.mapper.BasePipelineMapper;
import pps.core.base.mapper.BasePipelineViewMapper;
import pps.core.base.service.data.base_pipeline.QueryBasePipelineInput;
import pps.core.base.service.data.base_pipeline.QueryBasePipelineOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.XPageInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.Date;
import java.util.List;

@XText("管线接口")
@XService
public class BaseBasePipelineCloudServiceImpl implements BaseBasePipelineService {

    private final String ROOT_PARENT_ID = "11111111-1111-1111-1111-111111111111";


    @Override
    @XText("根据管线id获取管线信息")
    public XSingleResult<QueryPipelineByIdOutput> getPipelineById(XContext context, CommonIDInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        BasePipelineEnt ent = mapper.selectById(input.getId());
        QueryPipelineByIdOutput out = XCopyUtils.copyNewObject(ent, QueryPipelineByIdOutput.class);
        return XSingleResult.success(out);
    }

    @Override
    @XText("获取场站当前所属的管线信息")
    public XListResult<SelectPipelineListByStationIdOutput> selectPipelineListByStationId(XContext context, CommonIDInput input) {
        BasePipelineViewMapper mapper = context.getBean(BasePipelineViewMapper.class);
        List<BasePipelineView> list = mapper.selectPipelineListByStationId(input.getId());
        List<SelectPipelineListByStationIdOutput> outputs = XCopyUtils.copyNewList(list, SelectPipelineListByStationIdOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    @XText("根据管线id获取管线信息")
    public XListResult<QueryPipelineByIdsOutput> queryPipelineByPipelineIds(XContext context, QueryPipelineByIdsInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BasePipelineEnt> lambda = queryWrapper.lambda();
        lambda.in(BasePipelineEnt::getId, input.getPipelineIdList());
        XPageInput pageInput = new XPageInput();
        pageInput.setPage(1);
        pageInput.setLimit(1000);
        XPageResult<QueryPipelineByIdsOutput> query = XMapperHelper.query(mapper, pageInput, queryWrapper, QueryPipelineByIdsOutput.class);
        return XListResult.success(query.getResult().getItems());
    }


    @Override
    @XText("根据管线id获取1级管线信息")
    public XListResult<QueryPipelineByIdsOutput> queryPipelineByPipelineIdsOne(XContext context, QueryPipelineByIdsInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BasePipelineEnt> lambda = queryWrapper.lambda();
        lambda.in(BasePipelineEnt::getId, input.getPipelineIdList());
        lambda.eq(BasePipelineEnt::getParentPipelineId, ROOT_PARENT_ID);
        XPageInput pageInput = new XPageInput();
        pageInput.setPage(1);
        pageInput.setLimit(1000);
        XPageResult<QueryPipelineByIdsOutput> query = XMapperHelper.query(mapper, pageInput, queryWrapper, QueryPipelineByIdsOutput.class);
        return XListResult.success(query.getResult().getItems());
    }

    @Override
    @XText("根据管线名称获取管线信息")
    public XListResult<QueryPipelineByIdsOutput> queryPipelineByPipelineNames(XContext context, QueryPipelineByIdsInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BasePipelineEnt> lambda = queryWrapper.lambda();
        lambda.in(BasePipelineEnt::getPipelineName, input.getPipelineIdList());
        XPageInput pageInput = new XPageInput();
        pageInput.setPage(1);
        pageInput.setLimit(1000);
        XPageResult<QueryPipelineByIdsOutput> query = XMapperHelper.query(mapper, pageInput, queryWrapper, QueryPipelineByIdsOutput.class);
        return XListResult.success(query.getResult().getItems());
    }

    @Override
    @XText("根据管线名称获取管线信息")
    public XSingleResult<PipelineListOutput> selectPipelineByPipelineName(XContext context, BaseNameInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> qr = new QueryWrapper<>();
        qr.lambda().eq(BasePipelineEnt::getPipelineName, input.getName());
        BasePipelineEnt ent = mapper.selectOne(qr);
        return XSingleResult.success(XCopyUtils.copyNewObject(ent, PipelineListOutput.class));
    }

    @Override
    @XText("根据管线id获取下级管线信息")
    public XListResult<PipelineListOutput> selectDownPipelineByPipelineId(XContext context, PipelineIdInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, input.getId());
        List<BasePipelineEnt> list = mapper.selectList(queryWrapper);
        return XListResult.success(XCopyUtils.copyNewList(list, PipelineListOutput.class));
    }

    @Override
    @XText("根据管线id获取下级有调度台的管线信息")
    public XListResult<PipelineListOutput> selectDispatchingDownPipelineByPipelineId(XContext context, PipelineIdInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, input.getId());
        List<BasePipelineEnt> list = mapper.selectList(queryWrapper);
//        List<BasePipelineEnt> entList = new ArrayList<>();
        //调度台管线下级需求变更
//        if (null != list) {
//            for (int i = 0; i < list.size(); i++) {
//                String desk = list.get(i).getDispatchDesk();
//                if (StringUtils.isNotBlank(desk)) {
//                   entList.add(list.get(i));
//                }
//            }
//        }
        return XListResult.success(XCopyUtils.copyNewList(list, PipelineListOutput.class));
    }

    @Override
    @XText("根据调度台获取管线信息")
    public XListResult<PipelineListOutput> selectPipelineByDispatchDesk(XContext context, GetPipelineByDispatchInput input) {
        BasePipelineViewMapper mapper = context.getBean(BasePipelineViewMapper.class);
        BasePipelineView record = new BasePipelineView();
        record.setMediaType(input.getMediaType());
        record.setDispatchDeskList(input.getDispatchDeskList());
        record.setParentPipelineId(input.getParentId());
        List<BasePipelineView> list = mapper.selectByDispatchDesk(record);
        return XListResult.success(XCopyUtils.copyNewList(list,PipelineListOutput.class));
    }

    @Override
    public XSingleResult<QueryPipelineByIdsOutput> selectPipelinebyId(XContext context, CommonIDInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        BasePipelineEnt ent = mapper.selectById(input.getId());
        if (ent == null) {
            return XSingleResult.error(context, XError.NotFound);
        } else {
            QueryPipelineByIdsOutput output = new QueryPipelineByIdsOutput();
            XCopyUtils.copyObject(ent, output);
            return XSingleResult.success(output);
        }
    }

    @Override
    public XListResult<QueryPipelineByIdOutput> selectPipelineIdByenterprise(XContext context, QueryPipelineByIdInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineEnt::getId, input.getId());
        XPageInput pageInput = new XPageInput();
        pageInput.setPage(1);
        pageInput.setLimit(100);
        XPageResult<QueryPipelineByIdOutput> query = XMapperHelper.query(mapper, pageInput, queryWrapper, QueryPipelineByIdOutput.class);
        return XListResult.success(query.getResult().getItems());
    }

    @Override
    public XListResult<PipelineListOutput> selectPipelineByEnterpriseId(XContext context, GetPipelineByEnterpriseIdInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        Date now = new Date();
        if (XStringUtils.isNotEmpty(input.getEnterpriseId()))
            queryWrapper.lambda().eq(BasePipelineEnt::getEnterpriseId, input.getEnterpriseId());
        if (XStringUtils.isNotEmpty(input.getMediumType()))
            queryWrapper.lambda().eq(BasePipelineEnt::getMediumType, input.getMediumType());
        queryWrapper.lambda().isNotNull(BasePipelineEnt::getEnterpriseId);
        queryWrapper.lambda().le(true, BasePipelineEnt::getEffectTime, now);
        queryWrapper.lambda().ge(true, BasePipelineEnt::getEndTime, now);
        queryWrapper.lambda().orderByAsc(BasePipelineEnt::getEnterpriseId);
        queryWrapper.lambda().orderByAsc(BasePipelineEnt::getCreateTime);
        List<BasePipelineEnt> list = mapper.selectList(queryWrapper);
        List<PipelineListOutput> outputList = XCopyUtils.copyNewList(list, PipelineListOutput.class);
        return XListResult.success(outputList);
    }

    @Override
    public XListResult<SelectPipelineByOperatorOutput> selectPipelineByOperator(XContext context) {
        QueryBasePipelineInput pipelineInput = new QueryBasePipelineInput();
        pipelineInput.setIsHaveOuId("1");
        pipelineInput.setPage(1);
        pipelineInput.setLimit(1000);
        XPageResult<QueryBasePipelineOutput> queryBasePipelineOutputXPageResult = context.getBean(BasePipelineService.class).queryBasePipeline(context, pipelineInput);
        List<QueryBasePipelineOutput> items = queryBasePipelineOutputXPageResult.getResult().getItems();
        List<SelectPipelineByOperatorOutput> outputs = XCopyUtils.copyNewList(items, SelectPipelineByOperatorOutput.class);
        return XListResult.success(outputs);
    }
}
