package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.BaseBaseStationService;
import pps.cloud.base.service.data.*;
import pps.cloud.base.service.data.base_pipeline.QueryPipelineByIdInput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.entity.*;
import pps.core.base.entity.BaseStationView;
import pps.core.base.mapper.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@XService
public class BaseBaseStationServiceImpl implements BaseBaseStationService {

    @XText("根据id获取场站")
    public XSingleResult<QueryBaseStationOutput> selectStationById(XContext context, BaseStationIdInput input){
        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        BaseStationEnt ent = mapper.selectById(input.getStationId());
        QueryBaseStationOutput output = XCopyUtils.copyNewObject(ent, QueryBaseStationOutput.class);
        return XSingleResult.success(output);
    }

    @Override
    public XSingleResult<GetOuByStationIdOutput> getOuByStationId(XContext context, GetOuByStationIdInput input) {
        BaseStationOuRelMapper mapper = context.getBean(BaseStationOuRelMapper.class);
        QueryWrapper<BaseStationOuRelEnt> warpper = new QueryWrapper<>();
        warpper.lambda().eq(BaseStationOuRelEnt::getStationId, input.getStationId())
                .gt(BaseStationOuRelEnt::getEndTime, new Date()).last("limit 1");
        BaseStationOuRelEnt relEnt = mapper.selectOne(warpper);

        if (relEnt != null && relEnt.getOuId() != null) {
            SysOrganizationCloudService sysOrgService = context.getBean(SysOrganizationCloudService.class);
            GetSysOrganizationViewInput view = new GetSysOrganizationViewInput();
            view.setId(relEnt.getOuId());
            GetSysOrganizationViewOutput sysOrganization = sysOrgService.getSysOrganizationById(context, view).getResult();
            GetOuByStationIdOutput output = XCopyUtils.copyNewObject(sysOrganization, GetOuByStationIdOutput.class);
            return XSingleResult.success(output);
        }
        return XSingleResult.error(1001,"场站信息不完整");
    }

    @Override
    @XText("根据id获取场站")
    public XListResult<QueryBaseStationOutput> selectStationListByStationIds(XContext context, QueryBaseStationInput input) {
        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().in(BaseStationEnt::getId,input.getStationIds());
        List<BaseStationEnt> baseStationEnts = mapper.selectList(queryWrapper);
        PageInfo<BaseStationEnt> pageInfo = new PageInfo<>(baseStationEnts);
        List<QueryBaseStationOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    @XText("根据名称获取场站")
    public XListResult<QueryBaseStationOutput> selectStationListByStationNames(XContext context, QueryBaseStationInput input) {
        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().in(BaseStationEnt::getStationName,input.getStationIds());
        List<BaseStationEnt> baseStationEnts = mapper.selectList(queryWrapper);
        PageInfo<BaseStationEnt> pageInfo = new PageInfo<>(baseStationEnts);
        List<QueryBaseStationOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    @XText("根据场站名称获取场站")
    public XSingleResult<QueryBaseStationOutput> selectStationListByStationName(XContext context, GetBaseStationByNameInput input){
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        BaseStationView view = new BaseStationView();
        view.setStationName(input.getStationName());
        view.setPipelineId(input.getPipelineId());
        view.setStationId(input.getStationId());
        List<BaseStationView> list = mapper.selectStationByStationName(view);
        if(list.size()>0){
            return XSingleResult.success(XCopyUtils.copyNewObject(list.get(0),QueryBaseStationOutput.class));
        }else{
            return XSingleResult.error(400,"未找到场站");
        }
    }

    @Override
    @XText("根据组织机构id获取场站信息")
    public XListResult<QueryBaseStationOutput> selectStationListByOuId(XContext context, BaseOuIdInput input){
        if(input.getIsContainsChildren() != null && input.getIsContainsChildren()){
            BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
            List<BaseStationView> list = mapper.selectAllStationByOuId(input.getOuId() , "");
            return XListResult.success(XCopyUtils.copyNewList(list,QueryBaseStationOutput.class));
        }else{
            BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
            List<BaseStationView> list = mapper.selectStationByOuId(input.getOuId());
            return XListResult.success(XCopyUtils.copyNewList(list,QueryBaseStationOutput.class));
        }
    }
    @Override
    @XText("根据组织机构id获取场站信息")
    public XListResult<GetBaseStationForFlowMeterOutput> selectStationListByOuIdForFlowMeter(XContext context, GetBaseStationForFlowMeterInput input){
        BaseOuIdInput ouIdInput = new BaseOuIdInput();
        XCopyUtils.copyObject(input, ouIdInput);
        GetSysOrganizationViewOutput sysOrganization = getSysOrganization(context);
        if((sysOrganization.getOuLevel().equals("STATION")
                ||sysOrganization.getOuLevel().equals("4106"))){
            ouIdInput.setOuId(sysOrganization.getId());
        }
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        List<BaseStationView> stationOutputList = mapper.selectAllStationByOuId(ouIdInput.getOuId() , input.getMediaType());
        List<GetBaseStationForFlowMeterOutput> outputs = XCopyUtils.copyNewList(stationOutputList , GetBaseStationForFlowMeterOutput.class);
        List<QuerySysDictionaryViewOutput> dictionaryViewOutputs = getMenuDict(context , "MEDIUM_TYPE");
        outputs.stream().forEach(item->{
            if(XStringUtils.isNotEmpty(item.getMediumType())){
                item.setMediumTypeName(item.getStationName() + "-" + getDictName(item.getMediumType() , dictionaryViewOutputs));
            }else
                item.setMediumTypeName(item.getStationName());
        });
        return XListResult.success(outputs);
    }

    public List<QuerySysDictionaryViewOutput> getMenuDict(XContext context , String alias ){
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }

    public String getDictName(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return " ";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict.getDicName();
            }
        }
        return dictKey;
    }


    private GetSysOrganizationViewOutput getSysOrganization(XContext context ){
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysOrganizationCloudService system = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput getSysOrganizationViewInput = new GetSysOrganizationViewInput();
        getSysOrganizationViewInput.setId(session.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> outputXSingleResult = system.getSysOrganizationById(context , getSysOrganizationViewInput);
        outputXSingleResult.throwIfFail();
        return outputXSingleResult.getResult();
    }

    @Override
    @XText("根据场站id获取场站的组织机构信息")
    public XListResult<QueryBaseStationOutput> selectStationOuByStationId(XContext context, BaseStationIdInput input){
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        List<BaseStationView> list = mapper.selectStationOuByStationId(input.getStationId());
        return XListResult.success(XCopyUtils.copyNewList(list,QueryBaseStationOutput.class));
    }

    @Override
    @XText("获取用户范围授权的场站id")
    public XListResult<QueryBaseStationOutput> getStationIdsWithUser(XContext context, GetStationIdsWithUserInput input){
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        Map<String,String> map = new HashMap<>();
        if(StringUtils.isNotBlank(input.getUserId())){
            map.put("userId",input.getUserId());
        }
        if(StringUtils.isNotBlank(input.getMenuId())){
            map.put("menuId",input.getMenuId());
        }
        List<BaseStationView> list = mapper.selectUserStation(map);
        List<QueryBaseStationOutput> ouputs = XCopyUtils.copyNewList(list, QueryBaseStationOutput.class);
        return XListResult.success(ouputs);

    }

    @Override
    @XText("根据组织机构id获取所有下级信息")
    public XListResult<QueryBaseStationOutput> selectAllChildByOuId(XContext context, BaseOuIdInput input){
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        List<BaseStationView> list = mapper.selectAllChildByOuId(input.getOuId());
        return XListResult.success(XCopyUtils.copyNewList(list,QueryBaseStationOutput.class));
    }


    @Override
    public XListResult<QueryBaseStationOutput> selectStationByPipelineId(XContext context, QueryBaseStationInput input) {
        BasePipelineStationRelMapper pipelineStationRelMapper= context.getBean(BasePipelineStationRelMapper.class);
        BaseStationMapper stationMapper = context.getBean(BaseStationMapper.class);
        //根据管线id查询场站id
        QueryWrapper<BasePipelineStationRelEnt> queryStation = new QueryWrapper();
        queryStation.lambda().eq(BasePipelineStationRelEnt::getPipelineId,input.getPipelineId())
                .gt(BasePipelineStationRelEnt::getEndTime,new Date());
        List<BasePipelineStationRelEnt> stationEnts = pipelineStationRelMapper.selectList(queryStation);
        List<String> stationIds = stationEnts.stream().map(p -> {
            return p.getStationId();
        }).distinct().collect(Collectors.toList());
        //根据场站id查询场站信息
        QueryWrapper<BaseStationEnt> query = new QueryWrapper();
        query.lambda().in(BaseStationEnt::getId,stationIds);
        List<BaseStationEnt> stationList = stationMapper.selectList(query);
        List<QueryBaseStationOutput> list = XCopyUtils.copyNewList(stationList, QueryBaseStationOutput.class);
        return XListResult.success(list);
    }

    @Override
    public XListResult<QueryBaseStationExcelOutput> selectStationByPipelineIdExcel(XContext context, QueryPipelineByIdInput input) {
        BasePipelineStationRelMapper pipelineStationRelMapper= context.getBean(BasePipelineStationRelMapper.class);
        BaseStationMapper stationMapper = context.getBean(BaseStationMapper.class);
        //根据管线id查询场站id
        QueryWrapper<BasePipelineStationRelEnt> queryStation = new QueryWrapper();
        queryStation.lambda().eq(BasePipelineStationRelEnt::getPipelineId,input.getId())
                .gt(BasePipelineStationRelEnt::getEndTime,new Date());
        List<BasePipelineStationRelEnt> stationEnts = pipelineStationRelMapper.selectList(queryStation);
        List<String> stationIds = stationEnts.stream().map(p -> {
            return p.getStationId();
        }).distinct().collect(Collectors.toList());
        if (stationIds.size()==0){
            return null;
        }
        //根据场站id查询场站信息
        QueryWrapper<BaseStationEnt> query = new QueryWrapper();
        query.lambda().in(BaseStationEnt::getId,stationIds);
        List<BaseStationEnt> stationList = stationMapper.selectList(query);
        List<QueryBaseStationExcelOutput> list = XCopyUtils.copyNewList(stationList, QueryBaseStationExcelOutput.class);
        return XListResult.success(list);
    }

    @Override
    public XListResult<QueryBaseStationOutput> selectStationAllByStationName(XContext context, QueryBaseStationInput input) {
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        BaseStationView view = new BaseStationView();
        view.setStationName(input.getStationName());
        List<BaseStationView> baseStationViews = mapper.selectStationAllByStationName(view);
        List<QueryBaseStationOutput> list = XCopyUtils.copyNewList(baseStationViews, QueryBaseStationOutput.class);
        return XListResult.success(list);
    }

    @XText("创建场站")
    @XApiPost
    public XServiceResult createBaseStation(XContext context, CreateBaseStationInput input) {
        //1、新增场站基本信息 2、新增场站组织机构关联关系 3、添加场站和管线关系（新增场站与上游场站所在管线关系）
        //4、新增场站和上游场站自动生成管段（管段基础信息生成，别的信息需要手工维护）5、新增自动生成的管段和管线的关联关系
        //6、创建管段反输记录（默认不反输）。3、创建管段转供的所有关系（根据当前管段的结束场站作为转供管段的起始场站，查询所有管段添加转供关系）
        return XTransactionHelper.begin(context, () -> {
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            BaseStationViewMapper stationViewmapper = context.getBean(BaseStationViewMapper.class);
            BasePipelineStationRelMapper basePipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            BasePipelineMapper basePipelineMapper = context.getBean(BasePipelineMapper.class);
            BasePipelineStationRelViewMapper basePipelineStationRelViewMapper = context.getBean(BasePipelineStationRelViewMapper.class);
            BaseSegmentMapper segmentMapper = context.getBean(BaseSegmentMapper.class);
            BasePipelineSegmentRelMapper pipelineSegmentRel = context.getBean(BasePipelineSegmentRelMapper.class);
            BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
            BaseSegmentDirectionMapper baseSegmentDirectionMapper = context.getBean(BaseSegmentDirectionMapper.class);
            BaseSegmentViewMapper viewMapper = context.getBean(BaseSegmentViewMapper.class);
            BaseSegmentPipelineTransferRelMapper relMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            ArrayList<pps.cloud.base.service.data.BaseStationView> item = input.getItem();
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            //生成新增的场站id，新增场站
            String id = UUID.randomUUID().toString();
            BaseStationEnt entity = new BaseStationEnt();
            XCopyUtils.copyObject(input, entity);

            entity.setEffectTime(new Date());
            entity.setEndTime(endTime);
            entity.setId(id);
            entity.setCreateById(context.getSession(PpsUserSession.class).getId());
            entity.setCreateByName(context.getUser().getName());
            entity.setCreateTime(new Date());
            //场站默认设置管线的介质类型
            if (StringUtils.isNotBlank(input.getPipelineId())) {
                QueryWrapper<BasePipelineEnt> queryPipeline = new QueryWrapper<>();
                queryPipeline.lambda().eq(BasePipelineEnt::getId, input.getPipelineId());
                BasePipelineEnt basePipelineEnt = basePipelineMapper.selectOne(queryPipeline);
                if (ObjectUtils.isNotEmpty(basePipelineEnt)) {
                    entity.setMediumType(basePipelineEnt.getMediumType());
                }
            }
            mapper.insert(entity);
            //新增场站组织机构关系
            BaseStationOuRelEnt baseStationOuRelEnt = new BaseStationOuRelEnt();
            baseStationOuRelEnt.setStationId(id);
            baseStationOuRelEnt.setOuId(input.getOuId());
            baseStationOuRelEnt.setId(UUID.randomUUID().toString());
            baseStationOuRelEnt.setEffectTime(new Date());
            baseStationOuRelEnt.setEndTime(endTime);
            baseStationOuRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
            baseStationOuRelEnt.setCreateByName(context.getUser().getName());
            baseStationOuRelEnt.setCreateTime(new Date());
            stationOuRelMapper.insert(baseStationOuRelEnt);
            //新增当前管线和场站关系(新增场站和上游场站所在管线的关联关系)
            BasePipelineStationRelEnt pipelineStation = new BasePipelineStationRelEnt();
            pipelineStation.setId(UUID.randomUUID().toString());
            pipelineStation.setPipelineId(input.getPipelineId());
            pipelineStation.setStationId(id);
            pipelineStation.setEffectTime(new Date());
            pipelineStation.setEndTime(endTime);
            //sort赋值
            BaseStationView view = new BaseStationView();
            view.setPipelineId(input.getPipelineId());
            view.setEndTime(new Date());
            BaseStationView stationView = stationViewmapper.selectMaxSortByPipelineId(view);
            Integer sort = 0;
            if (stationView != null && stationView.getSort() != null) {
                sort = sort + stationView.getSort() + 1;
            }
            pipelineStation.setSort(sort);
            pipelineStation.setCreateById(context.getSession(PpsUserSession.class).getId());
            pipelineStation.setCreateByName(context.getUser().getName());
            pipelineStation.setCreateTime(new Date());
            basePipelineStationRelMapper.insert(pipelineStation);
            if (item != null && item.size() > 0) {
                for (pps.cloud.base.service.data.BaseStationView baseStationView : item) {
                    /*//新增管线和场站关系(新增场站和上游场站所在管线的关联关系)
                    BasePipelineStationRelEnt pipelineStationRel = new BasePipelineStationRelEnt();
                    pipelineStationRel.setId(UUID.randomUUID().toString());
                    pipelineStationRel.setPipelineId(baseStationView.getPipelineId());
                    pipelineStationRel.setStationId(id);
                    pipelineStationRel.setEffectTime(new Date());
                    pipelineStationRel.setEndTime(endTime);
                    //pipelineStationRel.setCreateById(context.getSession(PpsUserSession.class).getId());
                    pipelineStationRel.setCreateByName(context.getUser().getName());
                    pipelineStationRel.setCreateTime(new Date());
                    basePipelineStationRelMapper.insert(pipelineStationRel);*/
                    //自动生成管段base_segment
                    String segmentId = UUID.randomUUID().toString();
                    BaseSegmentEnt baseSegmentEnt = new BaseSegmentEnt();
                    baseSegmentEnt.setStartStationId(baseStationView.getId());
                    baseSegmentEnt.setEndStationId(id);
                    baseSegmentEnt.setSegmentName(input.getStationName() + "-" + baseStationView.getStationName());
                    baseSegmentEnt.setId(segmentId);
                    baseSegmentEnt.setEffectTime(new Date());
                    baseSegmentEnt.setEndTime(endTime);
                    baseSegmentEnt.setStartStationType("STATION");
                    baseSegmentEnt.setEndStationType("STATION");
                    baseSegmentEnt.setBeTraderType(0);
                    baseSegmentEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentEnt.setCreateByName(context.getUser().getName());
                    baseSegmentEnt.setCreateTime(new Date());
                    BasePipelineStationRelView station = new BasePipelineStationRelView();
                    station.setPipelineId(baseStationView.getPipelineId());
                    station.setEndTime(new Date());
                    //sort赋值
                    BasePipelineStationRelView stationRelView = basePipelineStationRelViewMapper.selectMaxSortByPipelineId(station);
                    Integer sortOrder = 0;
                    if (stationRelView != null && stationRelView.getSort() != null) {
                        sortOrder = sortOrder + stationRelView.getSort() + 1;
                    }
                    entity.setSort(sortOrder);
                    segmentMapper.insert(baseSegmentEnt);
                    //6、创建管段反输记录（默认不反输）。base_segment_direction
                    BaseSegmentDirectionEnt baseSegmentDirectionEnt = new BaseSegmentDirectionEnt();
                    baseSegmentDirectionEnt.setBeReverse(1);
                    baseSegmentDirectionEnt.setSegmentId(segmentId);
                    baseSegmentDirectionEnt.setId(UUID.randomUUID().toString());
                    baseSegmentDirectionEnt.setEffectTime(new Date());
                    baseSegmentDirectionEnt.setEndTime(endTime);
                    baseSegmentDirectionEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentDirectionEnt.setCreateByName(context.getUser().getName());
                    baseSegmentDirectionEnt.setCreateTime(new Date());
                    baseSegmentDirectionMapper.insert(baseSegmentDirectionEnt);
                    //创建管段转供关系(根据场站id，查询所有起始场站为当前场站的管段，管线信息)
                    BaseSegmentView baseSegmentView = new BaseSegmentView();
                    baseSegmentView.setStartStationId(id);
                    List<BaseSegmentView> baseSegmentEnts = viewMapper.selectListByStartStationId(baseSegmentView);
                    if (baseSegmentEnts != null && baseSegmentEnts.size() > 0) {
                        for (BaseSegmentView baseSegment : baseSegmentEnts) {
                            BaseSegmentPipelineTransferRelEnt baseSegmentPipelineTransferRel = new BaseSegmentPipelineTransferRelEnt();
                            baseSegmentPipelineTransferRel.setSegmentId(segmentId);
                            baseSegmentPipelineTransferRel.setStartPipelineId(input.getPipelineId());
                            baseSegmentPipelineTransferRel.setStartStationId(id);
                            baseSegmentPipelineTransferRel.setEndPipelineId(baseSegment.getPipelineId());
                            baseSegmentPipelineTransferRel.setEndStationId(baseSegment.getStartStationId());
                            baseSegmentPipelineTransferRel.setEffectTime(new Date());
                            baseSegmentPipelineTransferRel.setEndTime(endTime);
                            baseSegmentPipelineTransferRel.setCreateById(context.getSession(PpsUserSession.class).getId());
                            baseSegmentPipelineTransferRel.setCreateByName(context.getUser().getName());
                            baseSegmentPipelineTransferRel.setCreateTime(new Date());
                            relMapper.insert(baseSegmentPipelineTransferRel);
                        }
                    }
                    //新增管线管段关联表
                    BasePipelineSegmentRelEnt opsPipelineSegmentRelEnt = new BasePipelineSegmentRelEnt();
                    opsPipelineSegmentRelEnt.setPipelineId(baseStationView.getPipelineId());
                    opsPipelineSegmentRelEnt.setSegmentId(segmentId);
                    opsPipelineSegmentRelEnt.setId(UUID.randomUUID().toString());
                    opsPipelineSegmentRelEnt.setEffectTime(new Date());
                    opsPipelineSegmentRelEnt.setEndTime(endTime);
                    opsPipelineSegmentRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    opsPipelineSegmentRelEnt.setCreateByName(context.getUser().getName());
                    opsPipelineSegmentRelEnt.setCreateTime(new Date());
                    pipelineSegmentRel.insert(opsPipelineSegmentRelEnt);
                }
            }
            return XServiceResult.OK;
        });
    }
}
