package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.BaseBaseClientCloudService;
import pps.cloud.base.service.data.*;
import pps.cloud.base.service.data.base_pipeline.SelectClientByConditionInput;
import pps.core.base.entity.BaseClientEnt;
import pps.core.base.entity.BaseClientView;
import pps.core.base.mapper.BaseClientMapper;
import pps.core.base.mapper.BaseClientViewMapper;
import pps.core.base.service.data.base_client.UpdateBaseClientInput;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;

@XService
public class BaseClientCloudServiceImpl implements BaseBaseClientCloudService {

    @Override
    @XText("根据名称获取托运商数据")
    public XSingleResult<FindBaseClientOutput> selectShipperByName(XContext context, BaseNameInput input) {
        BaseClientEnt ent = new BaseClientEnt();
        ent.setClientName(input.getName());
        ent.setBeConsignor(1);
        FindBaseClientOutput result = this.selectBaseClient(context, ent);
        return XSingleResult.success(result);
    }

    @Override
    @XText("根据名称获取上游用户数据")
    public XSingleResult<FindBaseClientOutput> selectUpClientByName(XContext context, BaseNameInput input) {
        BaseClientEnt ent = new BaseClientEnt();
        ent.setClientName(input.getName());
        ent.setBeUpLient(1);
        FindBaseClientOutput result = this.selectBaseClient(context, ent);
        return XSingleResult.success(result);
    }

    @Override
    @XText("根据名称获取下游数据")
    public XSingleResult<FindBaseClientOutput> selectDownClientByName(XContext context, BaseNameInput input) {
        BaseClientEnt ent = new BaseClientEnt();
        ent.setClientName(input.getName());
        ent.setBeDownClient(1);
        FindBaseClientOutput result = this.selectBaseClient(context, ent);
        return XSingleResult.success(result);
    }

    @Override
    @XText("根据条件获取Client数据")
    public XListResult<FindBaseClientOutput> selectClientByCondition(XContext context, SelectClientByConditionInput input) {
        BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
        QueryWrapper<BaseClientEnt> qr = new QueryWrapper<>();
        qr.lambda().eq(BaseClientEnt::getBeDeleted,0).eq(BaseClientEnt::getOuId,input.getOuId());
        if(input.getBeConsignor() != null){
            qr.lambda().eq(BaseClientEnt::getBeConsignor,input.getBeConsignor());
        }else if(input.getBeDownClient() != null){
            qr.lambda().eq(BaseClientEnt::getBeDownClient,input.getBeDownClient());
        }else if(input.getBeUpLient() != null){
            qr.lambda().eq(BaseClientEnt::getBeUpLient,input.getBeUpLient());
        }
        List<BaseClientEnt> list = mapper.selectList(qr);
        return XListResult.success(XCopyUtils.copyNewList(list,FindBaseClientOutput.class));
    }

    private FindBaseClientOutput selectBaseClient(XContext context, BaseClientEnt ent){
        BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
        QueryWrapper<BaseClientEnt> qr = new QueryWrapper<>();
        qr.lambda().eq(BaseClientEnt::getClientName,ent.getClientName()).eq(BaseClientEnt::getBeDeleted,0);
        if(ent.getBeConsignor() != null){
            qr.lambda().eq(BaseClientEnt::getBeConsignor,ent.getBeConsignor());
        }else if(ent.getBeDownClient() != null){
            qr.lambda().eq(BaseClientEnt::getBeDownClient,ent.getBeDownClient());
        }else if(ent.getBeUpLient() != null){
            qr.lambda().eq(BaseClientEnt::getBeUpLient,ent.getBeUpLient());
        }
        List<BaseClientEnt> dbEnts = mapper.selectList(qr);
        if(dbEnts.size() == 0 ) return null;
        return XCopyUtils.copyNewObject(dbEnts.get(0),FindBaseClientOutput.class);
    }

    @XApiPost
    @XText("查询上游下游关系列表")
    @Override
    public XListResult<QueryBaseClientViewOutput> queryBaseClientStationRelByOuId(XContext context, QueryBaseClientViewInput input){
        BaseClientViewMapper mapper = context.getBean(BaseClientViewMapper.class);
        BaseClientView record = new BaseClientView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseClientView> list = mapper.selectListByStationByOuId(record);
        PageInfo<BaseClientView> pageInfo = new PageInfo<>(list);
        List<QueryBaseClientViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseClientViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XSingleResult<QueryBaseClientViewOutput> selectBaseClientById(XContext context, NameIdInput input) {
        BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
        BaseClientEnt ent = mapper.selectById(input.getId());
        if(ent == null){
            return XSingleResult.error(context, XError.NotFound);
        }
        QueryBaseClientViewOutput output = new QueryBaseClientViewOutput();
        XCopyUtils.copyObject(ent, output);
        return XSingleResult.success(output);
    }

    @XText("更新上下游用户与托运商")
    @Override
    public XServiceResult updateBaseClient(XContext context, UpdateBaseClientCloudInput input){
        return XTransactionHelper.begin(context,()-> {
            BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
            QueryWrapper<BaseClientEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseClientEnt::getId, input.getId());
            BaseClientEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            if (StringUtils.isBlank(input.getModifyById())) {
                PpsUserSession session = context.getSession(PpsUserSession.class);
                Date date = new Date();
                input.setModifyTime(date);
                input.setModifyByName(session.getUserName());
                input.setModifyById(session.getId() + "");
            }
            input.setBeDeleted(0);
            XCopyUtils.copyObject(input, entity);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }
}
