package pps.core.base.service;

import java.util.Date;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.data.AddressInfoModel;
import pps.cloud.system.service.data.GetSysAreaInput;
import pps.cloud.system.service.data.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.base_client.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.base.util.XDateUtils;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.UUID;

@XText("上下游用户与托运商")
@XService
public class BaseClientService {

    @XApiPost
    @XText("创建上下游用户与托运商")
    public XServiceResult createBaseClient(XContext context, CreateBaseClientInput input){
        return XTransactionHelper.begin(context,()-> {
            BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
            BaseClientViewMapper viewMapper = context.getBean(BaseClientViewMapper.class);
            BaseClientEnt entity = new BaseClientEnt();
            if(StringUtils.isBlank(input.getCreateById())){
                PpsUserSession session = context.getSession(PpsUserSession.class);
                Date date = new Date();
                input.setModifyTime(date);
                input.setModifyByName(session.getUserName());
                input.setModifyById(session.getId()+"");
                input.setCreateTime(date);
                input.setCreateByName(session.getUserName());
                input.setCreateById(session.getId()+"");
            }
            input.setBeDeleted(0);
            input.setId(UUID.randomUUID().toString());
            XCopyUtils.copyObject(input, entity);
            if(StringUtils.isNotBlank(input.getVendorType()) && StringUtils.isBlank(entity.getClientType())){
                entity.setClientType(input.getVendorType());
            }
            BaseClientView record = new BaseClientView();
            BaseClientView view = viewMapper.selectMaxSort(record);
            if(ObjectUtils.isNotEmpty(view)){
                Integer sort = view.getSort();
                entity.setSort(sort+1);
            }
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }


    @XApiPost
    @XText("更新上下游用户与托运商")
    public XServiceResult updateBaseClient(XContext context, UpdateBaseClientInput input){
        return XTransactionHelper.begin(context,()-> {
            BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
            QueryWrapper<BaseClientEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseClientEnt::getId, input.getId());
            BaseClientEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            PpsUserSession session = context.getSession(PpsUserSession.class);
            Date date = new Date();
            input.setModifyTime(date);
            input.setModifyByName(session.getUserName());
            input.setModifyById(session.getId() + "");
            input.setBeDeleted(0);
            XCopyUtils.copyObject(input, entity);
            if(StringUtils.isNotBlank(input.getVendorType())){
                entity.setClientType(input.getVendorType());
            }
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }


    @XApiPost
    @XText("删除上下游用户与托运商")
    public XServiceResult deleteBaseClient(XContext context, DeleteBaseClientInput input){
        BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
        QueryWrapper<BaseClientEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseClientEnt::getId, input.getId());
        BaseClientEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
//        mapper.deleteById(entity);
        //使用逻辑删除
        input.setBeDeleted(1);
        entity.setBeDeleted(1);
        entity.setEndTime(XDateUtils.addMunites(new Date(),-1));
        mapper.updateById(entity);
        return XServiceResult.OK;
    }


    @XApiPost
    @XText("获取上下游用户与托运商详情")
    public XSingleResult<GetBaseClientOutput> getBaseClient(XContext context, GetBaseClientInput input){
        BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
        QueryWrapper<BaseClientEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseClientEnt::getId, input.getId());
        BaseClientEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseClientOutput output = new GetBaseClientOutput();
        XCopyUtils.copyObject(entity, output);
        output.setVendorType(entity.getClientType());
        if(entity.getRegion() != null && StringUtils.isNotBlank(entity.getRegion())){
            int i = Integer.parseInt(entity.getRegion());
            output.setRegion(i);
        }
        if(entity.getCity() != null && StringUtils.isNotBlank(entity.getCity())){
            int i = Integer.parseInt(entity.getCity());
            output.setCity(i);
        }
        if(entity.getProvince() != null && StringUtils.isNotBlank(entity.getProvince())){
            int i = Integer.parseInt(entity.getProvince());
            output.setProvince(i);
        }
        if(entity.getCounty() != null && StringUtils.isNotBlank(entity.getCounty())){
            int i = Integer.parseInt(entity.getCounty());
            output.setCounty(i);
        }
        //获取组织机构名称
        if(StringUtils.isNotBlank(output.getOuId())){
            SysOrganizationCloudService sysOrganizationCloudService = context.getBean(SysOrganizationCloudService.class);
            GetSysOrganizationViewInput record = new GetSysOrganizationViewInput();
            record.setId(output.getOuId());
            XSingleResult<GetSysOrganizationViewOutput> ouResult = sysOrganizationCloudService.getSysOrganizationById(context, record);
            GetSysOrganizationViewOutput ou = ouResult.getResult();
            if(ou != null){
                output.setOuName(ou.getOuName());
            }
        }
        return XSingleResult.success(output);
    }


    @XApiPost
    @XText("分页上下游用户与托运商列表")
    public XPageResult<QueryBaseClientOutput> queryBaseClient(XContext context, QueryBaseClientInput input){
        BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
        /*BaseClientViewMapper viewMapper = context.getBean(BaseClientViewMapper.class);
        BaseClientView record = new BaseClientView();
        XCopyUtils.copyObject(input, record);
        if(StringUtils.equals("upUser",input.getBeConsignor())){
            record.setBeUpLient(1);
        }
        if(StringUtils.equals("downUser",input.getBeConsignor())){
            record.setBeDownClient(1);
        }
        if(StringUtils.equals("consignor",input.getBeConsignor())){
            record.setBeConsignor(1);
        }
        List<BaseClientView> viewList= viewMapper.selectListByPage(record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        PageInfo<BaseClientView> pageInfo = new PageInfo<>(viewList);
        List<QueryBaseClientOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseClientOutput.class);*/

        QueryWrapper<BaseClientEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseClientEnt::getBeDeleted,0); //未删除
        if(StringUtils.isNotBlank(input.getClientName())){
            queryWrapper.lambda().like(BaseClientEnt::getClientName, input.getClientName());
        }
        if(StringUtils.isNotBlank(input.getClientCode())){
            queryWrapper.lambda().like(BaseClientEnt::getClientCode, input.getClientCode());
        }
        if(StringUtils.isNotBlank(input.getBeConsignor()) && StringUtils.equals("consignor",input.getBeConsignor())){
            queryWrapper.lambda().eq(BaseClientEnt::getBeConsignor, "1");
        }
        if(StringUtils.isNotBlank(input.getBeConsignor()) && StringUtils.equals("downUser",input.getBeConsignor())){
            queryWrapper.lambda().eq(BaseClientEnt::getBeDownClient, "1");
        }
        if(StringUtils.isNotBlank(input.getBeConsignor()) && StringUtils.equals("upUser",input.getBeConsignor())){
            queryWrapper.lambda().eq(BaseClientEnt::getBeUpLient, "1");
        }
//        if(input.getBeConsignor() != null || input.getBeDownClient() != null || input.getBeUpLient() != null ){
//            queryWrapper.lambda().and(wrapper-> {
//                if(input.getBeConsignor() != null)
//                    wrapper.or(w->w.eq(BaseClientEnt::getBeConsignor,input.getBeConsignor()));
//                if(input.getBeDownClient() != null)
//                    wrapper.or(w->w.eq(BaseClientEnt::getBeDownClient,input.getBeDownClient()));
//                if(input.getBeUpLient() != null)
//                    wrapper.or(w->w.eq(BaseClientEnt::getBeUpLient,input.getBeUpLient()));
//            });
//        }
        queryWrapper.lambda().gt(BaseClientEnt::getEndTime,new Date());
        queryWrapper.lambda().orderByDesc(BaseClientEnt::getCreateTime,BaseClientEnt::getSort);
        XPageResult<QueryBaseClientOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QueryBaseClientOutput.class);
        List<QueryBaseClientOutput> items = query.getResult().getItems();
        if(ObjectUtils.isNotEmpty(items)){
            for(QueryBaseClientOutput item:items){
                StringBuffer str = new StringBuffer();
                if(item.getBeConsignor() != null && item.getBeConsignor() ==1){
                    str.append("托运商,");
                }
                if(item.getBeDownClient() != null && item.getBeDownClient() ==1){
                    str.append("下游用户,");
                }
                if(item.getBeUpLient() != null && item.getBeUpLient() ==1){
                    str.append("上游用户,");
                }
                if(str.length()>0){
                    item.setUserTypeStr(str.substring(0,str.length()-1));
                }
                if(item.getRegion() != null && StringUtils.isNotBlank(item.getRegion())){
                    int i = Integer.parseInt(item.getRegion());
                    item.setRegionInt(i);
                }
            }
        }
        return  query;
    }


    @XApiPost
    public XPageResult<QueryBaseClientOutput> dynamicQueryBaseClient(XContext context, CustomQueryInput input){
        BaseClientMapper mapper = context.getBean(BaseClientMapper.class);
        return XMapperHelper.query(mapper, input, BaseClientEnt.class, QueryBaseClientOutput.class);
    }


    @XApiPost
    public XSingleResult<GetBaseClientViewOutput> getBaseClientView(XContext context, GetBaseClientViewInput input){
        BaseClientViewMapper mapper = context.getBean(BaseClientViewMapper.class);
        BaseClientView record = new BaseClientView();
        XCopyUtils.copyObject(input, record);
        BaseClientView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseClientViewOutput output = new GetBaseClientViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }


    @XApiPost
    @XText("分页上下游用户与托运商列表")
    public XPageResult<QueryBaseClientViewOutput> queryBaseClientView(XContext context, QueryBaseClientViewInput input){
        BaseClientViewMapper mapper = context.getBean(BaseClientViewMapper.class);
        BaseClientView record = new BaseClientView();
        if(input.getType() != null){
            switch (input.getType()){
                case 0:
                    break;
                case 1:
                    input.setBeUpLient(1);
                    break;
                case 2:
                    input.setBeDownClient(1);
                    break;
                case 3:
                    input.setBeConsignor(1);
                    break;
            }
        }
        if(input.getBeDeleted() == null)
            input.setBeDeleted(0);
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseClientView> list = mapper.selectList(record);
        PageInfo<BaseClientView> pageInfo = new PageInfo<>(list);
        List<QueryBaseClientViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseClientViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiPost
    @XText("获取上下游用户与托运商列表-下拉")
    public XListResult<QueryBaseClientPullOutput> queryBaseClientViewList(XContext context, QueryBaseClientViewInput input){
        BaseClientViewMapper mapper = context.getBean(BaseClientViewMapper.class);
        BaseClientView record = new BaseClientView();
        if(input.getType() != null){
            switch (input.getType()){
                case 0:
                    break;
                case 1:
                    input.setBeUpLient(1);
                    break;
                case 2:
                    input.setBeDownClient(1);
                    break;
                case 3:
                    input.setBeConsignor(1);
                    break;
            }
        }
        if(input.getBeDeleted() == null)
            input.setBeDeleted(0);
        XCopyUtils.copyObject(input, record);
        List<BaseClientView> list = mapper.selectList(record);
        List<QueryBaseClientPullOutput> outputs = XCopyUtils.copyNewList(list, QueryBaseClientPullOutput.class);
        return XListResult.success(outputs);
    }
}
