package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.entity.BaseEquipmentExtendPropertyEnt;
import pps.core.base.entity.BaseEquipmentExtendPropertyView;
import pps.core.base.mapper.BaseEquipmentExtendPropertyMapper;
import pps.core.base.mapper.BaseEquipmentExtendPropertyViewMapper;
import pps.core.base.service.data.base_equipment_extend_property.*;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysDictionaryEnt;
import pps.core.system.entity.SysDictionaryView;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.mapper.SysDictionaryViewMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("设备扩展属性")
@XService
public class BaseEquipmentExtendPropertyService {

    @XText("创建设备扩展属性")
    @XApiPost
    public XServiceResult createBaseEquipmentExtendProperty(XContext context, CreateBaseEquipmentExtendPropertyInput input){
        BaseEquipmentExtendPropertyMapper mapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseEquipmentExtendPropertyEnt entity = new BaseEquipmentExtendPropertyEnt();
        XCopyUtils.copyObject(input, entity);
        entity.setId(UUID.randomUUID().toString());
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getLoginName());
        entity.setCreateTime(new Date());
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改设备扩展属性")
    @XApiPost
    public XServiceResult updateBaseEquipmentExtendProperty(XContext context, UpdateBaseEquipmentExtendPropertyInput input){
        BaseEquipmentExtendPropertyMapper mapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<BaseEquipmentExtendPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseEquipmentExtendPropertyEnt::getId, input.getId());
        BaseEquipmentExtendPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getLoginName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除设备扩展属性")
    @XApiPost
    public XServiceResult deleteBaseEquipmentExtendProperty(XContext context, DeleteBaseEquipmentExtendPropertyInput input){
        BaseEquipmentExtendPropertyMapper mapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        QueryWrapper<BaseEquipmentExtendPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseEquipmentExtendPropertyEnt::getId, input.getId());
        BaseEquipmentExtendPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("获取设备详情接口")
    @XApiPost
    public XSingleResult<GetBaseEquipmentExtendPropertyOutput> getBaseEquipmentExtendProperty(XContext context, GetBaseEquipmentExtendPropertyInput input){
        BaseEquipmentExtendPropertyMapper mapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        QueryWrapper<BaseEquipmentExtendPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseEquipmentExtendPropertyEnt::getId, input.getId());
        BaseEquipmentExtendPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEquipmentExtendPropertyOutput output = new GetBaseEquipmentExtendPropertyOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("获取设备扩展属性列表")
    @XApiPost
    public XPageResult<QueryBaseEquipmentExtendPropertyOutput> queryBaseEquipmentExtendProperty(XContext context, QueryBaseEquipmentExtendPropertyInput input){
        BaseEquipmentExtendPropertyMapper mapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        QueryWrapper<BaseEquipmentExtendPropertyEnt> queryWrapper = new QueryWrapper<>();
        if(StringUtils.isNotBlank(input.getPropertyName())){
            queryWrapper.lambda().like(BaseEquipmentExtendPropertyEnt::getPropertyName, input.getPropertyName());
        }
        if(StringUtils.isNotBlank(input.getPropertyType())){
            queryWrapper.lambda().like(BaseEquipmentExtendPropertyEnt::getPropertyType, input.getPropertyType());
        }
        if(StringUtils.isNotBlank(input.getUseEquipmentType())){
            queryWrapper.lambda().like(BaseEquipmentExtendPropertyEnt::getUseEquipmentType, input.getUseEquipmentType());
        }
        queryWrapper.lambda().orderByDesc(BaseEquipmentExtendPropertyEnt::getCreateTime);
        XPageResult<QueryBaseEquipmentExtendPropertyOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QueryBaseEquipmentExtendPropertyOutput.class);
        //调用微服务查询字典
        CreateSysDictionaryInput mediumType = new CreateSysDictionaryInput();
        mediumType.setAlias("PROPERTY_TYPE");
        List<QuerySysDictionaryViewOutput> propertyTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediumType).getResult();
        QuerySysDictionaryViewOutput dict = getDictByAlias(context , "equipment_type");
        List<QuerySysDictionaryViewOutput> dictList = queryChildByDicPath(context , dict);
        List<QueryBaseEquipmentExtendPropertyOutput> items = query.getResult().getItems();
        if(ObjectUtils.isNotEmpty(items)){
            items.stream().forEach(item->{
                item.setUseEquipmentTypeName(getDictName(item.getUseEquipmentType() + "" , dictList));
            });
            for(QueryBaseEquipmentExtendPropertyOutput out:items){
                for(QuerySysDictionaryViewOutput dic:propertyTypeList){
                    if(StringUtils.equals(dic.getDicKey(),out.getPropertyType())){
                        out.setPropertyTypeName(dic.getDicName());
                    }
                }
            }
        }


        return query;
    }

    public String getDictName(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return " ";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict.getDicName();
            }
        }
        return dictKey;
    }

    private List<QuerySysDictionaryViewOutput> queryChildByDicPath(XContext context , QuerySysDictionaryViewOutput dict){
        SystemDictionaryService systemDictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictionaryViewInput = new QuerySysDictionaryViewInput();
        dictionaryViewInput.setDicPath(dict.getId());
        dictionaryViewInput.setPage(1);
        dictionaryViewInput.setLimit(1000);
        List<QuerySysDictionaryViewOutput> dictList = systemDictionaryService.queryChildByDicPath(context , dictionaryViewInput).getResult();
        return dictList;
    }

    private QuerySysDictionaryViewOutput getDictByAlias(XContext context , String alias){
        SystemDictionaryService systemDictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictionaryViewInput = new QuerySysDictionaryViewInput();
        dictionaryViewInput.setPage(1);
        dictionaryViewInput.setLimit(1000);
        dictionaryViewInput.setAlias(alias);
        QuerySysDictionaryViewOutput dict = systemDictionaryService.queryByAlias(context , dictionaryViewInput).getResult();
        return dict;
    }

    @XApiPost
    @XText("查询设备扩展属性列表")
    public XListResult<QueryBaseEquipmentExtendPropertyOutput> getBaseEquipmentExtendPropertyList(XContext context, GetBaseEquipmentExtendPropertyInput input) {
        BaseEquipmentExtendPropertyMapper mapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        SysDictionaryViewMapper dicMapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryMapper dic = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryDic = new QueryWrapper<>();
        queryDic.lambda().like(SysDictionaryEnt::getDicPath,"4d35d185-125f-4d79-97d3-3dc387d47c20");
        queryDic.lambda().eq(SysDictionaryEnt::getDicKey,input.getEquipmentType());
        queryDic.lambda().gt(SysDictionaryEnt::getEndTime,new Date());
        SysDictionaryEnt dicObj = dic.selectOne(queryDic);

        SysDictionaryView record = new SysDictionaryView();
        record.setDicKey(input.getEquipmentType());
        record.setId(dicObj.getId());
        List<SysDictionaryView> list = dicMapper.selectListByEquipmentType(record);
        QueryWrapper<BaseEquipmentExtendPropertyEnt> queryExtend = new QueryWrapper();
        List<String> ids = new ArrayList<>();
        if(ObjectUtils.isNotEmpty(list)){
            ids = list.stream().map(p -> p.getId()).distinct().collect(Collectors.toList());
        }
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(SysDictionaryEnt::getId,ids);
        List<SysDictionaryEnt> sysDictionaryEnts = dic.selectList(queryWrapper);
        List<String> dicKeys = new ArrayList<>();
        if(ObjectUtils.isNotEmpty(list)){
            dicKeys = sysDictionaryEnts.stream().map(p -> p.getDicKey()).distinct().collect(Collectors.toList());
        }
        queryExtend.lambda().in(BaseEquipmentExtendPropertyEnt::getUseEquipmentType,dicKeys);
        List<BaseEquipmentExtendPropertyEnt> extendPropertyEnts = mapper.selectList(queryExtend);
        List<BaseEquipmentExtendPropertyEnt> queryList = new ArrayList<>();
        if(ObjectUtils.isNotEmpty(extendPropertyEnts)){
            for(BaseEquipmentExtendPropertyEnt ent:extendPropertyEnts){
                if(StringUtils.equals("1",String.valueOf(ent.getIsUseAll()))){
                    queryList.add(ent);
                }else if(StringUtils.equals(input.getEquipmentType(),String.valueOf(ent.getUseEquipmentType()))){
                    queryList.add(ent);
                }
            }
        }
        List<QueryBaseEquipmentExtendPropertyOutput> viewList = XCopyUtils.copyNewList(queryList, QueryBaseEquipmentExtendPropertyOutput.class);
        return XListResult.success(viewList);
    }

    @XText("")
    @XApiPost
    public XPageResult<QueryBaseEquipmentExtendPropertyOutput> dynamicQueryBaseEquipmentExtendProperty(XContext context, CustomQueryInput input){
        BaseEquipmentExtendPropertyMapper mapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        return XMapperHelper.query(mapper, input, BaseEquipmentExtendPropertyEnt.class, QueryBaseEquipmentExtendPropertyOutput.class);
    }

    @XText("")
    @XApiPost
    public XSingleResult<GetBaseEquipmentExtendPropertyViewOutput> queryBaseEquipmentExtendPropertyView(XContext context, GetBaseEquipmentExtendPropertyViewInput input){
        BaseEquipmentExtendPropertyViewMapper mapper = context.getBean(BaseEquipmentExtendPropertyViewMapper.class);
        BaseEquipmentExtendPropertyView record = new BaseEquipmentExtendPropertyView();
        XCopyUtils.copyObject(input, record);
        BaseEquipmentExtendPropertyView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEquipmentExtendPropertyViewOutput output = new GetBaseEquipmentExtendPropertyViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XText("")
    @XApiPost
    public XPageResult<QueryBaseEquipmentExtendPropertyViewOutput> queryBaseEquipmentExtendPropertyView(XContext context, QueryBaseEquipmentExtendPropertyViewInput input){
        BaseEquipmentExtendPropertyViewMapper mapper = context.getBean(BaseEquipmentExtendPropertyViewMapper.class);
        BaseEquipmentExtendPropertyView record = new BaseEquipmentExtendPropertyView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseEquipmentExtendPropertyView> list = mapper.selectList(record);
        PageInfo<BaseEquipmentExtendPropertyView> pageInfo = new PageInfo<>(list);
        List<QueryBaseEquipmentExtendPropertyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseEquipmentExtendPropertyViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
