package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.base_equipment_property.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XPageResult;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.Date;
import java.util.List;

@XService
public class BaseEquipmentPropertyService {

    @XApiPost
    public XServiceResult createBaseEquipmentProperty(XContext context, CreateBaseEquipmentPropertyInput input){
        BaseEquipmentPropertyMapper mapper = context.getBean(BaseEquipmentPropertyMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseEquipmentPropertyEnt entity = new BaseEquipmentPropertyEnt();
        XCopyUtils.copyObject(input, entity);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getLoginName());
        entity.setCreateTime(new Date());
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult updateBaseEquipmentProperty(XContext context, UpdateBaseEquipmentPropertyInput input){
        BaseEquipmentPropertyMapper mapper = context.getBean(BaseEquipmentPropertyMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<BaseEquipmentPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseEquipmentPropertyEnt::getId, input.getId());
        BaseEquipmentPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getLoginName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }


    @XApiPost
    public XServiceResult deleteBaseEquipmentProperty(XContext context, DeleteBaseEquipmentPropertyInput input){
        BaseEquipmentPropertyMapper mapper = context.getBean(BaseEquipmentPropertyMapper.class);
        QueryWrapper<BaseEquipmentPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseEquipmentPropertyEnt::getId, input.getId());
        BaseEquipmentPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }


    @XApiGet
    public XSingleResult<GetBaseEquipmentPropertyOutput> getBaseEquipmentProperty(XContext context, GetBaseEquipmentPropertyInput input){
        BaseEquipmentPropertyMapper mapper = context.getBean(BaseEquipmentPropertyMapper.class);
        QueryWrapper<BaseEquipmentPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseEquipmentPropertyEnt::getId, input.getId());
        BaseEquipmentPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEquipmentPropertyOutput output = new GetBaseEquipmentPropertyOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }


    @XApiGet
    public XPageResult<QueryBaseEquipmentPropertyOutput> queryBaseEquipmentProperty(XContext context, QueryBaseEquipmentPropertyInput input){
        BaseEquipmentPropertyMapper mapper = context.getBean(BaseEquipmentPropertyMapper.class);
        QueryWrapper<BaseEquipmentPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseEquipmentPropertyEnt::getId, input.getId())
            .like(BaseEquipmentPropertyEnt::getEquipmentId, input.getEquipmentId())
            .like(BaseEquipmentPropertyEnt::getPropertyId, input.getPropertyId())
            .like(BaseEquipmentPropertyEnt::getPropertyNameEn, input.getPropertyNameEn())
            .like(BaseEquipmentPropertyEnt::getPropertyValue, input.getPropertyValue())
            .like(BaseEquipmentPropertyEnt::getCreateById, input.getCreateById())
            .like(BaseEquipmentPropertyEnt::getCreateByName, input.getCreateByName())
            .like(BaseEquipmentPropertyEnt::getModifyById, input.getModifyById())
            .like(BaseEquipmentPropertyEnt::getModifyByName, input.getModifyByName());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseEquipmentPropertyOutput.class);
    }


    @XApiGet
    public XPageResult<QueryBaseEquipmentPropertyOutput> dynamicQueryBaseEquipmentProperty(XContext context, CustomQueryInput input){
        BaseEquipmentPropertyMapper mapper = context.getBean(BaseEquipmentPropertyMapper.class);
        return XMapperHelper.query(mapper, input, BaseEquipmentPropertyEnt.class, QueryBaseEquipmentPropertyOutput.class);
    }


    @XApiGet
    public XSingleResult<GetBaseEquipmentPropertyViewOutput> getBaseEquipmentPropertyView(XContext context, GetBaseEquipmentPropertyViewInput input){
        BaseEquipmentPropertyViewMapper mapper = context.getBean(BaseEquipmentPropertyViewMapper.class);
        BaseEquipmentPropertyView record = new BaseEquipmentPropertyView();
        XCopyUtils.copyObject(input, record);
        BaseEquipmentPropertyView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEquipmentPropertyViewOutput output = new GetBaseEquipmentPropertyViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }


    @XApiGet
    public XPageResult<QueryBaseEquipmentPropertyViewOutput> queryBaseEquipmentPropertyView(XContext context, QueryBaseEquipmentPropertyViewInput input){
        BaseEquipmentPropertyViewMapper mapper = context.getBean(BaseEquipmentPropertyViewMapper.class);
        BaseEquipmentPropertyView record = new BaseEquipmentPropertyView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseEquipmentPropertyView> list = mapper.selectList(record);
        PageInfo<BaseEquipmentPropertyView> pageInfo = new PageInfo<>(list);
        List<QueryBaseEquipmentPropertyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseEquipmentPropertyViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据设备id和属性英文名称查询设备属性值")
    @XApiGet
    public XSingleResult<BaseEquipmentPropertyView> queryBaseEquipmentPropertyView(XContext context, BaseEquipmentPropertyView input){
        BaseEquipmentPropertyMapper mapper = context.getBean(BaseEquipmentPropertyMapper.class);
        QueryWrapper<BaseEquipmentPropertyEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(BaseEquipmentPropertyEnt::getEquipmentId,input.getEquipmentId()).
                eq(BaseEquipmentPropertyEnt::getPropertyNameEn,input.getPropertyNameEn());
        BaseEquipmentPropertyEnt Ent = mapper.selectOne(queryWrapper);
        BaseEquipmentPropertyView view = XCopyUtils.copyNewObject(Ent, BaseEquipmentPropertyView.class);
        return XSingleResult.success(view);
    }

}
