package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_equipment.*;
import pps.core.base.service.data.base_equipment_client_rel.CreateBaseEquipmentClientRelInput;
import pps.core.base.service.data.base_equipment_extend_property.GetBaseEquipmentExtendPropertyInput;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysDictionaryEnt;
import pps.core.system.entity.SysDictionaryView;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.mapper.SysDictionaryViewMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("设备管理")
@XService
public class BaseEquipmentService {

    @XApiPost
    @XText("创建设备管理")
    public XServiceResult createBaseEquipment(XContext context, CreateBaseEquipmentInput input) {
        BaseEquipmentMapper mapper = context.getBean(BaseEquipmentMapper.class);
        BaseEquipmentPropertyMapper propertyMapper = context.getBean(BaseEquipmentPropertyMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseEquipmentEnt entity = new BaseEquipmentEnt();
        if(StringUtils.isBlank(input.getCreateById())){
            Date date = new Date();
            input.setModifyTime(date);
            input.setModifyByName(session.getUserName());
            input.setModifyById(session.getId()+"");
            input.setCreateTime(date);
            input.setCreateByName(session.getUserName());
            input.setCreateById(session.getId()+"");
        }
        XCopyUtils.copyObject(input, entity);
        String equipmentId = UUID.randomUUID().toString();
        entity.setId(equipmentId);
        mapper.insert(entity);
        //添加设备扩展属性列表
        if(ObjectUtils.isNotEmpty(input.getExtendModels())){
            List<EquipmentExtendModel> extendModels = input.getExtendModels();
            for(EquipmentExtendModel model:extendModels){
                List<String> propertyValue = model.getPropertyValue();
                if(ObjectUtils.isNotEmpty(propertyValue)){
                    for(String value:propertyValue){
                        BaseEquipmentPropertyEnt equipmentPropertyEnt = new BaseEquipmentPropertyEnt();
                        equipmentPropertyEnt.setId(UUID.randomUUID().toString());
                        equipmentPropertyEnt.setEquipmentId(equipmentId);
                        equipmentPropertyEnt.setPropertyId(model.getId());
                        equipmentPropertyEnt.setPropertyNameEn(model.getPropertyNameEn());
                        equipmentPropertyEnt.setPropertyValue(value);
                        equipmentPropertyEnt.setCreateTime(new Date());
                        equipmentPropertyEnt.setCreateByName(session.getLoginName());
                        equipmentPropertyEnt.setCreateById(session.getId());
                        propertyMapper.insert(equipmentPropertyEnt);
                    }
                }
            }
        }
        //1.添加关系表
        addRel(context , entity.getId() , input.getEquipmentClientRelList());
        return XServiceResult.OK;
    }

    @XApiPost
    @XText("修改设备管理")
    public XServiceResult updateBaseEquipment(XContext context, UpdateBaseEquipmentInput input) {
        BaseEquipmentMapper mapper = context.getBean(BaseEquipmentMapper.class);
        BaseEquipmentPropertyMapper propertyMapper = context.getBean(BaseEquipmentPropertyMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<BaseEquipmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseEquipmentEnt::getId, input.getId());
        BaseEquipmentEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        //1.先删除历史关系表
        deleteRel(context , input.getId());
        //2.添加新的关系表
        addRel(context , input.getId() , input.getEquipmentClientRelList());
        if(StringUtils.isBlank(input.getModifyById())){
            Date date = new Date();
            input.setModifyTime(date);
            input.setModifyByName(session.getUserName());
            input.setModifyById(session.getId()+"");
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        //删除旧设备扩展属性
        QueryWrapper<BaseEquipmentPropertyEnt> query = new QueryWrapper<>();
        query.lambda().eq(BaseEquipmentPropertyEnt::getEquipmentId,entity.getId());
        List<BaseEquipmentPropertyEnt> propertyEnts = propertyMapper.selectList(query);
        if(ObjectUtils.isNotEmpty(propertyEnts)) {
            for (BaseEquipmentPropertyEnt ent:propertyEnts) {
                propertyMapper.deleteById(ent);
            }
        }
        //添加设备扩展属性列表
        if(ObjectUtils.isNotEmpty(input.getExtendModels())){
            List<EquipmentExtendModel> extendModels = input.getExtendModels();
            for(EquipmentExtendModel model:extendModels){
                List<String> propertyValue = model.getPropertyValue();
                if(ObjectUtils.isNotEmpty(propertyValue)){
                    for(String value:propertyValue) {
                        BaseEquipmentPropertyEnt equipmentPropertyEnt = new BaseEquipmentPropertyEnt();
                        equipmentPropertyEnt.setId(UUID.randomUUID().toString());
                        equipmentPropertyEnt.setEquipmentId(entity.getId());
                        equipmentPropertyEnt.setPropertyId(model.getId());
                        equipmentPropertyEnt.setPropertyNameEn(model.getPropertyNameEn());
                        equipmentPropertyEnt.setPropertyValue(value);
                        equipmentPropertyEnt.setCreateTime(new Date());
                        equipmentPropertyEnt.setCreateByName(session.getLoginName());
                        equipmentPropertyEnt.setCreateById(session.getId());
                        propertyMapper.insert(equipmentPropertyEnt);
                    }
                }
            }
        }
        return XServiceResult.OK;
    }


    @XApiPost
    @XText("删除设备管理")
    public XServiceResult deleteBaseEquipment(XContext context, DeleteBaseEquipmentInput input) {
        BaseEquipmentMapper mapper = context.getBean(BaseEquipmentMapper.class);
        BaseEquipmentPropertyMapper propertyMapper = context.getBean(BaseEquipmentPropertyMapper.class);
        QueryWrapper<BaseEquipmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseEquipmentEnt::getId, input.getId());
        BaseEquipmentEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        deleteRel(context , input.getId());
        //查询设备扩展属性列表和属性值，组装页面对象
        QueryWrapper<BaseEquipmentPropertyEnt> query = new QueryWrapper<>();
        query.lambda().eq(BaseEquipmentPropertyEnt::getEquipmentId,entity.getId());
        List<BaseEquipmentPropertyEnt> propertyEnts = propertyMapper.selectList(query);
        if(ObjectUtils.isNotEmpty(propertyEnts)){
            for(BaseEquipmentPropertyEnt ent:propertyEnts){
                propertyMapper.deleteById(ent);
            }
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }


    @XApiGet
    @XText("获取设备管理详情")
    public XSingleResult<GetBaseEquipmentOutput> getBaseEquipment(XContext context, GetBaseEquipmentInput input) {
        BaseEquipmentViewMapper mapper = context.getBean(BaseEquipmentViewMapper.class);
        SysOrganizationCloudService orgService = context.getBean(SysOrganizationCloudService.class);
        BaseEquipmentPropertyMapper propertyMapper = context.getBean(BaseEquipmentPropertyMapper.class);
        BaseEquipmentExtendPropertyMapper extendPropertyMapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        BaseEquipmentView viewRecord = new BaseEquipmentView();
        XCopyUtils.copyObject(input, viewRecord);
        BaseEquipmentView entity = mapper.selectOne(viewRecord);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEquipmentOutput output = new GetBaseEquipmentOutput();
        XCopyUtils.copyObject(entity, output);
        BaseEquipmentClientRelViewMapper refMapper = context.getBean(BaseEquipmentClientRelViewMapper.class);
        BaseEquipmentClientRelView record = new BaseEquipmentClientRelView();
        record.setEquipmentId(input.getId());
        List<BaseEquipmentClientRelView> refList = refMapper.selectList(record);
        refList.stream().forEach(item->{
            item.setStationId(entity.getStationId());
            item.setStationName(entity.getStationName());
        });
        output.setEquipmentClientRelList(refList);

        //查询设备扩展属性列表和属性值，组装页面对象
        QueryWrapper<BaseEquipmentPropertyEnt> query = new QueryWrapper<>();
        query.lambda().eq(BaseEquipmentPropertyEnt::getEquipmentId,entity.getId());
        List<BaseEquipmentPropertyEnt> propertyEnts = propertyMapper.selectList(query);
        List<String> propertyIds = propertyEnts.stream().map(p -> {
            return p.getPropertyId();
        }).distinct().collect(Collectors.toList());
        List<EquipmentExtendModel> extendModels = new ArrayList<>();
        GetBaseEquipmentExtendPropertyInput view = new GetBaseEquipmentExtendPropertyInput();
        view.setEquipmentType(entity.getEquipmentType());
        List<BaseEquipmentExtendPropertyEnt> extendPropertyEnts = getBaseEquipmentExtendPropertyList(context,view);
        if(ObjectUtils.isNotEmpty(propertyIds)){
            /*ueryWrapper<BaseEquipmentExtendPropertyEnt> queryWrapper = new QueryWrapper();
            queryWrapper.lambda().in(BaseEquipmentExtendPropertyEnt::getUseEquipmentType,entity.getEquipmentType());
            List<BaseEquipmentExtendPropertyEnt> extendPropertyEnts = extendPropertyMapper.selectList(queryWrapper);*/
            if(ObjectUtils.isNotEmpty(extendPropertyEnts) && ObjectUtils.isNotEmpty(propertyEnts)){
                for(BaseEquipmentExtendPropertyEnt extendEnt:extendPropertyEnts){
                    EquipmentExtendModel equipmentExtendModel = XCopyUtils.copyNewObject(extendEnt, EquipmentExtendModel.class);
                    List<String> propertyValue = new ArrayList<>();
                    for(BaseEquipmentPropertyEnt propertyEnt:propertyEnts){
                        if(StringUtils.equals(propertyEnt.getPropertyId(),extendEnt.getId())){
                            propertyValue.add(propertyEnt.getPropertyValue());
                        }
                    }
                    equipmentExtendModel.setPropertyValue(propertyValue);
                    extendModels.add(equipmentExtendModel);
                }
                output.setExtendModels(extendModels);
            }
        }else{
            if(ObjectUtils.isNotEmpty(extendPropertyEnts)){
                for(BaseEquipmentExtendPropertyEnt extendEnt:extendPropertyEnts){
                    EquipmentExtendModel equipmentExtendModel = XCopyUtils.copyNewObject(extendEnt, EquipmentExtendModel.class);
                    extendModels.add(equipmentExtendModel);
                }
            }
            output.setExtendModels(extendModels);
        }
        GetSysOrganizationViewInput organizationViewInput = new GetSysOrganizationViewInput();
        organizationViewInput.setId(output.getOuid());
        GetSysOrganizationViewOutput org = orgService.getSysOrganizationById(context, organizationViewInput).getResult();
        if(ObjectUtils.isNotEmpty(org)){
            output.setOuName(org.getOuName());
        }
        return XSingleResult.success(output);
    }

    public List<BaseEquipmentExtendPropertyEnt> getBaseEquipmentExtendPropertyList(XContext context, GetBaseEquipmentExtendPropertyInput input) {
        BaseEquipmentExtendPropertyMapper mapper = context.getBean(BaseEquipmentExtendPropertyMapper.class);
        SysDictionaryViewMapper dicMapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryMapper dic = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryDic = new QueryWrapper<>();
        queryDic.lambda().like(SysDictionaryEnt::getDicPath,"4d35d185-125f-4d79-97d3-3dc387d47c20");
        queryDic.lambda().eq(SysDictionaryEnt::getDicKey,input.getEquipmentType());
        queryDic.lambda().gt(SysDictionaryEnt::getEndTime,new Date());
        SysDictionaryEnt dicObj = dic.selectOne(queryDic);
        List<BaseEquipmentExtendPropertyEnt> queryList = new ArrayList<>();
        if(null != dicObj){
            SysDictionaryView record = new SysDictionaryView();
            record.setDicKey(input.getEquipmentType());
            record.setId(dicObj.getId());
            List<SysDictionaryView> list = dicMapper.selectListByEquipmentType(record);
            QueryWrapper<BaseEquipmentExtendPropertyEnt> queryExtend = new QueryWrapper();
            List<String> ids = new ArrayList<>();
            if(ObjectUtils.isNotEmpty(list)){
                ids = list.stream().map(p -> p.getId()).distinct().collect(Collectors.toList());
            }
            QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().in(SysDictionaryEnt::getId,ids);
            List<SysDictionaryEnt> sysDictionaryEnts = dic.selectList(queryWrapper);
            List<String> dicKeys = new ArrayList<>();
            if(ObjectUtils.isNotEmpty(list)){
                dicKeys = sysDictionaryEnts.stream().map(p -> p.getDicKey()).distinct().collect(Collectors.toList());
            }
            queryExtend.lambda().in(BaseEquipmentExtendPropertyEnt::getUseEquipmentType,dicKeys).orderByDesc(BaseEquipmentExtendPropertyEnt::getCreateTime);
            List<BaseEquipmentExtendPropertyEnt> extendPropertyEnts = mapper.selectList(queryExtend);
            if(ObjectUtils.isNotEmpty(extendPropertyEnts)){
                for(BaseEquipmentExtendPropertyEnt ent:extendPropertyEnts){
                    if(StringUtils.equals("1",String.valueOf(ent.getIsUseAll()))){
                        queryList.add(ent);
                    }else if(StringUtils.equals(input.getEquipmentType(),String.valueOf(ent.getUseEquipmentType()))){
                        queryList.add(ent);
                    }
                }
            }
        }
        return queryList;
    }


    @XApiGet
    public XPageResult<QueryBaseEquipmentOutput> queryBaseEquipment(XContext context, QueryBaseEquipmentInput input) {
        BaseEquipmentMapper mapper = context.getBean(BaseEquipmentMapper.class);
        QueryWrapper<BaseEquipmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseEquipmentEnt::getId, input.getId())
                .like(BaseEquipmentEnt::getEquipmentTemplateId, input.getEquipmentTemplateId())
                .like(BaseEquipmentEnt::getOuid, input.getOuid())
                .like(BaseEquipmentEnt::getStationId, input.getStationId())
                .like(BaseEquipmentEnt::getSegmentId, input.getSegmentId())
                .eq(BaseEquipmentEnt::getEquipmentType, input.getEquipmentType())
                .like(BaseEquipmentEnt::getEquipmentName, input.getEquipmentName())
                .like(BaseEquipmentEnt::getEquipmentCode, input.getEquipmentCode())
                .like(BaseEquipmentEnt::getDescription, input.getDescription())
                .eq(BaseEquipmentEnt::getEffectTime, input.getEffectTime())
                .eq(BaseEquipmentEnt::getEndTime, input.getEndTime())
                .like(BaseEquipmentEnt::getCreateById, input.getCreateById())
                .like(BaseEquipmentEnt::getCreateByName, input.getCreateByName())
                .eq(BaseEquipmentEnt::getCreateTime, input.getCreateTime())
                .like(BaseEquipmentEnt::getModifyById, input.getModifyById())
                .like(BaseEquipmentEnt::getModifyByName, input.getModifyByName())
                .eq(BaseEquipmentEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseEquipmentOutput.class);
    }


    @XApiGet
    public XPageResult<QueryBaseEquipmentOutput> dynamicQueryBaseEquipment(XContext context, CustomQueryInput input) {
        BaseEquipmentMapper mapper = context.getBean(BaseEquipmentMapper.class);
        return XMapperHelper.query(mapper, input, BaseEquipmentEnt.class, QueryBaseEquipmentOutput.class);
    }


    @XApiGet
    public XSingleResult<GetBaseEquipmentViewOutput> getBaseEquipmentView(XContext context, GetBaseEquipmentViewInput input) {
        BaseEquipmentViewMapper mapper = context.getBean(BaseEquipmentViewMapper.class);
        BaseEquipmentView record = new BaseEquipmentView();
        XCopyUtils.copyObject(input, record);
        BaseEquipmentView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEquipmentViewOutput output = new GetBaseEquipmentViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }


    @XApiGet
    @XText("分页获取设备管理列表")
    public XPageResult<QueryBaseEquipmentViewOutput> queryBaseEquipmentView(XContext context, QueryBaseEquipmentViewInput input) {
        QuerySysDictionaryViewOutput dict = getDictByAlias(context , "equipment_type");
        List<QuerySysDictionaryViewOutput> dictList = queryChildByDicPath(context , dict);
        List<QuerySysDictionaryViewOutput> mediumList = getMenuDict(context , "MEDIUM_TYPE");

        BaseEquipmentViewMapper mapper = context.getBean(BaseEquipmentViewMapper.class);
        BaseEquipmentView record = new BaseEquipmentView();
        SysOrganizationCloudService sysOrganizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput getAllOuListByOuIdInput = new GetAllOuListByOuIdInput();
        getAllOuListByOuIdInput.setOuId(input.getOuid());
        List<GetOuListTreeOutput> getAllOuListByOuIdOutputList = sysOrganizationCloudService.getAllOuListByOuId(context , getAllOuListByOuIdInput).getResult();
        List<String> ouIds = getAllOuListByOuIdOutputList.stream().map(GetOuListTreeOutput::getOuId).collect(Collectors.toList());
        if(!CollectionUtils.isEmpty(ouIds)){
            ouIds.add(input.getOuid());
            record.setOuIdList(ouIds);
            input.setOuid("");
        }
        XCopyUtils.copyObject(input, record);
        if(XStringUtils.isNotEmpty(input.getEquipmentType())){
            QuerySysDictionaryViewOutput dictValue = getDict(input.getEquipmentType() + "" , dictList);
            List<QuerySysDictionaryViewOutput> outList = new ArrayList<>();
            outList.add(dictValue);
            getAllChildrenNodeByParent(dictList ,outList , Lists.newArrayList(dictValue));
            List<String> equipmentTypes = outList.stream().map(QuerySysDictionaryViewOutput::getDicKey).collect(Collectors.toList());
            record.setEquipmentTypeList(equipmentTypes);
            record.setEquipmentType(null);
        }
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseEquipmentView> list = mapper.selectList(record);
        PageInfo<BaseEquipmentView> pageInfo = new PageInfo<>(list);
        List<QueryBaseEquipmentViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseEquipmentViewOutput.class);
        outputs.stream().forEach(item->{
            String mediumTypeName = getDictName(item.getMediumType() + "" , mediumList);
            if(XStringUtils.isNotEmpty(mediumTypeName))
                item.setStationName(item.getStationName() + "-" + mediumTypeName);
            item.setEquipmentTypeName(getDictName(item.getEquipmentType() + "" , dictList));
        });
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }


    @XApiGet
    @XText("获取设备分类字典")
    public XListResult<EquipmentTypeDictOutput> queryEquipmentTypeDict(XContext context, GetBaseEquipmentViewInput input) {
        QuerySysDictionaryViewOutput dict = getDictByAlias(context , "equipment_type");
        List<QuerySysDictionaryViewOutput> dictList = queryChildByDicPath(context , dict);
        List<EquipmentTypeDictOutput> equipmentTypeDictOutputList = XCopyUtils.copyNewList(dictList , EquipmentTypeDictOutput.class);
        List<EquipmentTypeDictOutput> outList = streamToTree(equipmentTypeDictOutputList , dict.getId());
        return XListResult.success(outList);
    }

    private void deleteRel(XContext context, String id ){
        //删除关系表
        BaseEquipmentClientRelMapper relMapper = context.getBean(BaseEquipmentClientRelMapper.class);
        QueryWrapper<BaseEquipmentClientRelEnt> queryRelWrapper = new QueryWrapper<>();
        queryRelWrapper.lambda().eq(BaseEquipmentClientRelEnt::getEquipmentId, id);
        List<BaseEquipmentClientRelEnt> relEntitys = relMapper.selectList(queryRelWrapper);
        if (!CollectionUtils.isEmpty(relEntitys)) {
            List<String> ids = relEntitys.stream().map(BaseEquipmentClientRelEnt::getId).collect(Collectors.toList());
            relMapper.deleteBatchIds(ids);
        }
    }

    private void addRel(XContext context, String id , List<CreateBaseEquipmentClientRelInput> relInputList){
        //添加关系表
        if(!CollectionUtils.isEmpty(relInputList)){
            BaseEquipmentClientRelMapper relMapper = context.getBean(BaseEquipmentClientRelMapper.class);
            relInputList.stream().forEach(item->{
                BaseEquipmentClientRelEnt relEntity = new BaseEquipmentClientRelEnt();
                if(StringUtils.isBlank(item.getCreateById())){
                    PpsUserSession session = context.getSession(PpsUserSession.class);
                    Date date = new Date();
                    item.setModifyTime(date);
                    item.setModifyByName(session.getUserName());
                    item.setModifyById(session.getId()+"");
                    item.setCreateTime(date);
                    item.setCreateByName(session.getUserName());
                    item.setCreateById(session.getId()+"");
                }
                XCopyUtils.copyObject(item, relEntity);
                relEntity.setEquipmentId(id);
                relEntity.setId(UUID.randomUUID().toString());
                relMapper.insert(relEntity);
            });
        }
    }

    private QuerySysDictionaryViewOutput getDictByAlias(XContext context , String alias){
        SystemDictionaryService systemDictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictionaryViewInput = new QuerySysDictionaryViewInput();
        dictionaryViewInput.setPage(1);
        dictionaryViewInput.setLimit(1000);
        dictionaryViewInput.setAlias(alias);
        QuerySysDictionaryViewOutput dict = systemDictionaryService.queryByAlias(context , dictionaryViewInput).getResult();
        return dict;
    }
    public List<QuerySysDictionaryViewOutput> getMenuDict(XContext context , String alias ){
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }
    private List<QuerySysDictionaryViewOutput> queryChildByDicPath(XContext context , QuerySysDictionaryViewOutput dict){
        SystemDictionaryService systemDictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictionaryViewInput = new QuerySysDictionaryViewInput();
        dictionaryViewInput.setDicPath(dict.getId());
        dictionaryViewInput.setPage(1);
        dictionaryViewInput.setLimit(1000);
        List<QuerySysDictionaryViewOutput> dictList = systemDictionaryService.queryChildByDicPath(context , dictionaryViewInput).getResult();
        return dictList;
    }

    public String getDictName(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return "";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict.getDicName();
            }
        }
        return "";
    }

    public QuerySysDictionaryViewOutput getDict(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return null;
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict;
            }
        }
        return null;
    }

    private List<EquipmentTypeDictOutput> streamToTree( List<EquipmentTypeDictOutput> list  , String parentId){
        List<EquipmentTypeDictOutput> outList = list.stream()
                .filter(paren -> paren.getPid().equals(parentId))
                .map(child ->{
                    List<EquipmentTypeDictOutput> childList = streamToTree(list , child.getId());
                    if(CollectionUtils.isEmpty(childList))
                        child.setChildren(null);
                    else
                        child.setChildren(childList);
                    return child;
                }).collect(Collectors.toList());
        return outList;
    }

    private void getAllChildrenNodeByParent(List<QuerySysDictionaryViewOutput> dictList , List<QuerySysDictionaryViewOutput> outList ,  List<QuerySysDictionaryViewOutput> parentCodes){
        if (!CollectionUtils.isEmpty(dictList)){
            for(QuerySysDictionaryViewOutput parentCode : parentCodes){
                List<QuerySysDictionaryViewOutput> collect = dictList.stream().filter(item -> item.getPid().equals(parentCode.getId())).collect(Collectors.toList());
                if(!CollectionUtils.isEmpty(collect)){
                    getAllChildrenNodeByParent(dictList ,outList , collect);
                }
                outList.addAll(collect);
            }
        }
    }
}
