package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.entity.BaseOilTypeEnt;
import pps.core.base.mapper.BaseOilTypeMapper;
import pps.core.base.service.data.base_oil_type.*;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.ExportExcelUtils;
import pps.core.common.utils.ImportExcelUtils;
import pps.core.system.entity.SysOrganizationEnt;
import pps.core.system.mapper.SysOrganizationMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.*;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.io.InputStream;
import java.util.*;

@XText("油品信息表")
@XService
public class BaseOilTypeService {
    private HashMap<Integer, Boolean> isNeedNotEmptyMap = new HashMap<>();

    @XApiPost
    @XText("创建/更新油品信息")
    public XServiceResult createBaseOilType(XContext context, CreateBaseOilTypeInput input) {
        return XTransactionHelper.begin(context, () -> {
            BaseOilTypeMapper mapper = context.getBean(BaseOilTypeMapper.class);
            BaseOilTypeEnt entity = new BaseOilTypeEnt();
            PpsUserSession session = context.getSession(PpsUserSession.class);
            Date date = new Date();
            input.setModifyTime(date);
            input.setModifyByName(session.getUserName());
            input.setModifyById(session.getId() + "");
            input.setIsDelete(1);
            XCopyUtils.copyObject(input, entity);
            if (XStringUtils.isEmpty(input.getId())) {
                entity.setId(UUID.randomUUID().toString());
                entity.setCreateTime(date);
                entity.setCreateByName(session.getUserName());
                entity.setCreateById(session.getId() + "");
                mapper.insert(entity);
            } else {
                mapper.updateById(entity);
            }
            return XServiceResult.OK;
        });
    }

    @XApiPost
    @XText("删除油品信息")
    public XServiceResult deleteBaseOilType(XContext context, DeleteBaseOilTypeInput input) {
        BaseOilTypeMapper mapper = context.getBean(BaseOilTypeMapper.class);
        QueryWrapper<BaseOilTypeEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseOilTypeEnt::getId, input.getId());
        BaseOilTypeEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        //使用逻辑删除
        entity.setIsDelete(0);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }


    @XApiPost
    @XText("获取油品信息详情")
    public XSingleResult<GetBaseOilTypeOutput> getBaseOilType(XContext context, GetBaseOilTypeInput input) {
        BaseOilTypeMapper mapper = context.getBean(BaseOilTypeMapper.class);
        QueryWrapper<BaseOilTypeEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseOilTypeEnt::getId, input.getId());
        BaseOilTypeEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseOilTypeOutput output = new GetBaseOilTypeOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }


    @XApiPost
    @XText("分页油品信息列表")
    public XPageResult<QueryBaseOilTypeOutput> queryBaseOilType(XContext context, QueryBaseOilTypeInput input) {
        BaseOilTypeMapper mapper = context.getBean(BaseOilTypeMapper.class);
        BaseOilTypeEnt ent = XCopyUtils.copyNewObject(input, BaseOilTypeEnt.class);
        QueryWrapper<BaseOilTypeEnt> queryWrapper = setQueryWrapper(ent);
        XPageResult<QueryBaseOilTypeOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QueryBaseOilTypeOutput.class);
        List<QuerySysDictionaryViewOutput> dictList = getDictList(context , "OIL_CLASS");
        query.getResult().getItems().stream().forEach(item->{
            item.setResourceTypeName(getDictName(item.getResourceType() + "" , dictList));
        });
        return query;
    }

    @XText("导出油品信息数据")
    @XApiGet
    public XFileResult exportBaseOilTypeData(XContext context, GetBaseOilTypeInput input) {
        try {
            BaseOilTypeMapper mapper = context.getBean(BaseOilTypeMapper.class);
            BaseOilTypeEnt ent = XCopyUtils.copyNewObject(input, BaseOilTypeEnt.class);
            QueryWrapper<BaseOilTypeEnt> queryWrapper = setQueryWrapper(ent);
            List<BaseOilTypeEnt> baseOilTypeEntList = mapper.selectList(queryWrapper);
            List<QuerySysDictionaryViewOutput> dictList = getDictList(context , "OIL_CLASS");
            baseOilTypeEntList.stream().forEach(item->{
                item.setResourceTypeName(getDictName(item.getResourceType() + "" , dictList));
            });
            if (!CollectionUtils.isEmpty(baseOilTypeEntList)) {
                Map<String, Object> params = new HashMap<>();
                params.put("list", baseOilTypeEntList);
                String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/油品信息.xlsx");
                String templateFilePath = "template/exportBaseOilTypeTemplate.xlsx";
                InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
                byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, params);
                ExportExcelUtils.outputFileData(excelContent, fileSavePath);
                return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
            } else {
                return XFileResult.error(200, "暂无数据导出");
            }
        } catch (Exception ex) {
            context.getLogger().error("导出出错", ex);

            return XFileResult.error(500, "导出失败");
        }
    }

    @XText("导入油品信息数据")
    @XApiUpload
    public XServiceResult importBaseOilTypeData(XContext context, ImportBaseOilTypeInput input) {
        try {
            if (input.getFile() == null)
                return XServiceResult.error(500, "未获取上传文件");
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (StringUtils.isEmpty(input.getOuId()))
                input.setOuId(session.getOuId());
            if(StringUtils.isEmpty(input.getOuName())){
                SysOrganizationMapper sysOrganizationMapper = context.getBean(SysOrganizationMapper.class);
                QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysOrganizationEnt::getId , input.getOuId()).last("limit 1");
                SysOrganizationEnt sysOrganizationEnt = sysOrganizationMapper.selectOne(queryWrapper);
                if(null != sysOrganizationEnt)
                    input.setOuName(sysOrganizationEnt.getOuName());
            }
            List<BaseOilTypeEnt> detailsEntList = this.importExcelData(context, input);
            BaseOilTypeMapper mapper = context.getBean(BaseOilTypeMapper.class);
            XThreadHelper.async(new Runnable() {
                @Override
                public void run() {
                    for (int i = 0; i < detailsEntList.size(); i++) {
                        Date currentDate = new Date();
                        BaseOilTypeEnt record = detailsEntList.get(i);
                        record.setOuId(input.getOuId());
                        record.setOuName(input.getOuName());
                        record.setMediaType(input.getMediaType());
                        record.setId(UUID.randomUUID().toString());
                        record.setCreateById(session.getId());
                        record.setCreateByName(session.getUserName());
                        record.setCreateTime(currentDate);
                        record.setModifyById(session.getId());
                        record.setModifyByName(session.getUserName());
                        record.setModifyTime(currentDate);
                        mapper.insert(record);
                    }
                }
            });
            return XServiceResult.OK;
        } catch (Exception ex) {
            context.getLogger().error("导入失败", ex);
            return XServiceResult.error(500, "导入失败：" + ex.toString().replace("java.lang.Exception:", ""));
        }
    }

    private List<BaseOilTypeEnt> importExcelData(XContext context, ImportBaseOilTypeInput param) throws Exception {
        List<BaseOilTypeEnt> result = new ArrayList<>();
        String fileName = param.getFile().getFileName();
        InputStream input = param.getFile().getInputStream();
        this.checkError(!fileName.endsWith(".xlsx"), "导入模板文件格式不正确，请核对(文档格式应为.xlsx)");
        SystemDictionaryService systemDictionaryService = context.getBean(SystemDictionaryService.class);
        Workbook wb = new XSSFWorkbook(input);
        Sheet sheet = wb.getSheetAt(0);
        Iterator<Row> rows = sheet.rowIterator();
        this.checkError(rows == null || !rows.hasNext(), "未导入任何数据");

        DataFormatter formatter = new DataFormatter();
        List<String> tempRowTitle = createTitles();

        Row titleRowFromExcel = rows.next();
        String titleErrorMessage = "";
        isNeedNotEmptyMap.clear();

        for (int i = 0; i < tempRowTitle.size(); i++) {
            String titleFormExcel = formatter.formatCellValue(titleRowFromExcel.getCell(i)).trim();
            isNeedNotEmptyMap.put(i, titleFormExcel.startsWith("*"));
            titleFormExcel = titleFormExcel.replace("*", "");

            if (!tempRowTitle.get(i).equals(titleFormExcel)) {
                titleErrorMessage = titleErrorMessage.concat("第" + (i + 1) + "列表头不正确应为“" + tempRowTitle.get(i) + "”,");
            }
        }
        this.checkError(StringUtils.isNotBlank(titleErrorMessage), titleErrorMessage + "请根据模板修改；");
        StringBuilder errorMessageBuilder = new StringBuilder();
        while (rows.hasNext()) {
            Row row = rows.next();
            int rowIndex = row.getRowNum();

            if (ImportExcelUtils.isRowEmpty(row)) {
                errorMessageBuilder.append("第" + rowIndex + "行数据不能为空；");
                continue;
            }

            String dateErrorMessage = this.checkEmptyData(row, tempRowTitle);
            if (StringUtils.isNotBlank(dateErrorMessage)) {
                errorMessageBuilder.append(dateErrorMessage);
                continue;
            }
            BaseOilTypeEnt resultItem = new BaseOilTypeEnt();
            List<QuerySysDictionaryViewOutput> dictList = getDictList(context , "OIL_CLASS");
            for (int i = 0; i < tempRowTitle.size(); i++) {
                Cell cell = row.getCell(i);
                String cellText = formatter.formatCellValue(cell).trim();
                String titleName = tempRowTitle.get(i);

                if (StringUtils.isBlank(cellText))
                    continue;
                switch (i) {
                    case 0: //油源
                        if (resultItem != null) {
                            try {
                                resultItem.setResourceType(getDictKey(cellText + "" , dictList));
                            } catch (Exception ex) {
                                errorMessageBuilder.append("第" + rowIndex + "行'" + titleName + "'的格式不正确；");
                            }
                        }
                        break;
                    case 1: //油品编码
                        if (resultItem != null) {
                            try {
                                resultItem.setOilTypeCode(cellText);
                            } catch (Exception ex) {
                                errorMessageBuilder.append("第" + rowIndex + "行'" + titleName + "'的格式不正确；");
                            }
                        }
                        break;
                    case 2: //油品名称
                        if (resultItem != null) {
                            try {
                                resultItem.setOilTypeName(cellText);
                            } catch (Exception ex) {
                                errorMessageBuilder.append("第" + rowIndex + "行'" + titleName + "'的格式不正确；");
                            }
                        }
                        break;
                    case 3: //备注
                        if (resultItem != null) {
                            try {
                                resultItem.setRemark(cellText);
                            } catch (Exception ex) {
                                errorMessageBuilder.append("第" + rowIndex + "行'" + titleName + "'的格式不正确；");
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
            if (resultItem != null)
                result.add(resultItem);
        }
        this.checkError(errorMessageBuilder.length() > 0, errorMessageBuilder.toString());
        return result;
    }

    private List<String> createTitles() {
        List<String> titles = new ArrayList<String>();
        titles.add("油源");
        titles.add("油品编号");
        titles.add("油品名称");
        titles.add("备注");
        return titles;
    }

    private String checkEmptyData(Row row, List<String> tempRowTitle) {
        String result = "";
        DataFormatter formatter = new DataFormatter();

        for (int i = 0; i < isNeedNotEmptyMap.size(); i++) {
            Cell cell = row.getCell(i);
            Boolean isNeedNotEmpty = isNeedNotEmptyMap.get(i);
            Boolean isEmptyValue = StringUtils.isBlank(formatter.formatCellValue(cell));

            if (isNeedNotEmpty && isEmptyValue) {
                result = result.concat(String.format("“%s”", tempRowTitle.get(i).replace("*", "")));
            }
        }
        if (StringUtils.isNotBlank(result)) {
            int rowNum = row.getRowNum();
            result = "第" + rowNum + "行" + result + "不能为空；";
        }
        return result;
    }

    private void checkError(boolean isError, String errorMessage) throws Exception {
        if (isError)
            throw new Exception(errorMessage);
    }

    public QueryWrapper<BaseOilTypeEnt> setQueryWrapper(BaseOilTypeEnt input) {
        QueryWrapper<BaseOilTypeEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseOilTypeEnt::getIsDelete, 1); //未删除
        if (StringUtils.isNotBlank(input.getOilTypeName())) {
            queryWrapper.lambda().like(BaseOilTypeEnt::getOilTypeName, input.getOilTypeName());
        }
        if (StringUtils.isNotBlank(input.getMediaType())) {
            queryWrapper.lambda().eq(BaseOilTypeEnt::getMediaType, input.getMediaType());
        }
        if (StringUtils.isNotBlank(input.getOuId())) {
            queryWrapper.lambda().eq(BaseOilTypeEnt::getOuId, input.getOuId());
        }
        if (StringUtils.isNotBlank(input.getResourceType())) {
            queryWrapper.lambda().eq(BaseOilTypeEnt::getResourceType, input.getResourceType());
        }
        queryWrapper.lambda().orderByDesc(BaseOilTypeEnt::getCreateTime);
        return queryWrapper;
    }

    public List<QuerySysDictionaryViewOutput> getDictList(XContext context , String alias ){
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }

    public String getDictName(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return "";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict.getDicName();
            }
        }
        return "";
    }

    public String getDictKey(String dictName,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictName) || CollectionUtils.isEmpty(dictList)){
            return "";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicName().equals(dictName)){
                return dict.getDicKey();
            }
        }
        return "";
    }
}
