package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.logging.log4j.util.Strings;
import pps.cloud.base.service.BaseBaseOperationalConstraintCloudService;
import pps.cloud.base.service.data.base_operational_constraint.GetBaseOperationalConstraintInput;
import pps.cloud.base.service.data.base_operational_constraint.GetBaseOperationalConstraintOuput;
import pps.core.base.constant.OperationalConstraintConstant;
import pps.core.base.entity.BaseOperationalConstraintEnt;
import pps.core.base.entity.BaseOperationalConstraintExtendEnt;
import pps.core.base.mapper.BaseOperationalConstraintExtendMapper;
import pps.core.base.mapper.BaseOperationalConstraintMapper;
import pps.core.base.service.data.base_operational_constraint.BaseOperationalConstraintInput;
import pps.core.base.service.data.base_operational_constraint.BaseOperationalConstraintOuput;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.UUIDHelper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.text.SimpleDateFormat;
import java.util.Date;

@XText("操作约束相关接口")
@XService
public class BaseOperationalConstraintService {

    private final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");

    private final String BUSINESS_TYPE_SHIPPER_SPLIT = "SHIPPER_GAS_SPLIT";
    private final String DEFAULT_TIME = "11:00";

    @XText("获取托运商气量拆分约束信息")
    @XApiGet
    public XSingleResult<BaseOperationalConstraintOuput> getDataForShipperSplit(XContext context) {
        BaseOperationalConstraintMapper mapper = context.getBean(BaseOperationalConstraintMapper.class);
        QueryWrapper<BaseOperationalConstraintEnt> qr = new QueryWrapper<>();
        qr.lambda().eq(BaseOperationalConstraintEnt::getBusineType,BUSINESS_TYPE_SHIPPER_SPLIT).last(" limit 0,1");
        BaseOperationalConstraintEnt ent = mapper.selectOne(qr);
        if(ent == null){
            PpsUserSession session = context.getSession(PpsUserSession.class);
            ent = new BaseOperationalConstraintEnt();
            ent.setId(UUIDHelper.newUUID());
            ent.setBusineType(BUSINESS_TYPE_SHIPPER_SPLIT);
            ent.setCreateById(session.getId());
            ent.setCreateByName(session.getUserName());
            ent.setCreateTime(new Date());
            mapper.insert(ent);
        }
        BaseOperationalConstraintOuput output = XCopyUtils.copyNewObject(ent, BaseOperationalConstraintOuput.class);
        return XSingleResult.success(output);
    }

    @XText("更新操作约束信息")
    @XApiGet
    public XServiceResult updateData(XContext context, BaseOperationalConstraintInput input) {
        BaseOperationalConstraintMapper mapper = context.getBean(BaseOperationalConstraintMapper.class);
        BaseOperationalConstraintEnt ent = mapper.selectById(input.getId());
        if(ent == null){
            return XServiceResult.error(500,"数据不存在");
        }
        PpsUserSession session = context.getSession(PpsUserSession.class);
        XCopyUtils.copyObject(input,ent);
        ent.setModifyById(session.getId());
        ent.setModifyByName(session.getUserName());
        ent.setModifyTime(new Date());
        mapper.updateById(ent);
        return XServiceResult.OK;
    }
}
