package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.logging.log4j.util.Strings;
import pps.cloud.base.service.BaseBaseOperationalConstraintCloudService;
import pps.cloud.base.service.data.base_operational_constraint.GetBaseOperationalConstraintInput;
import pps.cloud.base.service.data.base_operational_constraint.GetBaseOperationalConstraintOuput;
import pps.core.base.constant.OperationalConstraintConstant;
import pps.core.base.entity.BaseOperationalConstraintEnt;
import pps.core.base.entity.BaseOperationalConstraintExtendEnt;
import pps.core.base.mapper.BaseOperationalConstraintExtendMapper;
import pps.core.base.mapper.BaseOperationalConstraintMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiPost;

import java.text.SimpleDateFormat;
import java.util.Date;

@XText("操作约束微服务接口")
@XService
public class BaseOperationalConstraintServiceImpl implements BaseBaseOperationalConstraintCloudService {

    private final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");

    @Override
    @XText("获取操作约束信息")
    @XApiPost
    public XSingleResult<GetBaseOperationalConstraintOuput> getBaseOperationalConstraint(XContext context, GetBaseOperationalConstraintInput input) {
        BaseOperationalConstraintMapper mapper = context.getBean(BaseOperationalConstraintMapper.class);
        QueryWrapper<BaseOperationalConstraintEnt> qr = new QueryWrapper<>();
        qr.lambda().eq(BaseOperationalConstraintEnt::getBusineType,input.getBusineType()).last(" limit 0,1");
        BaseOperationalConstraintEnt ent = mapper.selectOne(qr);
        //没有配置的话，当前业务不受时效限制
        if(ent == null){
            GetBaseOperationalConstraintOuput output = new GetBaseOperationalConstraintOuput();
            output.setCanSaveOrSubmit(true);
            output.setCanApplyChange(true);
            output.setCanAudit(true);
            return XSingleResult.success(output);
        }
        GetBaseOperationalConstraintOuput output = XCopyUtils.copyNewObject(ent, GetBaseOperationalConstraintOuput.class);
        output.setOperationalDate(input.getOperationalDate());
        output.setStationId(input.getStationId());
        this.parseData(context,output);
        return XSingleResult.success(output);
    }

    private void parseData(XContext context,GetBaseOperationalConstraintOuput output) {
        Date operationalDate = output.getOperationalDate();
        BaseOperationalConstraintExtendMapper extendMapper = context.getBean(BaseOperationalConstraintExtendMapper.class);
        output.setCanSaveOrSubmit(this.checkCanOperational(extendMapper,OperationalConstraintConstant.OPERATIONAL_TYPE_SAVE,operationalDate,output.getSaveSubmitTime(),output.getStationId()));
        output.setCanAudit(this.checkCanOperational(extendMapper, OperationalConstraintConstant.OPERATIONAL_TYPE_AUDIT,operationalDate,output.getAuditTime(),output.getStationId()));
        output.setCanApplyChange(this.checkCanOperational(extendMapper, OperationalConstraintConstant.OPERATIONAL_TYPE_APPLY_CHANGE,operationalDate,output.getApplyChangeTime(),output.getStationId()));
    }

    /**
     * 校验是否可以操作
     * @param operationalDate
     * @param operationalTime
     * @return true 可以操作
     */
    private boolean checkCanOperational(BaseOperationalConstraintExtendMapper extendMapper,String operationalType,
                                        Date operationalDate,String operationalTime,String stationId){
        if(operationalDate == null) return false;
        if(Strings.isBlank(operationalTime)) return true; //未设置操作时间限制
        try{
            //获取约束扩展数据
            QueryWrapper<BaseOperationalConstraintExtendEnt> qr = new QueryWrapper<>();
            qr.lambda().eq(BaseOperationalConstraintExtendEnt::getDataDate,operationalDate)
                    .eq(BaseOperationalConstraintExtendEnt::getStationId,stationId)
                    .eq(BaseOperationalConstraintExtendEnt::getOperationalType,operationalType).last("limit 0,1");
            BaseOperationalConstraintExtendEnt extendEnt = extendMapper.selectOne(qr);
            if(extendEnt != null){
                operationalDate = extendEnt.getConstraintDate();
            }else{
                operationalTime = operationalTime.trim();
                if(operationalTime.length()>5) operationalTime = operationalTime.substring(0,5);
                if(operationalTime.length() !=5 || operationalTime.charAt(2) != ':') return false;
                operationalDate = XDateUtils.getDayBeginTime(operationalDate);
                String tempTimeStr = XDateUtils.getString(operationalDate,"yyyy-MM-dd")+" "+operationalTime;
                operationalDate = simpleDateFormat.parse(tempTimeStr);
            }
            return operationalDate.getTime() > new Date().getTime();
        }catch (Exception e){
            e.printStackTrace();
            return false;
        }
    }
}
