package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BasePipelinePropertyEnt;
import pps.core.base.entity.BasePipelinePropertyView;
import pps.core.base.mapper.BasePipelinePropertyMapper;
import pps.core.base.mapper.BasePipelinePropertyViewMapper;
import pps.core.base.service.data.base_pipeline_property.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XText("管段属性接口")
@XService
public class BasePipelinePropertyService {

    @XText("创建管线属性")
    @XApiPost
    public XServiceResult createBasePipelineProperty(XContext context, CreateBasePipelinePropertyInput input) {
        BasePipelinePropertyMapper mapper = context.getBean(BasePipelinePropertyMapper.class);
        BasePipelinePropertyEnt entity = new BasePipelinePropertyEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改管线属性")
    @XApiPost
    public XServiceResult updateBasePipelineProperty(XContext context, UpdateBasePipelinePropertyInput input) {
        BasePipelinePropertyMapper mapper = context.getBean(BasePipelinePropertyMapper.class);
        QueryWrapper<BasePipelinePropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelinePropertyEnt::getId, input.getId());
        BasePipelinePropertyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除管线属性")
    @XApiPost
    public XServiceResult deleteBasePipelineProperty(XContext context, DeleteBasePipelinePropertyInput input) {
        BasePipelinePropertyMapper mapper = context.getBean(BasePipelinePropertyMapper.class);
        QueryWrapper<BasePipelinePropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelinePropertyEnt::getId, input.getId());
        BasePipelinePropertyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id获取管线属性")
    @XApiPost
    public XSingleResult<GetBasePipelinePropertyOutput> getBasePipelineProperty(XContext context, GetBasePipelinePropertyInput input) {
        BasePipelinePropertyMapper mapper = context.getBean(BasePipelinePropertyMapper.class);
        QueryWrapper<BasePipelinePropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelinePropertyEnt::getId, input.getId());
        BasePipelinePropertyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePipelinePropertyOutput output = new GetBasePipelinePropertyOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询管线属性")
    @XApiPost
    public XPageResult<QueryBasePipelinePropertyOutput> queryBasePipelineProperty(XContext context, QueryBasePipelinePropertyInput input) {
        BasePipelinePropertyMapper mapper = context.getBean(BasePipelinePropertyMapper.class);
        QueryWrapper<BasePipelinePropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BasePipelinePropertyEnt::getId, input.getId())
                .like(BasePipelinePropertyEnt::getPipelineId, input.getPipelineId())
                .like(BasePipelinePropertyEnt::getPropertyId, input.getPropertyId())
                .like(BasePipelinePropertyEnt::getPropertyValue, input.getPropertyValue())
                .eq(BasePipelinePropertyEnt::getEffectTime, input.getEffectTime())
                .eq(BasePipelinePropertyEnt::getEndTime, input.getEndTime())
                .like(BasePipelinePropertyEnt::getCreateById, input.getCreateById())
                .like(BasePipelinePropertyEnt::getCreateByName, input.getCreateByName())
                .eq(BasePipelinePropertyEnt::getCreateTime, input.getCreateTime())
                .like(BasePipelinePropertyEnt::getModifyById, input.getModifyById())
                .like(BasePipelinePropertyEnt::getModifyByName, input.getModifyByName())
                .eq(BasePipelinePropertyEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePipelinePropertyOutput.class);
    }

    @XApiPost
    public XPageResult<QueryBasePipelinePropertyOutput> dynamicQueryBasePipelineProperty(XContext context, CustomQueryInput input) {
        BasePipelinePropertyMapper mapper = context.getBean(BasePipelinePropertyMapper.class);
        return XMapperHelper.query(mapper, input, BasePipelinePropertyEnt.class, QueryBasePipelinePropertyOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBasePipelinePropertyViewOutput> getBasePipelinePropertyView(XContext context, GetBasePipelinePropertyViewInput input) {
        BasePipelinePropertyViewMapper mapper = context.getBean(BasePipelinePropertyViewMapper.class);
        BasePipelinePropertyView record = new BasePipelinePropertyView();
        XCopyUtils.copyObject(input, record);
        BasePipelinePropertyView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePipelinePropertyViewOutput output = new GetBasePipelinePropertyViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBasePipelinePropertyViewOutput> queryBasePipelinePropertyView(XContext context, QueryBasePipelinePropertyViewInput input) {
        BasePipelinePropertyViewMapper mapper = context.getBean(BasePipelinePropertyViewMapper.class);
        BasePipelinePropertyView record = new BasePipelinePropertyView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePipelinePropertyView> list = mapper.selectList(record);
        PageInfo<BasePipelinePropertyView> pageInfo = new PageInfo<>(list);
        List<QueryBasePipelinePropertyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePipelinePropertyViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
