package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BasePipelineSegmentRelEnt;
import pps.core.base.entity.BasePipelineSegmentRelView;
import pps.core.base.mapper.BasePipelineSegmentRelMapper;
import pps.core.base.mapper.BasePipelineSegmentRelViewMapper;
import pps.core.base.service.data.base_pipeline_segment_rel.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XText("管线管段关系接口")
@XService
public class BasePipelineSegmentRelService {

    @XText("创建管线管段关系")
    @XApiPost
    public XServiceResult createBasePipelineSegmentRel(XContext context, CreateBasePipelineSegmentRelInput input){
        BasePipelineSegmentRelMapper mapper = context.getBean(BasePipelineSegmentRelMapper.class);
        BasePipelineSegmentRelEnt entity = new BasePipelineSegmentRelEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("更新管线管段关系")
    @XApiPost
    public XServiceResult updateBasePipelineSegmentRel(XContext context, UpdateBasePipelineSegmentRelInput input){
        BasePipelineSegmentRelMapper mapper = context.getBean(BasePipelineSegmentRelMapper.class);
        QueryWrapper<BasePipelineSegmentRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineSegmentRelEnt::getId, input.getId());
        BasePipelineSegmentRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除管线管段关系")
    @XApiPost
    public XServiceResult deleteBasePipelineSegmentRel(XContext context, DeleteBasePipelineSegmentRelInput input){
        BasePipelineSegmentRelMapper mapper = context.getBean(BasePipelineSegmentRelMapper.class);
        QueryWrapper<BasePipelineSegmentRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineSegmentRelEnt::getId, input.getId());
        BasePipelineSegmentRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id获取管线管段关系")
    @XApiPost
    public XSingleResult<GetBasePipelineSegmentRelOutput> getBasePipelineSegmentRel(XContext context, GetBasePipelineSegmentRelInput input){
        BasePipelineSegmentRelMapper mapper = context.getBean(BasePipelineSegmentRelMapper.class);
        QueryWrapper<BasePipelineSegmentRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineSegmentRelEnt::getId, input.getId());
        BasePipelineSegmentRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePipelineSegmentRelOutput output = new GetBasePipelineSegmentRelOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询管线管线关系")
    @XApiPost
    public XPageResult<QueryBasePipelineSegmentRelOutput> queryBasePipelineSegmentRel(XContext context, QueryBasePipelineSegmentRelInput input){
        BasePipelineSegmentRelMapper mapper = context.getBean(BasePipelineSegmentRelMapper.class);
        QueryWrapper<BasePipelineSegmentRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BasePipelineSegmentRelEnt::getId, input.getId())
            .like(BasePipelineSegmentRelEnt::getPipelineId, input.getPipelineId())
            .like(BasePipelineSegmentRelEnt::getSegmentId, input.getSegmentId())
            .eq(BasePipelineSegmentRelEnt::getEffectTime, input.getEffectTime())
            .eq(BasePipelineSegmentRelEnt::getEndTime, input.getEndTime())
            .eq(BasePipelineSegmentRelEnt::getSort, input.getSort())
            .like(BasePipelineSegmentRelEnt::getCreateById, input.getCreateById())
            .like(BasePipelineSegmentRelEnt::getCreateByName, input.getCreateByName())
            .eq(BasePipelineSegmentRelEnt::getCreateTime, input.getCreateTime())
            .like(BasePipelineSegmentRelEnt::getModifyById, input.getModifyById())
            .like(BasePipelineSegmentRelEnt::getModifyByName, input.getModifyByName())
            .eq(BasePipelineSegmentRelEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePipelineSegmentRelOutput.class);
    }

    @XApiPost
    public XPageResult<QueryBasePipelineSegmentRelOutput> dynamicQueryBasePipelineSegmentRel(XContext context, CustomQueryInput input){
        BasePipelineSegmentRelMapper mapper = context.getBean(BasePipelineSegmentRelMapper.class);
        return XMapperHelper.query(mapper, input, BasePipelineSegmentRelEnt.class, QueryBasePipelineSegmentRelOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBasePipelineSegmentRelViewOutput> getBasePipelineSegmentRelView(XContext context, GetBasePipelineSegmentRelViewInput input){
        BasePipelineSegmentRelViewMapper mapper = context.getBean(BasePipelineSegmentRelViewMapper.class);
        BasePipelineSegmentRelView record = new BasePipelineSegmentRelView();
        XCopyUtils.copyObject(input, record);
        BasePipelineSegmentRelView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePipelineSegmentRelViewOutput output = new GetBasePipelineSegmentRelViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBasePipelineSegmentRelViewOutput> queryBasePipelineSegmentRelView(XContext context, QueryBasePipelineSegmentRelViewInput input){
        BasePipelineSegmentRelViewMapper mapper = context.getBean(BasePipelineSegmentRelViewMapper.class);
        BasePipelineSegmentRelView record = new BasePipelineSegmentRelView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePipelineSegmentRelView> list = mapper.selectList(record);
        PageInfo<BasePipelineSegmentRelView> pageInfo = new PageInfo<>(list);
        List<QueryBasePipelineSegmentRelViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePipelineSegmentRelViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
