package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.base.Strings;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

import pps.cloud.base.service.BaseBasePipelineService;
import pps.cloud.base.service.data.base_pipeline.QueryPipelineByIdsInput;
import pps.cloud.base.service.data.base_pipeline.QueryPipelineByIdsOutput;
import pps.cloud.framedata.service.CommonDataService;
import pps.cloud.framedata.service.data.common_data.SelectCommonInput;
import pps.cloud.framedata.service.data.common_data.SelectCommonOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_pipeline.*;
import pps.core.common.session.PpsUserSession;
import pps.core.common.session.SysOrganization;
import pps.core.common.utils.DateUtil;
import pps.core.common.utils.NamedServiceProvider;
import pps.core.common.utils.TraceHelper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.data.XPageInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;

@XText("管线接口")
@XService
public class BasePipelineService {

    private final String ROOT_PARENT_ID = "11111111-1111-1111-1111-111111111111";

    @XText("创建管线")
    @XApiPost
    public XServiceResult createBasePipeline(XContext context, CreateBasePipelineInput input) {
        BasePipelineMapper mapperCode = context.getBean(BasePipelineMapper.class);
        SystemAreaService systemArea = context.getBean(SystemAreaService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<BasePipelineEnt> queryCode = new QueryWrapper<>();
        queryCode.lambda().eq(BasePipelineEnt::getPipelineCode, input.getPipelineCode());
        BasePipelineEnt basePipelineCode = mapperCode.selectOne(queryCode);
        if (ObjectUtils.isNotEmpty(basePipelineCode)) {
            return XServiceResult.error(200, "管线编码已存在！");
        }
        //创建管线:新增管线、新增管线组织机构
        return XTransactionHelper.begin(context, () -> {
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
            BasePipelineOuRelMapper pipelineOuMapper = context.getBean(BasePipelineOuRelMapper.class);
            BasePipelineViewMapper pipelineViewMapper = context.getBean(BasePipelineViewMapper.class);
            SysOrganizationCloudService organizationService = context.getBean(SysOrganizationCloudService.class);
            BasePipelineEnt entity = new BasePipelineEnt();
            XCopyUtils.copyObject(input, entity);

            entity.setEffectTime(new Date());
            entity.setEndTime(endTime);
            String id = UUID.randomUUID().toString();
            entity.setId(id);
            if (StringUtils.isNotBlank(input.getEnterpriseId())) {
                GetSysOrganizationViewInput enterpriseInput = new GetSysOrganizationViewInput();
                enterpriseInput.setId(input.getEnterpriseId());
                XSingleResult<GetSysOrganizationViewOutput> enterprise = organizationService.getSysOrganizationById(context, enterpriseInput);
                if (ObjectUtils.isNotEmpty(enterprise) && ObjectUtils.isNotEmpty(enterprise.getResult()) && StringUtils.isNotBlank(enterprise.getResult().getShortName())) {
                    entity.setEnterpriseName(enterprise.getResult().getShortName());
                }
            }
            //判断父级管线id是否为空，为空表示是一级管线，不需要添加ParentPath，如果不为空表示可能为2级或者更低一级的管线，需要增加ParentPath路径
            BasePipelineView view = new BasePipelineView();
            if (StringUtils.isBlank(input.getParentPipelineId()) || StringUtils.equals(ROOT_PARENT_ID, input.getParentPipelineId())) {
                entity.setParentPipelineId(ROOT_PARENT_ID);
                entity.setParentPath(null);
                view.setParentPipelineId(ROOT_PARENT_ID);
            } else {
                QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(BasePipelineEnt::getId, input.getParentPipelineId());
                BasePipelineEnt basePipelineEnt = mapper.selectOne(queryWrapper);
                //查看父级管线有没有parentPath,如果没有说明是一级管线，直接parentPath=/父级id
                if (StringUtils.isNotBlank(basePipelineEnt.getParentPath())) {
                    entity.setParentPath(basePipelineEnt.getParentPath() + "/" + basePipelineEnt.getId());
                } else {
                    entity.setParentPath("/" + input.getParentPipelineId());
                }
                view.setParentPipelineId(input.getParentPipelineId());
            }
            //sort赋值
            BasePipelineView pipelineView = pipelineViewMapper.selectMaxSortByParentPipelineId(view);
            Integer sort = 0;
            if (pipelineView != null && pipelineView.getSort() != null) {
                sort = sort + pipelineView.getSort() + 1;
            }
            entity.setSort(sort);
            entity.setCreateById(session.getId());
            entity.setCreateByName(context.getUser().getName());
            entity.setCreateTime(new Date());
            int insert = mapper.insert(entity);

            if (ObjectUtils.isNotEmpty(input.getOrgs())) {
                List<BaseOrgModel> orgs = input.getOrgs();
                for (BaseOrgModel model : orgs) {
                    //新增管线组织机构
                    BasePipelineOuRelEnt pipelineOu = new BasePipelineOuRelEnt();
                    pipelineOu.setId(UUID.randomUUID().toString());
                    pipelineOu.setPipelineId(id);
                    pipelineOu.setEffectTime(new Date());
                    pipelineOu.setEndTime(endTime);
                    pipelineOu.setOuId(model.getOuId());
                    pipelineOu.setCreateById(session.getId());
                    pipelineOu.setCreateByName(context.getUser().getName());
                    pipelineOu.setCreateTime(new Date());
                    int num = pipelineOuMapper.insert(pipelineOu);
                }
            }
            return XServiceResult.OK;
        });
    }

    @XText("修改管线")
    @XApiPost
    public XServiceResult updateBasePipeline(XContext context, UpdateBasePipelineInput input) {
        //修改管线:修改管线详细信息、删除旧的管线组织机构、新增管线组织机构
        return XTransactionHelper.begin(context, () -> {
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BasePipelineOuRelMapper pipelineOuMapper = context.getBean(BasePipelineOuRelMapper.class);
            SysOrganizationCloudService organizationService = context.getBean(SysOrganizationCloudService.class);
            //更新管线详细信息
            QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BasePipelineEnt::getId, input.getId());
            BasePipelineEnt entity = mapper.selectOne(queryWrapper);
            XCopyUtils.copyObject(input, entity);
            //判断父级管线id是否为空，为空表示是一级管线，不需要添加ParentPath，如果不为空表示可能为2级或者更低一级的管线，需要增加ParentPath路径
            BasePipelineView view = new BasePipelineView();
            if (StringUtils.isBlank(input.getParentPipelineId()) || StringUtils.equals(ROOT_PARENT_ID, input.getParentPipelineId())) {
                entity.setParentPipelineId(ROOT_PARENT_ID);
                entity.setParentPath(null);
                view.setParentPipelineId(ROOT_PARENT_ID);
            } else {
                QueryWrapper<BasePipelineEnt> query = new QueryWrapper<>();
                query.lambda().eq(BasePipelineEnt::getId, input.getParentPipelineId());
                BasePipelineEnt basePipelineEnt = mapper.selectOne(query);
                //查看父级管线有没有parentPath,如果没有说明是一级管线，直接parentPath=/父级id
                if (StringUtils.isNotBlank(basePipelineEnt.getParentPath())) {
                    entity.setParentPath(basePipelineEnt.getParentPath() + "/" + basePipelineEnt.getId());
                } else {
                    entity.setParentPath("/" + input.getParentPipelineId());
                }
                view.setParentPipelineId(input.getParentPipelineId());
            }
            if (StringUtils.isNotBlank(input.getEnterpriseId())) {
                GetSysOrganizationViewInput enterpriseInput = new GetSysOrganizationViewInput();
                enterpriseInput.setId(input.getEnterpriseId());
                XSingleResult<GetSysOrganizationViewOutput> enterprise = organizationService.getSysOrganizationById(context, enterpriseInput);
                if (ObjectUtils.isNotEmpty(enterprise) && ObjectUtils.isNotEmpty(enterprise.getResult()) && StringUtils.isNotBlank(enterprise.getResult().getShortName())) {
                    entity.setEnterpriseName(enterprise.getResult().getOuName());
                }
            }
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getLoginName());
            entity.setModifyTime(new Date());
            mapper.updateById(entity);

            //删除之前的管线组织机构关系
            QueryWrapper<BasePipelineOuRelEnt> ouWrapper = new QueryWrapper<>();
            ouWrapper.lambda().eq(BasePipelineOuRelEnt::getPipelineId, input.getId())
                    .gt(BasePipelineOuRelEnt::getEndTime, new Date());
            List<BasePipelineOuRelEnt> basePipelineOuRelEnts = pipelineOuMapper.selectList(ouWrapper);
            if (ObjectUtils.isNotEmpty(basePipelineOuRelEnts)) {
                for (BasePipelineOuRelEnt pipeline : basePipelineOuRelEnts) {
                    pipeline.setEndTime(new Date());
                    pipeline.setModifyById(session.getId());
                    pipeline.setModifyByName(session.getLoginName());
                    pipeline.setModifyTime(new Date());
                    pipelineOuMapper.updateById(pipeline);
                }
            }
            if (ObjectUtils.isNotEmpty(input.getOrgs())) {
                List<BaseOrgModel> orgs = input.getOrgs();
                for (BaseOrgModel model : orgs) {
                    //新增管线组织机构
                    BasePipelineOuRelEnt pipelineOu = new BasePipelineOuRelEnt();
                    pipelineOu.setId(UUID.randomUUID().toString());
                    pipelineOu.setPipelineId(entity.getId());
                    pipelineOu.setEffectTime(new Date());
                    pipelineOu.setEndTime(endTime);
                    pipelineOu.setOuId(model.getOuId());
                    pipelineOu.setCreateById(session.getId());
                    pipelineOu.setCreateByName(context.getUser().getName());
                    pipelineOu.setCreateTime(new Date());
                    int num = pipelineOuMapper.insert(pipelineOu);
                }
            }
            return XServiceResult.OK;
        });
    }

    @XText("删除管线")
    @XApiPost
    public XServiceResult deleteBasePipeline(XContext context, DeleteBasePipelineInput input) {
        //删除管线:如果管线下面有管段，不能删除管线！更新管线endTime为当前时间、更新旧的管线组织机构endTime为当前时间
        return XTransactionHelper.begin(context, () -> {
            BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
            BasePipelineSegmentRelMapper relMapper = context.getBean(BasePipelineSegmentRelMapper.class);
            BasePipelineOuRelMapper pipelineOuMapper = context.getBean(BasePipelineOuRelMapper.class);
            QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BasePipelineEnt::getId, input.getId());
            BasePipelineEnt entity = mapper.selectOne(queryWrapper);
            //根据管线id查询管段，如果管线下面有管段，不能删除管线
            QueryWrapper<BasePipelineSegmentRelEnt> query = new QueryWrapper();
            query.lambda().eq(BasePipelineSegmentRelEnt::getPipelineId, input.getId())
                    .gt(BasePipelineSegmentRelEnt::getEndTime, new Date());
            List<BasePipelineSegmentRelEnt> basePipelineSegmentRelEnts = relMapper.selectList(query);
            if (basePipelineSegmentRelEnts != null && basePipelineSegmentRelEnts.size() > 0) {
                return XServiceResult.error(200, "当前管线包含管段，不能删除管线！");
            }
            BasePipelineEnt opsPipelineEnt = new BasePipelineEnt();
            opsPipelineEnt.setId(entity.getId());
            opsPipelineEnt.setEndTime(new Date());
            mapper.updateById(opsPipelineEnt);
            //删除之前的管线组织机构关系
            QueryWrapper<BasePipelineOuRelEnt> ouWrapper = new QueryWrapper<>();
            ouWrapper.lambda().eq(BasePipelineOuRelEnt::getPipelineId, input.getId())
                    .gt(BasePipelineOuRelEnt::getEndTime, new Date());
            BasePipelineOuRelEnt basePipelineOuRelEnt = pipelineOuMapper.selectOne(ouWrapper);
            if (ObjectUtils.isNotEmpty(basePipelineOuRelEnt)) {
                basePipelineOuRelEnt.setEndTime(new Date());
                pipelineOuMapper.updateById(basePipelineOuRelEnt);
            }
            return XServiceResult.OK;
        });
    }

    @XText("根据id获取管线")
    @XApiPost
    public XSingleResult<GetBasePipelineOutput> getBasePipeline(XContext context, GetBasePipelineInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        BasePipelineOuRelViewMapper relMapper = context.getBean(BasePipelineOuRelViewMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date())
                .eq(BasePipelineEnt::getId, input.getId());
        BasePipelineEnt entity = mapper.selectOne(queryWrapper);

        //查询管线组织机构
        BasePipelineOuRelView record = new BasePipelineOuRelView();
        record.setPipelineId(input.getId());
        record.setEndTimeStr(DateUtil.getCurrentDateTime());
        List<BasePipelineOuRelView> basePipelineOuRelEnts = relMapper.selectOrgList(record);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePipelineOutput output = new GetBasePipelineOutput();
        XCopyUtils.copyObject(entity, output);
        if (StringUtils.isNotBlank(entity.getParentPipelineId())) {
            QueryWrapper<BasePipelineEnt> queryParent = new QueryWrapper();
            queryParent.lambda().gt(BasePipelineEnt::getEndTime, new Date())
                    .eq(BasePipelineEnt::getId, entity.getParentPipelineId());
            BasePipelineEnt parentObj = mapper.selectOne(queryParent);
            if (ObjectUtils.isNotEmpty(parentObj)) {
                output.setParentPipelineName(parentObj.getPipelineName());
            }
        }
        if (ObjectUtils.isNotEmpty(basePipelineOuRelEnts)) {
            List<BaseOrgModel> list = new ArrayList<>();
            for (BasePipelineOuRelView ent : basePipelineOuRelEnts) {
                BaseOrgModel model = new BaseOrgModel();
                model.setOuId(ent.getId());
                model.setOuName(ent.getOuName());
                list.add(model);
            }
            output.setOrgs(list);
        }

        return XSingleResult.success(output);
    }

    @XText("分页条件查询管线")
    @XApiPost
    public XPageResult<QueryBasePipelineOutput> queryBasePipeline(XContext context, QueryBasePipelineInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        BasePipelineViewMapper viewMapper = context.getBean(BasePipelineViewMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        //调用微服务查询字典
        CreateSysDictionaryInput mediumType = new CreateSysDictionaryInput();
        mediumType.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediumTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediumType);
        CreateSysDictionaryInput pipelineType = new CreateSysDictionaryInput();
        pipelineType.setAlias("PIPELINE_TYPE");
        XListResult<QuerySysDictionaryViewOutput> pipelineTypeList = sysDictionaryService.querySysDictionarysByAlias(context, pipelineType);

        //查询符合条件的管线，通过parentPath获取一级管线id
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().le(BasePipelineEnt::getEffectTime, new Date())
                .gt(BasePipelineEnt::getEndTime, new Date());
        if (StringUtils.isNotBlank(input.getParentId()) && StringUtils.isNotBlank(input.getPipelineName())) {
            BasePipelineView record = new BasePipelineView();
            record.setPipelineName(input.getPipelineName());
            List<BasePipelineView> pipelineViewList = viewMapper.selectPipelineListByName(record);
            if (ObjectUtils.isNotEmpty(pipelineViewList)) {
                List<BasePipelineView> collect = pipelineViewList.stream().filter(p -> StringUtils.equals(p.getParentPipelineId(), input.getParentId())).distinct().collect(Collectors.toList());
                List<String> pipelineIds = collect.stream().map(p -> {
                    return p.getId();
                }).distinct().collect(Collectors.toList());
                if (ObjectUtils.isNotEmpty(pipelineIds)) {
                    queryWrapper.lambda().in(BasePipelineEnt::getId, pipelineIds);
                } else {
                    return XPageResult.success(new ArrayList<>(), 1, 1, 0);
                }
            }
//            queryWrapper.lambda().and(wrapper -> wrapper.eq(BasePipelineEnt::getParentPipelineId,parentId).or().like(BasePipelineEnt::getPipelineName,pipelineName));
        } else if (StringUtils.isNotBlank(input.getParentId())) {
            queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, input.getParentId());
        } else if (StringUtils.isNotBlank(input.getPipelineName())) {
            queryWrapper.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
        }
        if (StringUtils.isNotBlank(input.getMediaType()) && !StringUtils.equals("ALL", input.getMediaType())) {
            queryWrapper.lambda().eq(BasePipelineEnt::getMediumType, input.getMediaType());
        }
        if (StringUtils.isNotBlank(input.getDispatchDesk())) {
            queryWrapper.lambda().eq(BasePipelineEnt::getDispatchDesk, input.getDispatchDesk());
        }
        /*if(StringUtils.isBlank(input.getParentId())){
            queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId,"11111111-1111-1111-1111-111111111111");
        }*/

        /*//值为1  加载第一级，否则加载子菜单
        if(StringUtils.isNotBlank(input.getLoadType()) && StringUtils.equals("1",input.getLoadType())){
            input.setPage(1);
            input.setLimit(10000);
        }else{
            input.setPage(input.getPage());
            input.setLimit(input.getLimit());
        }*/
        //查询满足条件的管线
        QueryBasePipelineInput inputQuery = new QueryBasePipelineInput();
        XCopyUtils.copyObject(input, inputQuery);
        inputQuery.setPage(1);
        inputQuery.setLimit(100000);
        if (StringUtils.isNotBlank(input.getIsHaveOuId()) && StringUtils.equals(input.getIsHaveOuId(), "1")) {
            List<String> allOuIdsByUser = getAllOuIdsByUser(context, session, input.getMenuId());
            queryWrapper.lambda().in(BasePipelineEnt::getEnterpriseId, allOuIdsByUser);
        }
        queryWrapper.lambda().orderByDesc(BasePipelineEnt::getSort, BasePipelineEnt::getCreateTime);
        XPageResult<QueryBasePipelineOutput> query = XMapperHelper.query(mapper, inputQuery, queryWrapper, QueryBasePipelineOutput.class);
        List<QueryBasePipelineOutput> items = query.getResult().getItems();
        List<String> ids = new ArrayList<>();
        /*ids = items.stream().map(p -> {
            return p.getParentPath() == null || p.getParentPath() == ""?p.getId():p.getParentPath().split("/")[1];
        }).distinct().collect(Collectors.toList());*/
        if (StringUtils.isNotBlank(input.getParentId())) {
            ids = items.stream().map(p -> {
                return p.getId();
            }).distinct().collect(Collectors.toList());
        } else {
            ids = items.stream().map(p -> {
                return p.getParentPath() == null || p.getParentPath() == "" ? p.getId() : p.getParentPath().split("/")[1];
            }).distinct().collect(Collectors.toList());
        }

        QueryWrapper<BasePipelineEnt> search = new QueryWrapper();
        search.lambda().in(BasePipelineEnt::getId, ids);
        if (ids == null || ids.size() < 1) {
            return XPageResult.success(new ArrayList<>(), 1, 1, 0);
        }
        if (StringUtils.isNotBlank(input.getIsHaveOuId()) && StringUtils.equals(input.getIsHaveOuId(), "1")) {
            List<String> allOuIdsByUser = getAllOuIdsByUser(context, session, input.getMenuId());
            search.lambda().in(BasePipelineEnt::getEnterpriseId, allOuIdsByUser);
        }
        search.lambda().orderByAsc(BasePipelineEnt::getSort);
        //查询一级管线
        XPageResult<QueryBasePipelineOutput> parentPipelines = XMapperHelper.query(mapper, input, search, QueryBasePipelineOutput.class);
        List<QueryBasePipelineOutput> pipelines = parentPipelines.getResult().getItems();
        //根据上级管线id查询有下级管线信息
        QueryWrapper<BasePipelineEnt> wrapper = new QueryWrapper<>();
        XPageResult<QueryBasePipelineOutput> secPipeline = null;
        if (ids != null && ids.size() > 0) {
            if (StringUtils.isNotBlank(input.getPipelineName())) {
                BasePipelineView record = new BasePipelineView();
                record.setPipelineName(input.getPipelineName());
                List<BasePipelineView> pipelineViewList = viewMapper.selectPipelineListByName(record);
                List<String> idStr = new ArrayList<>();
                if (ObjectUtils.isNotEmpty(pipelineViewList)) {
                    for (BasePipelineView view : pipelineViewList) {
                        for (String id : ids) {
                            if (StringUtils.equals(id, view.getParentPipelineId())) {
                                idStr.add(view.getId());
                            }
                        }
                    }
                    if (ObjectUtils.isNotEmpty(idStr)) {
                        wrapper.lambda().in(BasePipelineEnt::getId, idStr);
                        secPipeline = XMapperHelper.query(mapper, inputQuery, wrapper, QueryBasePipelineOutput.class);
                    }
                }
            } else {
                wrapper.lambda().in(BasePipelineEnt::getParentPipelineId, ids);
                secPipeline = XMapperHelper.query(mapper, inputQuery, wrapper, QueryBasePipelineOutput.class);
            }
        }
        List<QueryBasePipelineOutput> basePipelineViews = null;
        if (ObjectUtils.isNotEmpty(secPipeline)) {
            basePipelineViews = secPipeline.getResult().getItems();
        }
        if (StringUtils.isNotBlank(input.getLoadType()) && StringUtils.equals("1", input.getLoadType())) {
            //添加前端显示字段值，null表示没有下级
            int i = 1;
            for (QueryBasePipelineOutput outPipeline : items) {
                if (basePipelineViews != null && basePipelineViews.size() > 0) {
                    boolean flag = false;
                    for (QueryBasePipelineOutput outPipeView : basePipelineViews) {
                        if (StringUtils.equals(outPipeline.getId(), outPipeView.getParentPipelineId())) {
                            flag = true;
                            break;
                        }
                    }
                    if (!flag) {
                        //添加子节点是否打开的标志
                        outPipeline.setChildren(null);
                    }
                } else {
                    //添加子节点是否打开的标志
                    outPipeline.setChildren(null);
                }
                outPipeline.setXh(i + "");
                i++;
            }
            int j = 1;
            for (QueryBasePipelineOutput outPipeView : items) {
                List<QuerySysDictionaryViewOutput> me = mediumTypeList.getResult();
                List<QuerySysDictionaryViewOutput> pi = pipelineTypeList.getResult();
                if (ObjectUtils.isNotEmpty(me)) {
                    for (QuerySysDictionaryViewOutput obj : me) {
                        if (StringUtils.equals(outPipeView.getMediumType(), obj.getDicKey())) {
                            outPipeView.setMediumTypeName(obj.getDicName());
                        }
                    }
                }
                if (ObjectUtils.isNotEmpty(pi)) {
                    for (QuerySysDictionaryViewOutput obj : pi) {
                        if (StringUtils.equals(outPipeView.getPipelineType(), obj.getDicKey())) {
                            outPipeView.setPipelineTypeName(obj.getDicName());
                        }
                    }
                }
                outPipeView.setXh(String.valueOf(j++));
            }
            return query;
        } else {
            //添加前端显示字段值，null表示没有下级
            int i = 1;
            for (QueryBasePipelineOutput outPipeline : pipelines) {
                if (basePipelineViews != null && basePipelineViews.size() > 0) {
                    boolean flag = false;
                    for (QueryBasePipelineOutput outPipeView : basePipelineViews) {
                        if (StringUtils.equals(outPipeline.getId(), outPipeView.getParentPipelineId())) {
                            flag = true;
                            break;
                        }
                    }
                    if (!flag) {
                        //添加子节点是否打开的标志
                        outPipeline.setChildren(null);
                    }
                } else {
                    //添加子节点是否打开的标志
                    outPipeline.setChildren(null);
                }
                outPipeline.setXh(i + "");
                i++;
            }
            int j = 1;
            for (QueryBasePipelineOutput outPipeView : pipelines) {
                List<QuerySysDictionaryViewOutput> me = mediumTypeList.getResult();
                List<QuerySysDictionaryViewOutput> pi = pipelineTypeList.getResult();
                if (ObjectUtils.isNotEmpty(me)) {
                    for (QuerySysDictionaryViewOutput obj : me) {
                        if (StringUtils.equals(outPipeView.getMediumType(), obj.getDicKey())) {
                            outPipeView.setMediumTypeName(obj.getDicName());
                        }
                    }
                }
                if (ObjectUtils.isNotEmpty(pi)) {
                    for (QuerySysDictionaryViewOutput obj : pi) {
                        if (StringUtils.equals(outPipeView.getPipelineType(), obj.getDicKey())) {
                            outPipeView.setPipelineTypeName(obj.getDicName());
                        }
                    }
                }
                outPipeView.setXh(String.valueOf(j++));
            }
            return parentPipelines;
        }
    }

    @XText("分页条件查询管线--根据调度台")
    @XApiPost
    public XPageResult<QueryBasePipelineOutput> queryBasePipelineByDispatchDesk(XContext context, QueryBasePipelineInput input) {
        //调用微服务查询字典
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        CreateSysDictionaryInput mediumType = new CreateSysDictionaryInput();
        mediumType.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediumTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediumType);
        CreateSysDictionaryInput pipelineType = new CreateSysDictionaryInput();
        pipelineType.setAlias("PIPELINE_TYPE");
        XListResult<QuerySysDictionaryViewOutput> pipelineTypeList = sysDictionaryService.querySysDictionarysByAlias(context, pipelineType);
        if (XStringUtils.isEmpty(input.getStationId())) {
            BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
            BasePipelineViewMapper viewMapper = context.getBean(BasePipelineViewMapper.class);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            //查询符合条件的管线，通过parentPath获取一级管线id
            QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().le(BasePipelineEnt::getEffectTime, new Date())
                    .gt(BasePipelineEnt::getEndTime, new Date());
            if (StringUtils.isNotBlank(input.getParentId())) {
                queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, input.getParentId());
                QueryWrapper<BasePipelineEnt> parentWrapper = new QueryWrapper<>();
                parentWrapper.lambda().eq(BasePipelineEnt::getId, input.getParentId());
                BasePipelineEnt basePipelineEnt = mapper.selectOne(parentWrapper);
                if (XStringUtils.isEmpty(basePipelineEnt.getParentPath()) && (XStringUtils.isEmpty(basePipelineEnt.getDispatchDesk()) || !basePipelineEnt.getDispatchDesk().contains(input.getDispatchDesk())))
                    queryWrapper.lambda().like(BasePipelineEnt::getDispatchDesk, input.getDispatchDesk());
                else {
                    queryWrapper.lambda().and((wrapper) -> {
                        wrapper.like(BasePipelineEnt::getDispatchDesk, input.getDispatchDesk()).or().isNull(BasePipelineEnt::getDispatchDesk).or().eq(BasePipelineEnt::getDispatchDesk, "");
                    });
                }
                //如果父级id不为空，判断当前管线是否包含查询的调度台，如果包含的话，查找所有管线，如果不包含的话，查找对应的
            } else {
                queryWrapper.lambda().like(BasePipelineEnt::getDispatchDesk, input.getDispatchDesk());
            }
            if (StringUtils.isNotBlank(input.getMediaType()) && !StringUtils.equals("ALL", input.getMediaType())) {
                queryWrapper.lambda().eq(BasePipelineEnt::getMediumType, input.getMediaType());
            }

            //查询满足条件的管线
            QueryBasePipelineInput inputQuery = new QueryBasePipelineInput();
            XCopyUtils.copyObject(input, inputQuery);
            inputQuery.setPage(1);
            inputQuery.setLimit(100000);
            if (StringUtils.isNotBlank(input.getIsHaveOuId()) && StringUtils.equals(input.getIsHaveOuId(), "1")) {
                List<String> allOuIdsByUser = getAllOuIdsByUser(context, session, input.getMenuId());
                queryWrapper.lambda().in(BasePipelineEnt::getEnterpriseId, allOuIdsByUser);
            }
            queryWrapper.lambda().orderByDesc(BasePipelineEnt::getSort, BasePipelineEnt::getCreateTime);
            XPageResult<QueryBasePipelineOutput> query = XMapperHelper.query(mapper, inputQuery, queryWrapper, QueryBasePipelineOutput.class);
            List<QueryBasePipelineOutput> items = query.getResult().getItems();
            List<String> ids = new ArrayList<>();
            if (StringUtils.isNotBlank(input.getParentId())) {
                ids = items.stream().map(p -> {
                    return p.getId();
                }).distinct().collect(Collectors.toList());
            } else {
                ids = items.stream().map(p -> {
                    return p.getParentPath() == null || p.getParentPath() == "" ? p.getId() : p.getParentPath().split("/")[1];
                }).distinct().collect(Collectors.toList());
            }

            QueryWrapper<BasePipelineEnt> search = new QueryWrapper();
            search.lambda().in(BasePipelineEnt::getId, ids);
            if (ids == null || ids.size() < 1) {
                return XPageResult.success(new ArrayList<>(), 1, 1, 0);
            }
            if (StringUtils.isNotBlank(input.getIsHaveOuId()) && StringUtils.equals(input.getIsHaveOuId(), "1")) {
                List<String> allOuIdsByUser = getAllOuIdsByUser(context, session, input.getMenuId());
                search.lambda().in(BasePipelineEnt::getEnterpriseId, allOuIdsByUser);
            }
            search.lambda().orderByAsc(BasePipelineEnt::getSort);
            //查询一级管线
            XPageResult<QueryBasePipelineOutput> parentPipelines = XMapperHelper.query(mapper, input, search, QueryBasePipelineOutput.class);
            List<QueryBasePipelineOutput> pipelines = parentPipelines.getResult().getItems();
            //根据上级管线id查询有下级管线信息
            QueryWrapper<BasePipelineEnt> wrapper = new QueryWrapper<>();
            XPageResult<QueryBasePipelineOutput> secPipeline = null;
            if (ids != null && ids.size() > 0) {
                if (StringUtils.isNotBlank(input.getPipelineName())) {
                    BasePipelineView record = new BasePipelineView();
                    record.setPipelineName(input.getPipelineName());
                    List<BasePipelineView> pipelineViewList = viewMapper.selectPipelineListByName(record);
                    List<String> idStr = new ArrayList<>();
                    if (ObjectUtils.isNotEmpty(pipelineViewList)) {
                        for (BasePipelineView view : pipelineViewList) {
                            for (String id : ids) {
                                if (StringUtils.equals(id, view.getParentPipelineId())) {
                                    idStr.add(view.getId());
                                }
                            }
                        }
                        if (ObjectUtils.isNotEmpty(idStr)) {
                            wrapper.lambda().in(BasePipelineEnt::getId, idStr);
                            secPipeline = XMapperHelper.query(mapper, inputQuery, wrapper, QueryBasePipelineOutput.class);
                        }
                    }
                } else {
                    wrapper.lambda().in(BasePipelineEnt::getParentPipelineId, ids);
                    secPipeline = XMapperHelper.query(mapper, inputQuery, wrapper, QueryBasePipelineOutput.class);
                }
            }
            List<QueryBasePipelineOutput> basePipelineViews = null;
            if (ObjectUtils.isNotEmpty(secPipeline)) {
                basePipelineViews = secPipeline.getResult().getItems();
            }
            if (StringUtils.isNotBlank(input.getLoadType()) && StringUtils.equals("1", input.getLoadType())) {
                //添加前端显示字段值，null表示没有下级
                int i = 1;
                for (QueryBasePipelineOutput outPipeline : items) {
                    if (basePipelineViews != null && basePipelineViews.size() > 0) {
                        boolean flag = false;
                        for (QueryBasePipelineOutput outPipeView : basePipelineViews) {
                            if (StringUtils.equals(outPipeline.getId(), outPipeView.getParentPipelineId())) {
                                flag = true;
                                break;
                            }
                        }
                        if (!flag) {
                            //添加子节点是否打开的标志
                            outPipeline.setChildren(null);
                        }
                    } else {
                        //添加子节点是否打开的标志
                        outPipeline.setChildren(null);
                    }
                    outPipeline.setXh(i + "");
                    i++;
                }
                int j = 1;
                for (QueryBasePipelineOutput outPipeView : items) {
                    List<QuerySysDictionaryViewOutput> me = mediumTypeList.getResult();
                    List<QuerySysDictionaryViewOutput> pi = pipelineTypeList.getResult();
                    if (ObjectUtils.isNotEmpty(me)) {
                        for (QuerySysDictionaryViewOutput obj : me) {
                            if (StringUtils.equals(outPipeView.getMediumType(), obj.getDicKey())) {
                                outPipeView.setMediumTypeName(obj.getDicName());
                            }
                        }
                    }
                    if (ObjectUtils.isNotEmpty(pi)) {
                        for (QuerySysDictionaryViewOutput obj : pi) {
                            if (StringUtils.equals(outPipeView.getPipelineType(), obj.getDicKey())) {
                                outPipeView.setPipelineTypeName(obj.getDicName());
                            }
                        }
                    }
                    outPipeView.setXh(String.valueOf(j++));
                }
                return query;
            } else {
                //添加前端显示字段值，null表示没有下级
                int i = 1;
                for (QueryBasePipelineOutput outPipeline : pipelines) {
                    if (basePipelineViews != null && basePipelineViews.size() > 0) {
                        boolean flag = false;
                        for (QueryBasePipelineOutput outPipeView : basePipelineViews) {
                            if (StringUtils.equals(outPipeline.getId(), outPipeView.getParentPipelineId())) {
                                flag = true;
                                break;
                            }
                        }
                        if (!flag) {
                            //添加子节点是否打开的标志
                            outPipeline.setChildren(null);
                        }
                    } else {
                        //添加子节点是否打开的标志
                        outPipeline.setChildren(null);
                    }
                    outPipeline.setXh(i + "");
                    i++;
                }
                int j = 1;
                for (QueryBasePipelineOutput outPipeView : pipelines) {
                    List<QuerySysDictionaryViewOutput> me = mediumTypeList.getResult();
                    List<QuerySysDictionaryViewOutput> pi = pipelineTypeList.getResult();
                    if (ObjectUtils.isNotEmpty(me)) {
                        for (QuerySysDictionaryViewOutput obj : me) {
                            if (StringUtils.equals(outPipeView.getMediumType(), obj.getDicKey())) {
                                outPipeView.setMediumTypeName(obj.getDicName());
                            }
                        }
                    }
                    if (ObjectUtils.isNotEmpty(pi)) {
                        for (QuerySysDictionaryViewOutput obj : pi) {
                            if (StringUtils.equals(outPipeView.getPipelineType(), obj.getDicKey())) {
                                outPipeView.setPipelineTypeName(obj.getDicName());
                            }
                        }
                    }
                    outPipeView.setXh(String.valueOf(j++));
                }
                return parentPipelines;
            }
        } else {
            BasePipelineViewMapper mapper = context.getBean(BasePipelineViewMapper.class);
            List<BasePipelineView> list = mapper.selectPipelineListByStationId(input.getStationId());
            List<QueryBasePipelineOutput> outputs = XCopyUtils.copyNewList(list, QueryBasePipelineOutput.class);
            outputs.stream().forEach(item->{
                item.setPipelineTypeName(getDictName(item.getPipelineType() + "" , pipelineTypeList.getResult()));
                item.setMediumTypeName(getDictName(item.getMediumType() + "" , mediumTypeList.getResult()));
            });
            return XPageResult.success(outputs, input, list.size());
        }
    }

    public String getDictName(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return "";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict.getDicName();
            }
        }
        return "";
    }

    @XText("条件获取管线树--根据调度台")
    @XApiPost
    public XListResult<BasePipelineTreeOutput> getBasePipelineTreeByDispatchDesk(XContext context, GetBasePipelineViewInput input) {
        //查询所有符合查询条件的数据(设置调度台数据)
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        BasePipelineViewMapper viewMapper = context.getBean(BasePipelineViewMapper.class);
        QueryWrapper<BasePipelineEnt> qr = new QueryWrapper<>();
        qr.lambda().gt(BasePipelineEnt::getEndTime, new Date());
        if (StringUtils.isNotBlank(input.getPipelineName())) {
            qr.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
        }
        if (StringUtils.isNotBlank(input.getDispatchDesk())) {
            qr.lambda().like(BasePipelineEnt::getDispatchDesk, input.getDispatchDesk());
        }
        List<BasePipelineEnt> list = mapper.selectList(qr);
        //查询所有符合查询条件的数据(未设置调度台数据，并判断父级是否属于调度台范围)
        qr.clear();
        if (StringUtils.isNotBlank(input.getPipelineName())) {
            qr.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
        }
        if (StringUtils.isNotBlank(input.getDispatchDesk())) {
            qr.lambda().and((wrapper) -> {
                wrapper.isNull(BasePipelineEnt::getDispatchDesk).or().eq(BasePipelineEnt::getDispatchDesk, "");
            });
            qr.lambda().isNotNull(BasePipelineEnt::getParentPath);
        }
        List<BasePipelineEnt> nullDispatchDeskList = mapper.selectList(qr);
        List<String> parentPathList = nullDispatchDeskList.stream().filter(item -> XStringUtils.isNotEmpty(item.getParentPath())).map(BasePipelineEnt::getParentPath).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(parentPathList)) {
            List<String> parentPathIds = new ArrayList<>();
            parentPathList.stream().forEach(item -> {
                item = item.substring(1);
                String[] ids = item.split("/");
                parentPathIds.addAll(Arrays.asList(ids));
            });
            qr.clear();
            qr.lambda().like(BasePipelineEnt::getDispatchDesk, input.getDispatchDesk());
            qr.lambda().in(BasePipelineEnt::getId, parentPathIds);
            List<BasePipelineEnt> dispatchDeskPrentList = mapper.selectList(qr);
            for (BasePipelineEnt item : nullDispatchDeskList) {
                for (BasePipelineEnt parent : dispatchDeskPrentList) {
                    if (item.getParentPath().contains(parent.getId()))
                        list.add(item);
                }
            }
        }
        List<String> ids = list.stream().map(p -> {
            return p.getId();
        }).distinct().collect(Collectors.toList());
        if (ObjectUtils.isNotEmpty(ids)) {
            StringBuffer pipelineId = new StringBuffer();
            for (int i = 0; i < ids.size(); i++) {
                if (i == 0) {
                    pipelineId.append("'" + ids.get(i) + "'");
                } else {
                    pipelineId.append(",'" + ids.get(i) + "'");
                }
            }
            BasePipelineView record = new BasePipelineView();
            record.setId(pipelineId.toString());
            List<BasePipelineView> pipelineViewList = viewMapper.selectPipelineListById(record);
            List<String> allIds = pipelineViewList.stream().map(p -> {
                return p.getId();
            }).distinct().collect(Collectors.toList());
            QueryWrapper<BasePipelineEnt> qrPipeline = new QueryWrapper<>();
            qrPipeline.lambda().in(BasePipelineEnt::getId, allIds);
            list = mapper.selectList(qrPipeline);
        }

        List<BasePipelineTreeOutput> treeList = new ArrayList<>();
        if (list.size() > 0) {
            List<BasePipelineTreeOutput> tempList = XCopyUtils.copyNewList(list, BasePipelineTreeOutput.class);
            for (BasePipelineTreeOutput tree : tempList) {
                if (ROOT_PARENT_ID.equals(tree.getParentPipelineId())) {
                    treeList.add(tree);
                }
                //子集
                for (BasePipelineTreeOutput item : tempList) {
                    if (tree.getId().equals(item.getParentPipelineId())) {
                        if (tree.getChildren() == null) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(item);
                    }
                }
                if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                    List<BasePipelineTreeOutput> children = tree.getChildren();
                    children.forEach(outputTree -> {
                        if (outputTree.getSort() == null) {
                            outputTree.setSort(999);
                        }
                    });
                    children = children.stream().sorted(Comparator.comparing(BasePipelineTreeOutput::getSort)).collect(Collectors.toList());
                    tree.setChildren(children);
                }
            }
        }
        return XListResult.success(treeList);
    }


    @XText("条件获取管线树--根据调度台王甜备用")
    @XApiPost
    public XListResult<BasePipelineTreeOutput> getDispatchingDeskByPipeline(XContext context, GetBasePipelineViewInput input) {
        //查询所有符合查询条件的数据(设置调度台数据)
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        BasePipelineViewMapper viewMapper = context.getBean(BasePipelineViewMapper.class);
        QueryWrapper<BasePipelineEnt> qr = new QueryWrapper<>();
        qr.lambda().gt(BasePipelineEnt::getEndTime, new Date());
        if (StringUtils.isNotBlank(input.getPipelineName())) {
            qr.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
        }
        if (StringUtils.isNotBlank(input.getConsoleDicId())) {
            qr.lambda().like(BasePipelineEnt::getDispatchDesk, input.getConsoleDicId());
        }
        List<BasePipelineEnt> list = mapper.selectList(qr);
        //查询所有符合查询条件的数据(未设置调度台数据，并判断父级是否属于调度台范围)
        qr.clear();
        if (StringUtils.isNotBlank(input.getPipelineName())) {
            qr.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
        }
        if (StringUtils.isNotBlank(input.getConsoleDicId())) {
            qr.lambda().and((wrapper) -> {
                wrapper.isNull(BasePipelineEnt::getDispatchDesk).or().eq(BasePipelineEnt::getDispatchDesk, "");
            });
            qr.lambda().isNotNull(BasePipelineEnt::getParentPath);
        }
        List<BasePipelineEnt> nullConsoleDicIdList = mapper.selectList(qr);
        List<String> parentPathList = nullConsoleDicIdList.stream().filter(item -> XStringUtils.isNotEmpty(item.getParentPath())).map(BasePipelineEnt::getParentPath).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(parentPathList)) {
            List<String> parentPathIds = new ArrayList<>();
            parentPathList.stream().forEach(item -> {
                item = item.substring(1);
                String[] ids = item.split("/");
                parentPathIds.addAll(Arrays.asList(ids));
            });
            qr.clear();
            qr.lambda().like(BasePipelineEnt::getDispatchDesk, input.getConsoleDicId());
            qr.lambda().in(BasePipelineEnt::getId, parentPathIds);
            List<BasePipelineEnt> ConsoleDicIdPrentList = mapper.selectList(qr);
            for (BasePipelineEnt item : nullConsoleDicIdList) {
                for (BasePipelineEnt parent : ConsoleDicIdPrentList) {
                    if (item.getParentPath().contains(parent.getId()))
                        list.add(item);
                }
            }
        }
        List<String> ids = list.stream().map(p -> {
            return p.getId();
        }).distinct().collect(Collectors.toList());
        if (ObjectUtils.isNotEmpty(ids)) {
            StringBuffer pipelineId = new StringBuffer();
            for (int i = 0; i < ids.size(); i++) {
                if (i == 0) {
                    pipelineId.append("'" + ids.get(i) + "'");
                } else {
                    pipelineId.append(",'" + ids.get(i) + "'");
                }
            }
            BasePipelineView record = new BasePipelineView();
            record.setId(pipelineId.toString());
            List<BasePipelineView> pipelineViewList = viewMapper.selectPipelineListById(record);
            List<String> allIds = pipelineViewList.stream().map(p -> {
                return p.getId();
            }).distinct().collect(Collectors.toList());
            QueryWrapper<BasePipelineEnt> qrPipeline = new QueryWrapper<>();
            qrPipeline.lambda().in(BasePipelineEnt::getId, allIds);
            list = mapper.selectList(qrPipeline);
        }

        List<BasePipelineTreeOutput> treeList = new ArrayList<>();
        if (list.size() > 0) {
            List<BasePipelineTreeOutput> tempList = XCopyUtils.copyNewList(list, BasePipelineTreeOutput.class);
            for (BasePipelineTreeOutput tree : tempList) {
                if (ROOT_PARENT_ID.equals(tree.getParentPipelineId())) {
                    treeList.add(tree);
                }
                //子集
                for (BasePipelineTreeOutput item : tempList) {
                    if (tree.getId().equals(item.getParentPipelineId())) {
                        if (tree.getChildren() == null) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(item);
                    }
                }
                if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                    List<BasePipelineTreeOutput> children = tree.getChildren();
                    children.forEach(outputTree -> {
                        if (outputTree.getSort() == null) {
                            outputTree.setSort(999);
                        }
                    });
                    children = children.stream().sorted(Comparator.comparing(BasePipelineTreeOutput::getSort)).collect(Collectors.toList());
                    tree.setChildren(children);
                }
            }
        }
        return XListResult.success(treeList);
    }

    @XText("暂时没用")
    @XApiPost
    public XPageResult<QueryBasePipelineOutput> dynamicQueryBasePipeline(XContext context, CustomQueryInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        return XMapperHelper.query(mapper, input, BasePipelineEnt.class, QueryBasePipelineOutput.class);
    }

    @XText("暂时没用")
    @XApiPost
    public XSingleResult<GetBasePipelineViewOutput> getBasePipelineView(XContext context, GetBasePipelineViewInput input) {
        BasePipelineViewMapper mapper = context.getBean(BasePipelineViewMapper.class);
        BasePipelineView record = new BasePipelineView();
        XCopyUtils.copyObject(input, record);
        BasePipelineView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePipelineViewOutput output = new GetBasePipelineViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XText("根据名称模糊查询管线列表")
    @XApiPost
    public XPageResult<QueryBasePipelineViewOutput> queryBasePipelineView(XContext context, QueryBasePipelineViewInput input) {
        BasePipelineViewMapper mapper = context.getBean(BasePipelineViewMapper.class);
        BasePipelineView record = new BasePipelineView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePipelineView> list = mapper.selectListByName(record);
        PageInfo<BasePipelineView> pageInfo = new PageInfo<>(list);
        List<QueryBasePipelineViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePipelineViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据管线id查询管线所有的站（暂时没用）")
    @XApiPost
    public XPageResult<QueryBasePipelineViewOutput> queryStationByPipelineId(XContext context, QueryBasePipelineViewInput input) {
        BasePipelineViewMapper mapper = context.getBean(BasePipelineViewMapper.class);
        BasePipelineView record = new BasePipelineView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePipelineView> list = mapper.selectStationListByPipelineId(record);
        PageInfo<BasePipelineView> pageInfo = new PageInfo<>(list);
        List<QueryBasePipelineViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePipelineViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("查询管段和场站经纬度")
    @XApiPost
    public XPageResult<QueryBasePipelineViewOutput> queryStationInfo(XContext context, QueryBasePipelineViewInput input) {
        BasePipelineViewMapper mapper = context.getBean(BasePipelineViewMapper.class);
        BasePipelineView record = new BasePipelineView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        //查询当前管线下所有子级管线
        if (StringUtils.isNotBlank(input.getId())) {
            List<String> ids = selectChildPipelines(context, input.getId());
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < ids.size(); i++) {
                if (i == 0) {
                    sb.append("'" + ids.get(i) + "'");
                } else {
                    sb.append(",'" + ids.get(i) + "'");
                }
            }
            record.setId(sb.toString());
        }
        record.setEndTimeStr(XDateUtils.getDateTimeString(new Date()));
        List<BasePipelineView> list = mapper.selectStationInfoList(record);
        PageInfo<BasePipelineView> pageInfo = new PageInfo<>(list);
        List<QueryBasePipelineViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePipelineViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    public List<String> selectChildPipelines(XContext context, String id) {
        List<String> ids = new ArrayList<>();
        ids.add(id);
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BasePipelineEnt::getParentPath, id);
        XPageInput page = new XPageInput();
        page.setPage(1);
        page.setLimit(100000);
        IPage<BasePipelineEnt> opsPipelineEnts = XMapperHelper.queryPage(mapper, page, queryWrapper);
        List<BasePipelineEnt> records = opsPipelineEnts.getRecords();
        List<String> collect = records.stream().map(p -> p.getId()).distinct().collect(Collectors.toList());
        ids.addAll(collect);
        return ids;
    }

    @XText("修改管线排序（暂时没用）")
    @XApiPost
    public XServiceResult updatePipelineSortById(XContext context, QueryBasePipelineViewInput input) {
        BasePipelineViewMapper viewMapper = context.getBean(BasePipelineViewMapper.class);
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> query = new QueryWrapper();
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(BasePipelineEnt::getId, input.getId());
        BasePipelineEnt basePipelineEnt = mapper.selectOne(queryWrapper);
        query.lambda().eq(BasePipelineEnt::getParentPipelineId, basePipelineEnt.getParentPipelineId());
        if (StringUtils.equals("up", input.getSortFlag())) {
            query.lambda().lt(BasePipelineEnt::getSort, basePipelineEnt.getSort());
        } else {
            query.lambda().gt(BasePipelineEnt::getSort, basePipelineEnt.getSort());
        }
        BasePipelineEnt basePipelineEnt1 = mapper.selectOne(query);
        Integer sort = basePipelineEnt1.getSort();
        basePipelineEnt1.setSort(basePipelineEnt.getSort());
        basePipelineEnt.setSort(sort);
        mapper.updateById(basePipelineEnt1);
        mapper.updateById(basePipelineEnt);
        return XServiceResult.OK;
    }


    @XText("初始化管线parentPath")
    @XApiPost
    public XServiceResult updateBasePipelinePath(XContext context, UpdateBasePipelineInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, ROOT_PARENT_ID);
        XPageInput page = new XPageInput();
        page.setPage(1);
        page.setLimit(100000);
        IPage<BasePipelineEnt> opsPipelineEnts = XMapperHelper.queryPage(mapper, page, queryWrapper);
        List<BasePipelineEnt> records = opsPipelineEnts.getRecords();
        updateBasePipelineParentPath(context, records);
        return XServiceResult.OK;
    }

    public XServiceResult updateBasePipelineParentPath(XContext context, List<BasePipelineEnt> pipelines) {
        for (BasePipelineEnt pipeline : pipelines) {
            BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
            if (StringUtils.isNotBlank(pipeline.getId())) {
                //更新二级管道parentPath,一级管道不更新
                QueryWrapper<BasePipelineEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, pipeline.getId());
                XPageInput pageInput = new XPageInput();
                pageInput.setPage(1);
                pageInput.setLimit(100000);
                IPage<BasePipelineEnt> child = XMapperHelper.queryPage(mapper, pageInput, wrapper);
                List<BasePipelineEnt> childPipelines = child.getRecords();
                for (BasePipelineEnt childPipeline : childPipelines) {
                    String parentPath = pipeline.getParentPath() == null || pipeline.getParentPath() == "null" ? "/" + pipeline.getId() : pipeline.getParentPath() + "/" + pipeline.getId();
                    childPipeline.setParentPath(parentPath);
                    mapper.updateById(childPipeline);
                }
                updateBasePipelineParentPath(context, childPipelines);
            }
        }
        return XServiceResult.OK;
    }

    @XText("获取管线树")
    @XApiPost
    public XListResult<BasePipelineTreeOutput> getBasePipelineTree(XContext context) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> qr = new QueryWrapper<>();
        qr.lambda().gt(BasePipelineEnt::getEndTime, new Date());
        List<BasePipelineEnt> list = mapper.selectList(qr);

        List<BasePipelineTreeOutput> treeList = new ArrayList<>();
        if (list.size() > 0) {
            List<BasePipelineTreeOutput> tempList = XCopyUtils.copyNewList(list, BasePipelineTreeOutput.class);
            for (BasePipelineTreeOutput tree : tempList) {
                if (ROOT_PARENT_ID.equals(tree.getParentPipelineId())) {
                    treeList.add(tree);
                }
                //子集
                for (BasePipelineTreeOutput item : tempList) {
                    if (tree.getId().equals(item.getParentPipelineId())) {
                        if (tree.getChildren() == null) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(item);
                    }
                }
                if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                    List<BasePipelineTreeOutput> children = tree.getChildren();
                    children.forEach(outputTree -> {
                        if (outputTree.getSort() == null) {
                            outputTree.setSort(999);
                        }
                    });
                    children = children.stream().sorted(Comparator.comparing(BasePipelineTreeOutput::getSort)).collect(Collectors.toList());
                    tree.setChildren(children);
                }
            }
        }
        return XListResult.success(treeList);
    }

    @XText("条件获取管线树")
    @XApiPost
    public XListResult<BasePipelineTreeOutput> getBasePipelineTreeByPipeline(XContext context, GetBasePipelineViewInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        BasePipelineViewMapper viewMapper = context.getBean(BasePipelineViewMapper.class);
        QueryWrapper<BasePipelineEnt> qr = new QueryWrapper<>();
        qr.lambda().gt(BasePipelineEnt::getEndTime, new Date());
        if (StringUtils.isNotBlank(input.getPipelineName())) {
            qr.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
        }
        if (StringUtils.isNotBlank(input.getDispatchDesk())) {
            qr.lambda().like(BasePipelineEnt::getDispatchDesk, input.getDispatchDesk());
        }
        //查询所有符合查询条件的数据
        List<BasePipelineEnt> list = mapper.selectList(qr);
        List<String> ids = list.stream().map(p -> {
            return p.getId();
        }).distinct().collect(Collectors.toList());
        if (ObjectUtils.isNotEmpty(ids)) {
            StringBuffer pipelineId = new StringBuffer();
            for (int i = 0; i < ids.size(); i++) {
                if (i == 0) {
                    pipelineId.append("'" + ids.get(i) + "'");
                } else {
                    pipelineId.append(",'" + ids.get(i) + "'");
                }
            }
            BasePipelineView record = new BasePipelineView();
            record.setId(pipelineId.toString());
            List<BasePipelineView> pipelineViewList = viewMapper.selectPipelineListById(record);
            List<String> allIds = pipelineViewList.stream().map(p -> {
                return p.getId();
            }).distinct().collect(Collectors.toList());
            QueryWrapper<BasePipelineEnt> qrPipeline = new QueryWrapper<>();
            qrPipeline.lambda().in(BasePipelineEnt::getId, allIds);
            if (StringUtils.isNotBlank(input.getIsHaveOuId()) && StringUtils.equals(input.getIsHaveOuId(), "1")) {
                List<String> allOuIdsByUser = getAllOuIdsByUser(context, context.getSession(PpsUserSession.class), input.getMenuId());
                qrPipeline.lambda().in(BasePipelineEnt::getEnterpriseId, allOuIdsByUser);
            }
            list = mapper.selectList(qrPipeline);
        }

        List<BasePipelineTreeOutput> treeList = new ArrayList<>();
        if (list.size() > 0) {
            List<BasePipelineTreeOutput> tempList = XCopyUtils.copyNewList(list, BasePipelineTreeOutput.class);
            for (BasePipelineTreeOutput tree : tempList) {
                if (ROOT_PARENT_ID.equals(tree.getParentPipelineId())) {
                    treeList.add(tree);
                }
                //子集
                for (BasePipelineTreeOutput item : tempList) {
                    if (tree.getId().equals(item.getParentPipelineId())) {
                        if (tree.getChildren() == null) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(item);
                    }
                }
                if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                    List<BasePipelineTreeOutput> children = tree.getChildren();
                    children.forEach(outputTree -> {
                        if (outputTree.getSort() == null) {
                            outputTree.setSort(999);
                        }
                    });
                    children = children.stream().sorted(Comparator.comparing(BasePipelineTreeOutput::getSort)).collect(Collectors.toList());
                    tree.setChildren(children);
                }
            }
        }
        return XListResult.success(treeList);
    }


    @XText("获取用户所属组织机构及授权的组织机构所管理的管线")
    @XApiPost
    public XListResult<BasePipelineTreeOutput> getAllAuthPipelineTree(XContext context, GetAllAuthPipelineInput input) {
        //获取用户所属组织机构及授权的组织机构
        PpsUserSession session = context.getSession(PpsUserSession.class);
        List<String> ouIdList = this.getAllOuIdsByUser(context, session, input.getMenuId());
        if (ouIdList.isEmpty()) return XListResult.success(new ArrayList<>());

        List<BasePipelineTreeOutput> resultList = new ArrayList<>();
        //获取管线信息
        BasePipelineViewMapper mapper = context.getBean(BasePipelineViewMapper.class);
        BasePipelineView record = new BasePipelineView();
        record.setOuIdList(ouIdList);
        List<BasePipelineView> list = mapper.selectByOuIds(record);
        List<BasePipelineTreeOutput> outList = XCopyUtils.copyNewList(list, BasePipelineTreeOutput.class);
        for (BasePipelineTreeOutput tree : outList) {
            //子集
            for (BasePipelineTreeOutput item : outList) {
                if (tree.getId().equals(item.getParentPipelineId())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(item);
                }
            }
            //父节点
            boolean isParent = true;
            for (BasePipelineTreeOutput item : outList) {
                if (tree.getParentPipelineId().equals(item.getId())) {
                    isParent = false;
                    break;
                }
            }
            if (isParent) {
                resultList.add(tree);
            }
        }
        return XListResult.success(resultList);
    }


    public List<String> getAllOuIdsByUser(XContext context, PpsUserSession session, String menuId) {
        List<String> ouIdList = new ArrayList<>();
        //获取用户所在的组织机构及其子级组织机构
        SysOrganizationCloudService service = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput record = new GetAllOuListByOuIdInput();
        record.setOuId(session.getOuId());
        XListResult<GetOuListTreeOutput> result = service.getAllOuListByOuId(context, record);
        result.throwIfFail();
        List<GetOuListTreeOutput> ouList = result.getResult();
        if (!CollectionUtils.isEmpty(ouList)) {
            ouList.forEach(item -> {
                ouIdList.add(item.getOuId());
            });
        }
        if (!Strings.isNullOrEmpty(menuId)) {
            //获取用户授权的组织机构
            Map<String, List<SysOrganization>> pageOrgMap = session.getPageOrgMap();
            if (pageOrgMap != null) {
                //获取当前用户在指定菜单下被授权的组织机构
                List<SysOrganization> authList = pageOrgMap.get(menuId);
                if (!CollectionUtils.isEmpty(authList)) {
                    authList.forEach(item -> {
                        ouIdList.add(item.getOuId());
                    });
                }
            }
        }
        return ouIdList;
    }

    public static List<LinkedHashMap<String, Object>> selectCommon(XContext context, String serviceName, String dbName, String sql) {
        TraceHelper.appendTraceLog(context, "BeginOfSelectCommon", sql + ", [SERVICENAME]:" + serviceName);
        CommonDataService mapper = NamedServiceProvider.getServiceByName(context, serviceName, CommonDataService.class);
        SelectCommonInput selectCommonInput = new SelectCommonInput();
        selectCommonInput.setCmd(sql);
        selectCommonInput.setTraceId(dbName);
        SelectCommonOutput result = mapper.selectCommon(context, selectCommonInput).getResult();
        if (result != null) {
//            TraceHelper.appendTraceLogWithSpan(context, "EndOfSelectCommon");
        } else {
            TraceHelper.appendTraceLogWithSpan(context, "EndOfSelectCommon", "ERROR,服务调用失败，请检查微服务是否运行正常。" + ", [SERVICENAME]:" + serviceName);
            TraceHelper.appendTraceLogTotalTime(context);
            throw new RuntimeException("执行SQL错误：" + sql);
        }
        return result.getResultList();


    }

    @XText("分页条件查询一级管线--根据用户的组织机构包含下级-未使用")
    @XApiPost
    public XPageResult<QueryBasePipelineOutput> queryBasePipelineOutputXPageResult(XContext context, QueryBasePipelineOuIdInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryBasePipelineOuIdInput inputQuery = new QueryBasePipelineOuIdInput();
        XCopyUtils.copyObject(input, inputQuery);
        inputQuery.setPage(1);
        inputQuery.setLimit(100000);
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        List<String> allOuIdsByUser = getAllOuIdsByUser(context, session, input.getMenuId());
        //生效时间
        queryWrapper.lambda().le(BasePipelineEnt::getEffectTime, new Date())
                .gt(BasePipelineEnt::getEndTime, new Date());
        //介质类型
        if (StringUtils.isNotBlank(input.getMediaType()) && !StringUtils.equals("ALL", input.getMediaType())) {
            queryWrapper.lambda().eq(BasePipelineEnt::getMediumType, input.getMediaType());
        }
        //allOuIdsByUser 当前用户所在组织机构及下级id
        if (allOuIdsByUser.size() > 0) {
            queryWrapper.lambda().in(BasePipelineEnt::getEnterpriseId, allOuIdsByUser);
        } else {
            return XPageResult.error(500, "该用户没有组织机构");
        }
        queryWrapper.lambda().orderByDesc(BasePipelineEnt::getSort, BasePipelineEnt::getCreateTime);
        XPageResult<QueryBasePipelineOutput> query = XMapperHelper.query(mapper, inputQuery, queryWrapper, QueryBasePipelineOutput.class);
        List<QueryBasePipelineOutput> items = query.getResult().getItems();
        List<String> ids = new ArrayList<>();

        if (StringUtils.isNotBlank(input.getParentId())) {
            ids = items.stream().map(p -> {
                return p.getId();
            }).distinct().collect(Collectors.toList());
        } else {
            ids = items.stream().map(p -> {
                return p.getParentPath() == null || p.getParentPath() == "" ? p.getId() : p.getParentPath().split("/")[1];
            }).distinct().collect(Collectors.toList());
        }
        QueryWrapper<BasePipelineEnt> search = new QueryWrapper();
        search.lambda().in(BasePipelineEnt::getId, ids);
        if (ids == null || ids.size() < 1) {
            return XPageResult.success(new ArrayList<>(), 1, 1, 0);
        }
        search.lambda().in(BasePipelineEnt::getEnterpriseId, allOuIdsByUser);
        search.lambda().orderByAsc(BasePipelineEnt::getSort);
        //查询一级管线
        return XMapperHelper.query(mapper, input, search, QueryBasePipelineOutput.class);
    }

    @XText("分页条件查询一级管线下级管线--根据用户的组织机构包含下级-未使用")
    @XApiPost
    public XPageResult<QueryBasePipelineOutput> queryBasePipelineOutputBelow(XContext context, QueryBasePipelineOuIdInput input) {
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        List<String> allOuIdsByUser = getAllOuIdsByUser(context, session, input.getMenuId());
        //根据上级管线id查询下级管线信息
        QueryWrapper<BasePipelineEnt> wrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(input.getId())) {
            wrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, input.getId());
        }
        //生效时间
        wrapper.lambda().le(BasePipelineEnt::getEffectTime, new Date())
                .gt(BasePipelineEnt::getEndTime, new Date());
        //介质类型
        if (StringUtils.isNotBlank(input.getMediaType()) && !StringUtils.equals("ALL", input.getMediaType())) {
            wrapper.lambda().eq(BasePipelineEnt::getMediumType, input.getMediaType());
        }
        wrapper.lambda().isNull(BasePipelineEnt::getEnterpriseId);
        //allOuIdsByUser 当前用户所在组织机构及下级id
        if (allOuIdsByUser.size() > 0) {
            wrapper.lambda().isNull(BasePipelineEnt::getEnterpriseId);
            wrapper.lambda().in(BasePipelineEnt::getEnterpriseId, allOuIdsByUser);
        } else {
            return XPageResult.error(500, "该用户没有组织机构");
        }
        wrapper.lambda().orderByDesc(BasePipelineEnt::getSort, BasePipelineEnt::getCreateTime);
        return XMapperHelper.query(mapper, input, wrapper, QueryBasePipelineOutput.class);
    }
}
