package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BasePipelineStationRelEnt;
import pps.core.base.entity.BasePipelineStationRelView;
import pps.core.base.mapper.BasePipelineStationRelMapper;
import pps.core.base.mapper.BasePipelineStationRelViewMapper;
import pps.core.base.service.data.base_pipeline_station_rel.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XText("管线场站关系接口")
@XService
public class BasePipelineStationRelService {

    @XText("创建管线场站关系")
    @XApiPost
    public XServiceResult createBasePipelineStationRel(XContext context, CreateBasePipelineStationRelInput input){
        BasePipelineStationRelMapper mapper = context.getBean(BasePipelineStationRelMapper.class);
        BasePipelineStationRelEnt entity = new BasePipelineStationRelEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改管线场站关系")
    @XApiPost
    public XServiceResult updateBasePipelineStationRel(XContext context, UpdateBasePipelineStationRelInput input){
        BasePipelineStationRelMapper mapper = context.getBean(BasePipelineStationRelMapper.class);
        QueryWrapper<BasePipelineStationRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineStationRelEnt::getId, input.getId());
        BasePipelineStationRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除管线场站关系")
    @XApiPost
    public XServiceResult deleteBasePipelineStationRel(XContext context, DeleteBasePipelineStationRelInput input){
        BasePipelineStationRelMapper mapper = context.getBean(BasePipelineStationRelMapper.class);
        QueryWrapper<BasePipelineStationRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineStationRelEnt::getId, input.getId());
        BasePipelineStationRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id查询管线场站关系")
    @XApiPost
    public XSingleResult<GetBasePipelineStationRelOutput> getBasePipelineStationRel(XContext context, GetBasePipelineStationRelInput input){
        BasePipelineStationRelMapper mapper = context.getBean(BasePipelineStationRelMapper.class);
        QueryWrapper<BasePipelineStationRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePipelineStationRelEnt::getId, input.getId());
        BasePipelineStationRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePipelineStationRelOutput output = new GetBasePipelineStationRelOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询管线场站关系")
    @XApiPost
    public XPageResult<QueryBasePipelineStationRelOutput> queryBasePipelineStationRel(XContext context, QueryBasePipelineStationRelInput input){
        BasePipelineStationRelMapper mapper = context.getBean(BasePipelineStationRelMapper.class);
        QueryWrapper<BasePipelineStationRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BasePipelineStationRelEnt::getId, input.getId())
            .like(BasePipelineStationRelEnt::getPipelineId, input.getPipelineId())
            .like(BasePipelineStationRelEnt::getStationId, input.getStationId())
            .eq(BasePipelineStationRelEnt::getEffectTime, input.getEffectTime())
            .eq(BasePipelineStationRelEnt::getEndTime, input.getEndTime())
            .like(BasePipelineStationRelEnt::getCreateById, input.getCreateById())
            .like(BasePipelineStationRelEnt::getCreateByName, input.getCreateByName())
            .eq(BasePipelineStationRelEnt::getCreateTime, input.getCreateTime())
            .like(BasePipelineStationRelEnt::getModifyById, input.getModifyById())
            .like(BasePipelineStationRelEnt::getModifyByName, input.getModifyByName())
            .eq(BasePipelineStationRelEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePipelineStationRelOutput.class);
    }

    @XApiPost
    public XPageResult<QueryBasePipelineStationRelOutput> dynamicQueryBasePipelineStationRel(XContext context, CustomQueryInput input){
        BasePipelineStationRelMapper mapper = context.getBean(BasePipelineStationRelMapper.class);
        return XMapperHelper.query(mapper, input, BasePipelineStationRelEnt.class, QueryBasePipelineStationRelOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBasePipelineStationRelViewOutput> getBasePipelineStationRelView(XContext context, GetBasePipelineStationRelViewInput input){
        BasePipelineStationRelViewMapper mapper = context.getBean(BasePipelineStationRelViewMapper.class);
        BasePipelineStationRelView record = new BasePipelineStationRelView();
        XCopyUtils.copyObject(input, record);
        BasePipelineStationRelView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePipelineStationRelViewOutput output = new GetBasePipelineStationRelViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBasePipelineStationRelViewOutput> queryBasePipelineStationRelView(XContext context, QueryBasePipelineStationRelViewInput input){
        BasePipelineStationRelViewMapper mapper = context.getBean(BasePipelineStationRelViewMapper.class);
        BasePipelineStationRelView record = new BasePipelineStationRelView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePipelineStationRelView> list = mapper.selectList(record);
        PageInfo<BasePipelineStationRelView> pageInfo = new PageInfo<>(list);
        List<QueryBasePipelineStationRelViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePipelineStationRelViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiGet
    @XText("获取场站与管线关系")
    public XListResult<GetBasePipelineStationRelOutput> getBaseStationOuRelList(XContext context, GetBasePipelineStationRelInput input){
        BasePipelineStationRelViewMapper mapper = context.getBean(BasePipelineStationRelViewMapper.class);
        BasePipelineStationRelView record = new BasePipelineStationRelView();
        XCopyUtils.copyObject(input, record);
        List<BasePipelineStationRelView> list = mapper.selectList(record);
        List<GetBasePipelineStationRelOutput> outputs = XCopyUtils.copyNewList(list, GetBasePipelineStationRelOutput.class);
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据管线id获取参与管存计算的场站信息")
    public XListResult<GetBasePipelineStationRelOutput> getStationByPipelineId(XContext context, PipelineIdInput input){
        BasePipelineStationRelViewMapper mapper = context.getBean(BasePipelineStationRelViewMapper.class);
        List<BasePipelineStationRelView> list = mapper.selectCustodyUseStation(input.getPipelineId(),null);
        List<GetBasePipelineStationRelOutput> outputs = XCopyUtils.copyNewList(list, GetBasePipelineStationRelOutput.class);
        return XListResult.success(outputs);
    }
}
