package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.cloud.system.service.data.OuIdAndOutNameOutput;
import pps.cloud.system.service.data.QueryUserIdInput;
import pps.core.base.entity.BasePipelineEnt;
import pps.core.base.entity.BasePipelineStationRelEnt;
import pps.core.base.entity.BasePipelineView;
import pps.core.base.entity.BaseStationOuRelEnt;
import pps.core.base.mapper.BasePipelineMapper;
import pps.core.base.mapper.BasePipelineStationRelMapper;
import pps.core.base.mapper.BasePipelineViewMapper;
import pps.core.base.mapper.BaseStationOuRelMapper;
import pps.core.base.service.data.base_pipeline.*;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysOrganizationRelEnt;
import pps.core.system.mapper.SysOrganizationRelMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;

import java.util.*;
import java.util.stream.Collectors;

import static java.util.Collections.*;

@XText("根据用户单位级别查询管线")
@XService
public class BasePipelineUserService {

    private final String ROOT_PARENT_ID = "11111111-1111-1111-1111-111111111111";

    @XText("根据用户单位级别获取一级管线")
    @XApiPost
    public XPageResult<GetBasePipelineUserViewOutput> getUserOuLevelPipeline(XContext context, GetBasePipelineUserViewInput input) {
        //查询管线
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        //组织机构关系表
        SysOrganizationRelMapper mapper1 = context.getBean(SysOrganizationRelMapper.class);
        //场站组织机构关系
        BaseStationOuRelMapper mapper2 = context.getBean(BaseStationOuRelMapper.class);
        //管线与场站关系表
        BasePipelineStationRelMapper mapper3 = context.getBean(BasePipelineStationRelMapper.class);
        SysOrganizationCloudService bean = context.getBean(SysOrganizationCloudService.class);
        QueryUserIdInput queryUserIdInput = new QueryUserIdInput();
        queryUserIdInput.setUserId(session.getId());
        XListResult<OuIdAndOutNameOutput> orgListByUserId = bean.getOrgListByUserId(context, queryUserIdInput);
        OuIdAndOutNameOutput ouIdAndOutNameOutput = orgListByUserId.getResult().get(0);
        GetSysOrganizationViewInput getSysOrganizationViewInput = new GetSysOrganizationViewInput();
        getSysOrganizationViewInput.setId(ouIdAndOutNameOutput.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> sysOrganizationById = bean.getSysOrganizationById(context, getSysOrganizationViewInput);
        //用户所在的单位级别
        String ouLevel = sysOrganizationById.getResult().getOuLevel();
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        QueryWrapper<SysOrganizationRelEnt> queryWrapper1 = new QueryWrapper<>();
        QueryWrapper<BaseStationOuRelEnt> queryWrapper2 = new QueryWrapper<>();
        QueryWrapper<BasePipelineStationRelEnt> queryWrapper3 = new QueryWrapper<>();
        //根据当前用户获取单位级别
        //单位级别为 所属企业（地区公司）
        if (ouLevel.equals("REGION_FIRM")) {
            queryWrapper.lambda().eq(BasePipelineEnt::getEnterpriseId, ouIdAndOutNameOutput.getOuId());
            queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, ROOT_PARENT_ID);
            queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
        }
        //单位级别为 二级分公司/管理处
        if (ouLevel.equals("ADMINISTRATIVE")) {
            String ouId = ouIdAndOutNameOutput.getOuId();
            queryWrapper1.lambda().eq(SysOrganizationRelEnt::getOuId, ouId);
            List<SysOrganizationRelEnt> sysOrganizationRelEnts = mapper1.selectList(queryWrapper1);
            queryWrapper.clear();
            for (SysOrganizationRelEnt sysOrganizationRelEnt : sysOrganizationRelEnts) {
                queryWrapper.lambda().eq(BasePipelineEnt::getEnterpriseId, sysOrganizationRelEnt.getParentOuId());
                queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, ROOT_PARENT_ID);
                queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
            }
        }
        //单位级别为 场站
        if (ouLevel.equals("STATION")) {
            String ouId = ouIdAndOutNameOutput.getOuId();
            queryWrapper2.lambda().eq(BaseStationOuRelEnt::getOuId, ouId);
            queryWrapper2.lambda().gt(BaseStationOuRelEnt::getEndTime, new Date());
            List<BaseStationOuRelEnt> baseStationOuRelEnts = mapper2.selectList(queryWrapper2);
            List<String> arrayList = new ArrayList<>();
            //根据组织机构获取所有场站id
            for (BaseStationOuRelEnt baseStationOuRelEnt : baseStationOuRelEnts) {
                arrayList.add(baseStationOuRelEnt.getStationId());
            }
            queryWrapper3.lambda().in(BasePipelineStationRelEnt::getStationId, arrayList);
            queryWrapper3.lambda().gt(BasePipelineStationRelEnt::getEndTime, new Date());
            List<BasePipelineStationRelEnt> basePipelineStationRelEntList = mapper3.selectList(queryWrapper3);
            List<String> arrayList1 = new ArrayList<>();
            for (BasePipelineStationRelEnt basePipelineStationRelEnt : basePipelineStationRelEntList) {
                arrayList1.add(basePipelineStationRelEnt.getPipelineId());
            }
            queryWrapper.clear();
            queryWrapper.lambda().in(BasePipelineEnt::getId, arrayList1);
            //queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, ROOT_PARENT_ID);
            queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
        }
        List<BasePipelineEnt> basePipelineEnts = mapper.selectList(queryWrapper);
        List<GetBasePipelineUserViewOutput> treeList = new ArrayList<>();
        XCopyUtils.copyList(basePipelineEnts, treeList, GetBasePipelineUserViewOutput.class);
        int size = treeList.size();
        treeList.sort(Comparator.comparing(GetBasePipelineUserViewOutput::getId).reversed());
        return XPageResult.success(treeList, 1, 999, size);
    }

    @XText("根据id获取下级管线")
    @XApiPost
    public XPageResult<GetBasePipelineUserViewOutput> getBasePipelineById(XContext context, GetBasePipelineUserViewInput input) {
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        //组织机构关系表
        SysOrganizationRelMapper mapper1 = context.getBean(SysOrganizationRelMapper.class);
        //场站组织机构关系
        BaseStationOuRelMapper mapper2 = context.getBean(BaseStationOuRelMapper.class);
        //管线与场站关系表
        BasePipelineStationRelMapper mapper3 = context.getBean(BasePipelineStationRelMapper.class);
        SysOrganizationCloudService bean = context.getBean(SysOrganizationCloudService.class);
        QueryUserIdInput queryUserIdInput = new QueryUserIdInput();
        queryUserIdInput.setUserId(session.getId());
        XListResult<OuIdAndOutNameOutput> orgListByUserId = bean.getOrgListByUserId(context, queryUserIdInput);
        OuIdAndOutNameOutput ouIdAndOutNameOutput = orgListByUserId.getResult().get(0);
        GetSysOrganizationViewInput getSysOrganizationViewInput = new GetSysOrganizationViewInput();
        getSysOrganizationViewInput.setId(ouIdAndOutNameOutput.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> sysOrganizationById = bean.getSysOrganizationById(context, getSysOrganizationViewInput);
        //用户所在的单位级别
        String ouLevel = sysOrganizationById.getResult().getOuLevel();
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        QueryWrapper<SysOrganizationRelEnt> queryWrapper1 = new QueryWrapper<>();
        QueryWrapper<BaseStationOuRelEnt> queryWrapper2 = new QueryWrapper<>();
        QueryWrapper<BasePipelineStationRelEnt> queryWrapper3 = new QueryWrapper<>();
        //根据当前用户获取单位级别
        //单位级别为 所属企业（地区公司）
        if (ouLevel.equals("REGION_FIRM")) {
            queryWrapper.lambda().eq(BasePipelineEnt::getEnterpriseId, ouIdAndOutNameOutput.getOuId());
            queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
            queryWrapper.lambda().orderByAsc(BasePipelineEnt::getId);
        }
        //单位级别为 二级分公司/管理处
        if (ouLevel.equals("ADMINISTRATIVE")) {
            String ouId = ouIdAndOutNameOutput.getOuId();
            queryWrapper1.lambda().eq(SysOrganizationRelEnt::getOuId, ouId);
            List<SysOrganizationRelEnt> sysOrganizationRelEnts = mapper1.selectList(queryWrapper1);
            queryWrapper.clear();
            for (SysOrganizationRelEnt sysOrganizationRelEnt : sysOrganizationRelEnts) {
                queryWrapper.lambda().eq(BasePipelineEnt::getEnterpriseId, sysOrganizationRelEnt.getParentOuId());
                queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
                queryWrapper.lambda().orderByAsc(BasePipelineEnt::getId);
            }
        }
        //单位级别为 场站
        if (ouLevel.equals("STATION")) {
            String ouId = ouIdAndOutNameOutput.getOuId();
            queryWrapper2.lambda().eq(BaseStationOuRelEnt::getOuId, ouId);
            queryWrapper2.lambda().gt(BaseStationOuRelEnt::getEndTime, new Date());
            List<BaseStationOuRelEnt> baseStationOuRelEnts = mapper2.selectList(queryWrapper2);
            List<String> arrayList = new ArrayList<>();
            //根据组织机构获取所有场站id
            for (BaseStationOuRelEnt baseStationOuRelEnt : baseStationOuRelEnts) {
                arrayList.add(baseStationOuRelEnt.getStationId());
            }
            queryWrapper3.lambda().in(BasePipelineStationRelEnt::getStationId, arrayList);
            queryWrapper3.lambda().gt(BasePipelineStationRelEnt::getEndTime, new Date());
            List<BasePipelineStationRelEnt> basePipelineStationRelEntList = mapper3.selectList(queryWrapper3);
            List<String> arrayList1 = new ArrayList<>();
            for (BasePipelineStationRelEnt basePipelineStationRelEnt : basePipelineStationRelEntList) {
                arrayList1.add(basePipelineStationRelEnt.getPipelineId());
            }
            queryWrapper.clear();
            queryWrapper.lambda().in(BasePipelineEnt::getId, arrayList1);
            queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
            queryWrapper.lambda().orderByAsc(BasePipelineEnt::getId);
        }
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        queryWrapper.lambda().eq(BasePipelineEnt::getParentPipelineId, input.getId());
        queryWrapper.lambda().orderByAsc(BasePipelineEnt::getId);
        List<BasePipelineEnt> basePipelineEnts = mapper.selectList(queryWrapper);
        if (basePipelineEnts == null) {
            return XPageResult.error(context, XError.NotFound);
        }
        ArrayList<GetBasePipelineUserViewOutput> arrayList = new ArrayList<>();
        XCopyUtils.copyList(basePipelineEnts, arrayList, GetBasePipelineUserViewOutput.class);
        int size = arrayList.size();
        arrayList.sort(Comparator.comparing(GetBasePipelineUserViewOutput::getId).reversed());
        return XPageResult.success(arrayList, 1, 999, size);
    }

    @XText("模糊查询条件获取管线树--根据用户单位级别")
    @XApiPost
    public XListResult<BasePipelineTreeUserOutput> getUserOuLevelBasePipelineTree(XContext context, GetBasePipelineUserViewInput input) {
        //查询管线
        BasePipelineMapper mapper = context.getBean(BasePipelineMapper.class);
        BasePipelineViewMapper viewMapper = context.getBean(BasePipelineViewMapper.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        //组织机构关系表
        SysOrganizationRelMapper mapper1 = context.getBean(SysOrganizationRelMapper.class);
        //场站组织机构关系
        BaseStationOuRelMapper mapper2 = context.getBean(BaseStationOuRelMapper.class);
        //管线与场站关系表
        BasePipelineStationRelMapper mapper3 = context.getBean(BasePipelineStationRelMapper.class);
        SysOrganizationCloudService bean = context.getBean(SysOrganizationCloudService.class);
        QueryUserIdInput queryUserIdInput = new QueryUserIdInput();
        queryUserIdInput.setUserId(session.getId());
        XListResult<OuIdAndOutNameOutput> orgListByUserId = bean.getOrgListByUserId(context, queryUserIdInput);
        OuIdAndOutNameOutput ouIdAndOutNameOutput = orgListByUserId.getResult().get(0);
        GetSysOrganizationViewInput getSysOrganizationViewInput = new GetSysOrganizationViewInput();
        getSysOrganizationViewInput.setId(ouIdAndOutNameOutput.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> sysOrganizationById = bean.getSysOrganizationById(context, getSysOrganizationViewInput);
        //用户所在的单位级别
        String ouLevel = sysOrganizationById.getResult().getOuLevel();
        QueryWrapper<BasePipelineEnt> queryWrapper = new QueryWrapper<>();
        QueryWrapper<SysOrganizationRelEnt> queryWrapper1 = new QueryWrapper<>();
        QueryWrapper<BaseStationOuRelEnt> queryWrapper2 = new QueryWrapper<>();
        QueryWrapper<BasePipelineStationRelEnt> queryWrapper3 = new QueryWrapper<>();
        //根据当前用户获取单位级别
        //单位级别为 所属企业（地区公司）
        if (ouLevel.equals("REGION_FIRM")) {
            queryWrapper.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
            queryWrapper.lambda().eq(BasePipelineEnt::getEnterpriseId, ouIdAndOutNameOutput.getOuId());
            queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
            queryWrapper.lambda().orderByAsc(BasePipelineEnt::getId);
        }
        //单位级别为 二级分公司/管理处
        if (ouLevel.equals("ADMINISTRATIVE")) {
            String ouId = ouIdAndOutNameOutput.getOuId();
            queryWrapper1.lambda().eq(SysOrganizationRelEnt::getOuId, ouId);
            List<SysOrganizationRelEnt> sysOrganizationRelEnts = mapper1.selectList(queryWrapper1);
            queryWrapper.clear();
            for (SysOrganizationRelEnt sysOrganizationRelEnt : sysOrganizationRelEnts) {
                queryWrapper.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
                queryWrapper.lambda().eq(BasePipelineEnt::getEnterpriseId, sysOrganizationRelEnt.getParentOuId());
                queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
            }
        }
        //单位级别为 场站
        if (ouLevel.equals("STATION")) {
            String ouId = ouIdAndOutNameOutput.getOuId();
            queryWrapper2.lambda().eq(BaseStationOuRelEnt::getOuId, ouId);
            queryWrapper2.lambda().gt(BaseStationOuRelEnt::getEndTime, new Date());
            List<BaseStationOuRelEnt> baseStationOuRelEnts = mapper2.selectList(queryWrapper2);
            List<String> arrayList = new ArrayList<>();
            //根据组织机构获取所有场站id
            for (BaseStationOuRelEnt baseStationOuRelEnt : baseStationOuRelEnts) {
                arrayList.add(baseStationOuRelEnt.getStationId());
            }
            queryWrapper3.lambda().in(BasePipelineStationRelEnt::getStationId, arrayList);
            queryWrapper3.lambda().gt(BasePipelineStationRelEnt::getEndTime, new Date());
            List<BasePipelineStationRelEnt> basePipelineStationRelEntList = mapper3.selectList(queryWrapper3);
            List<String> arrayList1 = new ArrayList<>();
            for (BasePipelineStationRelEnt basePipelineStationRelEnt : basePipelineStationRelEntList) {
                arrayList1.add(basePipelineStationRelEnt.getPipelineId());
            }
            queryWrapper.clear();
            queryWrapper.lambda().like(BasePipelineEnt::getPipelineName, input.getPipelineName());
            queryWrapper.lambda().in(BasePipelineEnt::getId, arrayList1);
            queryWrapper.lambda().gt(BasePipelineEnt::getEndTime, new Date());
            queryWrapper.lambda().orderByAsc(BasePipelineEnt::getId);
        }
        List<BasePipelineEnt> list = mapper.selectList(queryWrapper);
        queryWrapper.lambda().isNotNull(BasePipelineEnt::getParentPath);
        List<BasePipelineEnt> nullDispatchDeskList = mapper.selectList(queryWrapper);
        List<String> parentPathList = nullDispatchDeskList.stream().filter(item -> XStringUtils.isNotEmpty(item.getParentPath())).map(BasePipelineEnt::getParentPath).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(parentPathList)) {
            List<String> parentPathIds = new ArrayList<>();
            parentPathList.stream().forEach(item -> {
                item = item.substring(1);
                String[] ids = item.split("/");
                parentPathIds.addAll(Arrays.asList(ids));
            });
            queryWrapper.clear();
            queryWrapper.lambda().in(BasePipelineEnt::getId, parentPathIds);
            List<BasePipelineEnt> dispatchDeskPrentList = mapper.selectList(queryWrapper);
            for (BasePipelineEnt item : nullDispatchDeskList) {
                for (BasePipelineEnt parent : dispatchDeskPrentList) {
                    if (item.getParentPath().contains(parent.getId()))
                        list.add(item);
                }
            }
        }
        List<String> ids = list.stream().map(p -> {
            return p.getId();
        }).distinct().collect(Collectors.toList());
        if (ObjectUtils.isNotEmpty(ids)) {
            StringBuffer pipelineId = new StringBuffer();
            for (int i = 0; i < ids.size(); i++) {
                if (i == 0) {
                    pipelineId.append("'" + ids.get(i) + "'");
                } else {
                    pipelineId.append(",'" + ids.get(i) + "'");
                }
            }
            BasePipelineView record = new BasePipelineView();
            record.setId(pipelineId.toString());
            List<BasePipelineView> pipelineViewList = viewMapper.selectPipelineListById(record);
            List<String> allIds = pipelineViewList.stream().map(p -> {
                return p.getId();
            }).distinct().collect(Collectors.toList());
            QueryWrapper<BasePipelineEnt> qrPipeline = new QueryWrapper<>();
            qrPipeline.lambda().in(BasePipelineEnt::getId, allIds);
            list = mapper.selectList(qrPipeline);
        }
        List<BasePipelineTreeUserOutput> treeList = new ArrayList<>();
        if (list.size() > 0) {
            List<BasePipelineTreeUserOutput> tempList = XCopyUtils.copyNewList(list, BasePipelineTreeUserOutput.class);
            for (BasePipelineTreeUserOutput tree : tempList) {
                if (ROOT_PARENT_ID.equals(tree.getParentPipelineId())) {
                    treeList.add(tree);
                }
                //子集
                for (BasePipelineTreeUserOutput item : tempList) {
                    if (tree.getId().equals(item.getParentPipelineId())) {
                        if (tree.getChildren() == null) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(item);
                    }
                }
                if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                    List<BasePipelineTreeUserOutput> children = tree.getChildren();
                    children.forEach(outputTree -> {
                        if (outputTree.getSort() == null) {
                            outputTree.setSort(999);
                        }
                    });
                    children = children.stream().sorted(Comparator.comparing(BasePipelineTreeUserOutput::getSort)).collect(Collectors.toList());
                    tree.setChildren(children);
                }
            }
        }
        treeList.sort(Comparator.comparing(BasePipelineTreeUserOutput::getId).reversed());
        return XListResult.success(treeList);
    }


}
