package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BaseSegmentDirectionEnt;
import pps.core.base.entity.BaseSegmentDirectionView;
import pps.core.base.mapper.BaseSegmentDirectionMapper;
import pps.core.base.mapper.BaseSegmentDirectionViewMapper;
import pps.core.base.service.data.base_segment_direction.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XText("管段传输方向接口")
@XService
public class BaseSegmentDirectionService {

    @XText("创建管段传输方向")
    @XApiPost
    public XServiceResult createBaseSegmentDirection(XContext context, CreateBaseSegmentDirectionInput input){
        BaseSegmentDirectionMapper mapper = context.getBean(BaseSegmentDirectionMapper.class);
        BaseSegmentDirectionEnt entity = new BaseSegmentDirectionEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改管段传输方向")
    @XApiPost
    public XServiceResult updateBaseSegmentDirection(XContext context, UpdateBaseSegmentDirectionInput input){
        BaseSegmentDirectionMapper mapper = context.getBean(BaseSegmentDirectionMapper.class);
        QueryWrapper<BaseSegmentDirectionEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSegmentDirectionEnt::getId, input.getId());
        BaseSegmentDirectionEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除管段传输方向")
    @XApiPost
    public XServiceResult deleteBaseSegmentDirection(XContext context, DeleteBaseSegmentDirectionInput input){
        BaseSegmentDirectionMapper mapper = context.getBean(BaseSegmentDirectionMapper.class);
        QueryWrapper<BaseSegmentDirectionEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSegmentDirectionEnt::getId, input.getId());
        BaseSegmentDirectionEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id查询管段传输方向")
    @XApiPost
    public XSingleResult<GetBaseSegmentDirectionOutput> getBaseSegmentDirection(XContext context, GetBaseSegmentDirectionInput input){
        BaseSegmentDirectionMapper mapper = context.getBean(BaseSegmentDirectionMapper.class);
        QueryWrapper<BaseSegmentDirectionEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSegmentDirectionEnt::getId, input.getId());
        BaseSegmentDirectionEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSegmentDirectionOutput output = new GetBaseSegmentDirectionOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询管段传输方向")
    @XApiPost
    public XPageResult<QueryBaseSegmentDirectionOutput> queryBaseSegmentDirection(XContext context, QueryBaseSegmentDirectionInput input){
        BaseSegmentDirectionMapper mapper = context.getBean(BaseSegmentDirectionMapper.class);
        QueryWrapper<BaseSegmentDirectionEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseSegmentDirectionEnt::getId, input.getId())
            .like(BaseSegmentDirectionEnt::getSegmentId, input.getSegmentId())
            .eq(BaseSegmentDirectionEnt::getBeReverse, input.getBeReverse())
            .eq(BaseSegmentDirectionEnt::getEffectTime, input.getEffectTime())
            .eq(BaseSegmentDirectionEnt::getEndTime, input.getEndTime())
            .like(BaseSegmentDirectionEnt::getCreateById, input.getCreateById())
            .like(BaseSegmentDirectionEnt::getCreateByName, input.getCreateByName())
            .eq(BaseSegmentDirectionEnt::getCreateTime, input.getCreateTime())
            .like(BaseSegmentDirectionEnt::getModifyById, input.getModifyById())
            .like(BaseSegmentDirectionEnt::getModifyByName, input.getModifyByName())
            .eq(BaseSegmentDirectionEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseSegmentDirectionOutput.class);
    }

    @XApiPost
    public XPageResult<QueryBaseSegmentDirectionOutput> dynamicQueryBaseSegmentDirection(XContext context, CustomQueryInput input){
        BaseSegmentDirectionMapper mapper = context.getBean(BaseSegmentDirectionMapper.class);
        return XMapperHelper.query(mapper, input, BaseSegmentDirectionEnt.class, QueryBaseSegmentDirectionOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBaseSegmentDirectionViewOutput> getBaseSegmentDirectionView(XContext context, GetBaseSegmentDirectionViewInput input){
        BaseSegmentDirectionViewMapper mapper = context.getBean(BaseSegmentDirectionViewMapper.class);
        BaseSegmentDirectionView record = new BaseSegmentDirectionView();
        XCopyUtils.copyObject(input, record);
        BaseSegmentDirectionView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSegmentDirectionViewOutput output = new GetBaseSegmentDirectionViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBaseSegmentDirectionViewOutput> queryBaseSegmentDirectionView(XContext context, QueryBaseSegmentDirectionViewInput input){
        BaseSegmentDirectionViewMapper mapper = context.getBean(BaseSegmentDirectionViewMapper.class);
        BaseSegmentDirectionView record = new BaseSegmentDirectionView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseSegmentDirectionView> list = mapper.selectList(record);
        PageInfo<BaseSegmentDirectionView> pageInfo = new PageInfo<>(list);
        List<QueryBaseSegmentDirectionViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseSegmentDirectionViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
