package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BaseSegmentPipelineTransferRelEnt;
import pps.core.base.entity.BaseSegmentPipelineTransferRelView;
import pps.core.base.mapper.BaseSegmentPipelineTransferRelMapper;
import pps.core.base.mapper.BaseSegmentPipelineTransferRelViewMapper;
import pps.core.base.service.data.base_segment_pipeline_transfer_rel.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XText("管段管线转供关系接口")
@XService
public class BaseSegmentPipelineTransferRelService {

    @XText("创建管段场站转供关系")
    @XApiPost
    public XServiceResult createBaseSegmentPipelineTransferRel(XContext context, CreateBaseSegmentPipelineTransferRelInput input){
        BaseSegmentPipelineTransferRelMapper mapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
        BaseSegmentPipelineTransferRelEnt entity = new BaseSegmentPipelineTransferRelEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改管段场站转供关系")
    @XApiPost
    public XServiceResult updateBaseSegmentPipelineTransferRel(XContext context, UpdateBaseSegmentPipelineTransferRelInput input){
        BaseSegmentPipelineTransferRelMapper mapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
        QueryWrapper<BaseSegmentPipelineTransferRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSegmentPipelineTransferRelEnt::getId, input.getId());
        BaseSegmentPipelineTransferRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除管段场站转供关系")
    @XApiPost
    public XServiceResult deleteBaseSegmentPipelineTransferRel(XContext context, DeleteBaseSegmentPipelineTransferRelInput input){
        BaseSegmentPipelineTransferRelMapper mapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
        QueryWrapper<BaseSegmentPipelineTransferRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSegmentPipelineTransferRelEnt::getId, input.getId());
        BaseSegmentPipelineTransferRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id查询管段场站转供关系")
    @XApiPost
    public XSingleResult<GetBaseSegmentPipelineTransferRelOutput> getBaseSegmentPipelineTransferRel(XContext context, GetBaseSegmentPipelineTransferRelInput input){
        BaseSegmentPipelineTransferRelMapper mapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
        QueryWrapper<BaseSegmentPipelineTransferRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSegmentPipelineTransferRelEnt::getId, input.getId());
        BaseSegmentPipelineTransferRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSegmentPipelineTransferRelOutput output = new GetBaseSegmentPipelineTransferRelOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询管段场站转供关系")
    @XApiPost
    public XPageResult<QueryBaseSegmentPipelineTransferRelOutput> queryBaseSegmentPipelineTransferRel(XContext context, QueryBaseSegmentPipelineTransferRelInput input){
        BaseSegmentPipelineTransferRelMapper mapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
        QueryWrapper<BaseSegmentPipelineTransferRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseSegmentPipelineTransferRelEnt::getId, input.getId())
            .like(BaseSegmentPipelineTransferRelEnt::getSegmentId, input.getSegmentId())
            .like(BaseSegmentPipelineTransferRelEnt::getStartStationId, input.getStartStationId())
            .like(BaseSegmentPipelineTransferRelEnt::getEndStationId, input.getEndStationId())
            .like(BaseSegmentPipelineTransferRelEnt::getStartPipelineId, input.getStartPipelineId())
            .like(BaseSegmentPipelineTransferRelEnt::getEndPipelineId, input.getEndPipelineId())
            .eq(BaseSegmentPipelineTransferRelEnt::getEffectTime, input.getEffectTime())
            .eq(BaseSegmentPipelineTransferRelEnt::getEndTime, input.getEndTime())
            .like(BaseSegmentPipelineTransferRelEnt::getCreateById, input.getCreateById())
            .like(BaseSegmentPipelineTransferRelEnt::getCreateByName, input.getCreateByName())
            .eq(BaseSegmentPipelineTransferRelEnt::getCreateTime, input.getCreateTime())
            .like(BaseSegmentPipelineTransferRelEnt::getModifyById, input.getModifyById())
            .like(BaseSegmentPipelineTransferRelEnt::getModifyByName, input.getModifyByName())
            .eq(BaseSegmentPipelineTransferRelEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseSegmentPipelineTransferRelOutput.class);
    }

    @XApiPost
    public XPageResult<QueryBaseSegmentPipelineTransferRelOutput> dynamicQueryBaseSegmentPipelineTransferRel(XContext context, CustomQueryInput input){
        BaseSegmentPipelineTransferRelMapper mapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
        return XMapperHelper.query(mapper, input, BaseSegmentPipelineTransferRelEnt.class, QueryBaseSegmentPipelineTransferRelOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBaseSegmentPipelineTransferRelViewOutput> getBaseSegmentPipelineTransferRelView(XContext context, GetBaseSegmentPipelineTransferRelViewInput input){
        BaseSegmentPipelineTransferRelViewMapper mapper = context.getBean(BaseSegmentPipelineTransferRelViewMapper.class);
        BaseSegmentPipelineTransferRelView record = new BaseSegmentPipelineTransferRelView();
        XCopyUtils.copyObject(input, record);
        BaseSegmentPipelineTransferRelView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSegmentPipelineTransferRelViewOutput output = new GetBaseSegmentPipelineTransferRelViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBaseSegmentPipelineTransferRelViewOutput> queryBaseSegmentPipelineTransferRelView(XContext context, QueryBaseSegmentPipelineTransferRelViewInput input){
        BaseSegmentPipelineTransferRelViewMapper mapper = context.getBean(BaseSegmentPipelineTransferRelViewMapper.class);
        BaseSegmentPipelineTransferRelView record = new BaseSegmentPipelineTransferRelView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseSegmentPipelineTransferRelView> list = mapper.selectList(record);
        PageInfo<BaseSegmentPipelineTransferRelView> pageInfo = new PageInfo<>(list);
        List<QueryBaseSegmentPipelineTransferRelViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseSegmentPipelineTransferRelViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
