package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_segment.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@XText("管段接口")
@XService
public class BaseSegmentService {

    @XText("创建管段")
    @XApiPost
    public XServiceResult createBaseSegment(XContext context, CreateBaseSegmentInput input){
        //创建管段：1、创建管段基础信息。2、创建管段反输记录。3、创建管段转供的所有关系（根据当前管段的结束场站作为转供管段的起始场站，查询所有管段添加转供关系）
        //4、创建管段的起始场站和结束场站与管线的关联关系5、创建管线与管段的关联关系
        return XTransactionHelper.begin(context,()-> {
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            BaseSegmentMapper mapper = context.getBean(BaseSegmentMapper.class);
            BaseSegmentDirectionMapper segmentDirectionmapper = context.getBean(BaseSegmentDirectionMapper.class);
            BasePipelineSegmentRelMapper basePipelineSegmentRel = context.getBean(BasePipelineSegmentRelMapper.class);
            BasePipelineMapper basePipeline = context.getBean(BasePipelineMapper.class);
            BasePipelineSegmentRelViewMapper basePipelineSegmentRelView = context.getBean(BasePipelineSegmentRelViewMapper.class);
            BaseSegmentPipelineTransferRelMapper relMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            BaseSegmentViewMapper viewMapper = context.getBean(BaseSegmentViewMapper.class);
            BasePipelineStationRelMapper basePipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            //创建管段
            BaseSegmentEnt entity = new BaseSegmentEnt();
            XCopyUtils.copyObject(input, entity);

            BaseSegmentView view = new BaseSegmentView();
            view.setPipelineId(input.getPipelineId());
            view.setEndTime(new Date());

            String segmentId = UUID.randomUUID().toString();
            entity.setId(segmentId);
            entity.setEffectTime(new Date());
            entity.setEndTime(endTime);
            entity.setStartStationType("STATION");
            entity.setEndStationType("STATION");
            entity.setBeTraderType(0);
            entity.setCreateById(context.getSession(PpsUserSession.class).getId());
            entity.setCreateByName(context.getUser().getName());
            entity.setCreateTime(new Date());
            mapper.insert(entity);
            //创建反输关系  管段传输方向关系表
            //管段传输方向ops_segment_direction
            BaseSegmentDirectionEnt baseSegmentDirectionEnt = new BaseSegmentDirectionEnt();
            baseSegmentDirectionEnt.setSegmentId(segmentId);
            baseSegmentDirectionEnt.setBeReverse(input.getBeReverse());
            baseSegmentDirectionEnt.setEffectTime(new Date());
            baseSegmentDirectionEnt.setEndTime(endTime);
            entity.setCreateById(context.getSession(PpsUserSession.class).getId());
            baseSegmentDirectionEnt.setCreateByName(context.getUser().getName());
            baseSegmentDirectionEnt.setCreateTime(new Date());
            segmentDirectionmapper.insert(baseSegmentDirectionEnt);
            //创建管段转供关系
            //根据结束场站id，查询所有起始场站为当前结束场站的管段，管线信息
            BaseSegmentView baseSegmentView = new BaseSegmentView();
            baseSegmentView.setStartStationId(input.getEndStationId());
            List<BaseSegmentView> baseSegmentEnts = viewMapper.selectListByStartStationId(baseSegmentView);
            for (BaseSegmentView baseSegmentEnt : baseSegmentEnts) {
                BaseSegmentPipelineTransferRelEnt baseSegmentPipelineTransferRel = new BaseSegmentPipelineTransferRelEnt();
                baseSegmentPipelineTransferRel.setSegmentId(segmentId);
                baseSegmentPipelineTransferRel.setStartPipelineId(input.getPipelineId());
                baseSegmentPipelineTransferRel.setStartStationId(input.getStartStationId());
                baseSegmentPipelineTransferRel.setEndPipelineId(baseSegmentEnt.getPipelineId());
                baseSegmentPipelineTransferRel.setEndStationId(baseSegmentEnt.getStartStationId());
                baseSegmentPipelineTransferRel.setEffectTime(new Date());
                baseSegmentPipelineTransferRel.setEndTime(endTime);
                baseSegmentPipelineTransferRel.setCreateById(context.getSession(PpsUserSession.class).getId());
                baseSegmentPipelineTransferRel.setCreateByName(context.getUser().getName());
                baseSegmentPipelineTransferRel.setCreateTime(new Date());
                relMapper.insert(baseSegmentPipelineTransferRel);
            }

            //创建管线和管段起始场站关系
            BasePipelineStationRelEnt pipelineStationRel = new BasePipelineStationRelEnt();
            pipelineStationRel.setId(UUID.randomUUID().toString());
            pipelineStationRel.setPipelineId(input.getPipelineId());
            pipelineStationRel.setStationId(input.getStartStationId());
            pipelineStationRel.setEffectTime(new Date());
            pipelineStationRel.setEndTime(endTime);
            pipelineStationRel.setCreateById(context.getSession(PpsUserSession.class).getId());
            pipelineStationRel.setCreateByName(context.getUser().getName());
            pipelineStationRel.setCreateTime(new Date());
            basePipelineStationRelMapper.insert(pipelineStationRel);

            //创建管线和管段结束场站关系
            BasePipelineStationRelEnt pipelineStation = new BasePipelineStationRelEnt();
            pipelineStation.setId(UUID.randomUUID().toString());
            pipelineStation.setPipelineId(input.getPipelineId());
            pipelineStation.setStationId(input.getStartStationId());
            pipelineStation.setEffectTime(new Date());
            pipelineStation.setEndTime(endTime);
            pipelineStation.setCreateById(context.getSession(PpsUserSession.class).getId());
            pipelineStation.setCreateByName(context.getUser().getName());
            pipelineStation.setCreateTime(new Date());
            basePipelineStationRelMapper.insert(pipelineStation);

            //创建管线和管段关联关系ops_pipeline_segment_rel
            BasePipelineSegmentRelEnt opsPipelineSegmentRelEnt = new BasePipelineSegmentRelEnt();
            opsPipelineSegmentRelEnt.setPipelineId(input.getPipelineId());
            opsPipelineSegmentRelEnt.setSegmentId(segmentId);
            opsPipelineSegmentRelEnt.setId(UUID.randomUUID().toString());
            opsPipelineSegmentRelEnt.setEffectTime(new Date());
            opsPipelineSegmentRelEnt.setEndTime(endTime);
            if(ObjectUtils.isNotEmpty(input.getSort())){
                QueryWrapper<BasePipelineSegmentRelEnt> queryWrapper = new QueryWrapper();
                queryWrapper.lambda().eq(BasePipelineSegmentRelEnt::getPipelineId,input.getPipelineId())
                        .eq(BasePipelineSegmentRelEnt::getSort,input.getSort()).gt(BasePipelineSegmentRelEnt::getEndTime,new Date());
                List<BasePipelineSegmentRelEnt> ents = basePipelineSegmentRel.selectList(queryWrapper);
                if(ObjectUtils.isNotEmpty(ents)){
                    BasePipelineSegmentRelView record = new BasePipelineSegmentRelView();
                    record.setPipelineId(input.getPipelineId());
                    record.setSort(new BigDecimal(input.getSort()));
                    basePipelineSegmentRelView.updateSortByPipelineIdAndSort(record);
                }
                opsPipelineSegmentRelEnt.setSort(new BigDecimal(input.getSort()));
            }else{
                BasePipelineSegmentRelView record = new BasePipelineSegmentRelView();
                record.setPipelineId(input.getPipelineId());
                BasePipelineSegmentRelView ents = basePipelineSegmentRelView.selectMaxSort(record);
                if(ObjectUtils.isNotEmpty(ents) && ObjectUtils.isNotEmpty(ents.getSort())){
                    opsPipelineSegmentRelEnt.setSort(ents.getSort());
                }
            }

            opsPipelineSegmentRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
            opsPipelineSegmentRelEnt.setCreateByName(context.getUser().getName());
            opsPipelineSegmentRelEnt.setCreateTime(new Date());
            basePipelineSegmentRel.insert(opsPipelineSegmentRelEnt);

            //计算管线长度
            BasePipelineSegmentRelView record = new BasePipelineSegmentRelView();
            record.setPipelineId(input.getPipelineId());
            record.setEndTime(new Date());
            List<BasePipelineSegmentRelView> segmentList = basePipelineSegmentRelView.selectSegmentList(record);
            BigDecimal totalLength = new BigDecimal("0");
            if(ObjectUtils.isNotEmpty(segmentList)){
                for(BasePipelineSegmentRelView segment:segmentList){
                    if(segment != null && segment.getSegmentLength() != null){
                        totalLength = totalLength.add(segment.getSegmentLength());
                    }
                }
            }
            if(entity.getSegmentLength() != null){
                totalLength = totalLength.add(entity.getSegmentLength());
            }
            BasePipelineEnt pipelineEnt = new BasePipelineEnt();
            pipelineEnt.setId(input.getPipelineId());
            pipelineEnt.setTotalLength(totalLength);
            basePipeline.updateById(pipelineEnt);
            return XServiceResult.OK;
        });
    }

    @XText("修改管段")
    @XApiPost
    public XServiceResult updateBaseSegment(XContext context, UpdateBaseSegmentInput input){
        //修改不能修改起始场站和结束场站，如果场站错了，删除当前管段，重新添加新的管段
        //修改管段：1、修改管段基础信息。2、判断管段的是否反输有没有修改，如果修改：先删除管段反输记录，再新增一条反输记录。
        //3、管段的场站不能修改  所以转供关系不会发生变化
        //4、创建管段的起始场站和结束场站与管线的关联关系 5、创建管线与管段的关联关系
        return XTransactionHelper.begin(context,()-> {
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            BaseSegmentMapper mapper = context.getBean(BaseSegmentMapper.class);
            BasePipelineSegmentRelMapper basePipelineSegmentRel = context.getBean(BasePipelineSegmentRelMapper.class);
            BasePipelineSegmentRelViewMapper basePipelineSegmentRelView = context.getBean(BasePipelineSegmentRelViewMapper.class);
            BasePipelineMapper basePipeline = context.getBean(BasePipelineMapper.class);
            QueryWrapper<BaseSegmentEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseSegmentEnt::getId, input.getId());
            BaseSegmentEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            entity.setModifyById(context.getSession(PpsUserSession.class).getId());
            entity.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
            entity.setModifyTime(new Date());
            mapper.updateById(entity);
            //查询当前管段所在管线的关联关系对象
            QueryWrapper<BasePipelineSegmentRelEnt> query = new QueryWrapper();
            query.lambda().eq(BasePipelineSegmentRelEnt::getPipelineId,input.getPipelineId())
                    .eq(BasePipelineSegmentRelEnt::getSegmentId,entity.getId()).gt(BasePipelineSegmentRelEnt::getEndTime,new Date());
            BasePipelineSegmentRelEnt opsPipelineSegmentRelEnt = basePipelineSegmentRel.selectOne(query);
            if(ObjectUtils.isNotEmpty(input.getSort())){
                if(ObjectUtils.isNotEmpty(opsPipelineSegmentRelEnt) && ObjectUtils.isNotEmpty(opsPipelineSegmentRelEnt.getSort())
                    && opsPipelineSegmentRelEnt.getSort().compareTo(new BigDecimal(input.getSort()))!=0){
                    QueryWrapper<BasePipelineSegmentRelEnt> queryWrappe = new QueryWrapper();
                    queryWrappe.lambda().eq(BasePipelineSegmentRelEnt::getPipelineId,input.getPipelineId())
                            .eq(BasePipelineSegmentRelEnt::getSort,input.getSort()).gt(BasePipelineSegmentRelEnt::getEndTime,new Date());
                    List<BasePipelineSegmentRelEnt> ents = basePipelineSegmentRel.selectList(queryWrappe);
                    if(ObjectUtils.isNotEmpty(ents)){
                        BasePipelineSegmentRelView record = new BasePipelineSegmentRelView();
                        record.setPipelineId(input.getPipelineId());
                        record.setSort(new BigDecimal(input.getSort()));
                        basePipelineSegmentRelView.updateSortByPipelineIdAndSort(record);
                    }
                    opsPipelineSegmentRelEnt.setSort(new BigDecimal(input.getSort()));
                }
            }else{
                BasePipelineSegmentRelView record = new BasePipelineSegmentRelView();
                record.setPipelineId(input.getPipelineId());
                BasePipelineSegmentRelView ents = basePipelineSegmentRelView.selectMaxSort(record);
                if(ObjectUtils.isNotEmpty(ents) && ObjectUtils.isNotEmpty(ents.getSort())){
                    opsPipelineSegmentRelEnt.setSort(ents.getSort());
                }
            }

            /*//创建反输关系  管段传输方向关系表
            QueryWrapper<BaseSegmentDirectionEnt> query = new QueryWrapper<>();
            query.lambda().eq(BaseSegmentDirectionEnt::getSegmentId, entity.getId())
                    .gt(BaseSegmentDirectionEnt::getEndTime, new Date());
            BaseSegmentDirectionEnt baseSegmentDirection = segmentDirectionmapper.selectOne(query);
            if(ObjectUtils.isNotEmpty(baseSegmentDirection)){
                if (baseSegmentDirection.getBeReverse() != input.getBeReverse()) {
                    //删除管段传输方向数据
                    baseSegmentDirection.setEndTime(new Date());
                    segmentDirectionmapper.updateById(baseSegmentDirection);
                    //创建管段传输方向ops_segment_direction
                    BaseSegmentDirectionEnt baseSegmentDirectionEnt = new BaseSegmentDirectionEnt();
                    baseSegmentDirectionEnt.setSegmentId(entity.getId());
                    baseSegmentDirectionEnt.setBeReverse(input.getBeReverse());
                    baseSegmentDirectionEnt.setEffectTime(new Date());
                    baseSegmentDirectionEnt.setEndTime(endTime);
                    //entity.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentDirectionEnt.setCreateByName(context.getUser().getName());
                    baseSegmentDirectionEnt.setCreateTime(new Date());
                    segmentDirectionmapper.insert(baseSegmentDirectionEnt);
                }
            }*/

            //计算管线长度
            BasePipelineSegmentRelView record = new BasePipelineSegmentRelView();
            record.setPipelineId(input.getPipelineId());
            record.setEndTime(new Date());
            List<BasePipelineSegmentRelView> segmentList = basePipelineSegmentRelView.selectSegmentList(record);
            BigDecimal totalLength = new BigDecimal("0");
            if(ObjectUtils.isNotEmpty(segmentList)){
                for(BasePipelineSegmentRelView segment:segmentList){
                    if(segment != null && segment.getSegmentLength() != null){
                        totalLength = totalLength.add(segment.getSegmentLength());
                    }
                }
            }
            if(entity.getSegmentLength() != null){
                totalLength = totalLength.add(entity.getSegmentLength());
            }
            BasePipelineEnt pipelineEnt = new BasePipelineEnt();
            pipelineEnt.setId(input.getPipelineId());
            pipelineEnt.setTotalLength(totalLength);
            basePipeline.updateById(pipelineEnt);
        /*if(!StringUtils.equals(entity.getStartStationId(),input.getStartStationId())){
            //删除管线场站关系
            QueryWrapper<BasePipelineStationRelEnt> query = new QueryWrapper();
            query.lambda().eq(BasePipelineStationRelEnt::getPipelineId,input.getPipelineId())
                    .le(BasePipelineStationRelEnt::getEffectTime, new Date()).gt(BasePipelineStationRelEnt::getEndTime, new Date())
                    .eq(BasePipelineStationRelEnt::getStationId,input.getStartStationId());
            BasePipelineStationRelEnt basePipelineStationRelEnt = basePipelineStationRelMapper.selectOne(query);
            basePipelineStationRelEnt.setEndTime(new Date());
            basePipelineStationRelMapper.updateById(basePipelineStationRelEnt);
            //创建管线和管段起始场站关系
            BasePipelineStationRelEnt pipelineStationRel = new BasePipelineStationRelEnt();
            pipelineStationRel.setId(UUID.randomUUID().toString());
            pipelineStationRel.setPipelineId(input.getPipelineId());
            pipelineStationRel.setStationId(input.getStartStationId());
            pipelineStationRel.setEffectTime(new Date());
            pipelineStationRel.setEndTime(endTime);
            //pipelineStationRel.setCreateById(context.getSession(PpsUserSession.class).getId());
            pipelineStationRel.setCreateByName(context.getUser().getName());
            pipelineStationRel.setCreateTime(new Date());
            basePipelineStationRelMapper.insert(pipelineStationRel);
        }
        if(!StringUtils.equals(entity.getEndStationId(),input.getEndStationId())){
            //删除管线场站关系
            QueryWrapper<BasePipelineStationRelEnt> query = new QueryWrapper();
            query.lambda().eq(BasePipelineStationRelEnt::getPipelineId,input.getPipelineId())
            .le(BasePipelineStationRelEnt::getEffectTime, new Date()).gt(BasePipelineStationRelEnt::getEndTime, new Date())
            .eq(BasePipelineStationRelEnt::getStationId,input.getEndStationId());
            BasePipelineStationRelEnt basePipelineStationRelEnt = basePipelineStationRelMapper.selectOne(query);
            basePipelineStationRelEnt.setEndTime(new Date());
            basePipelineStationRelMapper.updateById(basePipelineStationRelEnt);
            //创建管线和管段结束场站关系
            BasePipelineStationRelEnt pipelineStation = new BasePipelineStationRelEnt();
            pipelineStation.setId(UUID.randomUUID().toString());
            pipelineStation.setPipelineId(input.getPipelineId());
            pipelineStation.setStationId(input.getStartStationId());
            pipelineStation.setEffectTime(new Date());
            pipelineStation.setEndTime(endTime);
            //pipelineStation.setCreateById(context.getSession(PpsUserSession.class).getId());
            pipelineStation.setCreateByName(context.getUser().getName());
            pipelineStation.setCreateTime(new Date());
            basePipelineStationRelMapper.insert(pipelineStation);
        }*/
            //创建管段转供关系

            //管线场站表修改
            return XServiceResult.OK;
        });
    }

    @XText("删除管段")
    @XApiPost
    public XServiceResult deleteBaseSegment(XContext context, DeleteBaseSegmentInput input){
        //删除管段( 删除关系，不删除站)：1、删除管段基础信息。2、删除管段反输记录。3、删除管段转供的所有关系（根据当前管段的结束场站作为转供管段的起始场站，查询所有管段添加转供关系）
        //4、珊瑚管段的起始场站和结束场站与管线的关联关系5、删除管线与管段的关联关系
        return XTransactionHelper.begin(context,()-> {
            BaseSegmentMapper mapper = context.getBean(BaseSegmentMapper.class);
            BaseSegmentPipelineTransferRelMapper relMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            BaseSegmentPipelineTransferRelViewMapper relViewMapper = context.getBean(BaseSegmentPipelineTransferRelViewMapper.class);
            BasePipelineSegmentRelMapper basePipelineSegmentRelMapper = context.getBean(BasePipelineSegmentRelMapper.class);
            BasePipelineSegmentRelViewMapper basePipelineSegmentRelViewMapper = context.getBean(BasePipelineSegmentRelViewMapper.class);
            BaseSegmentDirectionMapper segmentDirectionmapper = context.getBean(BaseSegmentDirectionMapper.class);
            BaseSegmentDirectionMapper segmentDirectionViewmapper = context.getBean(BaseSegmentDirectionMapper.class);
            BasePipelineStationRelMapper basePipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            //删除管段（更新管段结束时间）
            QueryWrapper<BaseSegmentEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseSegmentEnt::getId, input.getId());
            BaseSegmentEnt entity = mapper.selectOne(queryWrapper);
            entity.setEndTime(new Date());
            entity.setModifyById(context.getSession(PpsUserSession.class).getId());
            entity.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
            entity.setModifyTime(new Date());
            mapper.updateById(entity);
            //查询管段转供关系relMapper
            BaseSegmentPipelineTransferRelView basePipelineSegmentRel = new BaseSegmentPipelineTransferRelView();
            basePipelineSegmentRel.setSegmentId(entity.getId());
            List<BaseSegmentPipelineTransferRelView> baseSegmentPipelineTransferRelViews = relViewMapper.selectList(basePipelineSegmentRel);
            for (BaseSegmentPipelineTransferRelView obj : baseSegmentPipelineTransferRelViews) {
                BaseSegmentPipelineTransferRelEnt begmentPipelineTransferRel = new BaseSegmentPipelineTransferRelEnt();
                begmentPipelineTransferRel.setEndTime(new Date());
                begmentPipelineTransferRel.setId(obj.getId());
                relMapper.updateById(begmentPipelineTransferRel);
            }
            //删除查询管段传输方向
            QueryWrapper<BaseSegmentDirectionEnt> query = new QueryWrapper<>();
            query.lambda().eq(BaseSegmentDirectionEnt::getSegmentId, input.getId())
                    .gt(BaseSegmentDirectionEnt::getEndTime, new Date());
            BaseSegmentDirectionEnt baseSegmentDirectionEnt = segmentDirectionmapper.selectOne(query);
            if(null != baseSegmentDirectionEnt){
                baseSegmentDirectionEnt.setEndTime(new Date());
                segmentDirectionViewmapper.updateById(baseSegmentDirectionEnt);
            }
            //查找管段对应的管线
            BasePipelineSegmentRelView basePipelineSegmentRelView = new BasePipelineSegmentRelView();
            basePipelineSegmentRelView.setSegmentId(entity.getId());
            List<BasePipelineSegmentRelView> basePipelineSegmentRelViewList = basePipelineSegmentRelViewMapper.selectListPipelineSegmentRelBySegmentId(basePipelineSegmentRelView);
            if (basePipelineSegmentRelViewList != null && basePipelineSegmentRelViewList.size() > 0) {
                //删除管段对应的管线和管段的关系（更新管段结束时间来删除）
                BasePipelineSegmentRelEnt basePipelineSegmentRelEnt = new BasePipelineSegmentRelEnt();
                basePipelineSegmentRelEnt.setId(basePipelineSegmentRelViewList.get(0).getId());
                basePipelineSegmentRelEnt.setEndTime(new Date());
                basePipelineSegmentRelEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
                basePipelineSegmentRelEnt.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                basePipelineSegmentRelEnt.setModifyTime(new Date());
                basePipelineSegmentRelMapper.updateById(basePipelineSegmentRelEnt);
            }
            /*//珊瑚管段的起始场站和结束场站与管线的关联关系
            //删除起始场站和管线的关联关系
            QueryWrapper<BasePipelineStationRelEnt> queryPipelineStart = new QueryWrapper<>();
            queryPipelineStart.lambda().eq(BasePipelineStationRelEnt::getStationId, entity.getStartStationId());
            BasePipelineStationRelEnt basePipelineStationRelStart = basePipelineStationRelMapper.selectOne(queryPipelineStart);
            basePipelineStationRelStart.setEndTime(new Date());
            basePipelineStationRelMapper.updateById(basePipelineStationRelStart);
            //删除结束场站和管线的关联关系
            QueryWrapper<BasePipelineStationRelEnt> queryPipelineEnd = new QueryWrapper<>();
            queryPipelineEnd.lambda().eq(BasePipelineStationRelEnt::getStationId, entity.getStartStationId());
            BasePipelineStationRelEnt basePipelineStationRelEnd = basePipelineStationRelMapper.selectOne(queryPipelineEnd);
            basePipelineStationRelEnd.setEndTime(new Date());
            basePipelineStationRelMapper.updateById(basePipelineStationRelEnd);*/
            return XServiceResult.OK;
        });
    }

    @XText("根据id查询管段")
    @XApiPost
    public XSingleResult<GetBaseSegmentOutput> getBaseSegment(XContext context, GetBaseSegmentInput input){
        BaseSegmentMapper mapper = context.getBean(BaseSegmentMapper.class);
        QueryWrapper<BaseSegmentEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSegmentEnt::getId, input.getId());
        BaseSegmentEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSegmentOutput output = new GetBaseSegmentOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询管段（暂时没用）")
    @XApiPost
    public XPageResult<QueryBaseSegmentOutput> queryBaseSegment(XContext context, QueryBaseSegmentInput input){
        BaseSegmentMapper mapper = context.getBean(BaseSegmentMapper.class);
        QueryWrapper<BaseSegmentEnt> queryWrapper = new QueryWrapper<>();
        if(StringUtils.isNotBlank(input.getSegmentName())){
            queryWrapper.lambda().like(BaseSegmentEnt::getSegmentName, input.getSegmentName());
        }

        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseSegmentOutput.class);
    }

    @XText("分页条件查询管段")
    @XApiPost
    public XPageResult<QueryBaseSegmentViewOutput> queryBaseSegmentList(XContext context, QueryBaseSegmentViewInput input){
        BaseSegmentViewMapper mapper = context.getBean(BaseSegmentViewMapper.class);
        BaseSegmentView record = new BaseSegmentView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setEndTime(XDateUtils.addMunites(new Date(),1));
        List<BaseSegmentView> list = mapper.queryBaseSegmentList(record);
        PageInfo<BaseSegmentView> pageInfo = new PageInfo<>(list);
        List<QueryBaseSegmentViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseSegmentViewOutput.class);
        if(ObjectUtils.isNotEmpty(list)){
            for(BaseSegmentView view:list){
                for(QueryBaseSegmentViewOutput out:outputs){
                    if(StringUtils.equals(view.getId(),out.getId()) && view.getBePipelineTransfer() != null){
                        out.setBePipelineTransfer(String.valueOf(view.getBePipelineTransfer()));
                    }
                }
            }
        }

        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("修改管段排序")
    @XApiPost
    public XServiceResult updatePipelineSortById(XContext context, QueryBaseSegmentViewInput input){
        BaseSegmentMapper mapper = context.getBean(BaseSegmentMapper.class);
        BasePipelineSegmentRelMapper relMapper = context.getBean(BasePipelineSegmentRelMapper.class);
        QueryWrapper<BasePipelineSegmentRelEnt> query = new QueryWrapper();
        QueryWrapper<BaseSegmentEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(BaseSegmentEnt::getId,input.getId());
        //查询修改的管线sort值
        BaseSegmentEnt baseSegmentEnt = mapper.selectOne(queryWrapper);
        //查询当前管段所属管线
        query.lambda().eq(BasePipelineSegmentRelEnt::getSegmentId,baseSegmentEnt.getId()).gt(BasePipelineSegmentRelEnt::getEndTime,new Date());
        List<BasePipelineSegmentRelEnt> basePipelineSegmentRelEnts = relMapper.selectList(query);
        //查询管线中下一条管段或者上一条管段
        QueryWrapper<BasePipelineSegmentRelEnt> segmentWrapper = new QueryWrapper();
        BasePipelineSegmentRelEnt basePipelineSegmentRel = basePipelineSegmentRelEnts.get(0);
        segmentWrapper.lambda().eq(BasePipelineSegmentRelEnt::getPipelineId,basePipelineSegmentRel.getPipelineId());
        if(StringUtils.equals("up",input.getSortFlag())){
            segmentWrapper.lambda().lt(BasePipelineSegmentRelEnt::getPipelineId,basePipelineSegmentRel.getPipelineId())
                    .lt(BasePipelineSegmentRelEnt::getSort,basePipelineSegmentRel.getSort()).orderByDesc(BasePipelineSegmentRelEnt::getSort);
        }else{
            segmentWrapper.lambda().gt(BasePipelineSegmentRelEnt::getPipelineId,basePipelineSegmentRel.getPipelineId())
                    .gt(BasePipelineSegmentRelEnt::getSort,basePipelineSegmentRel.getSort()).orderByAsc(BasePipelineSegmentRelEnt::getSort);
        }
        List<BasePipelineSegmentRelEnt> basePipelineSegmentRelEnts1 = relMapper.selectList(segmentWrapper);
        //根据上一条或者下一条管段id查询管段
        if(basePipelineSegmentRelEnts1 != null && basePipelineSegmentRelEnts1.size()>0){
            BasePipelineSegmentRelEnt basePipelineSegmentRelEnt = basePipelineSegmentRelEnts1.get(0);
            //交换sort
            BigDecimal sort = basePipelineSegmentRel.getSort();
            basePipelineSegmentRel.setSort(basePipelineSegmentRelEnt.getSort());
            basePipelineSegmentRelEnt.setSort(sort);
            relMapper.updateById(basePipelineSegmentRel);
            relMapper.updateById(basePipelineSegmentRelEnt);
        }

        return XServiceResult.OK;
    }

    @XApiPost
    public XPageResult<QueryBaseSegmentOutput> dynamicQueryBaseSegment(XContext context, CustomQueryInput input){
        BaseSegmentMapper mapper = context.getBean(BaseSegmentMapper.class);
        return XMapperHelper.query(mapper, input, BaseSegmentEnt.class, QueryBaseSegmentOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBaseSegmentViewOutput> getBaseSegmentView(XContext context, GetBaseSegmentViewInput input){
        BaseSegmentViewMapper mapper = context.getBean(BaseSegmentViewMapper.class);
        BaseSegmentView record = new BaseSegmentView();
        XCopyUtils.copyObject(input, record);
        BaseSegmentView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSegmentViewOutput output = new GetBaseSegmentViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBaseSegmentViewOutput> queryBaseSegmentView(XContext context, QueryBaseSegmentViewInput input){
        BaseSegmentViewMapper mapper = context.getBean(BaseSegmentViewMapper.class);
        BaseSegmentView record = new BaseSegmentView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseSegmentView> list = mapper.selectList(record);
        PageInfo<BaseSegmentView> pageInfo = new PageInfo<>(list);
        List<QueryBaseSegmentViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseSegmentViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据场站id获取管段数据")
    @XApiGet
    public XListResult<GetBaseSegmentOutput> getBaseSegmentList(XContext context, GetBaseSegmentInput input){
        BaseSegmentViewMapper mapper = context.getBean(BaseSegmentViewMapper.class);
        BaseSegmentView record = new BaseSegmentView();
        record.setStartStationId(input.getStationId());
        List<BaseSegmentView> list = mapper.selectBaseSegmentByStationId(record);
        List<GetBaseSegmentOutput> outputs = XCopyUtils.copyNewList(list, GetBaseSegmentOutput.class);
        return XListResult.success(outputs);
    }
}
