package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.base_small_metering_appliances.GetBaseStationOutput;
import pps.core.base.service.data.base_small_metering_appliances.QueryBaseSmallMeteringAppliancesOutput;
import pps.core.base.service.data.base_small_metering_appliances.UnitStationParamView;
import pps.core.base.service.data.base_small_metering_appliances_group.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XPageResult;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import com.github.pagehelper.PageHelper;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("小型计量器具组")
@XService
public class BaseSmallMeteringAppliancesGroupService {

    @XText("创建小型计量器具组")
    @XApiPost
    public XServiceResult createBaseSmallMeteringAppliancesGroup(XContext context, CreateBaseSmallMeteringAppliancesGroupInput input){
        BaseSmallMeteringAppliancesGroupMapper mapper = context.getBean(BaseSmallMeteringAppliancesGroupMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseSmallMeteringAppliancesGroupEnt entity = new BaseSmallMeteringAppliancesGroupEnt();
        XCopyUtils.copyObject(input, entity);
        SysOrganizationCloudService service = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput organizationViewInput = new GetSysOrganizationViewInput();
        organizationViewInput.setId(input.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> result = service.getSysOrganizationById(context , organizationViewInput);
        result.throwIfFail();
        GetSysOrganizationViewOutput viewOutput = result.getResult();
        entity.setOuName(viewOutput.getOuName());
        String s = UUID.randomUUID().toString();
        entity.setId(s);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getLoginName());
        entity.setCreateTime(new Date());
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改小型计量器具组")
    @XApiPost
    public XServiceResult updateBaseSmallMeteringAppliancesGroup(XContext context, UpdateBaseSmallMeteringAppliancesGroupInput input){
        BaseSmallMeteringAppliancesGroupMapper mapper = context.getBean(BaseSmallMeteringAppliancesGroupMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<BaseSmallMeteringAppliancesGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSmallMeteringAppliancesGroupEnt::getId, input.getId());
        BaseSmallMeteringAppliancesGroupEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        SysOrganizationCloudService service = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput organizationViewInput = new GetSysOrganizationViewInput();
        organizationViewInput.setId(input.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> result = service.getSysOrganizationById(context , organizationViewInput);
        result.throwIfFail();
        GetSysOrganizationViewOutput viewOutput = result.getResult();
        entity.setOuName(viewOutput.getOuName());
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getLoginName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除小型计量器具组")
    @XApiPost
    public XServiceResult deleteBaseSmallMeteringAppliancesGroup(XContext context, DeleteBaseSmallMeteringAppliancesGroupInput input){
        BaseSmallMeteringAppliancesGroupMapper mapper = context.getBean(BaseSmallMeteringAppliancesGroupMapper.class);
        QueryWrapper<BaseSmallMeteringAppliancesGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSmallMeteringAppliancesGroupEnt::getId, input.getId());
        BaseSmallMeteringAppliancesGroupEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("获取小型计量器具组详情")
    @XApiPost
    public XSingleResult<GetBaseSmallMeteringAppliancesGroupOutput> getBaseSmallMeteringAppliancesGroup(XContext context, GetBaseSmallMeteringAppliancesGroupInput input){
        BaseSmallMeteringAppliancesGroupMapper mapper = context.getBean(BaseSmallMeteringAppliancesGroupMapper.class);
        QueryWrapper<BaseSmallMeteringAppliancesGroupEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSmallMeteringAppliancesGroupEnt::getId, input.getId());
        BaseSmallMeteringAppliancesGroupEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseSmallMeteringAppliancesGroupOutput output = new GetBaseSmallMeteringAppliancesGroupOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页查询小型计量器具组")
    @XApiPost
    public XPageResult<QueryBaseSmallMeteringAppliancesGroupOutput> queryBaseSmallMeteringAppliancesGroup(XContext context, QueryBaseSmallMeteringAppliancesGroupInput input){
        BaseSmallMeteringAppliancesGroupMapper mapper = context.getBean(BaseSmallMeteringAppliancesGroupMapper.class);
        BaseSmallMeteringAppliancesMapper stationMapper = context.getBean(BaseSmallMeteringAppliancesMapper.class);
        BaseStationMapper baseStationMapper = context.getBean(BaseStationMapper.class);
        QueryWrapper<BaseSmallMeteringAppliancesGroupEnt> queryWrapper = new QueryWrapper<>();
        BaseSmallMeteringAppliancesService service = context.getBean(BaseSmallMeteringAppliancesService.class);
        UnitStationParamView unitStationParamView = service.setUnitStationParam(context);
        if(!CollectionUtils.isEmpty(unitStationParamView.getStationList()))
            queryWrapper.lambda().in(BaseSmallMeteringAppliancesGroupEnt::getStationId, unitStationParamView.getStationList());
        if(!CollectionUtils.isEmpty(unitStationParamView.getUnitList()))
            queryWrapper.lambda().in(BaseSmallMeteringAppliancesGroupEnt::getOuId, unitStationParamView.getUnitList());
        if(StringUtils.isNotBlank(input.getOuId())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesGroupEnt::getOuId, input.getOuId());
        }
        if(StringUtils.isNotBlank(input.getToolGroupName())){
            queryWrapper.lambda().like(BaseSmallMeteringAppliancesGroupEnt::getToolGroupName, input.getToolGroupName());
        }
        if(StringUtils.isNotBlank( input.getStationId())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesGroupEnt::getStationId, input.getStationId());
        }
        XPageResult<QueryBaseSmallMeteringAppliancesGroupOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QueryBaseSmallMeteringAppliancesGroupOutput.class);
        List<QueryBaseSmallMeteringAppliancesGroupOutput> items = query.getResult().getItems();
        if(ObjectUtils.isNotEmpty(items)){
            //查询场站
            List<String> stationIds = items.stream().map(p -> p.getStationId()).distinct().collect(Collectors.toList());
            QueryWrapper<BaseSmallMeteringAppliancesEnt> queryMeter = new QueryWrapper<>();
            queryMeter.lambda().in(BaseSmallMeteringAppliancesEnt::getStationId,stationIds);
            List<BaseSmallMeteringAppliancesEnt> meterList = stationMapper.selectList(queryMeter);
            //设置场站名称
            QueryWrapper<BaseStationEnt> queryStation = new QueryWrapper<>();
            queryStation.lambda().in(BaseStationEnt::getId,stationIds);
            List<BaseStationEnt> stationEnts = baseStationMapper.selectList(queryStation);
            for(QueryBaseSmallMeteringAppliancesGroupOutput obj:items){
                if(stationEnts != null && stationEnts.size()>0){
                    for(BaseStationEnt stationEnt:stationEnts) {
                        if(StringUtils.equals(obj.getStationId(),stationEnt.getId())){
                            obj.setStationName(stationEnt.getStationName());
                        }
                    }
                }
                if(meterList != null && meterList.size()>0){
                    for(BaseSmallMeteringAppliancesEnt meterEnt:meterList) {
                        if(StringUtils.equals(obj.getMeasureOilRule(),meterEnt.getId())){
                            obj.setMeasureOilRuleNo(meterEnt.getManufacturingNo());
                        }
                        if(StringUtils.equals(obj.getMercuryThermometer(),meterEnt.getId())){
                            obj.setMercuryThermometerNo(meterEnt.getManufacturingNo());
                        }
                        if(StringUtils.equals(obj.getThermometer(),meterEnt.getId())){
                            obj.setThermometerNo(meterEnt.getManufacturingNo());
                        }
                        if(StringUtils.equals(obj.getSeventyDensitometer(),meterEnt.getId())){
                            obj.setSeventyDensitometerNo(meterEnt.getManufacturingNo());
                        }
                        if(StringUtils.equals(obj.getSeventyFiveDensitometer(),meterEnt.getId())){
                            obj.setSeventyFiveDensitometerNo(meterEnt.getManufacturingNo());
                        }
                        if(StringUtils.equals(obj.getEightyDensitometer(),meterEnt.getId())){
                            obj.setEightyDensitometerNo(meterEnt.getManufacturingNo());
                        }
                    }
                }
            }
        }
        return query;
    }

    @XText("获取小型计量器具组（下拉框选择）")
    @XApiPost
    public XPageResult<QueryBaseSmallMeteringAppliancesGroupOutput> getBaseSmallMeteringAppliancesGroupForSelect(XContext context, GetMeteringAppliancesGroupForSelectInput input){
        BaseSmallMeteringAppliancesGroupMapper mapper = context.getBean(BaseSmallMeteringAppliancesGroupMapper.class);
        QueryWrapper<BaseSmallMeteringAppliancesGroupEnt> queryWrapper = new QueryWrapper<>();
        if(StringUtils.isNotBlank( input.getStationId())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesGroupEnt::getStationId, input.getStationId());
        }
        XPageResult<QueryBaseSmallMeteringAppliancesGroupOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QueryBaseSmallMeteringAppliancesGroupOutput.class);
        return query;
    }

    @XText("查询小型计量器具组列表")
    @XApiPost
    public XListResult<QueryBaseSmallMeteringAppliancesGroupViewOutput> queryBaseSmallMeteringAppliancesGroupView(XContext context, QueryBaseSmallMeteringAppliancesGroupViewInput input){
        BaseSmallMeteringAppliancesGroupViewMapper mapper = context.getBean(BaseSmallMeteringAppliancesGroupViewMapper.class);
        BaseSmallMeteringAppliancesGroupView record = new BaseSmallMeteringAppliancesGroupView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseSmallMeteringAppliancesGroupView> list = mapper.selectList(record);
        List<QueryBaseSmallMeteringAppliancesGroupViewOutput> outputs = XCopyUtils.copyNewList(list, QueryBaseSmallMeteringAppliancesGroupViewOutput.class);
        return XListResult.success(outputs);
    }

}
