package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.base.Strings;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.BaseBaseStationService;
import pps.cloud.base.service.data.BaseOuIdInput;
import pps.cloud.base.service.data.FindBaseClientOutput;
import pps.cloud.base.service.data.QueryBaseStationOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.GetSysOrganizationViewOutput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.CommonImportError;
import pps.core.base.service.data.base_small_metering_appliances.*;
import pps.core.base.service.data.base_station.QueryBaseStationViewOutput;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.*;
import pps.core.common.utils.DateUtil;
import pps.core.system.entity.SysOrganizationEnt;
import pps.core.system.mapper.SysOrganizationMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.*;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import com.github.pagehelper.PageHelper;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@XText("小型计量器具")
@XService
public class BaseSmallMeteringAppliancesService {

    @XText("创建小型计量器具")
    @XApiPost
    public XServiceResult createBaseSmallMeteringAppliances(XContext context, CreateBaseSmallMeteringAppliancesInput input){
        BaseSmallMeteringAppliancesMapper mapper = context.getBean(BaseSmallMeteringAppliancesMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseSmallMeteringAppliancesEnt entity = new BaseSmallMeteringAppliancesEnt();
        XCopyUtils.copyObject(input, entity);
        SysOrganizationCloudService service = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput organizationViewInput = new GetSysOrganizationViewInput();
        organizationViewInput.setId(input.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> result = service.getSysOrganizationById(context , organizationViewInput);
        result.throwIfFail();
        GetSysOrganizationViewOutput viewOutput = result.getResult();
        entity.setOuName(viewOutput.getOuName());
        String s = UUID.randomUUID().toString();
        entity.setId(s);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getLoginName());
        entity.setCreateTime(new Date());
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("编辑小型计量器具")
    @XApiPost
    public XServiceResult updateBaseSmallMeteringAppliances(XContext context, UpdateBaseSmallMeteringAppliancesInput input){
        BaseSmallMeteringAppliancesMapper mapper = context.getBean(BaseSmallMeteringAppliancesMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<BaseSmallMeteringAppliancesEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getId, input.getId());
        BaseSmallMeteringAppliancesEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        SysOrganizationCloudService service = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput organizationViewInput = new GetSysOrganizationViewInput();
        organizationViewInput.setId(input.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> result = service.getSysOrganizationById(context , organizationViewInput);
        result.throwIfFail();
        GetSysOrganizationViewOutput viewOutput = result.getResult();
        entity.setOuName(viewOutput.getOuName());
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getLoginName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除小型计量器具")
    @XApiPost
    public XServiceResult deleteBaseSmallMeteringAppliance(XContext context, DeleteBaseSmallMeteringAppliancesInput input){
        return XTransactionHelper.begin(context,()->{
            BaseSmallMeteringAppliancesMapper mapper = context.getBean(BaseSmallMeteringAppliancesMapper.class);
            QueryWrapper<BaseSmallMeteringAppliancesEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getId, input.getId());
            BaseSmallMeteringAppliancesEnt entity = mapper.selectOne(queryWrapper);
            if(entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            mapper.deleteById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("分页查询小型计量器具列表")
    @XApiPost
    public XPageResult<QueryBaseSmallMeteringAppliancesOutput> queryBaseSmallMeteringAppliances(XContext context, QueryBaseSmallMeteringAppliancesInput input){
        BaseSmallMeteringAppliancesMapper mapper = context.getBean(BaseSmallMeteringAppliancesMapper.class);
        Map<String,String> dicMap = this.getDicMap(context,"METER_TYPE","k-v");
        BaseStationMapper baseStationMapper = context.getBean(BaseStationMapper.class);
        QueryWrapper<BaseSmallMeteringAppliancesEnt> queryWrapper = new QueryWrapper<>();
        UnitStationParamView unitStationParamView = setUnitStationParam(context);
        if(!CollectionUtils.isEmpty(unitStationParamView.getStationList()))
            queryWrapper.lambda().in(BaseSmallMeteringAppliancesEnt::getStationId, unitStationParamView.getStationList());
        if(!CollectionUtils.isEmpty(unitStationParamView.getUnitList()))
            queryWrapper.lambda().in(BaseSmallMeteringAppliancesEnt::getOuId, unitStationParamView.getUnitList());
        if(StringUtils.isNotBlank(input.getOuId())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getOuId, input.getOuId());
        }
        if(StringUtils.isNotBlank(input.getStationId())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getStationId, input.getStationId());
        }
        if(StringUtils.isNotBlank(input.getMeterType())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getMeterType, input.getMeterType());
        }
        if(StringUtils.isNotBlank(input.getUseStatus())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getUseStatus, input.getUseStatus());
        }
        if(StringUtils.equals("1",input.getWarnType())){//1、预警设备
            String date = DateUtil.minusMonths(1L, DateUtil.FormatEnum.FORMAT_DATA_TIME);
            queryWrapper.lambda().gt(BaseSmallMeteringAppliancesEnt::getVerificationDate, date);
        }
        if(StringUtils.equals("2",input.getWarnType())){//2、到期设备
            queryWrapper.apply("ADD_MONTHS(verification_date,coalesce(verification_period,0))>now()");
        }
        queryWrapper.lambda().orderByAsc(BaseSmallMeteringAppliancesEnt::getOuId , BaseSmallMeteringAppliancesEnt::getStationId , BaseSmallMeteringAppliancesEnt::getMeterType);
        XPageResult<QueryBaseSmallMeteringAppliancesOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QueryBaseSmallMeteringAppliancesOutput.class);
        List<QueryBaseSmallMeteringAppliancesOutput> items = query.getResult().getItems();

        if(ObjectUtils.isNotEmpty(items)){
            for(QueryBaseSmallMeteringAppliancesOutput obj:items){
                //查询场站
                List<String> stationIds = items.stream().map(p -> p.getStationId()).distinct().collect(Collectors.toList());
                QueryWrapper<BaseStationEnt> queryStation = new QueryWrapper<>();
                queryStation.lambda().in(BaseStationEnt::getId,stationIds);
                List<BaseStationEnt> stationEnts = baseStationMapper.selectList(queryStation);
                obj.setMeterTypeName(dicMap.get(obj.getMeterType()));
                if(stationEnts != null && stationEnts.size()>0){
                    for(BaseStationEnt stationEnt:stationEnts) {
                        if(StringUtils.equals(obj.getStationId(),stationEnt.getId())){
                            obj.setStationName(stationEnt.getStationName());
                        }
                    }
                }
            }
        }
        return query;
    }

    private Map<String,String> getDicMap(XContext context,String alias,String type){
        Map<String,String> map = new HashMap<>();
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        CreateSysDictionaryInput sysDictionaryView = new CreateSysDictionaryInput();
        sysDictionaryView.setAlias(alias);
        XListResult<QuerySysDictionaryViewOutput> dicList = sysDictionaryService.querySysDictionarysByAlias(context, sysDictionaryView);
        List<QuerySysDictionaryViewOutput> dics = dicList.getResult();
        if(!CollectionUtils.isEmpty(dics)){
            for(QuerySysDictionaryViewOutput item:dics){
                if("k-v".equals(type)){
                    map.put(item.getDicKey(),item.getDicName());
                }else{
                    map.put(item.getDicName(),item.getDicKey());
                }
            }
        }
        return map;
    }

    @XText("下载导入模板")
    @XApiGet(anonymous = true)
    public XFileResult downloadImportTemplate(XContext context){
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/小型器具台账导入模板.xlsx");
        String templateFilePath = "template/import_small_metering.xlsx";
        InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
        byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, null);
        try {
            ExportExcelUtils.outputFileData(excelContent,fileSavePath);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    @XText("导入数据")
    @XApiUpload
    public XListResult importData(XContext context, SmallMeteringFileUploadInput input){
        XUploadFile file = input.getFile();
        Map<String, Object> map = this.importFile(context,file);
        String code = String.valueOf(map.get("code"));
        if("-1".equals(code)){
            Object errors = map.get("errors");
            if(errors != null)
                return XListResult.success((List<CommonImportError>)errors);
            else
                return XListResult.success(new ArrayList<>());
        }else if("0".equals(code)){ //
            List<BaseSmallMeteringAppliancesView> viewList = (List<BaseSmallMeteringAppliancesView>)map.get("list");
            //添加数据
            if(viewList.size()>0){
                BaseSmallMeteringAppliancesViewMapper viewMapper = context.getBean(BaseSmallMeteringAppliancesViewMapper.class);
                int len = viewList.size();
                int lastIndex = 100;
                for(int i=0;i<len;i+=100){
                    if(i+100>len){
                        lastIndex = len-1;
                    }
                    List<BaseSmallMeteringAppliancesView> tempInsertList = viewList.subList(i,lastIndex);
                    XThreadHelper.async(new Runnable() {
                        @Override
                        public void run() {
                            viewMapper.batchInsert(tempInsertList);
                        }
                    });
                }
            }
            return XListResult.success();
        }else{
            return XListResult.error(500,String.valueOf(map.get("msg")));
        }
    }

    private Map<String, Object> importFile(XContext context,XUploadFile file) {
        Map map = new HashMap();
        List<CommonImportError> errors = new ArrayList<>();
        try {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            Date date  = new Date();

            String fileName = file.getFileName();
            InputStream input = file.getInputStream();
            boolean isE2007 = false;    //判断是否是excel2007格式
            if (fileName.endsWith(".xlsx")){
                isE2007 = true;
            }else{
                map.put("code", "500");
                map.put("msg", "上传文件不超过100M，文件格式为xlsx");
                return map;
            }

            List<BaseSmallMeteringAppliancesView> viewList = new ArrayList<>();

            Workbook wb = null;
            //根据文件格式(2003或者2007)来初始化
            if (isE2007) {
                wb = new XSSFWorkbook(input);//xlsx后缀
            } else {
                wb = new HSSFWorkbook(input);//xls后缀
            }

            SysOrganizationMapper organizationMapper = context.getBean(SysOrganizationMapper.class);
            BaseStationMapper baseStationMapper = context.getBean(BaseStationMapper.class);
            Map<String,String> typeDicMap = this.getDicMap(context,"METER_TYPE","v-k");
            Map<String,String> useStatusDicMap = this.getDicMap(context,"USE_STATUS","v-k");

            List<String> titles = this.createTitles();

            Sheet sheet = wb.getSheetAt(0);     //获得第一个sheet
            DataFormatter formatter = new DataFormatter();
            Iterator<Row> rows = sheet.rowIterator(); //获得第一个sheet的行迭代器
            while (rows.hasNext()) {
                Row row = rows.next();  //获得行数据

                int currRow = row.getRowNum();  //获取当前行，第一行为标题，不做保存
                if(currRow == 0) continue;
                if(currRow == 1) {
                    for (int i = 0; i < titles.size(); i++) {
                        String title = formatter.formatCellValue(row.getCell(i));
                        if(!titles.get(i).equals(title)){
                            map.put("code", "500");
                            map.put("msg", "导入模板不正确，请使用下载的模板");
                            return map;
                        }
                    }
                    continue;
                }

                if (ImportExcelUtils.isRowEmpty(row)) {
                    break;
                }

                BaseSmallMeteringAppliancesView view = null;
                if (currRow > 0) {
                    view = new BaseSmallMeteringAppliancesView();
                    view.setId(UUIDHelper.newUUID());
                    view.setCreateById(session.getId());
                    view.setCreateByName(session.getUserName());
                    view.setCreateTime(date);
                }
                //校验数据
                String msg = ImportExcelUtils.checkData(row,titles);

                if(msg != null){
                    errors.add(new CommonImportError((currRow+1),"校验数据",msg));
                    continue;
                }

                for (int i = 0; i < titles.size(); i++) {
                    Cell cell = row.getCell(i);
                    String cellText = formatter.formatCellValue(cell);
                    if (view != null) {
                        try {
                            switch (i) {  //针对列去解析，放进viewList
                                case 0: //器具检字号
                                    view.setMeterNo(cellText);
                                    break;
                                case 1: //输油部
                                    QueryWrapper<SysOrganizationEnt> ouQr = new QueryWrapper<>();
                                    ouQr.lambda().eq(SysOrganizationEnt::getIsDeleted,1).and(w->w.eq(SysOrganizationEnt::getOuName,cellText).or().eq(SysOrganizationEnt::getShortName,cellText)).last("limit 0,1");
                                    SysOrganizationEnt sysOrganizationEnt = organizationMapper.selectOne(ouQr);
                                    if(sysOrganizationEnt == null){
                                       errors.add(new CommonImportError((currRow+1),"校验数据","输油部不存在"));
                                    }else{
                                        view.setOuId(sysOrganizationEnt.getId());
                                        view.setOuName(sysOrganizationEnt.getOuName());
                                    }
                                    break;
                                case 2: //所在油库
                                    QueryWrapper<BaseStationEnt> stationQr = new QueryWrapper<>();
                                    stationQr.lambda().gt(BaseStationEnt::getEndTime,new Date()).and(w->w.eq(BaseStationEnt::getStationName,cellText).or().eq(BaseStationEnt::getShortName,cellText)).last("limit 0,1");
                                    BaseStationEnt baseStationEnt = baseStationMapper.selectOne(stationQr);
                                    if(baseStationEnt == null){
                                        errors.add(new CommonImportError((currRow+1),"校验数据","所在油库不存在"));
                                    }else{
                                        view.setStationId(baseStationEnt.getId());
                                    }
                                    break;
                                case 3: //工具自编号
                                    view.setToolNo(cellText);
                                    break;
                                case 4: //证书号
                                    view.setCertificateNo(cellText);
                                    break;
                                case 5: //类型
                                    String type = typeDicMap.get(cellText);
                                    if(type == null){
                                        errors.add(new CommonImportError((currRow+1),"校验数据","类型不存在"));
                                    }else{
                                        view.setMeterType(type);
                                    }
                                    break;
                                case 6: //使用项目
                                    view.setUseProject(cellText);
                                    break;
                                case 7: //规格型号
                                    view.setSpecification(cellText);
                                    break;
                                case 8: //制造厂
                                    view.setManufactory(cellText);
                                    break;
                                case 9: //出厂编号
                                    view.setManufacturingNo(cellText);
                                    break;
                                case 10: //主管
                                    view.setButler(cellText);
                                    break;
                                case 11: //核验
                                    view.setCheckName(cellText);
                                    break;
                                case 12: //鉴定员
                                    view.setVerificationUserId(cellText);
                                    break;
                                case 13: //送检人
                                    view.setSendVerificationId(cellText);
                                    break;
                                case 14:  //检定周期（天）
                                    try{
                                        view.setVerificationPeriod(Integer.parseInt(cellText));
                                    }catch (Exception e){
                                        errors.add(new CommonImportError((currRow+1),"校验数据格式","检定周期格式不正确"));
                                    }
                                    break;
                                case 15:  //检定日期
                                    try{
                                        view.setVerificationDate(XDateUtils.parse(cellText));
                                    }catch (Exception e){
                                        errors.add(new CommonImportError((currRow+1),"校验数据格式","检定日期格式不正确"));
                                    }
                                    break;
                                case 16:  //有效日期
                                    try{
                                        view.setValidDate(XDateUtils.parse(cellText));
                                    }catch (Exception e){
                                        errors.add(new CommonImportError((currRow+1),"校验数据格式","有效日期格式不正确"));
                                    }
                                    break;
                                case 17:  //使用状态
                                    String useStatus = useStatusDicMap.get(cellText);
                                    if(useStatus == null){
                                        errors.add(new CommonImportError((currRow+1),"校验数据","使用状态不存在"));
                                    }else{
                                        view.setUseStatus(useStatus);
                                    }
                                    break;
                                case 18:  //检定结果
                                    view.setVerificationResult(cellText);
                                    break;
                                case 19:  //备注
                                    view.setRemark(cellText);
                                    break;
                                default:
                                    break;
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                            context.getLogger().error("=====parse_data_error: "+e.toString());
                            map.put("code","500");
                            map.put("msg","数据解析报错："+e.toString());
                            return map;
                        }
                    } else if (currRow == 0) {
                        continue;
                    }
                }
                if(view != null){
                    viewList.add(view);
                }
            }
            if(errors.isEmpty()){
                map.put("list", viewList);
                map.put("code", "0");
            }else{
                map.put("errors", errors);
                map.put("code", "-1");
            }
        }catch (Exception e){
            e.printStackTrace();
            map.put("code", "500");
            map.put("msg", "导入失败");
        }
        return map;
    }

    /**
     * 创建标题列表
     * @return
     */
    private List<String> createTitles(){
        List<String> titles = new ArrayList<String>();
        titles.add("*器具检字号");
        titles.add("*输油部");
        titles.add("*所在油库");
        titles.add("*工具自编号");
        titles.add("证书号");
        titles.add("*类型");
        titles.add("使用项目");
        titles.add("规格型号");
        titles.add("制造厂");
        titles.add("*出厂编号");
        titles.add("主管");
        titles.add("核验");
        titles.add("鉴定员");
        titles.add("送检人");
        titles.add("*检定周期（天）");
        titles.add("*检定日期");
        titles.add("*有效日期");
        titles.add("*使用状态");
        titles.add("检定结果");
        titles.add("备注");
        return titles;
    }

    @XText("导出小型计量器具列表")
    @XApiGet(anonymous = true)
    public XFileResult exportData(XContext context, ExportSmallMeteringInput input){
        Map<String,Object> params = new HashMap<>();
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/小型器具组台账"+XDateUtils.getString(new Date(),"yyyyMMddHHmmss") +".xlsx");
        String templateFilePath = "template/exportSmallMetering.xlsx";
        InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);

        BaseSmallMeteringAppliancesMapper mapper = context.getBean(BaseSmallMeteringAppliancesMapper.class);
        Map<String,String> dicMap = this.getDicMap(context,"METER_TYPE","k-v");
        BaseStationMapper baseStationMapper = context.getBean(BaseStationMapper.class);
        QueryWrapper<BaseSmallMeteringAppliancesEnt> queryWrapper = new QueryWrapper<>();
        UnitStationParamView unitStationParamView = setUnitStationParam(context);
        if(!CollectionUtils.isEmpty(unitStationParamView.getStationList()))
            queryWrapper.lambda().in(BaseSmallMeteringAppliancesEnt::getStationId, unitStationParamView.getStationList());
        if(!CollectionUtils.isEmpty(unitStationParamView.getUnitList()))
            queryWrapper.lambda().in(BaseSmallMeteringAppliancesEnt::getOuId, unitStationParamView.getUnitList());
        if(StringUtils.isNotBlank(input.getOuId())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getOuId, input.getOuId());
        }
        if(StringUtils.isNotBlank(input.getStationId())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getStationId, input.getStationId());
        }
        if(StringUtils.isNotBlank(input.getMeterType())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getMeterType, input.getMeterType());
        }
        if(StringUtils.isNotBlank(input.getUseStatus())){
            queryWrapper.lambda().eq(BaseSmallMeteringAppliancesEnt::getUseStatus, input.getUseStatus());
        }
        if(StringUtils.equals("1",input.getWarnType())){//1、预警设备
            String date = DateUtil.minusMonths(1L, DateUtil.FormatEnum.FORMAT_DATA_TIME);
            queryWrapper.lambda().gt(BaseSmallMeteringAppliancesEnt::getVerificationDate, date);
        }
        if(StringUtils.equals("2",input.getWarnType())){//2、到期设备
            queryWrapper.apply("ADD_MONTHS(verification_date,ifnull(verification_period,0))>now()");
        }
        queryWrapper.lambda().orderByAsc(BaseSmallMeteringAppliancesEnt::getOuId , BaseSmallMeteringAppliancesEnt::getStationId , BaseSmallMeteringAppliancesEnt::getMeterType);
        List<BaseSmallMeteringAppliancesEnt> entList = mapper.selectList(queryWrapper);
        List<QueryBaseSmallMeteringAppliancesOutput> outputList =XCopyUtils.copyNewList(entList,QueryBaseSmallMeteringAppliancesOutput.class);
        //调用微服务查询字典
        if(ObjectUtils.isNotEmpty(outputList)){
            for(QueryBaseSmallMeteringAppliancesOutput obj:outputList){
                //查询场站
                List<String> stationIds = outputList.stream().map(p -> p.getStationId()).distinct().collect(Collectors.toList());
                QueryWrapper<BaseStationEnt> queryStation = new QueryWrapper<>();
                queryStation.lambda().in(BaseStationEnt::getId,stationIds);
                List<BaseStationEnt> stationEnts = baseStationMapper.selectList(queryStation);
                obj.setMeterTypeName(dicMap.get(obj.getMeterType()));
                if(stationEnts != null && stationEnts.size()>0){
                    for(BaseStationEnt stationEnt:stationEnts) {
                        if(StringUtils.equals(obj.getStationId(),stationEnt.getId())){
                            obj.setStationName(stationEnt.getStationName());
                        }
                    }
                }
            }
        }
        params.put("dataList", outputList);
        byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, params);
        try {
            ExportExcelUtils.outputFileData(excelContent,fileSavePath);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XFileResult.success(XFileType.APPLICATION_XLSX,fileSavePath);
    }

    @XText("查询小型计量器具列表")
    @XApiPost
    public XListResult<QueryBaseSmallMeteringAppliancesViewOutput> queryBaseSmallMeteringAppliancesView(XContext context, QueryBaseSmallMeteringAppliancesViewInput input){
        BaseSmallMeteringAppliancesViewMapper mapper = context.getBean(BaseSmallMeteringAppliancesViewMapper.class);
        BaseSmallMeteringAppliancesView record = new BaseSmallMeteringAppliancesView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseSmallMeteringAppliancesView> list = mapper.selectList(record);
        List<QueryBaseSmallMeteringAppliancesViewOutput> out = XCopyUtils.copyNewList(list, QueryBaseSmallMeteringAppliancesViewOutput.class);
        return XListResult.success(out);
    }

    @XApiGet
    @XText("获取二级单位")
    public XSingleResult<GetUnitStationOutput> selectUnitListByUser(XContext context) {
        GetUnitStationOutput stationOutput = new GetUnitStationOutput();
        stationOutput.setIsChoice(0);
        List<GetBaseStationOutput> outputs = new ArrayList<>();
        BaseBaseStationService stationService = context.getBean(BaseBaseStationService.class);
        GetSysOrganizationViewOutput sysOrganization = getSysOrganization(context );
        if((sysOrganization.getOuLevel().equals("REGION_FIRM")
                ||sysOrganization.getOuLevel().equals("4097"))){
            //1.所属企业
            //获取二级单位
            BaseOuIdInput child = new BaseOuIdInput();
            child.setOuId(sysOrganization.getId());
            child.setIsContainsChildren(true);
            XListResult<QueryBaseStationOutput> result = stationService.selectAllChildByOuId(context , child);
            result.throwIfFail();
            List<QueryBaseStationOutput> stationOutputList = result.getResult();
            List<QueryBaseStationOutput> filterList = stationOutputList.stream().filter(item-> XStringUtils.isNotEmpty(item.getOuLevel()) && (item.getOuLevel().equals("ADMINISTRATIVE") || item.getOuLevel().equals("4099"))).collect(Collectors.toList());
            for(QueryBaseStationOutput output : filterList){
                GetBaseStationOutput o = new GetBaseStationOutput();
                o.setId(output.getId());
                o.setStationName(output.getOuName());
                outputs.add(o);
            }
        }else if(sysOrganization.getOuLevel().equals("ADMINISTRATIVE")
                ||sysOrganization.getOuLevel().equals("4099")) {
            //2.二级单位,展示自身
            GetBaseStationOutput output = new GetBaseStationOutput();
            output.setId(sysOrganization.getId());
            output.setStationName(sysOrganization.getOuName());
            outputs.add(output);
            stationOutput.setIsChoice(1);
        }else if(sysOrganization.getOuLevel().equals("STATION")
                ||sysOrganization.getOuLevel().equals("4106")){
            //3.场站，获取父级二级单位
            SysOrganizationCloudService sysOrganizationService = context.getBean(SysOrganizationCloudService.class);
            GetSysOrganizationViewInput sysOrganizationViewInput = new GetSysOrganizationViewInput();
            sysOrganizationViewInput.setId(sysOrganization.getId());
            XSingleResult<GetSysOrganizationViewOutput> result = sysOrganizationService.getOrganizationBySubOuId(context , sysOrganizationViewInput);
            result.throwIfFail();
            GetSysOrganizationViewOutput sysOrganizationViewOutput = result.getResult();
            GetBaseStationOutput output = new GetBaseStationOutput();
            output.setId(sysOrganizationViewOutput.getId());
            output.setStationName(sysOrganizationViewOutput.getOuName());
            outputs.add(output);
            stationOutput.setIsChoice(1);
        }
        stationOutput.setList(outputs);
        return XSingleResult.success(stationOutput);
    }

    private GetSysOrganizationViewOutput getSysOrganization(XContext context ){
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysOrganizationCloudService system = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput getSysOrganizationViewInput = new GetSysOrganizationViewInput();
        getSysOrganizationViewInput.setId(session.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> outputXSingleResult = system.getSysOrganizationById(context , getSysOrganizationViewInput);
        outputXSingleResult.throwIfFail();
        return outputXSingleResult.getResult();
    }

    public UnitStationParamView setUnitStationParam(XContext context){
        UnitStationParamView unitStationParamView = new UnitStationParamView();
        List<String> unitList = new ArrayList<>();
        List<String> stationList = new ArrayList<>();
        BaseBaseStationService stationService = context.getBean(BaseBaseStationService.class);
        GetSysOrganizationViewOutput sysOrganization = getSysOrganization(context );
        if((sysOrganization.getOuLevel().equals("REGION_FIRM")
                ||sysOrganization.getOuLevel().equals("4097"))){
            //1.所属企业
            //获取二级单位
            BaseOuIdInput child = new BaseOuIdInput();
            child.setOuId(sysOrganization.getId());
            child.setIsContainsChildren(true);
            XListResult<QueryBaseStationOutput> result = stationService.selectAllChildByOuId(context , child);
            result.throwIfFail();
            List<QueryBaseStationOutput> stationOutputList = result.getResult();
            List<QueryBaseStationOutput> filterList = stationOutputList.stream().filter(item-> XStringUtils.isNotEmpty(item.getOuLevel()) && (item.getOuLevel().equals("ADMINISTRATIVE") || item.getOuLevel().equals("4099"))).collect(Collectors.toList());
            for(QueryBaseStationOutput output : filterList){
                unitList.add(output.getId());
            }
        }else if(sysOrganization.getOuLevel().equals("ADMINISTRATIVE")
                ||sysOrganization.getOuLevel().equals("4099")) {
            //2.二级单位,展示自身
            unitList.add(sysOrganization.getId());
        }else if(sysOrganization.getOuLevel().equals("STATION")
                ||sysOrganization.getOuLevel().equals("4106")){
            //3.场站，获取父级二级单位
            SysOrganizationCloudService sysOrganizationService = context.getBean(SysOrganizationCloudService.class);
            GetSysOrganizationViewInput sysOrganizationViewInput = new GetSysOrganizationViewInput();
            sysOrganizationViewInput.setId(sysOrganization.getId());
            XSingleResult<GetSysOrganizationViewOutput> result = sysOrganizationService.getOrganizationBySubOuId(context , sysOrganizationViewInput);
            result.throwIfFail();
            GetSysOrganizationViewOutput sysOrganizationViewOutput = result.getResult();
            unitList.add(sysOrganizationViewOutput.getId());
            BaseOuIdInput baseOuIdInput = new BaseOuIdInput();
            baseOuIdInput.setOuId(sysOrganization.getId());
            XListResult<QueryBaseStationOutput> xListResult = stationService.selectStationListByOuId(context,baseOuIdInput);
            xListResult.throwIfFail();
            List<QueryBaseStationOutput> stationOutputs = xListResult.getResult();
            stationList.addAll(stationOutputs.stream().map(QueryBaseStationOutput::getId).collect(Collectors.toList()));
        }else{
            unitList.add("false");
            stationList.add("false");
        }
        unitStationParamView.setStationList(stationList);
        unitStationParamView.setUnitList(unitList);
        return unitStationParamView;
    }
}
