package pps.core.base.service;

import pps.cloud.base.service.BaseSmsSendService;
import pps.cloud.dispatch.service.DispSmsSendCloudService;
import pps.cloud.dispatch.service.data.GetOndutyMsgSettingOutput;
import pps.cloud.framedata.service.CommonDataService;
import pps.cloud.framedata.service.data.common_data.SelectCommonInput;
import pps.cloud.framedata.service.data.common_data.SelectCommonOutput;
import pps.cloud.framedata.service.data.common_data.SelectCommonParamInput;
import pps.cloud.sms.service.SmsService;
import pps.cloud.sms.service.data.MessageParam;
import pps.core.base.entity.BaseSmsConfEnt;
import pps.core.base.entity.ScadaLogEnt;
import pps.core.base.mapper.BaseSmsConfMapper;
import pps.core.base.mapper.ScadaLogEntMapper;
import pps.core.base.service.data.base_sms.SmsTemplate;
import pps.core.base.service.data.base_sms.SmsValueConf;
import pps.core.common.utils.DateUtil;
import pps.core.common.utils.NamedServiceProvider;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XJsonUtils;
import xstartup.data.XRawResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Pattern;

/**
 * 短信模版json串格式
 * {
 * 	"contentTemplate": "",
 * 	"smsValueConfs": [{
 * 		"sort": 1,
 *       "serviceName":"",
 * 		"sql": ""
 *        }, {
 * 		"sort": 2,
 *      "serviceName":"",
 * 		"sql": ""
 *    }]
 * }
 * contentTemplate 发送内容 例如 "{1}月{2}日{10}点国内SCADA采集压力温度采集到SCADA_TagsData共{3}条" 花括号为占位符，是要替换的。
 * serviceName：微服务配置在字典表中的编码 例如：pps-base-info
 */

/**
 * @Title： [标题]
 * @Description: [功能描述]
 * @author：lyy
 */
@XText("自定义短信发送服务")
@XService
public class BaseSmsSendServiceImpl implements BaseSmsSendService {

    @XText("自定义短信定时发送")
    @XApiGet
    @XApiAnonymous
    @Override
    public XServiceResult smsTimingSend(XContext context) {
        Date startDate = new Date();
        //读取所有自定义短信配置
        BaseSmsConfMapper baseSmsConfMapper = context.getBean(BaseSmsConfMapper.class);
        ScadaLogEntMapper scadaLogEntMapper = context.getBean(ScadaLogEntMapper.class);
        SmsService smsService = context.getBean(SmsService.class);
        List<BaseSmsConfEnt> baseSmsConfEntList = baseSmsConfMapper.selectList(null);
        for (BaseSmsConfEnt ent:  baseSmsConfEntList ) {
            String strContext ="";
            try
            {
                //判断是否能发送
                if(this.isSendVerification(ent.getSendTime(),startDate,scadaLogEntMapper)){
                   strContext = this.smsTemplateParse(ent.getContentTemplate(),context);
                   MessageParam param = new MessageParam();
                   param.setVariable(strContext);
                   param.setPhoneNum(ent.getSendPhoneCodes());
                   XServiceResult xServiceResult = smsService.sendMessage(context, param);
                    ScadaLogEnt logEnt = new ScadaLogEnt(UUID.randomUUID().toString(),"pps-base-info","smsTimingSend",strContext
                            ,"",new Date(),ent.getSmsTitle(),ent.getSendPhoneCodes(),ent.getSendTime());
                   if(xServiceResult.isSuccess()){
                        logEnt.setExeResult("成功");
                   }
                   else {
                       logEnt.setExeResult("失败");
                   }
                   scadaLogEntMapper.insert(logEnt);
                }
            }
            catch (Exception ex){
                //记录日志
                ScadaLogEnt logEnt = new ScadaLogEnt(UUID.randomUUID().toString(),"pps-base-info","smsTimingSend",ex.getMessage()
                                                     ,"失败",new Date(),ent.getSmsTitle(),strContext,ent.getSendTime());
                scadaLogEntMapper.insert(logEnt);
            }
        }
        return XServiceResult.OK;
    }

    /**
     * 是否发送验证 写验证方法
     * @param sendTime  发送时间
     * @return
     */
    private  Boolean isSendVerification(String sendTime, Date currentDate,ScadaLogEntMapper mapper){
        SimpleDateFormat format = new SimpleDateFormat("HH:mm:00");
        String strCurrentTime = format.format(currentDate);
        //记录日志
       /* ScadaLogEnt logEnt = new ScadaLogEnt(UUID.randomUUID().toString(),"pps-base-info","smsTimingSend",sendTime
                ,"时间对比",new Date(),"","",strCurrentTime);
        mapper.insert(logEnt);*/
        return sendTime.equals(strCurrentTime);
    }

    /**
     * 解析模版生成发送字符串
     * @param smsContextTemplate
     * @return
     */
    private  String smsTemplateParse(String smsContextTemplate,XContext context){
        SmsTemplate smsTemplate =   XJsonUtils.toObject(smsContextTemplate, SmsTemplate.class);
        String strTemplateContext = smsTemplate.getContentTemplate();
        //解析sql语句
        for (SmsValueConf conf: smsTemplate.getSmsValueConfs()) {
            CommonDataService mapper = NamedServiceProvider.getServiceByName(context, conf.getServiceName(), CommonDataService.class);
            SelectCommonInput selectCommonInput = new SelectCommonInput();
            selectCommonInput.setCmd(conf.getSql());
            selectCommonInput.setTraceId(context.getTraceId());
            SelectCommonOutput selectCommonOutput = mapper.selectCommon(context, selectCommonInput).getResult();
            if(selectCommonOutput.getResultList().size() > 0){
               String strValue =  selectCommonOutput.getResultList().get(0).get("value").toString();
               strTemplateContext = strTemplateContext.replaceAll("\\{".concat(conf.getSort().toString()).concat("\\}"),strValue);
            }
        }
        return strTemplateContext;
    }


    private boolean isPhoneNumber(String num){
        if(num!=null&&!num.isEmpty()){
            return Pattern.matches("^1[3-9]\\d{9}$",num);
        }
        return false;
    }

    @XText("自定义短信定时发送")
    @XApiGet
    @XApiAnonymous
    @Override
    public XServiceResult ondutySmsTimingSend(XContext context){
        DispSmsSendCloudService bean=context.getBean(DispSmsSendCloudService.class);
        SmsService smsService = context.getBean(SmsService.class);
        Date date = new Date();
        SimpleDateFormat formatTool=new SimpleDateFormat("yyyy-MM");

        Calendar calendar=Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DATE,1);
        int monthDay = calendar.get(Calendar.DAY_OF_MONTH);
        Date nextDay=calendar.getTime();
        String monthString= formatTool.format(nextDay);
        GetOndutyMsgSettingOutput con=bean.ondutySmsTimingSend(context,monthString,monthDay);

        if(con!=null){
            boolean exitSecondStaff=false;
            if(!con.getOndutyStaffFirstPhoneNumber().isEmpty()&&isPhoneNumber(con.getOndutyStaffFirstPhoneNumber())){
                MessageParam param = new MessageParam();
                param.setVariable("【应急值班室】根据国家管网集团应急值班安排，"+monthString+"月"+monthDay+"日由"+con.getOndutyStaffFirst()+"担任应急值班干部，请保持手机畅通。");
                param.setPhoneNum(con.getOndutyStaffFirstPhoneNumber());
                XServiceResult xServiceResult = smsService.sendMessage(context, param);
            }
            if(!con.getOndutyStaffSecondPhoneNumber().isEmpty()&&isPhoneNumber(con.getOndutyStaffSecondPhoneNumber())){
                MessageParam param = new MessageParam();
                param.setVariable("【应急值班室】根据国家管网集团应急值班安排，"+monthString+"月"+monthDay+"日由"+con.getOndutyStaffSecond()+"担任应急值班干部，请保持手机畅通。");
                param.setPhoneNum(con.getOndutyStaffSecondPhoneNumber());
                XServiceResult xServiceResult = smsService.sendMessage(context, param);
                exitSecondStaff=true;
            }
            if(exitSecondStaff){
                if(!con.getLeadershipPhoneNumber().isEmpty()&&isPhoneNumber(con.getLeadershipPhoneNumber())){
                    MessageParam param = new MessageParam();
                    param.setVariable("【应急值班室】根据国家管网集团应急值班安排，"+monthString+"月"+monthDay+"日由"+con.getOndutyStaffFirst()+"("+con.getOndutyStaffFirstPhoneNumber()+")担任应急值班干部，请保持手机畅通。/n" +
                            "【应急值班室】根据国家管网集团应急值班安排，"+monthString+"月"+monthDay+"日由"+con.getOndutyStaffSecond()+"("+con.getOndutyStaffSecondPhoneNumber()+")担任应急值班干部，请保持手机畅通。");
                    param.setPhoneNum(con.getLeadershipPhoneNumber());
                    XServiceResult xServiceResult = smsService.sendMessage(context, param);
                }
            }else{
                if(!con.getLeadershipPhoneNumber().isEmpty()&&isPhoneNumber(con.getLeadershipPhoneNumber())){
                    MessageParam param = new MessageParam();
                    param.setVariable("【应急值班室】根据国家管网集团应急值班安排，"+monthString+"月"+monthDay+"日由"+con.getOndutyStaffFirst()+"("+con.getOndutyStaffFirstPhoneNumber()+")担任应急值班干部，请保持手机畅通。");
                    param.setPhoneNum(con.getLeadershipPhoneNumber());
                    XServiceResult xServiceResult = smsService.sendMessage(context, param);
                }
            }

        }

        return XServiceResult.OK;
    }

}
