package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BaseStationPropertyEnt;
import pps.core.base.entity.BaseStationPropertyView;
import pps.core.base.mapper.BaseStationPropertyMapper;
import pps.core.base.mapper.BaseStationPropertyViewMapper;
import pps.core.base.service.data.base_station_property.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XText("场站属性接口")
@XService
public class BaseStationPropertyService {

    @XText("创建场站属性")
    @XApiPost
    public XServiceResult createBaseStationProperty(XContext context, CreateBaseStationPropertyInput input){
        BaseStationPropertyMapper mapper = context.getBean(BaseStationPropertyMapper.class);
        BaseStationPropertyEnt entity = new BaseStationPropertyEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改场站属性")
    @XApiPost
    public XServiceResult updateBaseStationProperty(XContext context, UpdateBaseStationPropertyInput input){
        BaseStationPropertyMapper mapper = context.getBean(BaseStationPropertyMapper.class);
        QueryWrapper<BaseStationPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseStationPropertyEnt::getId, input.getId());
        BaseStationPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除场站属性")
    @XApiPost
    public XServiceResult deleteBaseStationProperty(XContext context, DeleteBaseStationPropertyInput input){
        BaseStationPropertyMapper mapper = context.getBean(BaseStationPropertyMapper.class);
        QueryWrapper<BaseStationPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseStationPropertyEnt::getId, input.getId());
        BaseStationPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id查询场站属性")
    @XApiPost
    public XSingleResult<GetBaseStationPropertyOutput> getBaseStationProperty(XContext context, GetBaseStationPropertyInput input){
        BaseStationPropertyMapper mapper = context.getBean(BaseStationPropertyMapper.class);
        QueryWrapper<BaseStationPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseStationPropertyEnt::getId, input.getId());
        BaseStationPropertyEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseStationPropertyOutput output = new GetBaseStationPropertyOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询场站属性")
    @XApiPost
    public XPageResult<QueryBaseStationPropertyOutput> queryBaseStationProperty(XContext context, QueryBaseStationPropertyInput input){
        BaseStationPropertyMapper mapper = context.getBean(BaseStationPropertyMapper.class);
        QueryWrapper<BaseStationPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BaseStationPropertyEnt::getId, input.getId())
            .like(BaseStationPropertyEnt::getStationId, input.getStationId())
            .like(BaseStationPropertyEnt::getPropertyId, input.getPropertyId())
            .like(BaseStationPropertyEnt::getPropertyValue, input.getPropertyValue())
            .eq(BaseStationPropertyEnt::getEffectTime, input.getEffectTime())
            .eq(BaseStationPropertyEnt::getEndTime, input.getEndTime())
            .like(BaseStationPropertyEnt::getCreateById, input.getCreateById())
            .like(BaseStationPropertyEnt::getCreateByName, input.getCreateByName())
            .eq(BaseStationPropertyEnt::getCreateTime, input.getCreateTime())
            .like(BaseStationPropertyEnt::getModifyById, input.getModifyById())
            .like(BaseStationPropertyEnt::getModifyByName, input.getModifyByName())
            .eq(BaseStationPropertyEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseStationPropertyOutput.class);
    }

    @XApiPost
    public XPageResult<QueryBaseStationPropertyOutput> dynamicQueryBaseStationProperty(XContext context, CustomQueryInput input){
        BaseStationPropertyMapper mapper = context.getBean(BaseStationPropertyMapper.class);
        return XMapperHelper.query(mapper, input, BaseStationPropertyEnt.class, QueryBaseStationPropertyOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBaseStationPropertyViewOutput> getBaseStationPropertyView(XContext context, GetBaseStationPropertyViewInput input){
        BaseStationPropertyViewMapper mapper = context.getBean(BaseStationPropertyViewMapper.class);
        BaseStationPropertyView record = new BaseStationPropertyView();
        XCopyUtils.copyObject(input, record);
        BaseStationPropertyView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseStationPropertyViewOutput output = new GetBaseStationPropertyViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBaseStationPropertyViewOutput> queryBaseStationPropertyView(XContext context, QueryBaseStationPropertyViewInput input){
        BaseStationPropertyViewMapper mapper = context.getBean(BaseStationPropertyViewMapper.class);
        BaseStationPropertyView record = new BaseStationPropertyView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BaseStationPropertyView> list = mapper.selectList(record);
        PageInfo<BaseStationPropertyView> pageInfo = new PageInfo<>(list);
        List<QueryBaseStationPropertyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationPropertyViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
