package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.data.BaseOuIdInput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SysRoleCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_segment.QueryBaseSegmentViewInput;
import pps.core.base.service.data.base_station.*;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.DateHelper;
import pps.core.common.utils.ExportExcelUtils;
import pps.core.common.utils.ImportExcelUtils;
import pps.core.common.utils.MD5Util;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_user_import_log.GetSysUserImportLogDetailInput;
import pps.core.system.service.data.sys_user_import_log.ImportSysUserImportLogDetailInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.*;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.io.InputStream;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@XText("场站接口")
@XService
public class BaseStationService {
    private HashMap<Integer,Boolean> isNeedNotEmptyMap=new HashMap<>();

    @XText("根据组织机构获取所有场站：ouId or stationName")
    @XApiPost
    public XListResult<GetBaseStationOutput> getStationAllByOuIdOrStationName(XContext context, GetBaseStationInput input) {
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        BaseStationView record = new BaseStationView();
        XCopyUtils.copyObject(input, record);
        if (XStringUtils.isNotEmpty(record.getMediumType()) && "ALL".equals(record.getMediumType())){
            record.setMediumType(null);
        }
        record.setEndTime(new Date());
        List<BaseStationView> list = mapper.selectStationByOuIdOrStationName(record);
        List<GetBaseStationOutput> outputs = XCopyUtils.copyNewList(list, GetBaseStationOutput.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        //调用微服务查询字典
        CreateSysDictionaryInput mediumType = new CreateSysDictionaryInput();
        mediumType.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediumTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediumType);
        for (GetBaseStationOutput stationOutput : outputs) {
            for (QuerySysDictionaryViewOutput dict : mediumTypeList.getResult()) {
                if (dict.getDicKey().equals((stationOutput.getMediumType()))){
                    stationOutput.setMediumTypeName(dict.getDicName());
                }
            }
        }
        return XListResult.success(outputs);
    }

    @XText("创建场站")
    @XApiPost
    public XServiceResult createBaseStation(XContext context, CreateBaseStationInput input) {
        //1、新增场站基本信息 2、新增场站组织机构关联关系 3、添加场站和管线关系（新增场站与上游场站所在管线关系）
        //4、新增场站和上游场站自动生成管段（管段基础信息生成，别的信息需要手工维护）5、新增自动生成的管段和管线的关联关系
        //6、创建管段反输记录（默认不反输）。3、创建管段转供的所有关系（根据当前管段的结束场站作为转供管段的起始场站，查询所有管段添加转供关系）
        return XTransactionHelper.begin(context, () -> {
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            BaseStationViewMapper stationViewmapper = context.getBean(BaseStationViewMapper.class);
            BasePipelineStationRelMapper basePipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            BasePipelineMapper basePipelineMapper = context.getBean(BasePipelineMapper.class);
            BasePipelineStationRelViewMapper basePipelineStationRelViewMapper = context.getBean(BasePipelineStationRelViewMapper.class);
            BaseSegmentMapper segmentMapper = context.getBean(BaseSegmentMapper.class);
            BasePipelineSegmentRelMapper pipelineSegmentRel = context.getBean(BasePipelineSegmentRelMapper.class);
            BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
            BaseSegmentDirectionMapper baseSegmentDirectionMapper = context.getBean(BaseSegmentDirectionMapper.class);
            BaseSegmentViewMapper viewMapper = context.getBean(BaseSegmentViewMapper.class);
            BaseSegmentPipelineTransferRelMapper relMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            ArrayList<BaseStationView> item = input.getItem();
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            //生成新增的场站id，新增场站
            String id = UUID.randomUUID().toString();
            BaseStationEnt entity = new BaseStationEnt();
            XCopyUtils.copyObject(input, entity);

            entity.setEffectTime(new Date());
            entity.setEndTime(endTime);
            entity.setId(id);
            entity.setCreateById(context.getSession(PpsUserSession.class).getId());
            entity.setCreateByName(context.getUser().getName());
            entity.setCreateTime(new Date());
            //场站默认设置管线的介质类型
            if (StringUtils.isNotBlank(input.getPipelineId())) {
                QueryWrapper<BasePipelineEnt> queryPipeline = new QueryWrapper<>();
                queryPipeline.lambda().eq(BasePipelineEnt::getId, input.getPipelineId());
                BasePipelineEnt basePipelineEnt = basePipelineMapper.selectOne(queryPipeline);
                if (ObjectUtils.isNotEmpty(basePipelineEnt)) {
                    entity.setMediumType(basePipelineEnt.getMediumType());
                }
            }
            mapper.insert(entity);
            //新增场站组织机构关系
            BaseStationOuRelEnt baseStationOuRelEnt = new BaseStationOuRelEnt();
            baseStationOuRelEnt.setStationId(id);
            baseStationOuRelEnt.setOuId(input.getOuId());
            baseStationOuRelEnt.setId(UUID.randomUUID().toString());
            baseStationOuRelEnt.setEffectTime(new Date());
            baseStationOuRelEnt.setEndTime(endTime);
            baseStationOuRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
            baseStationOuRelEnt.setCreateByName(context.getUser().getName());
            baseStationOuRelEnt.setCreateTime(new Date());
            stationOuRelMapper.insert(baseStationOuRelEnt);
            //新增当前管线和场站关系(新增场站和上游场站所在管线的关联关系)
            BasePipelineStationRelEnt pipelineStation = new BasePipelineStationRelEnt();
            pipelineStation.setId(UUID.randomUUID().toString());
            pipelineStation.setPipelineId(input.getPipelineId());
            pipelineStation.setStationId(id);
            pipelineStation.setEffectTime(new Date());
            pipelineStation.setEndTime(endTime);
            //sort赋值
            BaseStationView view = new BaseStationView();
            view.setPipelineId(input.getPipelineId());
            view.setEndTime(new Date());
            BaseStationView stationView = stationViewmapper.selectMaxSortByPipelineId(view);
            Integer sort = 0;
            if (stationView != null && stationView.getSort() != null) {
                sort = sort + stationView.getSort() + 1;
            }
            pipelineStation.setSort(sort);
            pipelineStation.setCreateById(context.getSession(PpsUserSession.class).getId());
            pipelineStation.setCreateByName(context.getUser().getName());
            pipelineStation.setCreateTime(new Date());
            basePipelineStationRelMapper.insert(pipelineStation);
            if (item != null && item.size() > 0) {
                for (BaseStationView baseStationView : item) {
                    /*//新增管线和场站关系(新增场站和上游场站所在管线的关联关系)
                    BasePipelineStationRelEnt pipelineStationRel = new BasePipelineStationRelEnt();
                    pipelineStationRel.setId(UUID.randomUUID().toString());
                    pipelineStationRel.setPipelineId(baseStationView.getPipelineId());
                    pipelineStationRel.setStationId(id);
                    pipelineStationRel.setEffectTime(new Date());
                    pipelineStationRel.setEndTime(endTime);
                    //pipelineStationRel.setCreateById(context.getSession(PpsUserSession.class).getId());
                    pipelineStationRel.setCreateByName(context.getUser().getName());
                    pipelineStationRel.setCreateTime(new Date());
                    basePipelineStationRelMapper.insert(pipelineStationRel);*/
                    //自动生成管段base_segment
                    String segmentId = UUID.randomUUID().toString();
                    BaseSegmentEnt baseSegmentEnt = new BaseSegmentEnt();
                    baseSegmentEnt.setStartStationId(baseStationView.getId());
                    baseSegmentEnt.setEndStationId(id);
                    baseSegmentEnt.setSegmentName(input.getStationName() + "-" + baseStationView.getStationName());
                    baseSegmentEnt.setId(segmentId);
                    baseSegmentEnt.setEffectTime(new Date());
                    baseSegmentEnt.setEndTime(endTime);
                    baseSegmentEnt.setStartStationType("STATION");
                    baseSegmentEnt.setEndStationType("STATION");
                    baseSegmentEnt.setBeTraderType(0);
                    baseSegmentEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentEnt.setCreateByName(context.getUser().getName());
                    baseSegmentEnt.setCreateTime(new Date());
                    BasePipelineStationRelView station = new BasePipelineStationRelView();
                    station.setPipelineId(baseStationView.getPipelineId());
                    station.setEndTime(new Date());
                    //sort赋值
                    BasePipelineStationRelView stationRelView = basePipelineStationRelViewMapper.selectMaxSortByPipelineId(station);
                    Integer sortOrder = 0;
                    if (stationRelView != null && stationRelView.getSort() != null) {
                        sortOrder = sortOrder + stationRelView.getSort() + 1;
                    }
                    entity.setSort(sortOrder);
                    segmentMapper.insert(baseSegmentEnt);
                    //6、创建管段反输记录（默认不反输）。base_segment_direction
                    BaseSegmentDirectionEnt baseSegmentDirectionEnt = new BaseSegmentDirectionEnt();
                    baseSegmentDirectionEnt.setBeReverse(1);
                    baseSegmentDirectionEnt.setSegmentId(segmentId);
                    baseSegmentDirectionEnt.setId(UUID.randomUUID().toString());
                    baseSegmentDirectionEnt.setEffectTime(new Date());
                    baseSegmentDirectionEnt.setEndTime(endTime);
                    baseSegmentDirectionEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentDirectionEnt.setCreateByName(context.getUser().getName());
                    baseSegmentDirectionEnt.setCreateTime(new Date());
                    baseSegmentDirectionMapper.insert(baseSegmentDirectionEnt);
                    //创建管段转供关系(根据场站id，查询所有起始场站为当前场站的管段，管线信息)
                    BaseSegmentView baseSegmentView = new BaseSegmentView();
                    baseSegmentView.setStartStationId(id);
                    List<BaseSegmentView> baseSegmentEnts = viewMapper.selectListByStartStationId(baseSegmentView);
                    if (baseSegmentEnts != null && baseSegmentEnts.size() > 0) {
                        for (BaseSegmentView baseSegment : baseSegmentEnts) {
                            BaseSegmentPipelineTransferRelEnt baseSegmentPipelineTransferRel = new BaseSegmentPipelineTransferRelEnt();
                            baseSegmentPipelineTransferRel.setSegmentId(segmentId);
                            baseSegmentPipelineTransferRel.setStartPipelineId(input.getPipelineId());
                            baseSegmentPipelineTransferRel.setStartStationId(id);
                            baseSegmentPipelineTransferRel.setEndPipelineId(baseSegment.getPipelineId());
                            baseSegmentPipelineTransferRel.setEndStationId(baseSegment.getStartStationId());
                            baseSegmentPipelineTransferRel.setEffectTime(new Date());
                            baseSegmentPipelineTransferRel.setEndTime(endTime);
                            baseSegmentPipelineTransferRel.setCreateById(context.getSession(PpsUserSession.class).getId());
                            baseSegmentPipelineTransferRel.setCreateByName(context.getUser().getName());
                            baseSegmentPipelineTransferRel.setCreateTime(new Date());
                            relMapper.insert(baseSegmentPipelineTransferRel);
                        }
                    }
                    //新增管线管段关联表
                    BasePipelineSegmentRelEnt opsPipelineSegmentRelEnt = new BasePipelineSegmentRelEnt();
                    opsPipelineSegmentRelEnt.setPipelineId(baseStationView.getPipelineId());
                    opsPipelineSegmentRelEnt.setSegmentId(segmentId);
                    opsPipelineSegmentRelEnt.setId(UUID.randomUUID().toString());
                    opsPipelineSegmentRelEnt.setEffectTime(new Date());
                    opsPipelineSegmentRelEnt.setEndTime(endTime);
                    opsPipelineSegmentRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    opsPipelineSegmentRelEnt.setCreateByName(context.getUser().getName());
                    opsPipelineSegmentRelEnt.setCreateTime(new Date());
                    pipelineSegmentRel.insert(opsPipelineSegmentRelEnt);
                }
            }
            return XServiceResult.OK;
        });
    }

    @XText("创建场站（动态配置）")
    @XApiPost
    public XServiceResult createBaseStationByConfig(XContext context, CreateBaseStationInput input) {
        //1、新增场站基本信息 2、新增场站组织机构关联关系 3、添加场站和管线关系（新增场站与上游场站所在管线关系）
        //4、新增场站和上游场站自动生成管段（管段基础信息生成，别的信息需要手工维护）5、新增自动生成的管段和管线的关联关系
        //6、创建管段反输记录（默认不反输）。3、创建管段转供的所有关系（根据当前管段的结束场站作为转供管段的起始场站，查询所有管段添加转供关系）
        return XTransactionHelper.begin(context, () -> {
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            BaseStationViewMapper stationViewmapper = context.getBean(BaseStationViewMapper.class);
            BasePipelineStationRelMapper basePipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            BasePipelineMapper basePipelineMapper = context.getBean(BasePipelineMapper.class);
            BasePipelineStationRelViewMapper basePipelineStationRelViewMapper = context.getBean(BasePipelineStationRelViewMapper.class);
            BaseSegmentMapper segmentMapper = context.getBean(BaseSegmentMapper.class);
            BasePipelineSegmentRelMapper pipelineSegmentRel = context.getBean(BasePipelineSegmentRelMapper.class);
            BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
            BaseSegmentDirectionMapper baseSegmentDirectionMapper = context.getBean(BaseSegmentDirectionMapper.class);
            BaseSegmentViewMapper viewMapper = context.getBean(BaseSegmentViewMapper.class);
            BaseSegmentPipelineTransferRelMapper relMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            ArrayList<BaseStationView> item = input.getItem();
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            //生成新增的场站id，新增场站
            String id = UUID.randomUUID().toString();
            BaseStationEnt entity = new BaseStationEnt();
            XCopyUtils.copyObject(input, entity);

            entity.setEffectTime(new Date());
            entity.setEndTime(endTime);
            entity.setId(id);
            entity.setCreateById(context.getSession(PpsUserSession.class).getId());
            entity.setCreateByName(context.getUser().getName());
            entity.setCreateTime(new Date());

            //新增场站组织机构关系
            BaseStationOuRelEnt baseStationOuRelEnt = new BaseStationOuRelEnt();
            baseStationOuRelEnt.setStationId(id);
            baseStationOuRelEnt.setOuId(input.getOuId());
            baseStationOuRelEnt.setId(UUID.randomUUID().toString());
            baseStationOuRelEnt.setEffectTime(new Date());
            baseStationOuRelEnt.setEndTime(endTime);
            baseStationOuRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
            baseStationOuRelEnt.setCreateByName(context.getUser().getName());
            baseStationOuRelEnt.setCreateTime(new Date());
            stationOuRelMapper.insert(baseStationOuRelEnt);
            String pipelineId = "";
            if (item != null && item.size() > 0) {
                for (BaseStationView baseStationView : item) {
                    pipelineId = baseStationView.getPipelineId();
                    //新增管线和场站关系(新增场站和上游场站所在管线的关联关系)
                    BasePipelineStationRelEnt pipelineStation = new BasePipelineStationRelEnt();
                    pipelineStation.setId(UUID.randomUUID().toString());
                    pipelineStation.setPipelineId(baseStationView.getPipelineId());
                    pipelineStation.setStationId(id);
                    pipelineStation.setEffectTime(new Date());
                    pipelineStation.setEndTime(endTime);
                    //sort赋值
                    BaseStationView view = new BaseStationView();
                    view.setPipelineId(baseStationView.getPipelineId());
                    view.setEndTime(new Date());
                    BaseStationView stationView = stationViewmapper.selectMaxSortByPipelineId(view);
                    Integer sort = 0;
                    if (stationView != null && stationView.getSort() != null) {
                        sort = sort + stationView.getSort() + 1;
                    }
                    pipelineStation.setSort(sort);
                    pipelineStation.setCreateById(context.getSession(PpsUserSession.class).getId());
                    pipelineStation.setCreateByName(context.getUser().getName());
                    pipelineStation.setCreateTime(new Date());
                    basePipelineStationRelMapper.insert(pipelineStation);
                    //自动生成管段base_segment
                    String segmentId = UUID.randomUUID().toString();
                    BaseSegmentEnt baseSegmentEnt = new BaseSegmentEnt();
                    baseSegmentEnt.setStartStationId(baseStationView.getId());
                    baseSegmentEnt.setEndStationId(id);
                    baseSegmentEnt.setSegmentName(baseStationView.getStationName() + "-" + input.getStationName());
                    baseSegmentEnt.setId(segmentId);
                    baseSegmentEnt.setEffectTime(new Date());
                    baseSegmentEnt.setEndTime(endTime);
                    baseSegmentEnt.setBeTraderType(0);
                    baseSegmentEnt.setStartStationType("STATION");
                    baseSegmentEnt.setEndStationType("STATION");
                    baseSegmentEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentEnt.setCreateByName(context.getUser().getName());
                    baseSegmentEnt.setCreateTime(new Date());
                    BasePipelineStationRelView station = new BasePipelineStationRelView();
                    station.setPipelineId(baseStationView.getPipelineId());
                    station.setEndTime(new Date());
                    //sort赋值
                    BasePipelineStationRelView stationRelView = basePipelineStationRelViewMapper.selectMaxSortByPipelineId(station);
                    Integer sortOrder = 0;
                    if (stationRelView != null && stationRelView.getSort() != null) {
                        sortOrder = sortOrder + stationRelView.getSort() + 1;
                    }
                    entity.setSort(sortOrder);
                    segmentMapper.insert(baseSegmentEnt);
                    //6、创建管段反输记录（默认不反输）。base_segment_direction
                    BaseSegmentDirectionEnt baseSegmentDirectionEnt = new BaseSegmentDirectionEnt();
                    baseSegmentDirectionEnt.setBeReverse(1);
                    baseSegmentDirectionEnt.setSegmentId(segmentId);
                    baseSegmentDirectionEnt.setId(UUID.randomUUID().toString());
                    baseSegmentDirectionEnt.setEffectTime(new Date());
                    baseSegmentDirectionEnt.setEndTime(endTime);
                    baseSegmentDirectionEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentDirectionEnt.setCreateByName(context.getUser().getName());
                    baseSegmentDirectionEnt.setCreateTime(new Date());
                    baseSegmentDirectionMapper.insert(baseSegmentDirectionEnt);
                    //创建管段转供关系(根据场站id，查询所有起始场站为当前场站的管段，管线信息)
                    BaseSegmentView baseSegmentView = new BaseSegmentView();
                    baseSegmentView.setStartStationId(id);
                    List<BaseSegmentView> baseSegmentEnts = viewMapper.selectListByStartStationId(baseSegmentView);
                    if (baseSegmentEnts != null && baseSegmentEnts.size() > 0) {
                        for (BaseSegmentView baseSegment : baseSegmentEnts) {
                            BaseSegmentPipelineTransferRelEnt baseSegmentPipelineTransferRel = new BaseSegmentPipelineTransferRelEnt();
                            baseSegmentPipelineTransferRel.setSegmentId(segmentId);
                            baseSegmentPipelineTransferRel.setStartPipelineId(baseStationView.getPipelineId());
                            baseSegmentPipelineTransferRel.setStartStationId(id);
                            baseSegmentPipelineTransferRel.setEndPipelineId(baseSegment.getPipelineId());
                            baseSegmentPipelineTransferRel.setEndStationId(baseSegment.getStartStationId());
                            baseSegmentPipelineTransferRel.setEffectTime(new Date());
                            baseSegmentPipelineTransferRel.setEndTime(endTime);
                            baseSegmentPipelineTransferRel.setCreateById(context.getSession(PpsUserSession.class).getId());
                            baseSegmentPipelineTransferRel.setCreateByName(context.getUser().getName());
                            baseSegmentPipelineTransferRel.setCreateTime(new Date());
                            relMapper.insert(baseSegmentPipelineTransferRel);
                        }
                    }
                    //新增管线管段关联表
                    BasePipelineSegmentRelEnt opsPipelineSegmentRelEnt = new BasePipelineSegmentRelEnt();
                    opsPipelineSegmentRelEnt.setPipelineId(baseStationView.getPipelineId());
                    opsPipelineSegmentRelEnt.setSegmentId(segmentId);
                    opsPipelineSegmentRelEnt.setId(UUID.randomUUID().toString());
                    opsPipelineSegmentRelEnt.setEffectTime(new Date());
                    opsPipelineSegmentRelEnt.setEndTime(endTime);
                    opsPipelineSegmentRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    opsPipelineSegmentRelEnt.setCreateByName(context.getUser().getName());
                    opsPipelineSegmentRelEnt.setCreateTime(new Date());
                    pipelineSegmentRel.insert(opsPipelineSegmentRelEnt);
                }
            }
            //场站默认设置管线的介质类型
            if (StringUtils.isNotBlank(pipelineId)) {
                QueryWrapper<BasePipelineEnt> queryPipeline = new QueryWrapper<>();
                queryPipeline.lambda().eq(BasePipelineEnt::getId, pipelineId);
                BasePipelineEnt basePipelineEnt = basePipelineMapper.selectOne(queryPipeline);
                if (ObjectUtils.isNotEmpty(basePipelineEnt)) {
                    entity.setMediumType(basePipelineEnt.getMediumType());
                }
            }
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    @XText("创建场站关联管线（添加已有场站到管线）")
    @XApiPost
    public XServiceResult createBaseStationByPipelineId(XContext context, CreateBaseStationInput input) {
        //1、新增管线和场站关系
        return XTransactionHelper.begin(context, () -> {
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
            BasePipelineStationRelMapper basePipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }

            //新增管线和场站关系
            BasePipelineStationRelEnt pipelineStationRel = new BasePipelineStationRelEnt();
            pipelineStationRel.setId(UUID.randomUUID().toString());
            pipelineStationRel.setPipelineId(input.getPipelineId());
            pipelineStationRel.setStationId(input.getId());
            pipelineStationRel.setEffectTime(new Date());
            pipelineStationRel.setEndTime(endTime);
            pipelineStationRel.setCreateById(context.getSession(PpsUserSession.class).getId());
            pipelineStationRel.setCreateByName(context.getUser().getName());
            pipelineStationRel.setCreateTime(new Date());
            basePipelineStationRelMapper.insert(pipelineStationRel);

            return XServiceResult.OK;
        });
    }

    @XText("修改场站")
    @XApiPost
    public XServiceResult updateBaseStation(XContext context, UpdateBaseStationInput input) {
        //1、修改场站基本信息 2、修改场站组织机构关联关系(先删除旧的关系，创建新的关系) 3、修改旧的场站和管线关系（修改场站与上游场站所在管线关系）
        //4、新增场站和上游场站自动生成管段（管段基础信息生成，别的信息需要手工维护）5、新增自动生成的管段和管线的关联关系
        //6、创建管段反输记录（默认不反输）。3、创建管段转供的所有关系（根据当前管段的结束场站作为转供管段的起始场站，查询所有管段添加转供关系）
        return XTransactionHelper.begin(context, () -> {
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
            BaseStationOuRelViewMapper stationOuRelViewMapper = context.getBean(BaseStationOuRelViewMapper.class);
            BaseSegmentMapper segmentMapper = context.getBean(BaseSegmentMapper.class);
            BasePipelineStationRelMapper pipelineStationMapper = context.getBean(BasePipelineStationRelMapper.class);
            BaseSegmentViewMapper segmentViewMapper = context.getBean(BaseSegmentViewMapper.class);
            BasePipelineSegmentRelMapper pipelineSegmentMapper = context.getBean(BasePipelineSegmentRelMapper.class);
            BaseSegmentPipelineTransferRelMapper transferRelMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            BaseSegmentDirectionMapper directionMapper = context.getBean(BaseSegmentDirectionMapper.class);
            BasePipelineSegmentRelMapper pipelineSegmentRel = context.getBean(BasePipelineSegmentRelMapper.class);
            BasePipelineStationRelMapper basePipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            BaseSegmentPipelineTransferRelMapper relMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            //更新场站详细信息
            QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseStationEnt::getId, input.getId());
            BaseStationEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            if(input.getStatus() == null || input.getStatus() == 0)
                entity.setEndTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00"));
            else
                entity.setEndTime(new Date());
            XCopyUtils.copyObject(input, entity);
            entity.setModifyById(context.getSession(PpsUserSession.class).getId());
            entity.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
            entity.setModifyTime(new Date());
            mapper.updateById(entity);
            //删除旧的场站组织关系
            QueryWrapper<BaseStationOuRelEnt> ouWrapper = new QueryWrapper<>();
            ouWrapper.lambda().eq(BaseStationOuRelEnt::getStationId, input.getId())
                    .gt(BaseStationOuRelEnt::getEndTime, new Date());
            List<BaseStationOuRelEnt> baseStationOuRelEnts = stationOuRelMapper.selectList(ouWrapper);
            if (ObjectUtils.isNotEmpty(baseStationOuRelEnts)) {
                for (BaseStationOuRelEnt stationEnt : baseStationOuRelEnts) {
                    if (ObjectUtils.isNotEmpty(stationEnt)) {
                        stationEnt.setEndTime(new Date());
                        stationEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
                        stationEnt.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                        stationEnt.setModifyTime(new Date());
                        stationOuRelMapper.updateById(stationEnt);
                    }
                }
            }

            //添加新的场站和组织关联表
            BaseStationOuRelEnt baseStationOuRelEnt = new BaseStationOuRelEnt();
            baseStationOuRelEnt.setStationId(input.getId());
            baseStationOuRelEnt.setOuId(input.getOuId());
            baseStationOuRelEnt.setId(UUID.randomUUID().toString());
            baseStationOuRelEnt.setEffectTime(new Date());
            baseStationOuRelEnt.setEndTime(endTime);
            baseStationOuRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
            baseStationOuRelEnt.setCreateByName(context.getUser().getName());
            baseStationOuRelEnt.setCreateTime(new Date());
            stationOuRelMapper.insert(baseStationOuRelEnt);
            //更新场站和上游场站关系，删除旧的关系添加新的关系
            ArrayList<BaseStationView> item = input.getItem();
            //根据管线查询管段
            QueryWrapper<BasePipelineSegmentRelEnt> psRel = new QueryWrapper<>();
            psRel.lambda().eq(BasePipelineSegmentRelEnt::getPipelineId,input.getPipelineId())
                    .gt(BasePipelineSegmentRelEnt::getEndTime, new Date());
            List<BasePipelineSegmentRelEnt> bpsList = pipelineSegmentMapper.selectList(psRel);
            QueryWrapper<BaseSegmentEnt> segmentWrapper = new QueryWrapper<>();
            segmentWrapper.lambda().eq(BaseSegmentEnt::getEndStationId, input.getId())
                    .gt(BaseSegmentEnt::getEndTime, new Date());
            if(ObjectUtils.isNotEmpty(bpsList)){
                List<String> segIds = bpsList.stream().map(p -> {
                    return p.getSegmentId();
                }).distinct().collect(Collectors.toList());
                segmentWrapper.lambda().in(BaseSegmentEnt::getId,segIds);
            }
            List<BaseSegmentEnt> segments = segmentMapper.selectList(segmentWrapper);
            //对比页面返回的管段和数据库查出的管段，取出删除掉的管段
            List<String> delSegment = new ArrayList<>();
            if (segments != null && segments.size() > 0) {
                if (item != null && item.size() > 0) {
                    for (BaseSegmentEnt segment : segments) {
                        boolean flag = true;
                        for (BaseStationView baseStationView : item) {
                            if (StringUtils.isNotBlank(baseStationView.getSegmentId())) {
                                if (StringUtils.equals(baseStationView.getSegmentId(), segment.getId())) {
                                    flag = false;
                                }
                            }
                        }
                        if (flag) {
                            delSegment.add(segment.getId());
                        }
                    }
                }
            }
            if (delSegment != null && delSegment.size() > 0) {
                for (String segmentId : delSegment) {
                    //删除管段
                    QueryWrapper<BaseSegmentEnt> querySegment = new QueryWrapper();
                    querySegment.lambda().eq(BaseSegmentEnt::getId, segmentId);
                    BaseSegmentEnt baseSegmentEnt = segmentMapper.selectOne(querySegment);
                    baseSegmentEnt.setEndTime(new Date());
                    segmentMapper.updateById(baseSegmentEnt);
                    //查询所有需要删除的管线管段关系
                    QueryWrapper<BasePipelineSegmentRelEnt> delPipelineWrapper = new QueryWrapper<>();
                    delPipelineWrapper.lambda().eq(BasePipelineSegmentRelEnt::getSegmentId, segmentId)
                            .gt(BasePipelineSegmentRelEnt::getEndTime, new Date());
                    BasePipelineSegmentRelEnt basePipelineSegmentRelEnt = pipelineSegmentMapper.selectOne(delPipelineWrapper);
                    //删除管线和管段关系
                    basePipelineSegmentRelEnt.setEndTime(new Date());
                    pipelineSegmentMapper.updateById(basePipelineSegmentRelEnt);

                    //删除转供关系base_segment_pipeline_transfer_rel
                    QueryWrapper<BaseSegmentPipelineTransferRelEnt> query = new QueryWrapper();
                    query.lambda().eq(BaseSegmentPipelineTransferRelEnt::getSegmentId, segmentId)
                            .gt(BaseSegmentPipelineTransferRelEnt::getEndTime, new Date());
                    List<BaseSegmentPipelineTransferRelEnt> transferRels = transferRelMapper.selectList(query);
                    if (transferRels != null && transferRels.size() > 0) {
                        for (BaseSegmentPipelineTransferRelEnt transferRel : transferRels) {
                            transferRel.setEndTime(new Date());
                            transferRelMapper.updateById(transferRel);
                        }
                    }
                    //删除反输记录base_segment_direction
                    QueryWrapper<BaseSegmentDirectionEnt> directionQuery = new QueryWrapper();
                    directionQuery.lambda().eq(BaseSegmentDirectionEnt::getSegmentId, segmentId)
                            .gt(BaseSegmentDirectionEnt::getEndTime, new Date());
                    BaseSegmentDirectionEnt baseSegmentDirectionEnt = directionMapper.selectOne(directionQuery);
                    if(ObjectUtils.isNotEmpty(baseSegmentDirectionEnt)){
                        baseSegmentDirectionEnt.setEndTime(new Date());
                        directionMapper.updateById(baseSegmentDirectionEnt);
                    }
                }
            }
            List<BaseStationView> collect = item.stream().filter((BaseStationView view) -> !delSegment.contains(view.getSegmentId())).collect(Collectors.toList());
            // 新增管段，管段管线关系、管段传输方向记录、管段转供关系、管线和场站关系
            if (collect != null && collect.size() > 0) {
                Set<String> pipelineSet = new HashSet<>();
                for (BaseStationView baseStationView : collect) {
                    pipelineSet.add(baseStationView.getPipelineId());
                    //自动生成管段base_segment
                    BaseSegmentEnt baseSegmentEnt = new BaseSegmentEnt();
                    baseSegmentEnt.setStartStationId(baseStationView.getId());
                    baseSegmentEnt.setEndStationId(input.getId());
                    baseSegmentEnt.setSegmentName(input.getStationName() + "-" + baseStationView.getStationName());
                    baseSegmentEnt.setEffectTime(new Date());
                    baseSegmentEnt.setEndTime(endTime);
                    baseSegmentEnt.setBeTraderType(0);
                    baseSegmentEnt.setStartStationType("STATION");
                    baseSegmentEnt.setEndStationType("STATION");
                    baseSegmentEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentEnt.setCreateByName(context.getUser().getName());
                    baseSegmentEnt.setCreateTime(new Date());
                    if (StringUtils.isBlank(baseStationView.getSegmentId())) {
                        String segmentId = UUID.randomUUID().toString();
                        baseSegmentEnt.setId(segmentId);
                        segmentMapper.insert(baseSegmentEnt);
                        //新增管线管段关联表
                        BasePipelineSegmentRelEnt opsPipelineSegmentRelEnt = new BasePipelineSegmentRelEnt();
                        opsPipelineSegmentRelEnt.setPipelineId(input.getPipelineId());
                        opsPipelineSegmentRelEnt.setSegmentId(baseSegmentEnt.getId());
                        opsPipelineSegmentRelEnt.setId(UUID.randomUUID().toString());
                        opsPipelineSegmentRelEnt.setEffectTime(new Date());
                        opsPipelineSegmentRelEnt.setEndTime(endTime);
                        opsPipelineSegmentRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                        opsPipelineSegmentRelEnt.setCreateByName(context.getUser().getName());
                        opsPipelineSegmentRelEnt.setCreateTime(new Date());
                        pipelineSegmentRel.insert(opsPipelineSegmentRelEnt);
                        //6、创建管段反输记录（默认不反输）。base_segment_direction
                        BaseSegmentDirectionEnt baseSegmentDirectionEnt = new BaseSegmentDirectionEnt();
                        baseSegmentDirectionEnt.setBeReverse(1);
                        baseSegmentDirectionEnt.setSegmentId(baseSegmentEnt.getId());
                        baseSegmentDirectionEnt.setId(UUID.randomUUID().toString());
                        baseSegmentDirectionEnt.setEffectTime(new Date());
                        baseSegmentDirectionEnt.setEndTime(endTime);
                        baseSegmentDirectionEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                        baseSegmentDirectionEnt.setCreateByName(context.getUser().getName());
                        baseSegmentDirectionEnt.setCreateTime(new Date());
                        directionMapper.insert(baseSegmentDirectionEnt);
                        //创建管段转供关系(根据场站id，查询所有起始场站为当前场站的管段，管线信息)
                        BaseSegmentView baseSegmentView = new BaseSegmentView();
                        baseSegmentView.setStartStationId(input.getId());
                        List<BaseSegmentView> baseSegmentEnts = segmentViewMapper.selectListByStartStationId(baseSegmentView);
                        if (baseSegmentEnts != null && baseSegmentEnts.size() > 0) {
                            for (BaseSegmentView baseSegment : baseSegmentEnts) {
                                BaseSegmentPipelineTransferRelEnt baseSegmentPipelineTransferRel = new BaseSegmentPipelineTransferRelEnt();
                                baseSegmentPipelineTransferRel.setSegmentId(baseSegmentEnt.getId());
                                baseSegmentPipelineTransferRel.setStartPipelineId(baseStationView.getPipelineId());
                                baseSegmentPipelineTransferRel.setStartStationId(entity.getId());
                                baseSegmentPipelineTransferRel.setEndPipelineId(baseSegment.getPipelineId());
                                baseSegmentPipelineTransferRel.setEndStationId(baseSegment.getStartStationId());
                                baseSegmentPipelineTransferRel.setEffectTime(new Date());
                                baseSegmentPipelineTransferRel.setEndTime(endTime);
                                baseSegmentPipelineTransferRel.setCreateById(context.getSession(PpsUserSession.class).getId());
                                baseSegmentPipelineTransferRel.setCreateByName(context.getUser().getName());
                                baseSegmentPipelineTransferRel.setCreateTime(new Date());
                                relMapper.insert(baseSegmentPipelineTransferRel);
                            }
                        }
                    }
                }
            }
            return XServiceResult.OK;
        });
    }

    @XText("修改场站（动态配置）")
    @XApiPost
    public XServiceResult updateBaseStationByConfig(XContext context, UpdateBaseStationInput input) {
        //1、修改场站基本信息 2、修改场站组织机构关联关系(先删除旧的关系，创建新的关系) 3、修改旧的场站和管线关系（修改场站与上游场站所在管线关系）
        //4、新增场站和上游场站自动生成管段（管段基础信息生成，别的信息需要手工维护）5、新增自动生成的管段和管线的关联关系
        //6、创建管段反输记录（默认不反输）。3、创建管段转供的所有关系（根据当前管段的结束场站作为转供管段的起始场站，查询所有管段添加转供关系）
        return XTransactionHelper.begin(context, () -> {
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
            BaseSegmentMapper segmentMapper = context.getBean(BaseSegmentMapper.class);
            BaseSegmentViewMapper segmentViewMapper = context.getBean(BaseSegmentViewMapper.class);
            BasePipelineStationRelMapper pipelineStationMapper = context.getBean(BasePipelineStationRelMapper.class);
            BasePipelineSegmentRelMapper pipelineSegmentMapper = context.getBean(BasePipelineSegmentRelMapper.class);
            BaseSegmentPipelineTransferRelMapper transferRelMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            BaseSegmentDirectionMapper directionMapper = context.getBean(BaseSegmentDirectionMapper.class);
            BasePipelineSegmentRelMapper pipelineSegmentRel = context.getBean(BasePipelineSegmentRelMapper.class);
            BasePipelineStationRelMapper basePipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            BaseSegmentPipelineTransferRelMapper relMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            //更新场站详细信息
            QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseStationEnt::getId, input.getId());
            BaseStationEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            if(input.getStatus() == null || input.getStatus() == 0)
                entity.setEndTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00"));
            else
                entity.setEndTime(new Date());
            entity.setModifyById(context.getSession(PpsUserSession.class).getId());
            entity.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
            entity.setModifyTime(new Date());
            mapper.updateById(entity);
            //删除旧的场站组织关系
            QueryWrapper<BaseStationOuRelEnt> ouWrapper = new QueryWrapper<>();
            ouWrapper.lambda().eq(BaseStationOuRelEnt::getStationId, input.getId())
                    .gt(BaseStationOuRelEnt::getEndTime, new Date());
            List<BaseStationOuRelEnt> baseStationOuRelEnts = stationOuRelMapper.selectList(ouWrapper);
            if (ObjectUtils.isNotEmpty(baseStationOuRelEnts)) {
                for (BaseStationOuRelEnt stationEnt : baseStationOuRelEnts) {
                    if (ObjectUtils.isNotEmpty(stationEnt)) {
                        stationEnt.setEndTime(new Date());
                        stationEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
                        stationEnt.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                        stationEnt.setModifyTime(new Date());
                        stationOuRelMapper.updateById(stationEnt);
                    }
                }
            }
            //添加新的场站和组织关联表
            BaseStationOuRelEnt baseStationOuRelEnt = new BaseStationOuRelEnt();
            baseStationOuRelEnt.setStationId(input.getId());
            baseStationOuRelEnt.setOuId(input.getOuId());
            baseStationOuRelEnt.setId(UUID.randomUUID().toString());
            baseStationOuRelEnt.setEffectTime(new Date());
            baseStationOuRelEnt.setEndTime(endTime);
            baseStationOuRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
            baseStationOuRelEnt.setCreateByName(context.getUser().getName());
            baseStationOuRelEnt.setCreateTime(new Date());
            stationOuRelMapper.insert(baseStationOuRelEnt);
            //更新场站和上游场站关系，删除旧的关系添加新的关系
            ArrayList<BaseStationView> item = input.getItem();
            //根据管线查询管段
            QueryWrapper<BasePipelineSegmentRelEnt> psRel = new QueryWrapper<>();
            psRel.lambda().eq(BasePipelineSegmentRelEnt::getPipelineId,input.getPipelineId())
                    .gt(BasePipelineSegmentRelEnt::getEndTime, new Date());
            List<BasePipelineSegmentRelEnt> bpsList = pipelineSegmentMapper.selectList(psRel);
            QueryWrapper<BaseSegmentEnt> segmentWrapper = new QueryWrapper<>();
            segmentWrapper.lambda().eq(BaseSegmentEnt::getEndStationId, input.getId())
                    .gt(BaseSegmentEnt::getEndTime, new Date());
            if(ObjectUtils.isNotEmpty(bpsList)){
                List<String> segIds = bpsList.stream().map(p -> {
                    return p.getSegmentId();
                }).distinct().collect(Collectors.toList());
                segmentWrapper.lambda().in(BaseSegmentEnt::getId,segIds);
            }
            List<BaseSegmentEnt> segments = segmentMapper.selectList(segmentWrapper);
            //对比页面返回的管段和数据库查出的管段，取出删除掉的管段
            List<String> delSegment = new ArrayList<>();
            if (segments != null && segments.size() > 0) {
                if (item != null && item.size() > 0) {
                    for (BaseSegmentEnt segment : segments) {
                        boolean flag = true;
                        for (BaseStationView baseStationView : item) {
                            if (StringUtils.isNotBlank(baseStationView.getSegmentId())) {
                                if (StringUtils.equals(baseStationView.getSegmentId(), segment.getId())) {
                                    flag = false;
                                }
                            }
                        }
                        if (flag) {
                            delSegment.add(segment.getId());
                        }
                    }
                }
            }
            if (delSegment != null && delSegment.size() > 0) {
                for (String segmentId : delSegment) {
                    //删除管段
                    QueryWrapper<BaseSegmentEnt> querySegment = new QueryWrapper();
                    querySegment.lambda().eq(BaseSegmentEnt::getId, segmentId);
                    BaseSegmentEnt baseSegmentEnt = segmentMapper.selectOne(querySegment);
                    baseSegmentEnt.setEndTime(new Date());
                    segmentMapper.updateById(baseSegmentEnt);
                    //查询所有需要删除的管线管段关系
                    QueryWrapper<BasePipelineSegmentRelEnt> delPipelineWrapper = new QueryWrapper<>();
                    delPipelineWrapper.lambda().eq(BasePipelineSegmentRelEnt::getSegmentId, segmentId)
                            .gt(BasePipelineSegmentRelEnt::getEndTime, new Date());
                    BasePipelineSegmentRelEnt basePipelineSegmentRelEnt = pipelineSegmentMapper.selectOne(delPipelineWrapper);
                    //删除管线和管段关系
                    basePipelineSegmentRelEnt.setEndTime(new Date());
                    pipelineSegmentMapper.updateById(basePipelineSegmentRelEnt);
                    //删除转供关系base_segment_pipeline_transfer_rel
                    QueryWrapper<BaseSegmentPipelineTransferRelEnt> query = new QueryWrapper();
                    query.lambda().eq(BaseSegmentPipelineTransferRelEnt::getSegmentId, segmentId)
                            .gt(BaseSegmentPipelineTransferRelEnt::getEndTime, new Date());
                    List<BaseSegmentPipelineTransferRelEnt> transferRels = transferRelMapper.selectList(query);
                    if (transferRels != null && transferRels.size() > 0) {
                        for (BaseSegmentPipelineTransferRelEnt transferRel : transferRels) {
                            transferRel.setEndTime(new Date());
                            transferRelMapper.updateById(transferRel);
                        }
                    }
                    //删除反输记录base_segment_direction
                    QueryWrapper<BaseSegmentDirectionEnt> directionQuery = new QueryWrapper();
                    directionQuery.lambda().eq(BaseSegmentDirectionEnt::getSegmentId, segmentId)
                            .gt(BaseSegmentDirectionEnt::getEndTime, new Date());
                    BaseSegmentDirectionEnt baseSegmentDirectionEnt = directionMapper.selectOne(directionQuery);
                    if (ObjectUtils.isNotEmpty(baseSegmentDirectionEnt)) {
                        baseSegmentDirectionEnt.setEndTime(new Date());
                        directionMapper.updateById(baseSegmentDirectionEnt);
                    }
                }
            }
            List<BaseStationView> collect = item.stream().filter((BaseStationView view) -> !delSegment.contains(view.getSegmentId())).collect(Collectors.toList());
            // 新增管段，管段管线关系、管段传输方向记录、管段转供关系、管线和场站关系
            Set<String> pipelineSet = new HashSet<>();
            if (collect != null && collect.size() > 0) {
                for (BaseStationView baseStationView : collect) {
                    pipelineSet.add(baseStationView.getPipelineId());
                    //自动生成管段base_segment
                    BaseSegmentEnt baseSegmentEnt = new BaseSegmentEnt();
                    baseSegmentEnt.setStartStationId(baseStationView.getId());
                    baseSegmentEnt.setEndStationId(input.getId());
                    baseSegmentEnt.setSegmentName(input.getStationName() + "-" + baseStationView.getStationName());
                    baseSegmentEnt.setBeTraderType(0);
                    baseSegmentEnt.setEffectTime(new Date());
                    baseSegmentEnt.setEndTime(endTime);
                    baseSegmentEnt.setStartStationType("STATION");
                    baseSegmentEnt.setEndStationType("STATION");
                    baseSegmentEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                    baseSegmentEnt.setCreateByName(context.getUser().getName());
                    baseSegmentEnt.setCreateTime(new Date());
                    if (StringUtils.isBlank(baseStationView.getSegmentId())) {
                        String segmentId = UUID.randomUUID().toString();
                        baseSegmentEnt.setId(segmentId);
                        segmentMapper.insert(baseSegmentEnt);
                        //新增管线管段关联表
                        BasePipelineSegmentRelEnt opsPipelineSegmentRelEnt = new BasePipelineSegmentRelEnt();
                        opsPipelineSegmentRelEnt.setPipelineId(baseStationView.getPipelineId());
                        opsPipelineSegmentRelEnt.setSegmentId(baseSegmentEnt.getId());
                        opsPipelineSegmentRelEnt.setId(UUID.randomUUID().toString());
                        opsPipelineSegmentRelEnt.setEffectTime(new Date());
                        opsPipelineSegmentRelEnt.setEndTime(endTime);
                        opsPipelineSegmentRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                        opsPipelineSegmentRelEnt.setCreateByName(context.getUser().getName());
                        opsPipelineSegmentRelEnt.setCreateTime(new Date());
                        pipelineSegmentRel.insert(opsPipelineSegmentRelEnt);
                        //6、创建管段反输记录（默认不反输）。base_segment_direction
                        BaseSegmentDirectionEnt baseSegmentDirectionEnt = new BaseSegmentDirectionEnt();
                        baseSegmentDirectionEnt.setBeReverse(1);
                        baseSegmentDirectionEnt.setSegmentId(baseSegmentEnt.getId());
                        baseSegmentDirectionEnt.setId(UUID.randomUUID().toString());
                        baseSegmentDirectionEnt.setEffectTime(new Date());
                        baseSegmentDirectionEnt.setEndTime(endTime);
                        baseSegmentDirectionEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                        baseSegmentDirectionEnt.setCreateByName(context.getUser().getName());
                        baseSegmentDirectionEnt.setCreateTime(new Date());
                        directionMapper.insert(baseSegmentDirectionEnt);
                        //创建管段转供关系(根据场站id，查询所有起始场站为当前场站的管段，管线信息)
                        BaseSegmentView baseSegmentView = new BaseSegmentView();
                        baseSegmentView.setStartStationId(input.getId());
                        List<BaseSegmentView> baseSegmentEnts = segmentViewMapper.selectListByStartStationId(baseSegmentView);
                        if (baseSegmentEnts != null && baseSegmentEnts.size() > 0) {
                            for (BaseSegmentView baseSegment : baseSegmentEnts) {
                                BaseSegmentPipelineTransferRelEnt baseSegmentPipelineTransferRel = new BaseSegmentPipelineTransferRelEnt();
                                baseSegmentPipelineTransferRel.setSegmentId(baseSegmentEnt.getId());
                                baseSegmentPipelineTransferRel.setStartPipelineId(baseStationView.getPipelineId());
                                baseSegmentPipelineTransferRel.setStartStationId(entity.getId());
                                baseSegmentPipelineTransferRel.setEndPipelineId(baseSegment.getPipelineId());
                                baseSegmentPipelineTransferRel.setEndStationId(baseSegment.getStartStationId());
                                baseSegmentPipelineTransferRel.setEffectTime(new Date());
                                baseSegmentPipelineTransferRel.setEndTime(endTime);
                                baseSegmentPipelineTransferRel.setCreateById(context.getSession(PpsUserSession.class).getId());
                                baseSegmentPipelineTransferRel.setCreateByName(context.getUser().getName());
                                baseSegmentPipelineTransferRel.setCreateTime(new Date());
                                relMapper.insert(baseSegmentPipelineTransferRel);
                            }
                        }
                    }
                }
            }
            return XServiceResult.OK;
        });
    }

    @XText("删除场站")
    @XApiPost
    public XServiceResult deleteBaseStation(XContext context, DeleteBaseStationInput input) {
        //TODO 需要考虑删除的流程
        //删除站，删除站所有管段关系（包括上游和下游）
        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        BasePipelineStationRelMapper stationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
        BaseSegmentPipelineTransferRelViewMapper relViewMapper = context.getBean(BaseSegmentPipelineTransferRelViewMapper.class);
        BaseStationViewMapper viewMapper = context.getBean(BaseStationViewMapper.class);
        BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
        BaseSegmentMapper baseSegmentMapper = context.getBean(BaseSegmentMapper.class);
        BaseSegmentPipelineTransferRelMapper relMapper = context.getBean(BaseSegmentPipelineTransferRelMapper.class);
        BaseSegmentDirectionMapper segmentDirectionViewmapper = context.getBean(BaseSegmentDirectionMapper.class);
        BasePipelineSegmentRelMapper basePipelineSegmentRelMapper = context.getBean(BasePipelineSegmentRelMapper.class);
        BaseSegmentDirectionMapper segmentDirectionmapper = context.getBean(BaseSegmentDirectionMapper.class);
        BasePipelineSegmentRelViewMapper basePipelineSegmentRelViewMapper = context.getBean(BasePipelineSegmentRelViewMapper.class);
        //查询删除场站所属管段列表
        BaseStationView record = new BaseStationView();
        XCopyUtils.copyObject(input, record);
        /*//删除场站（更新结束时间来删除）
        BaseStationEnt baseStationEnt = new BaseStationEnt();
        baseStationEnt.setId(input.getId());
        baseStationEnt.setEndTime(new Date());
        baseStationEnt.setModifyByName(context.getUser().getName());
        //baseStationOuRelEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
        mapper.updateById(baseStationEnt);
        //删除场站组织对应关系
        QueryWrapper<BaseStationOuRelEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(BaseStationOuRelEnt::getStationId,input.getId());
        BaseStationOuRelEnt baseStationOuRelEnt = stationOuRelMapper.selectOne(queryWrapper);
        baseStationOuRelEnt.setEndTime(new Date());
        baseStationOuRelEnt.setModifyTime(new Date());
        baseStationOuRelEnt.setModifyByName(context.getUser().getName());
        //baseStationOuRelEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
        stationOuRelMapper.updateById(baseStationOuRelEnt);*/

        record.setEndTimeStr(XDateUtils.getString(new Date()));
        List<BaseSegmentView> list = viewMapper.selectListSegmentByStationId(record);
        //删除场站对应管段关联关系
        if (list != null && list.size() > 0) {
            return XServiceResult.error(200, "删除失败，场站已关联别的管段！");
            /*for (BaseSegmentView baseSegmentView : list) {
                //删除管段（更新管段结束时间）
                BaseSegmentEnt entity = new BaseSegmentEnt();
                entity.setEndTime(new Date());
                entity.setId(baseSegmentView.getId());
                entity.setModifyById(context.getSession(PpsUserSession.class).getId());
                entity.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                entity.setModifyTime(new Date());
                baseSegmentMapper.updateById(entity);
                //查询管段转供关系relMapper
                BaseSegmentPipelineTransferRelView basePipelineSegmentRel = new BaseSegmentPipelineTransferRelView();
                basePipelineSegmentRel.setSegmentId(entity.getId());
                List<BaseSegmentPipelineTransferRelView> baseSegmentPipelineTransferRelViews = relViewMapper.selectList(basePipelineSegmentRel);
                for (BaseSegmentPipelineTransferRelView obj : baseSegmentPipelineTransferRelViews) {
                    BaseSegmentPipelineTransferRelEnt begmentPipelineTransferRel = new BaseSegmentPipelineTransferRelEnt();
                    begmentPipelineTransferRel.setEndTime(new Date());
                    begmentPipelineTransferRel.setId(obj.getId());
                    begmentPipelineTransferRel.setModifyById(context.getSession(PpsUserSession.class).getId());
                    begmentPipelineTransferRel.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                    begmentPipelineTransferRel.setModifyTime(new Date());
                    relMapper.updateById(begmentPipelineTransferRel);
                }
                //删除查询管段传输方向
                QueryWrapper<BaseSegmentDirectionEnt> query = new QueryWrapper<>();
                query.lambda().eq(BaseSegmentDirectionEnt::getSegmentId, baseSegmentView.getId())
                        .gt(BaseSegmentDirectionEnt::getEndTime, new Date());
                BaseSegmentDirectionEnt baseSegmentDirectionEnt = segmentDirectionmapper.selectOne(query);
                baseSegmentDirectionEnt.setEndTime(new Date());
                baseSegmentDirectionEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
                baseSegmentDirectionEnt.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                baseSegmentDirectionEnt.setModifyTime(new Date());
                segmentDirectionViewmapper.updateById(baseSegmentDirectionEnt);

                //查找管段对应的管线
                BasePipelineSegmentRelView basePipelineSegmentRelView = new BasePipelineSegmentRelView();
                basePipelineSegmentRelView.setSegmentId(entity.getId());
                List<BasePipelineSegmentRelView> basePipelineSegmentRelViewList = basePipelineSegmentRelViewMapper.selectListPipelineSegmentRelBySegmentId(basePipelineSegmentRelView);
                if (basePipelineSegmentRelViewList != null && basePipelineSegmentRelViewList.size() > 0) {
                    //删除管段对应的管线和管段的关系（更新管段结束时间来删除）
                    BasePipelineSegmentRelEnt basePipelineSegmentRelEnt = new BasePipelineSegmentRelEnt();
                    basePipelineSegmentRelEnt.setId(basePipelineSegmentRelViewList.get(0).getId());
                    basePipelineSegmentRelEnt.setEndTime(new Date());
                    basePipelineSegmentRelEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
                    basePipelineSegmentRelEnt.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                    basePipelineSegmentRelEnt.setModifyTime(new Date());
                    basePipelineSegmentRelMapper.updateById(basePipelineSegmentRelEnt);
                }
            }
            return XServiceResult.OK;*/
        } else {
            QueryWrapper<BasePipelineStationRelEnt> queryWrapper = new QueryWrapper();
            queryWrapper.lambda().eq(BasePipelineStationRelEnt::getStationId, input.getId())
                    .eq(BasePipelineStationRelEnt::getPipelineId, input.getPipelineId())
                    .gt(BasePipelineStationRelEnt::getEndTime, new Date());
            BasePipelineStationRelEnt basePipelineStationRelEnt = stationRelMapper.selectOne(queryWrapper);
            basePipelineStationRelEnt.setEndTime(new Date());
            basePipelineStationRelEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
            basePipelineStationRelEnt.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
            basePipelineStationRelEnt.setModifyTime(new Date());
            stationRelMapper.updateById(basePipelineStationRelEnt);
            return XServiceResult.OK;
        }
    }

    @XText("根据id查询场站")
    @XApiPost
    public XSingleResult<GetBaseStationOutput> getBaseStation(XContext context, GetBaseStationInput input) {
        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseStationEnt::getId, input.getId());
        BaseStationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseStationOutput output = new GetBaseStationOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询场站")
    @XApiPost
    public XPageResult<QueryBaseStationOutput> queryBaseStation(XContext context, QueryBaseStationInput input) {
        SysRoleCloudService service = context.getBean(SysRoleCloudService.class);
        service.getSysRole(context, new GetSysRoleInput());

        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().gt(BaseStationEnt::getEndTime, new Date());
        if (StringUtils.isNotBlank(input.getStationName())) {
            queryWrapper.lambda().like(BaseStationEnt::getStationName, input.getStationName());
        }
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseStationOutput.class);
    }

    @XText("根据id查询场站和上游场站关系")
    @XApiPost
    public XSingleResult<GetBaseStationOutput> getBaseStationAndRel(XContext context, GetBaseStationInput input) {
        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        SysOrganizationCloudService sysOrgService = context.getBean(SysOrganizationCloudService.class);
        BaseStationViewMapper mapperView = context.getBean(BaseStationViewMapper.class);
        BasePipelineStationRelMapper pipelineStationRel = context.getBean(BasePipelineStationRelMapper.class);
        BaseStationOuRelMapper stationOuView = context.getBean(BaseStationOuRelMapper.class);
        QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseStationEnt::getId, input.getId());
        BaseStationEnt entity = mapper.selectOne(queryWrapper);
        //查询上游场站
        BaseStationView record = new BaseStationView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        record.setPipelineId(input.getPipelineId());
        List<BaseStationView> list = mapperView.selectListRel(record);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseStationOutput output = new GetBaseStationOutput();
        XCopyUtils.copyObject(entity, output);
        if (StringUtils.isNotBlank(entity.getProvince())) {
            output.setProvince(Integer.parseInt(entity.getProvince()));
        }
        if (StringUtils.isNotBlank(entity.getCity())) {
            output.setCity(Integer.parseInt(entity.getCity()));
        }
        if (StringUtils.isNotBlank(entity.getCounty())) {
            output.setCounty(Integer.parseInt(entity.getCounty()));
        }
        //查询场站所属管线
        QueryWrapper<BasePipelineStationRelEnt> queryPipeline = new QueryWrapper<>();
        queryPipeline.lambda().eq(BasePipelineStationRelEnt::getStationId, input.getId()).
                gt(BasePipelineStationRelEnt::getEndTime, new Date());
        List<BasePipelineStationRelEnt> basePipelineStationRelEnts = pipelineStationRel.selectList(queryPipeline);
        if (ObjectUtils.isNotEmpty(basePipelineStationRelEnts)) {
            BasePipelineStationRelEnt basePipelineStationRelEnt = basePipelineStationRelEnts.get(0);
            if (ObjectUtils.isNotEmpty(basePipelineStationRelEnt)) {
                output.setPipelineId(basePipelineStationRelEnt.getPipelineId());
            }
        }
        //查询场站关系
        QueryWrapper<BaseStationOuRelEnt> warpper = new QueryWrapper<>();
        warpper.lambda().eq(BaseStationOuRelEnt::getStationId, input.getId())
                .gt(BaseStationOuRelEnt::getEndTime, new Date());
        List<BaseStationOuRelEnt> baseStationOuRelEnts = stationOuView.selectList(warpper);
        if (ObjectUtils.isNotEmpty(baseStationOuRelEnts)) {
            BaseStationOuRelEnt baseStationOuRelEnt = baseStationOuRelEnts.get(0);
            if (baseStationOuRelEnt != null && baseStationOuRelEnt.getOuId() != null) {
                GetSysOrganizationViewInput view = new GetSysOrganizationViewInput();
                view.setId(baseStationOuRelEnt.getOuId());
                GetSysOrganizationViewOutput sysOrganization = sysOrgService.getSysOrganizationById(context, view).getResult();
                output.setOuId(baseStationOuRelEnt.getOuId());
                if (ObjectUtils.isNotEmpty(sysOrganization)) {
                    output.setOuName(sysOrganization.getOuName());
                }
            }
        }
        if (list != null && list.size() > 0) {
            output.setItem(list);
        } else {
            output.setItem(new ArrayList<>());
        }
        if(null != output.getEndTime() && XDateUtils.compare(output.getEndTime() , new Date()) <= 0)
            output.setStatus(1);
        else
            output.setStatus(0);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QueryBaseStationOutput> dynamicQueryBaseStation(XContext context, CustomQueryInput input) {
        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        return XMapperHelper.query(mapper, input, BaseStationEnt.class, QueryBaseStationOutput.class);
    }

    @XApiPost
    public XSingleResult<GetBaseStationViewOutput> getBaseStationView(XContext context, GetBaseStationViewInput input) {
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        BaseStationView record = new BaseStationView();
        XCopyUtils.copyObject(input, record);
        BaseStationView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseStationViewOutput output = new GetBaseStationViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XText("根据管线id获取场站列表")
    @XApiPost
    public XPageResult<QueryBaseStationViewOutput> queryBaseStationView(XContext context, QueryBaseStationViewInput input) {
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        SysOrganizationCloudService sysOrganizationCloudService = context.getBean(SysOrganizationCloudService.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        BaseStationView record = new BaseStationView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setEndTime(new Date());
        List<BaseStationView> list = mapper.selectList(record);
        PageInfo<BaseStationView> pageInfo = new PageInfo<>(list);
        List<QueryBaseStationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationViewOutput.class);
        //调用微服务查询字典
        CreateSysDictionaryInput sysDictionaryView = new CreateSysDictionaryInput();
        sysDictionaryView.setAlias("STATION_TYPE");
        XListResult<QuerySysDictionaryViewOutput> dicList = sysDictionaryService.querySysDictionarysByAlias(context, sysDictionaryView);
        List<QuerySysDictionaryViewOutput> dics = dicList.getResult();
        /*//调用微服务查询地区
        GetSysAreaInput sysOrgView = new GetSysAreaInput();
        XListResult<GetSysAreaOutput> sysAreaList = sysAreaService.getSysAreaList(context, sysOrgView);
        List<GetSysAreaOutput> items = sysAreaList.getResult();*/
        //调用微服务查询组织机构
        QuerySysOrganizationViewInput sysOrgView = new QuerySysOrganizationViewInput();
        sysOrgView.setPage(1);
        sysOrgView.setLimit(9999);
        XPageResult<GetSysOrganizationViewOutput> sysOrganizationList = sysOrganizationCloudService.getSysOrganizationList(context, sysOrgView);
        List<GetSysOrganizationViewOutput> items = sysOrganizationList.getResult().getItems();
        if (outputs != null && outputs.size() > 0) {
            int i = 1;
            for (QueryBaseStationViewOutput baseStation : outputs) {
                if (items != null && items.size() > 0) {
                    for (GetSysOrganizationViewOutput orgs : items) {
                        if (StringUtils.equals(baseStation.getOuId(), String.valueOf(orgs.getId()))) {
                            baseStation.setOuName(orgs.getOuName());
                        }
                    }
                }
                if (dics != null && dics.size() > 0) {
                    for (QuerySysDictionaryViewOutput dic : dics) {
                        if (StringUtils.equals(baseStation.getStationType(), dic.getDicKey())) {
                            baseStation.setStationTypeName(dic.getDicName());
                        }
                    }
                }
                baseStation.setXh(i++ + "");
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据调度台获取场站列表")
    @XApiPost
    public XPageResult<QueryBaseStationViewOutput> queryBaseStationViewByDispatchDesk(XContext context, QueryBaseStationViewInput input) {
        //1.先获取调度台下所有的管线
        BasePipelineMapper pipelineMapper = context.getBean(BasePipelineMapper.class);
        QueryWrapper<BasePipelineEnt> queryPipelineWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(input.getMediaType()) && !StringUtils.equals("ALL", input.getMediaType())) {
            queryPipelineWrapper.lambda().eq(BasePipelineEnt::getMediumType, input.getMediaType());
        }
        queryPipelineWrapper.lambda().le(BasePipelineEnt::getEffectTime, new Date())
                .gt(BasePipelineEnt::getEndTime, new Date());
        if(XStringUtils.isNotEmpty(input.getDispatchDesk()))
            queryPipelineWrapper.lambda().like(BasePipelineEnt::getDispatchDesk, input.getDispatchDesk());
        if(XStringUtils.isNotEmpty(input.getConsoleId()))
            queryPipelineWrapper.lambda().like(BasePipelineEnt::getDispatchDesk, input.getConsoleId());
        List<BasePipelineEnt> pipelineList = pipelineMapper.selectList(queryPipelineWrapper);
        List<String> pilelineIds = pipelineList.stream().map(BasePipelineEnt::getId).collect(Collectors.toList());
        if(CollectionUtils.isEmpty(pilelineIds))
            pilelineIds.add("1111-1111-cs");
        //2.根据管线获取场站
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        BaseStationView record = new BaseStationView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setEndTime(new Date());
        record.setPipelineIdList(pilelineIds);
        List<BaseStationView> list = mapper.selectListByPipelineIds(record);
        PageInfo<BaseStationView> pageInfo = new PageInfo<>(list);
        List<QueryBaseStationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据管线id获取所有场站列表")
    @XApiPost
    public XPageResult<QueryBaseStationViewOutput> queryBaseStationViewAll(XContext context, QueryBaseStationViewInput input) {
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        SysOrganizationCloudService sysOrganizationCloudService = context.getBean(SysOrganizationCloudService.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        BaseStationView record = new BaseStationView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setEndTime(new Date());
        List<BaseStationView> list = mapper.selectAllList(record);
        PageInfo<BaseStationView> pageInfo = new PageInfo<>(list);
        List<QueryBaseStationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationViewOutput.class);
        //调用微服务查询字典
        CreateSysDictionaryInput sysDictionaryView = new CreateSysDictionaryInput();
        sysDictionaryView.setAlias("STATION_TYPE");
        XListResult<QuerySysDictionaryViewOutput> dicList = sysDictionaryService.querySysDictionarysByAlias(context, sysDictionaryView);
        List<QuerySysDictionaryViewOutput> dics = dicList.getResult();
        QuerySysOrganizationViewInput sysOrgView = new QuerySysOrganizationViewInput();
        sysOrgView.setPage(1);
        sysOrgView.setLimit(9999);
        XPageResult<GetSysOrganizationViewOutput> sysOrganizationList = sysOrganizationCloudService.getSysOrganizationList(context, sysOrgView);
        List<GetSysOrganizationViewOutput> items = sysOrganizationList.getResult().getItems();
        if (outputs != null && outputs.size() > 0) {
            int i = 1;
            for (QueryBaseStationViewOutput baseStation : outputs) {
                if (items != null && items.size() > 0) {
                    for (GetSysOrganizationViewOutput orgs : items) {
                        if (StringUtils.equals(baseStation.getOuId(), String.valueOf(orgs.getId()))) {
                            baseStation.setOuName(orgs.getOuName());
                        }
                    }
                }
                if (dics != null && dics.size() > 0) {
                    for (QuerySysDictionaryViewOutput dic : dics) {
                        if (StringUtils.equals(baseStation.getStationType(), dic.getDicKey())) {
                            baseStation.setStationTypeName(dic.getDicName());
                        }
                    }
                }
                baseStation.setXh(i++ + "");
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据管线id获取场站列表（不包含当前管线场站）")
    @XApiPost
    public XPageResult<QueryBaseStationViewOutput> queryBaseStationUnIncludePipelineIdView(XContext context, QueryBaseStationViewInput input) {
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        SysOrganizationCloudService sysOrganizationCloudService = context.getBean(SysOrganizationCloudService.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        BaseStationView record = new BaseStationView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setEndTime(new Date());
        List<BaseStationView> list = mapper.selectUnIncludePipelineList(record);
        PageInfo<BaseStationView> pageInfo = new PageInfo<>(list);
        List<QueryBaseStationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationViewOutput.class);
        /*//调用微服务查询地区
        GetSysAreaInput sysOrgView = new GetSysAreaInput();
        XListResult<GetSysAreaOutput> sysAreaList = sysAreaService.getSysAreaList(context, sysOrgView);
        List<GetSysAreaOutput> items = sysAreaList.getResult();*/
        if (outputs != null && outputs.size() > 0) {
            int i = 1;
            for (QueryBaseStationViewOutput baseStation : outputs) {
                baseStation.setXh(i++ + "");
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }


    @XText("初始化场站颜色(暂时不用)")
    @XApiPost
    public XSingleResult updateStationColor(XContext context, QueryBaseStationViewInput input) {
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        BaseStationMapper baseStationMapper = context.getBean(BaseStationMapper.class);
        BaseStationView baseStation = new BaseStationView();
        List<BaseStationView> baseStationViews = mapper.selectList(baseStation);
        for (BaseStationView obj : baseStationViews) {
            BaseStationEnt baseStationEnt = new BaseStationEnt();
            baseStationEnt.setId(obj.getId());
            baseStationEnt.setStationColor(colorGenar());
            baseStationMapper.updateById(baseStationEnt);
        }

        return XSingleResult.success();
    }

    private String colorGenar() {
        //红色
        String red;
        //绿色
        String green;
        //蓝色
        String blue;
        //生成随机对象
        Random random = new Random();
        //生成红色颜色代码
        red = Integer.toHexString(random.nextInt(256)).toUpperCase();
        //生成绿色颜色代码
        green = Integer.toHexString(random.nextInt(256)).toUpperCase();
        //生成蓝色颜色代码
        blue = Integer.toHexString(random.nextInt(256)).toUpperCase();

        //判断红色代码的位数
        red = red.length() == 1 ? "0" + red : red;
        //判断绿色代码的位数
        green = green.length() == 1 ? "0" + green : green;
        //判断蓝色代码的位数
        blue = blue.length() == 1 ? "0" + blue : blue;
        //生成十六进制颜色值
        String color = "#" + red + green + blue;

        return color;
    }

    @XText("修改管段排序(暂时不用)")
    @XApiPost
    public XServiceResult updatePipelineSortById(XContext context, QueryBaseSegmentViewInput input) {
        BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
        BasePipelineStationRelMapper relMapper = context.getBean(BasePipelineStationRelMapper.class);
        QueryWrapper<BasePipelineStationRelEnt> query = new QueryWrapper();
        QueryWrapper<BaseStationEnt> querySegent = new QueryWrapper();
        QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(BaseStationEnt::getId, input.getId());
        //查询修改的场站sort值
        BaseStationEnt baseStationEnt = mapper.selectOne(queryWrapper);
        //查询当前场站所属管线
        query.lambda().eq(BasePipelineStationRelEnt::getStationId, baseStationEnt.getId()).gt(BasePipelineStationRelEnt::getEndTime, new Date());
        List<BasePipelineStationRelEnt> basePipelineStationRelEnts = relMapper.selectList(query);
        //查询管线中下一条管段或者上一条管段
        QueryWrapper<BasePipelineStationRelEnt> segmentWrapper = new QueryWrapper();
        String pipelineId = basePipelineStationRelEnts.get(0).getPipelineId();
        segmentWrapper.lambda().eq(BasePipelineStationRelEnt::getPipelineId, pipelineId);
        if (StringUtils.equals("up", input.getSortFlag())) {
            segmentWrapper.lambda().lt(BasePipelineStationRelEnt::getStationId, baseStationEnt.getId()).orderByDesc(BasePipelineStationRelEnt::getSort);
        } else {
            segmentWrapper.lambda().gt(BasePipelineStationRelEnt::getStationId, baseStationEnt.getId()).orderByAsc(BasePipelineStationRelEnt::getSort);
        }
        List<BasePipelineStationRelEnt> basePipelineStationRel = relMapper.selectList(segmentWrapper);
        //根据上一条或者下一条管段id查询管段
        if (basePipelineStationRel != null && basePipelineStationRel.size() > 0) {
            BasePipelineStationRelEnt basePipelineStationRelEnt = basePipelineStationRel.get(0);
            querySegent.lambda().eq(BaseStationEnt::getId, basePipelineStationRelEnt.getStationId());
        }
        BaseStationEnt baseStation = mapper.selectOne(querySegent);
        //交换sort
        Integer sort = baseStation.getSort();
        baseStation.setSort(baseStationEnt.getSort());
        baseStationEnt.setSort(sort);
        mapper.updateById(baseStation);
        mapper.updateById(baseStationEnt);
        return XServiceResult.OK;
    }

    @XApiGet
    @XText("根据ouId获取场站")
    public XListResult<GetBaseStationOutput> selectStationListByOuId(XContext context, GetBaseStationInput input) {
        BaseOuIdInput ouIdInput = new BaseOuIdInput();
        XCopyUtils.copyObject(input, ouIdInput);
        GetSysOrganizationViewOutput sysOrganization = getSysOrganization(context);
        if((sysOrganization.getOuLevel().equals("STATION")
                ||sysOrganization.getOuLevel().equals("4106"))){
            ouIdInput.setOuId(sysOrganization.getId());
        }
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        List<BaseStationView> stationOutputList = mapper.selectAllStationByOuId(ouIdInput.getOuId() , input.getMediaType());
        List<GetBaseStationOutput> outputs = XCopyUtils.copyNewList(stationOutputList , GetBaseStationOutput.class);
        List<QuerySysDictionaryViewOutput> dictionaryViewOutputs = getMenuDict(context , "MEDIUM_TYPE");
        outputs.stream().forEach(item->{
            if(XStringUtils.isNotEmpty(item.getMediumType())){
                item.setMediumTypeName(item.getStationName() + "-" + getDictName(item.getMediumType() , dictionaryViewOutputs));
            }else
                item.setMediumTypeName(item.getStationName());
        });
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据ouId和介质类型获取场站")
    public XListResult<GetBaseStationOutput> selectStationListByOuIdAndMediumType(XContext context, GetBaseStationInput input) {
        BaseOuIdInput ouIdInput = new BaseOuIdInput();
        XCopyUtils.copyObject(input, ouIdInput);
        GetSysOrganizationViewOutput sysOrganization = getSysOrganization(context);
        if((sysOrganization.getOuLevel().equals("STATION")
                ||sysOrganization.getOuLevel().equals("4106"))){
            ouIdInput.setOuId(sysOrganization.getId());
        }
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        List<BaseStationView> stationOutputList = mapper.selectAllStationByOuIdAndMediumType(ouIdInput.getOuId() , input.getMediaType());
        List<GetBaseStationOutput> outputs = XCopyUtils.copyNewList(stationOutputList , GetBaseStationOutput.class);
        List<QuerySysDictionaryViewOutput> dictionaryViewOutputs = getMenuDict(context , "MEDIUM_TYPE");
        outputs.stream().forEach(item->{
            if(XStringUtils.isNotEmpty(item.getMediumType())){
                item.setMediumTypeName(item.getStationName() + "-" + getDictName(item.getMediumType() , dictionaryViewOutputs));
            }else
                item.setMediumTypeName(item.getStationName());
        });
        return XListResult.success(outputs);
    }
    private GetSysOrganizationViewOutput getSysOrganization(XContext context ){
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysOrganizationCloudService system = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput getSysOrganizationViewInput = new GetSysOrganizationViewInput();
        getSysOrganizationViewInput.setId(session.getOuId());
        XSingleResult<GetSysOrganizationViewOutput> outputXSingleResult = system.getSysOrganizationById(context , getSysOrganizationViewInput);
        outputXSingleResult.throwIfFail();
        return outputXSingleResult.getResult();
    }


    public List<QuerySysDictionaryViewOutput> getMenuDict(XContext context , String alias ){
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }

    public String getDictName(String dictKey,List<QuerySysDictionaryViewOutput> dictList){
        if(StringUtils.isBlank(dictKey) || CollectionUtils.isEmpty(dictList)){
            return " ";
        }
        for (QuerySysDictionaryViewOutput dict : dictList) {
            if(dict.getDicKey().equals(dictKey)){
                return dict.getDicName();
            }
        }
        return dictKey;
    }

    @XText("分页条件查询场站")
    @XApiPost
    public XPageResult<QueryBaseStationOutput> queryBaseStationAll(XContext context, QueryBaseStationInput input) {
        BaseStationViewMapper mapper = context.getBean(BaseStationViewMapper.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        BaseStationView record = new BaseStationView();
        if(XStringUtils.isNotEmpty(input.getMediumType()) && input.getMediumType().equals("ALL"))
            input.setMediumType("");
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setEndTime(new Date());
        List<BaseStationView> list = mapper.selectStationAllList(record);
        PageInfo<BaseStationView> pageInfo = new PageInfo<>(list);
        List<QueryBaseStationOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseStationOutput.class);
        //调用微服务查询字典
        CreateSysDictionaryInput sysDictionaryView = new CreateSysDictionaryInput();
        sysDictionaryView.setAlias("STATION_TYPE");
        XListResult<QuerySysDictionaryViewOutput> dicList = sysDictionaryService.querySysDictionarysByAlias(context, sysDictionaryView);
        sysDictionaryView.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediList = sysDictionaryService.querySysDictionarysByAlias(context, sysDictionaryView);
        List<QuerySysDictionaryViewOutput> dics = dicList.getResult();
        List<QuerySysDictionaryViewOutput> medis = mediList.getResult();
        if (outputs != null && outputs.size() > 0) {
            for (QueryBaseStationOutput baseStation : outputs) {
                if (dics != null && dics.size() > 0) {
                    for (QuerySysDictionaryViewOutput dic : dics) {
                        if (StringUtils.equals(baseStation.getStationType(), dic.getDicKey())) {
                            baseStation.setStationTypeName(dic.getDicName());
                        }
                    }
                    for (QuerySysDictionaryViewOutput dic : medis) {
                        if (StringUtils.equals(baseStation.getMediumType(), dic.getDicKey())) {
                            baseStation.setMediumTypeName(dic.getDicName());
                        }
                    }
                }
                baseStation.setStatus(0);
                if(XDateUtils.compare(baseStation.getEndTime() , new Date()) <= 0)
                    baseStation.setStatus(1);
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("删除场站")
    @XApiPost
    public XServiceResult deleteBaseStationAllRel(XContext context, DeleteBaseStationInput input) {
        return XTransactionHelper.begin(context, () -> {
            //删除站，逻辑删除，is_delete
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseStationEnt::getId , input.getId());
            queryWrapper.lambda().eq(BaseStationEnt::getIsDelete , 0);
            BaseStationEnt baseStationEnt = mapper.selectOne(queryWrapper);
            Date currentDate = new Date();
            if(null != baseStationEnt){
                baseStationEnt.setIsDelete(1);
                baseStationEnt.setEndTime(currentDate);
                baseStationEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
                baseStationEnt.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                baseStationEnt.setModifyTime(new Date());
                mapper.updateById(baseStationEnt);
            }
            //删除组织机构关系,失效处理
            BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
            QueryWrapper<BaseStationOuRelEnt> queryStationOuRelWrapper = new QueryWrapper();
            queryStationOuRelWrapper.lambda().eq(BaseStationOuRelEnt::getStationId, baseStationEnt.getId())
                    .gt(BaseStationOuRelEnt::getEndTime, new Date());
            List<BaseStationOuRelEnt> baseStationOuRelEntList = stationOuRelMapper.selectList(queryStationOuRelWrapper);
            baseStationOuRelEntList.stream().forEach(item->{
                item.setEndTime(new Date());
                item.setModifyById(context.getSession(PpsUserSession.class).getId());
                item.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                item.setModifyTime(new Date());
                stationOuRelMapper.updateById(item);
            });
            //删除管线关系,失效处理
            BasePipelineStationRelMapper pipelineStationRelMapper = context.getBean(BasePipelineStationRelMapper.class);
            QueryWrapper<BasePipelineStationRelEnt> queryPipelineStationRelWrapper = new QueryWrapper();
            queryPipelineStationRelWrapper.lambda().eq(BasePipelineStationRelEnt::getStationId, input.getId())
                    .gt(BasePipelineStationRelEnt::getEndTime, new Date());
            List<BasePipelineStationRelEnt> basePipelineStationRelEntList = pipelineStationRelMapper.selectList(queryPipelineStationRelWrapper);
            basePipelineStationRelEntList.stream().forEach(item->{
                item.setEndTime(new Date());
                item.setModifyById(context.getSession(PpsUserSession.class).getId());
                item.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                item.setModifyTime(new Date());
                pipelineStationRelMapper.updateById(item);
            });
            //删除场站和客户管线，失效处理
            BaseClientStationRelMapper baseClientStationRelMapper = context.getBean(BaseClientStationRelMapper.class);
            QueryWrapper<BaseClientStationRelEnt> queryClientStationRelWrapper = new QueryWrapper();
            queryClientStationRelWrapper.lambda().eq(BaseClientStationRelEnt::getStationId, input.getId())
                    .gt(BaseClientStationRelEnt::getEndTime, new Date());
            List<BaseClientStationRelEnt> baseClientStationRelList = baseClientStationRelMapper.selectList(queryClientStationRelWrapper);
            baseClientStationRelList.stream().forEach(item->{
                item.setEndTime(new Date());
                item.setModifyById(context.getSession(PpsUserSession.class).getId());
                item.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                item.setModifyTime(new Date());
                baseClientStationRelMapper.updateById(item);
            });
            return XServiceResult.OK;

        });

    }

    @XText("修改场站状态")
    @XApiPost
    public XServiceResult updateBaseStationStatus(XContext context, UpdateBaseStationInput input) {
        //1、修改场站的有效时间
        return XTransactionHelper.begin(context, () -> {
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            //更新场站详细信息
            QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BaseStationEnt::getId, input.getId());
            BaseStationEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            if(input.getStatus() == null || input.getStatus() == 0)
                entity.setEndTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00"));
            else
                entity.setEndTime(new Date());
            XCopyUtils.copyObject(input, entity);
            entity.setModifyById(context.getSession(PpsUserSession.class).getId());
            entity.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
            entity.setModifyTime(new Date());
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("模板导出")
    @XApiGet
    public XFileResult exportBaseStationData(XContext context, GetBaseStationInput input) {
        try {
            List<BaseStationEnt> baseStationEnts = new ArrayList<>();
            String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/场站信息.xlsx");
            String templateFilePath = "template/exportBaseStationTemplate.xlsx";
            BaseStationEnt example = new BaseStationEnt();
            example.setStationName("场站");
            example.setStationCode("1111");
            example.setShortName("场站");
            example.setSort(0);
            baseStationEnts.add(example);
            if (!CollectionUtils.isEmpty(baseStationEnts)) {
                Map<String, Object> params = new HashMap<>();
                params.put("list", baseStationEnts);
                InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
                byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, params);
                ExportExcelUtils.outputFileData(excelContent, fileSavePath);
                return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
            } else {
                return XFileResult.error(200, "暂无数据导出");
            }
        } catch (Exception ex) {
            context.getLogger().error("导出出错", ex);
            return XFileResult.error(500, "导出失败");
        }
    }

    @XText("导入场站信息数据")
    @XApiUpload
    public XServiceResult importBaseStationData(XContext context, ImportBaseStationInput input) {
        try {
            if (input.getFile() == null)
                return XServiceResult.error(500, "未获取上传文件");
            PpsUserSession session = context.getSession(PpsUserSession.class);
            List<BaseStationEnt> successEntList =  this.importExcelData(context, input );
            BaseStationMapper detailMapper = context.getBean(BaseStationMapper.class);
            XThreadHelper.async(new Runnable() {
                @Override
                public void run() {
                    for (int i = 0; i < successEntList.size(); i++) {
                        Date currentDate = new Date();
                        BaseStationEnt record = successEntList.get(i);
                        record.setId(UUID.randomUUID().toString());
                        record.setCreateById(session.getId());
                        record.setCreateByName(session.getUserName());
                        record.setCreateTime(currentDate);
                        record.setEndTime(DateHelper.ENDTIME);
                        record.setEffectTime(new Date());
                        record.setIsDelete(0);
                        record.setIsTurnoverUse(0);
                        record.setIsCustodyUse(0);
                        detailMapper.insert(record);
                    }
                }
            });
            return XServiceResult.OK;
        } catch (Exception ex) {
            context.getLogger().error("导入失败", ex);
            return XServiceResult.error(500, "导入失败：" + ex.toString().replace("java.lang.Exception:", ""));
        }
    }

    private List<BaseStationEnt> importExcelData(XContext context, ImportBaseStationInput param) throws Exception {
        List<BaseStationEnt> result = new ArrayList<>();
        String fileName = param.getFile().getFileName();
        InputStream input = param.getFile().getInputStream();
        this.checkError(!fileName.endsWith(".xlsx"),"导入模板文件格式不正确，请核对(文档格式应为.xlsx)");
        Workbook wb = new XSSFWorkbook(input);
        Sheet sheet = wb.getSheetAt(0);
        Iterator<Row> rows = sheet.rowIterator();
        this.checkError(rows==null || !rows.hasNext(),"未导入任何数据");
        DataFormatter formatter = new DataFormatter();
        List<String> tempRowTitle= this.createTitles();
        Row titleRowFromExcel = rows.next();
        String titleErrorMessage="";
        isNeedNotEmptyMap.clear();
        for (int i = 0; i < tempRowTitle.size(); i++) {
            String titleFormExcel = formatter.formatCellValue(titleRowFromExcel.getCell(i)).trim();
            isNeedNotEmptyMap.put(i,titleFormExcel.startsWith("*"));
            titleFormExcel=titleFormExcel.replace("*","");

            if(!tempRowTitle.get(i).equals(titleFormExcel)){
                titleErrorMessage = titleErrorMessage.concat("第"+ (i+1)+"列表头不正确应为“"+tempRowTitle.get(i)+"”,");
            }
        }
        this.checkError(StringUtils.isNotBlank(titleErrorMessage),titleErrorMessage+"请根据模板修改；");

        StringBuilder errorMessageBuilder=new StringBuilder();
        while (rows.hasNext()) {
            Row row = rows.next();
            int rowIndex=row.getRowNum();

            if (ImportExcelUtils.isRowEmpty(row))
            {
                errorMessageBuilder.append("第"+rowIndex+"行数据不能为空；");
                continue;
            }

            String dateErrorMessage = this.checkEmptyData(row , tempRowTitle);
            if(StringUtils.isNotBlank(dateErrorMessage))
            {
                errorMessageBuilder.append(dateErrorMessage);
                continue;
            }
            BaseStationEnt resultItem=new BaseStationEnt();
            for (int i = 0; i < tempRowTitle.size(); i++) {
                Cell cell = row.getCell(i);
                String cellText=formatter.formatCellValue(cell).trim();
                String titleName=tempRowTitle.get(i);

                if(StringUtils.isBlank(cellText))
                    continue;
                switch (i) {
                    case 0: //场站名称
                        resultItem.setStationName(cellText);
                        break;
                    case 1: //场站编码
                        resultItem.setStationCode(cellText);
                        break;
                    case 2: //简称
                        resultItem.setShortName(cellText);
                        break;
                    case 4: //排序
                        try {
                            resultItem.setSort(Integer.valueOf(cellText));
                        } catch (Exception ex) {
                            errorMessageBuilder.append("第" + rowIndex + "行'" + titleName + "'的格式不正确；");
                        }
                        break;
                    default:
                        break;
                }
            }
            if(resultItem!=null)
                result.add(resultItem);
        }
        this.checkError(errorMessageBuilder.length()>0,errorMessageBuilder.toString());
        return result;
    }

    private List<String> createTitles() {
        List<String> titles = new ArrayList<String>();
        titles.add("场站名称");
        titles.add("场站编码");
        titles.add("场站简称");
        titles.add("排序");
        return titles;
    }

    private void checkError(boolean isError, String errorMessage) throws Exception {
        if (isError)
            throw new Exception(errorMessage);
    }

    private String checkEmptyData(Row row , List<String> tempRowTitle){
        String result="";
        DataFormatter formatter = new DataFormatter();
        for (int i = 0; i < isNeedNotEmptyMap.size(); i++) {
            Cell cell = row.getCell(i);
            Boolean isNeedNotEmpty=isNeedNotEmptyMap.get(i);
            Boolean isEmptyValue=StringUtils.isBlank(formatter.formatCellValue(cell));

            if(isNeedNotEmpty && isEmptyValue){
                result = result.concat(String.format( "“%s”",tempRowTitle.get(i).replace("*","")));
            }
        }
        if(StringUtils.isNotBlank(result))
        {
            int rowNum=row.getRowNum();
            result="第"+rowNum+"行"+result+"不能为空；";
        }
        return result;
    }

    @XText("批量修改场站组织机构")
    @XApiPost
    public XServiceResult updateBaseStationOuId(XContext context, UpdateBaseStationInput input) {
        //1、修改场站基本信息 2、修改场站组织机构关联关系(先删除旧的关系，创建新的关系)
        return XTransactionHelper.begin(context, () -> {
            BaseStationMapper mapper = context.getBean(BaseStationMapper.class);
            BaseStationOuRelMapper stationOuRelMapper = context.getBean(BaseStationOuRelMapper.class);
            Date endTime = new Date();
            try {
                endTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }

            for(String id : input.getIdList()){
                //更新场站详细信息
                QueryWrapper<BaseStationEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(BaseStationEnt::getId, id);
                BaseStationEnt entity = mapper.selectOne(queryWrapper);
                if (entity == null) {
                    return XServiceResult.error(context, XError.NotFound);
                }
                entity.setModifyById(context.getSession(PpsUserSession.class).getId());
                entity.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                entity.setModifyTime(new Date());
                mapper.updateById(entity);

                //删除旧的场站组织关系
                QueryWrapper<BaseStationOuRelEnt> ouWrapper = new QueryWrapper<>();
                ouWrapper.lambda().eq(BaseStationOuRelEnt::getStationId, id)
                        .gt(BaseStationOuRelEnt::getEndTime, new Date());
                List<BaseStationOuRelEnt> baseStationOuRelEnts = stationOuRelMapper.selectList(ouWrapper);
                if (ObjectUtils.isNotEmpty(baseStationOuRelEnts)) {
                    for (BaseStationOuRelEnt stationEnt : baseStationOuRelEnts) {
                        if (ObjectUtils.isNotEmpty(stationEnt)) {
                            stationEnt.setEndTime(new Date());
                            stationEnt.setModifyById(context.getSession(PpsUserSession.class).getId());
                            stationEnt.setModifyByName(context.getSession(PpsUserSession.class).getLoginName());
                            stationEnt.setModifyTime(new Date());
                            stationOuRelMapper.updateById(stationEnt);
                        }
                    }
                }
                //添加新的场站和组织关联表
                BaseStationOuRelEnt baseStationOuRelEnt = new BaseStationOuRelEnt();
                baseStationOuRelEnt.setStationId(id);
                baseStationOuRelEnt.setOuId(input.getOuId());
                baseStationOuRelEnt.setId(UUID.randomUUID().toString());
                baseStationOuRelEnt.setEffectTime(new Date());
                baseStationOuRelEnt.setEndTime(endTime);
                baseStationOuRelEnt.setCreateById(context.getSession(PpsUserSession.class).getId());
                baseStationOuRelEnt.setCreateByName(context.getUser().getName());
                baseStationOuRelEnt.setCreateTime(new Date());
                stationOuRelMapper.insert(baseStationOuRelEnt);
            }
            return XServiceResult.OK;
        });
    }
}
