package pps.core.base.service;

import java.util.ArrayList;
import java.util.Date;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.BaseBaseStationService;
import pps.cloud.base.service.data.BaseOuIdInput;
import pps.cloud.base.service.data.QueryBaseStationOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.scada_gather_box_m_station.*;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XStringUtils;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XService
@XText("scada权限配置管理")
public class ScadaGatherBoxMStationService {

    @XText("查询scada权限场站：GatherBoxId")
    @XApiPost
    public XListResult<GetScadaGatherBoxMStationOutput> getStationAllByGatherBoxId(XContext context, GetScadaGatherBoxMStationInput input){
        ScadaGatherBoxMStationMapper mapper = context.getBean(ScadaGatherBoxMStationMapper.class);
        QueryWrapper<ScadaGatherBoxMStationEnt> queryWrapper = new QueryWrapper<>();
        if (XStringUtils.isNotEmpty(input.getStationName())){
            queryWrapper.lambda().like(ScadaGatherBoxMStationEnt::getStationName,input.getStationName());
        }
        //获取组织机构下面的，所有场站id
        if (XStringUtils.isNotEmpty(input.getOuId())){
            //queryWrapper.lambda().eq(ScadaGatherBoxMStationEnt::getOuId,input.getOuId());
            BaseOuIdInput ouIdInput = new BaseOuIdInput();
            ouIdInput.setOuId(input.getOuId());
            if(!XStringUtils.isEmpty(input.getOuId()))
                ouIdInput.setOuId(input.getOuId());
            List<QueryBaseStationOutput> stationOutputList = selectStationListByOuId(context , ouIdInput);
            List<String> stringList = stationOutputList.stream().filter(item -> XStringUtils.isNotEmpty(item.getId())).map(QueryBaseStationOutput::getId).distinct().collect(Collectors.toList());
            if(CollectionUtils.isEmpty(stringList))
                stringList.add(input.getOuId());
            queryWrapper.lambda().in(ScadaGatherBoxMStationEnt::getStationId,stringList);
        }
        queryWrapper.lambda().eq(ScadaGatherBoxMStationEnt::getGatherBoxId, input.getGatherBoxId()).orderByDesc(ScadaGatherBoxMStationEnt::getCreateTime);
        List<ScadaGatherBoxMStationEnt> stationEnts = mapper.selectList(queryWrapper);
        List<GetScadaGatherBoxMStationOutput> outputs = XCopyUtils.copyNewList(stationEnts, GetScadaGatherBoxMStationOutput.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        //调用微服务查询字典
        CreateSysDictionaryInput businessType = new CreateSysDictionaryInput();
        businessType.setAlias("business_type");
        XListResult<QuerySysDictionaryViewOutput> businessTypeList = sysDictionaryService.querySysDictionarysByAlias(context, businessType);

        CreateSysDictionaryInput mediaType = new CreateSysDictionaryInput();
        mediaType.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediaType);
        for (GetScadaGatherBoxMStationOutput output : outputs) {
            if (XStringUtils.isNotEmpty(output.getBusinessType())){
                businessTypeList.getResult().forEach(querySysDictionaryViewOutput -> {
                    if (output.getBusinessType().equals(querySysDictionaryViewOutput.getDicKey())){
                        output.setBusinessTypeName(querySysDictionaryViewOutput.getDicName());
                    }
                });
            }

            if (XStringUtils.isNotEmpty(output.getMediaType())){
                mediaTypeList.getResult().forEach(querySysDictionaryViewOutput -> {
                    if (output.getMediaType().equals(querySysDictionaryViewOutput.getDicKey())){
                        output.setMediaTypeName(querySysDictionaryViewOutput.getDicName());
                    }
                });
            }
        }
        return XListResult.success(outputs);
    }

    @XText("查询scada权限场站：ouId")
    @XApiPost
    public XListResult<GetScadaGatherBoxMStationOutput> getStationAllByOuId(XContext context, GetScadaGatherBoxMStationInput input){
        SysOrganizationCloudService sysOuMapper = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(input.getOuId());
        XListResult<GetOuListTreeOutput> listByOuId = sysOuMapper.getAllOuListByOuId(context, ouIdInput);
        List<String> ouIds = listByOuId.getResult().stream().map(GetOuListTreeOutput::getOuId).collect(Collectors.toList());
        ScadaGatherBoxMStationMapper mapper = context.getBean(ScadaGatherBoxMStationMapper.class);
        QueryWrapper<ScadaGatherBoxMStationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("DISTINCT ou_id,ou_name,station_id,station_name,media_type").lambda().in(ScadaGatherBoxMStationEnt::getOuId, ouIds).orderByDesc(ScadaGatherBoxMStationEnt::getStationId);
        List<ScadaGatherBoxMStationEnt> stationEnts = mapper.selectList(queryWrapper);
        List<GetScadaGatherBoxMStationOutput> outputs = XCopyUtils.copyNewList(stationEnts, GetScadaGatherBoxMStationOutput.class);
        /*SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        //调用微服务查询字典
        CreateSysDictionaryInput businessType = new CreateSysDictionaryInput();
        businessType.setAlias("business_type");
        XListResult<QuerySysDictionaryViewOutput> businessTypeList = sysDictionaryService.querySysDictionarysByAlias(context, businessType);

        CreateSysDictionaryInput mediaType = new CreateSysDictionaryInput();
        mediaType.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediaType);
        for (GetScadaGatherBoxMStationOutput output : outputs) {
            if (XStringUtils.isNotEmpty(output.getBusinessType())){
                businessTypeList.getResult().forEach(querySysDictionaryViewOutput -> {
                    if (output.getBusinessType().equals(querySysDictionaryViewOutput.getDicKey())){
                        output.setBusinessTypeName(querySysDictionaryViewOutput.getDicName());
                    }
                });
            }

            if (XStringUtils.isNotEmpty(output.getMediaType())){
                mediaTypeList.getResult().forEach(querySysDictionaryViewOutput -> {
                    if (output.getMediaType().equals(querySysDictionaryViewOutput.getDicKey())){
                        output.setMediaTypeName(querySysDictionaryViewOutput.getDicName());
                    }
                });
            }
        }*/
        return XListResult.success(outputs);
    }

    @XText("新增权限配置")
    @XApiPost
    public XServiceResult createScadaGatherBoxMStation(XContext context, CreateScadaGatherBoxMStationInputs input)throws XServiceException {
        return XTransactionHelper.begin(context,()-> {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            ScadaGatherBoxMStationMapper mapper = context.getBean(ScadaGatherBoxMStationMapper.class);
            if (!CollectionUtils.isEmpty(input.getInputs())){
                List<String> stationIds = input.getInputs().stream().map(CreateScadaGatherBoxMStationInput::getStationId).collect(Collectors.toList());
                QueryWrapper<ScadaGatherBoxMStationEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(ScadaGatherBoxMStationEnt::getBusinessType,input.getBusinessType())
                        .in(ScadaGatherBoxMStationEnt::getStationId,stationIds);
                List<ScadaGatherBoxMStationEnt> stationEnts = mapper.selectList(wrapper);

                List<ScadaGatherBoxMStationEnt> mStationEnts = XCopyUtils.copyNewList(input.getInputs(), ScadaGatherBoxMStationEnt.class);
                mStationEnts.forEach(scadaGatherBoxMStationEnt -> {
                    for (ScadaGatherBoxMStationEnt stationEnt : stationEnts) {
                        if (stationEnt.getStationId().equals(scadaGatherBoxMStationEnt.getStationId())){
                            throw new XServiceException("当前业务类型已存在" + scadaGatherBoxMStationEnt.getStationName() + "!");
                        }
                    }
                    scadaGatherBoxMStationEnt.setId(UUID.randomUUID().toString());
                    scadaGatherBoxMStationEnt.setGatherBoxId(input.getGatherBoxId());
                    scadaGatherBoxMStationEnt.setBusinessType(input.getBusinessType());
                    scadaGatherBoxMStationEnt.setCreateById(session.getId());
                    scadaGatherBoxMStationEnt.setCreateByName(session.getUserName());
                    scadaGatherBoxMStationEnt.setCreateTime(new Date());
                    mapper.insert(scadaGatherBoxMStationEnt);
                });
                //XMapperHelper.saveBatch(context,ScadaGatherBoxMStationMapper.class,mStationEnts);
            }


            return XServiceResult.OK;
        });

    }


    @XText("删除权限配置中已选场站")
    @XApiPost
    public XServiceResult deleteScadaGatherBoxMStations(XContext context, DeleteScadaGatherBoxMStationInput input)throws XServiceException {
        return XTransactionHelper.begin(context,()-> {
            ScadaGatherBoxMStationMapper mapper = context.getBean(ScadaGatherBoxMStationMapper.class);
            QueryWrapper<ScadaGatherBoxMStationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().in(ScadaGatherBoxMStationEnt::getId, input.getIds());
            List<ScadaGatherBoxMStationEnt> stationEnts = mapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(stationEnts)){
                List<String> ids = stationEnts.stream().map(ScadaGatherBoxMStationEnt::getId).collect(Collectors.toList());
                mapper.deleteBatchIds(ids);
            }
            return XServiceResult.OK;
        });

    }

    public List<QueryBaseStationOutput> selectStationListByOuId(XContext context, BaseOuIdInput ouIdInput) {
        BaseBaseStationService stationService = context.getBean(BaseBaseStationService.class);
        ouIdInput.setIsContainsChildren(true);
        XListResult<QueryBaseStationOutput> result =  stationService.selectStationListByOuId(context, ouIdInput);
        result.throwIfFail();
        List<QueryBaseStationOutput> stationOutputList = result.getResult();
        return stationOutputList;
    }
}
