package pps.core.base.service;

import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.constant.ScadaConstant;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.scada_gather_box_rel_detail.*;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.SmartRecord;
import pps.core.system.service.data.sys_menu.QuerySysMenuOutputTree;
import xstartup.annotation.XService;
import xstartup.base.exception.XServiceException;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.stream.Collectors;

@XService
@XText("模型关系配置列表的配置管理")
public class ScadaGatherBoxRelDetailService {

    @XText("采集配置列表")
    @XApiPost
    public XListResult<ScadaGatherConfContent> getScadaGatherConfTable(XContext context, GetScadaGatherBoxRelDetailInput input){

        //获取模型关系
        ScadaGatherBoxRelMapper relMapper = context.getBean(ScadaGatherBoxRelMapper.class);
        QueryWrapper<ScadaGatherBoxRelEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(ScadaGatherBoxRelEnt::getGatherBoxId,input.getScadaGatherBoxId());
        List<ScadaGatherBoxRelEnt> boxRelEnts = relMapper.selectList(wrapper);

        List<String> relIds = boxRelEnts.stream().map(ScadaGatherBoxRelEnt::getId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(relIds)){
            return XListResult.error(200,"没有配置模型关系配置");
        }
        //出参
        ScadaGatherBoxRelDetailMapper mapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
        QueryWrapper<ScadaGatherBoxRelDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(ScadaGatherBoxRelDetailEnt::getScadaGatherBoxRelId, relIds);
        List<ScadaGatherBoxRelDetailEnt> detailEnts = mapper.selectList(queryWrapper);

        ScadaGatherConfInMapper confInMapper = context.getBean(ScadaGatherConfInMapper.class);
        QueryWrapper<ScadaGatherConfInEnt> inWrapper = new QueryWrapper<>();
        inWrapper.lambda().in(ScadaGatherConfInEnt::getScadaGatherBoxRelId,relIds);
        List<ScadaGatherConfInEnt> confInEnts = confInMapper.selectList(inWrapper);


        ScadaGatherBoxViewMapper boxViewMapper = context.getBean(ScadaGatherBoxViewMapper.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        //调用微服务查询字典
        CreateSysDictionaryInput appServiceConf = new CreateSysDictionaryInput();
        appServiceConf.setAlias("APP_SERVICE_CONF");
        XListResult<QuerySysDictionaryViewOutput> appServiceConfList = sysDictionaryService.querySysDictionarysByAlias(context, appServiceConf);

        CreateSysDictionaryInput scadaInParam = new CreateSysDictionaryInput();
        scadaInParam.setAlias("SCADA_IN_PARAM");
        XListResult<QuerySysDictionaryViewOutput> scadaInParamList = sysDictionaryService.querySysDictionarysByAlias(context, scadaInParam);

        List<ScadaGatherConfContent> list = new ArrayList<>();
        for (ScadaGatherBoxRelEnt boxRelEnt : boxRelEnts) {
            ScadaGatherConfContent contentEntity = new ScadaGatherConfContent();
            contentEntity.setTitle(boxRelEnt.getTitle());
            contentEntity.setTimerGather(boxRelEnt.getTimerGather());
            if (boxRelEnt.getSort()!= null){
                contentEntity.setSort(boxRelEnt.getSort());
            }else{
                contentEntity.setSort(999);
            }
            contentEntity.setScadaGatherBoxRelId(boxRelEnt.getId());
            List<HeadValue> columns = new ArrayList<>();
            StringBuilder sql = new StringBuilder();
            sql.append("select ");
            sql.append("id,");
            for (ScadaGatherBoxRelDetailEnt detailEnt : detailEnts) {
                if (boxRelEnt.getId().equals(detailEnt.getScadaGatherBoxRelId())){
                    if (detailEnt.getCheckShow() != null && detailEnt.getCheckShow()){
                        HeadValue headValue = new HeadValue();
                        headValue.setName(detailEnt.getTableColumnDescription());
                        headValue.setCode(detailEnt.getTableColumn());
                        headValue.setBoxDetailType(detailEnt.getBoxDetailType());
                        headValue.setCodeAlready(detailEnt.getTableColumn()+"already");
                        headValue.setColumnWidth(detailEnt.getColumnWidth());
                        if (detailEnt.getSort() != null){
                            headValue.setSort(detailEnt.getSort());
                        }else{
                            headValue.setSort(999);
                        }
                        columns.add(headValue);
                    }


                    sql.append("(case when ");
                    sql.append(detailEnt.getTableColumn());
                    sql.append(" is not null ");
                    sql.append(" then ");
                    sql.append(detailEnt.getTableColumn());
                    sql.append(" else 'null' ");
                    sql.append(" end) as ");
                    sql.append(detailEnt.getTableColumn());
                    sql.append(",");
                }
            }
            //列名
            columns=columns.stream().sorted(Comparator.comparing(HeadValue::getSort)).collect(Collectors.toList());
            contentEntity.setColumns(columns);
            //查找内容
            sql = new StringBuilder(sql.substring(0, sql.length() - 1));
            sql.append(" from ");
            for (QuerySysDictionaryViewOutput dict : appServiceConfList.getResult()) {
                if (dict.getDicKey().equals(boxRelEnt.getBusinessModule())) {
                    sql.append(dict.getShortName());
                    break;
                }
            }
            sql.append(".");
            sql.append(boxRelEnt.getTableName());
            for (ScadaGatherConfInEnt confInEnt : confInEnts) {
                if (boxRelEnt.getId().equals(confInEnt.getScadaGatherBoxRelId())){
                    sql.append(" where ");
                    sql.append(confInEnt.getParamCode());
                    sql.append(" = '");
                    if (ScadaConstant.IN_PARAM_TYPE_FIXED.equals(confInEnt.getParamType())){
                        sql.append(confInEnt.getConfInParam());
                        sql.append("'");
                    }else{
                        if ("station".equals(confInEnt.getConfInParam())){
                            sql.append(input.getStationId());
                        }else{
                            sql.append(input.getOuId());
                        }
                        sql.append("'");
                    }
                    break;
                }
            }
            sql.append(" order by sort ");
            List<SmartRecord> smartRecords = boxViewMapper.selectBySql(sql.toString());
            contentEntity.setContent(smartRecords);
            //只展示有数据的
            if (!CollectionUtils.isEmpty(smartRecords) && smartRecords.size()>0){
                list.add(contentEntity);
            }
        }
        if (CollectionUtils.isEmpty(list)){
            return XListResult.error(200,"config表中未查到所选场站!");
        }
        //关系排序
        list=list.stream().sorted(Comparator.comparing(ScadaGatherConfContent::getSort)).collect(Collectors.toList());

        StringBuilder sql = new StringBuilder();

        for (ScadaGatherBoxRelDetailEnt detailEnt : detailEnts) {
            if (ScadaConstant.SCADA_DATA.equals(detailEnt.getBoxDetailType())) {
                sql.append(" union select id,scada_gather_box_id,scada_gather_box_rel_id,tag_name,config_column_id " +
                        "from pps_base_info.scada_gather_conf where ");
                sql.append("  scada_gather_box_id = '");
                sql.append(input.getScadaGatherBoxId());
                sql.append("'");
                sql.append(" and scada_gather_box_rel_id = '");
                sql.append(detailEnt.getScadaGatherBoxRelId());
                sql.append("'");
                sql.append(" and tag_name = '");
                sql.append(detailEnt.getTableColumn());
                sql.append("'");

                for (ScadaGatherConfContent content : list) {
                    if (content.getScadaGatherBoxRelId().equals(detailEnt.getScadaGatherBoxRelId())) {
                        List<SmartRecord> records = content.getContent();
                        if (!CollectionUtils.isEmpty(records)){
                            sql.append(" and config_column_id in ( ");
                            for (SmartRecord record : records) {
                                sql.append("'");
                                sql.append(record.get("id").toString());
                                sql.append("'");
                                sql.append(",");
                            }
                            sql= new StringBuilder(sql.substring(0, sql.length() - 1));
                            sql.append(" ) ");
                        }

                    }
                }
            }
        }

        String substring = sql.substring(6, sql.length());
        List<SmartRecord> smartRecords = boxViewMapper.selectBySql(substring);
        //循环当前模型id下所有模型关系
        for (ScadaGatherConfContent content : list) {
            //循环当前模型id下所有scada需要配置的列
            for (SmartRecord smartRecord : smartRecords) {
                if (content.getScadaGatherBoxRelId().equals(smartRecord.get("scada_gather_box_rel_id").toString())) {
                    //循环当前模型关系下所有数据
                    List<SmartRecord> content1 = content.getContent();
                    for (SmartRecord record : content1) {
                        if (smartRecord.get("config_column_id").toString().equals(record.get("id").toString())){
                            //找到已配置的scada列
                            Set<String> keys = record.keySet();
                            for (String key : keys) {
                                if (key.equals(String.valueOf(smartRecord.get("tag_name")))){
                                    record.put(key+"already","已配置");
                                    break;
                                }
                            }
                        }

                    }
                }

            }
        }

        return XListResult.success(list);
    }


    @XText("静态字段保存")
    @XApiPost
    public XServiceResult createScadaGatherBoxRelDetail(XContext context, CreateScadaGatherBoxRelDetailInputs input)throws XServiceException {
        return XTransactionHelper.begin(context,()-> {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            ScadaGatherConfInMapper confInMapper = context.getBean(ScadaGatherConfInMapper.class);
            QueryWrapper<ScadaGatherConfInEnt> confInEntQueryWrapper = new QueryWrapper<>();
            confInEntQueryWrapper.lambda().eq(ScadaGatherConfInEnt::getScadaGatherBoxRelId,input.getScadaGatherBoxRelId());
            List<ScadaGatherConfInEnt> confInEnts = confInMapper.selectList(confInEntQueryWrapper);
            if (!CollectionUtils.isEmpty(confInEnts)){
                List<String> ids = confInEnts.stream().map(ScadaGatherConfInEnt::getId).collect(Collectors.toList());
                confInMapper.deleteBatchIds(ids);
            }
            //入参
            if (input.getConfInInput() != null){
                ScadaGatherConfInEnt confInEnt = new ScadaGatherConfInEnt();
                XCopyUtils.copyObject(input.getConfInInput(),confInEnt);
                confInEnt.setId(UUID.randomUUID().toString());
                confInEnt.setScadaGatherBoxRelId(input.getScadaGatherBoxRelId());
                confInEnt.setCreateTime(new Date());
                confInEnt.setCreateByName(session.getUserName());
                confInEnt.setCreateById(session.getId());
                confInMapper.insert(confInEnt);
            }
            //出参
            ScadaGatherBoxRelDetailMapper mapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
            QueryWrapper<ScadaGatherBoxRelDetailEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(ScadaGatherBoxRelDetailEnt::getScadaGatherBoxRelId,input.getScadaGatherBoxRelId());
            List<ScadaGatherBoxRelDetailEnt> detailEnts = mapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(detailEnts)){
                List<String> ids = detailEnts.stream().map(ScadaGatherBoxRelDetailEnt::getId).collect(Collectors.toList());
                mapper.deleteBatchIds(ids);
            }

            if (!CollectionUtils.isEmpty(input.getInputs())){
                for (CreateScadaGatherBoxRelDetailInput detail : input.getInputs()) {
                    ScadaGatherBoxRelDetailEnt entity = new ScadaGatherBoxRelDetailEnt();
                    XCopyUtils.copyObject(detail, entity);
                    entity.setId(UUID.randomUUID().toString());
                    entity.setScadaGatherBoxRelId(input.getScadaGatherBoxRelId());
                    entity.setCreateTime(new Date());
                    entity.setCreateByName(session.getUserName());
                    entity.setCreateById(session.getId());
                    mapper.insert(entity);
                }
            }


            return XServiceResult.OK;
        });

    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateScadaGatherBoxRelDetail(XContext context, UpdateScadaGatherBoxRelDetailInput input){
        ScadaGatherBoxRelDetailMapper mapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
        QueryWrapper<ScadaGatherBoxRelDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaGatherBoxRelDetailEnt::getId, input.getId());
        ScadaGatherBoxRelDetailEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteScadaGatherBoxRelDetail(XContext context, DeleteScadaGatherBoxRelDetailInput input){
        ScadaGatherBoxRelDetailMapper mapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
        QueryWrapper<ScadaGatherBoxRelDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaGatherBoxRelDetailEnt::getId, input.getId());
        ScadaGatherBoxRelDetailEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }



    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryScadaGatherBoxRelDetailOutput> queryScadaGatherBoxRelDetail(XContext context, QueryScadaGatherBoxRelDetailInput input){
        ScadaGatherBoxRelDetailMapper mapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
        QueryWrapper<ScadaGatherBoxRelDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(ScadaGatherBoxRelDetailEnt::getId, input.getId())
            .like(ScadaGatherBoxRelDetailEnt::getScadaGatherBoxRelId, input.getScadaGatherBoxRelId())
            .like(ScadaGatherBoxRelDetailEnt::getTableColumn, input.getTableColumn())
            .like(ScadaGatherBoxRelDetailEnt::getTableColumnDescription, input.getTableColumnDescription())
            .like(ScadaGatherBoxRelDetailEnt::getDefultColumn, input.getDefultColumn())
            .eq(ScadaGatherBoxRelDetailEnt::getBoxDetailType, input.getBoxDetailType())
            .eq(ScadaGatherBoxRelDetailEnt::getCreateTime, input.getCreateTime())
            .like(ScadaGatherBoxRelDetailEnt::getCreateById, input.getCreateById())
            .like(ScadaGatherBoxRelDetailEnt::getCreateByName, input.getCreateByName())
            .eq(ScadaGatherBoxRelDetailEnt::getModifyTime, input.getModifyTime())
            .like(ScadaGatherBoxRelDetailEnt::getModifyById, input.getModifyById())
            .like(ScadaGatherBoxRelDetailEnt::getModifyByName, input.getModifyByName());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryScadaGatherBoxRelDetailOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryScadaGatherBoxRelDetailOutput> dynamicQueryScadaGatherBoxRelDetail(XContext context, CustomQueryInput input){
        ScadaGatherBoxRelDetailMapper mapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
        return XMapperHelper.query(mapper, input, ScadaGatherBoxRelDetailEnt.class, QueryScadaGatherBoxRelDetailOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetScadaGatherBoxRelDetailViewOutput> getScadaGatherBoxRelDetailView(XContext context, GetScadaGatherBoxRelDetailViewInput input){
        ScadaGatherBoxRelDetailViewMapper mapper = context.getBean(ScadaGatherBoxRelDetailViewMapper.class);
        ScadaGatherBoxRelDetailView record = new ScadaGatherBoxRelDetailView();
        XCopyUtils.copyObject(input, record);
        ScadaGatherBoxRelDetailView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetScadaGatherBoxRelDetailViewOutput output = new GetScadaGatherBoxRelDetailViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryScadaGatherBoxRelDetailViewOutput> queryScadaGatherBoxRelDetailView(XContext context, QueryScadaGatherBoxRelDetailViewInput input){
        ScadaGatherBoxRelDetailViewMapper mapper = context.getBean(ScadaGatherBoxRelDetailViewMapper.class);
        ScadaGatherBoxRelDetailView record = new ScadaGatherBoxRelDetailView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<ScadaGatherBoxRelDetailView> list = mapper.selectList(record);
        PageInfo<ScadaGatherBoxRelDetailView> pageInfo = new PageInfo<>(list);
        List<QueryScadaGatherBoxRelDetailViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryScadaGatherBoxRelDetailViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
