package pps.core.base.service;

import java.util.ArrayList;
import java.util.Date;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.springframework.util.CollectionUtils;
import pps.cloud.framedata.service.CommonDataService;
import pps.cloud.framedata.service.data.common_data.GetColumnListInput;
import pps.cloud.framedata.service.data.common_data.GetColumnListOutput;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.scada_gather_box_rel.*;
import pps.core.base.service.data.scada_gather_box_rel_detail.CreateScadaGatherBoxRelDetailInput;
import pps.core.base.service.data.scada_gather_conf_in.CreateScadaGatherConfInInput;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.NamedServiceProvider;
import xstartup.annotation.XService;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XStringUtils;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XService
@XText("模型关系配置管理")
public class ScadaGatherBoxRelService {

    @XText("查询模型管理配置列表")
    @XApiGet
    public XPageResult<QueryScadaGatherBoxRelViewOutput> queryScadaGatherBoxRelView(XContext context, QueryScadaGatherBoxRelViewInput input){
        ScadaGatherBoxRelViewMapper mapper = context.getBean(ScadaGatherBoxRelViewMapper.class);
        ScadaGatherBoxRelView record = new ScadaGatherBoxRelView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<ScadaGatherBoxRelView> list = mapper.selectListByGatherBoxId(record);
        PageInfo<ScadaGatherBoxRelView> pageInfo = new PageInfo<>(list);
        List<QueryScadaGatherBoxRelViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryScadaGatherBoxRelViewOutput.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        //调用微服务查询字典
        CreateSysDictionaryInput syncRule = new CreateSysDictionaryInput();
        syncRule.setAlias("SYNC_RULE");
        XListResult<QuerySysDictionaryViewOutput> syncRuleList = sysDictionaryService.querySysDictionarysByAlias(context, syncRule);

        CreateSysDictionaryInput businessType = new CreateSysDictionaryInput();
        businessType.setAlias("business_type");
        XListResult<QuerySysDictionaryViewOutput> businessTypeList = sysDictionaryService.querySysDictionarysByAlias(context, businessType);
        for (QueryScadaGatherBoxRelViewOutput output : outputs) {
            for (QuerySysDictionaryViewOutput dict : syncRuleList.getResult()) {
                if (dict.getDicKey().equals(output.getSyncRule())){
                    output.setSyncRuleName(dict.getDicName());
                    break;
                }
            }

            for (QuerySysDictionaryViewOutput dict : businessTypeList.getResult()) {
                if (dict.getDicKey().equals(output.getBusinessType())){
                    output.setBusinessTypeName(dict.getDicName());
                    break;
                }
            }
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("创建模型关系配置")
    @XApiPost
    public XServiceResult createScadaGatherBoxRel(XContext context, CreateScadaGatherBoxRelInput input)throws XServiceException {
        return XTransactionHelper.begin(context,()-> {
            ScadaGatherBoxRelMapper mapper = context.getBean(ScadaGatherBoxRelMapper.class);

            QueryWrapper<ScadaGatherBoxRelEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(ScadaGatherBoxRelEnt::getBusinessType,input.getBusinessType())
                    .eq(ScadaGatherBoxRelEnt::getBoxRelType,input.getBoxRelType())
                    .eq(ScadaGatherBoxRelEnt::getBusinessModule,input.getBusinessModule())
                    .eq(ScadaGatherBoxRelEnt::getTableName,input.getTableName());
            List<ScadaGatherBoxRelEnt> relEnts = mapper.selectList(wrapper);
            if (!CollectionUtils.isEmpty(relEnts)){
                throw new XServiceException("标题为："+relEnts.get(0).getTitle()+"和当前业务类型已配置了"+input.getBusinessModule()+"库里的"+input.getTableName());
            }
            ScadaGatherBoxRelEnt entity = new ScadaGatherBoxRelEnt();
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setId(UUID.randomUUID().toString());
            entity.setCreateById(session.getId());
            entity.setCreateByName(session.getUserName());
            entity.setCreateTime(new Date());
            mapper.insert(entity);
            return XServiceResult.OK;
        });

    }

    @XText("修改模型关系配置")
    @XApiPost
    public XServiceResult updateScadaGatherBoxRel(XContext context, UpdateScadaGatherBoxRelInput input)throws XServiceException{
        return XTransactionHelper.begin(context,()-> {
            ScadaGatherBoxRelMapper mapper = context.getBean(ScadaGatherBoxRelMapper.class);
            QueryWrapper<ScadaGatherBoxRelEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ScadaGatherBoxRelEnt::getId, input.getId());
            ScadaGatherBoxRelEnt entity = mapper.selectOne(queryWrapper);
            if(entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }

            if (!input.getBusinessType().equals(entity.getBusinessType()) || !input.getBoxRelType().equals(entity.getBoxRelType())
                || !input.getBusinessModule().equals(entity.getBusinessModule()) || !input.getTableName().equals(entity.getTableName())){
                QueryWrapper<ScadaGatherBoxRelEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(ScadaGatherBoxRelEnt::getBusinessType,input.getBusinessType())
                        .eq(ScadaGatherBoxRelEnt::getBoxRelType,input.getBoxRelType())
                        .eq(ScadaGatherBoxRelEnt::getBusinessModule,input.getBusinessModule())
                        .eq(ScadaGatherBoxRelEnt::getTableName,input.getTableName());
                List<ScadaGatherBoxRelEnt> relEnts = mapper.selectList(wrapper);
                if (!CollectionUtils.isEmpty(relEnts)){
                    throw new XServiceException("当前业务类型已配置了"+input.getBusinessModule()+"库里的"+input.getTableName());
                }
            }

            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            entity.setModifyTime(new Date());
            mapper.updateById(entity);
            return XServiceResult.OK;
        });

    }

    @XText("删除模型关系配置")
    @XApiPost
    public XServiceResult deleteScadaGatherBoxRel(XContext context, DeleteScadaGatherBoxRelInput input)throws XServiceException{
        return XTransactionHelper.begin(context,()-> {
            ScadaGatherBoxRelMapper mapper = context.getBean(ScadaGatherBoxRelMapper.class);
            QueryWrapper<ScadaGatherBoxRelEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ScadaGatherBoxRelEnt::getId, input.getId());
            ScadaGatherBoxRelEnt entity = mapper.selectOne(queryWrapper);
            if(entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }


            ScadaGatherConfInMapper confInMapper = context.getBean(ScadaGatherConfInMapper.class);
            QueryWrapper<ScadaGatherConfInEnt> confInWrapper = new QueryWrapper<>();
            confInWrapper.lambda().eq(ScadaGatherConfInEnt::getScadaGatherBoxRelId,input.getId());
            List<ScadaGatherConfInEnt> confInEnts = confInMapper.selectList(confInWrapper);
            if (!CollectionUtils.isEmpty(confInEnts)){
                confInMapper.deleteBatchIds(confInEnts.stream().map(ScadaGatherConfInEnt::getId).collect(Collectors.toList()));
            }

            ScadaGatherBoxRelDetailMapper detailMapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
            QueryWrapper<ScadaGatherBoxRelDetailEnt> boxRelDetailEntQueryWrapper = new QueryWrapper<>();
            boxRelDetailEntQueryWrapper.lambda().eq(ScadaGatherBoxRelDetailEnt::getScadaGatherBoxRelId,input.getId());
            List<ScadaGatherBoxRelDetailEnt> relDetailEnts = detailMapper.selectList(boxRelDetailEntQueryWrapper);
            if (!CollectionUtils.isEmpty(relDetailEnts)){
                detailMapper.deleteBatchIds(relDetailEnts.stream().map(ScadaGatherBoxRelDetailEnt::getId).collect(Collectors.toList()));
            }

            mapper.deleteById(entity);
            return XServiceResult.OK;
        });

    }

    @XText("模型关系页面：配置")
    @XApiGet
    public XSingleResult<GetScadaGatherBoxRelOutput> getScadaGatherBoxRel(XContext context, GetScadaGatherBoxRelInput input){
        ScadaGatherBoxRelMapper mapper = context.getBean(ScadaGatherBoxRelMapper.class);
        ScadaGatherBoxRelDetailMapper detailMapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
        ScadaGatherConfInMapper confInMapper = context.getBean(ScadaGatherConfInMapper.class);
        QueryWrapper<ScadaGatherBoxRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaGatherBoxRelEnt::getId, input.getId());
        ScadaGatherBoxRelEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetScadaGatherBoxRelOutput output = new GetScadaGatherBoxRelOutput();
        QueryWrapper<ScadaGatherConfInEnt> wrapper1 = new QueryWrapper<>();
        wrapper1.lambda().eq(ScadaGatherConfInEnt::getScadaGatherBoxRelId,entity.getId());
        ScadaGatherConfInEnt scadaGatherConfInEnt = confInMapper.selectOne(wrapper1);
        if (scadaGatherConfInEnt != null){
            CreateScadaGatherConfInInput inInput = new CreateScadaGatherConfInInput();
            XCopyUtils.copyObject(scadaGatherConfInEnt, inInput);
            output.setConfInInput(inInput);
        }

        XCopyUtils.copyObject(entity, output);
        if (XStringUtils.isNotEmpty(entity.getBusinessModule()) && XStringUtils.isNotEmpty(entity.getTableName())){
            //查询已保存的config表字段
            QueryWrapper<ScadaGatherBoxRelDetailEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(ScadaGatherBoxRelDetailEnt::getScadaGatherBoxRelId,input.getId());
            List<ScadaGatherBoxRelDetailEnt> detailEnts = detailMapper.selectList(wrapper);
            //获取config表字段信息
            List<GetColumnListOutput> tableColumns = getTableColumns(context, entity.getBusinessModule(),entity.getTableName());
            List<CreateScadaGatherBoxRelDetailInput> list = new ArrayList<>();
            for (GetColumnListOutput config : tableColumns) {
                CreateScadaGatherBoxRelDetailInput detailInput = new CreateScadaGatherBoxRelDetailInput();

                for (ScadaGatherBoxRelDetailEnt detailEnt : detailEnts) {
                    //已经存在的
                    if (config.getColumnName().equals(detailEnt.getTableColumn())){
                        XCopyUtils.copyObject(detailEnt,detailInput);
                        detailInput.setIsShow(true);
                        break;
                    }
                }
                detailInput.setTableColumn(config.getColumnName());
                if (XStringUtils.isEmpty(detailInput.getTableColumnDescription())){
                    detailInput.setTableColumnDescription(config.getColumnDesc());
                }
                if (detailInput.getIsShow() == null){
                    detailInput.setIsShow(false);
                }

                list.add(detailInput);
            }
            output.setTableColumns(list);

        }
        //查询默认值的存储表
        if (XStringUtils.isNotEmpty(entity.getDefultBusinessModule()) && XStringUtils.isNotEmpty(entity.getDefultTableName())){
            List<GetColumnListOutput> defaultTableColumns = getTableColumns(context, entity.getDefultBusinessModule(),entity.getDefultTableName());
            List<CreateScadaGatherBoxRelDetailInput> defaultList = new ArrayList<>();
            for (GetColumnListOutput tableColumn : defaultTableColumns) {
                CreateScadaGatherBoxRelDetailInput detailInput = new CreateScadaGatherBoxRelDetailInput();
                detailInput.setDefultColumn(tableColumn.getColumnName());
                detailInput.setTableColumnDescription(tableColumn.getColumnDesc());
                defaultList.add(detailInput);
            }
            output.setDefaultTableColumns(defaultList);
        }

        return XSingleResult.success(output);
    }
    //获取表字段
    private List<GetColumnListOutput> getTableColumns(XContext context, String BusinessModule, String tableName){
        //data数据表
        CommonDataService service = NamedServiceProvider.getServiceByName(context,BusinessModule, CommonDataService.class);
        SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
        GetColumnListInput relInput = new GetColumnListInput();
        //调用微服务查询字典
        CreateSysDictionaryInput mediumType = new CreateSysDictionaryInput();
        mediumType.setAlias("APP_SERVICE_CONF");
        XListResult<QuerySysDictionaryViewOutput> mediumTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediumType);
        for (QuerySysDictionaryViewOutput dict : mediumTypeList.getResult()) {
            if (dict.getDicKey().equals(BusinessModule)){
                relInput.setModelName(dict.getShortName());
                break;
            }
        }
        relInput.setServiceName(BusinessModule);
        relInput.setTableName(tableName);
        XListResult<GetColumnListOutput> columnList = service.getColumnList(context, relInput);

        return columnList.getResult();
    }




}
