package pps.core.base.service;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.*;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.util.CollectionUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.CreateSysDictionaryInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.constant.ScadaConstant;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.base_station.ImportBaseStationInput;
import pps.core.base.service.data.scada_gather_box.*;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import pps.core.base.service.data.scada_gather_box_m_ou.CreateScadaGatherBoxMOuInput;
import pps.core.base.service.data.scada_gather_box_m_ou.GetScadaGatherBoxMOuOutput;
import pps.core.base.service.data.scada_gather_box_m_ou.UpdateScadaGatherBoxMOuInput;
import pps.core.base.service.data.scada_gather_box_rel.ConfData;
import pps.core.base.service.data.scada_gather_box_rel_detail.HeadValue;
import pps.core.base.service.data.scada_gather_box_rel_detail.ScadaGatherConfContent;
import pps.core.base.service.data.scada_gather_conf.QueryScadaGatherConfViewInput;
import pps.core.base.service.data.scada_gather_conf.QueryScadaGatherConfViewOutput;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.DateHelper;
import pps.core.common.utils.ExportExcelUtils;
import pps.core.common.utils.ImportExcelUtils;
import pps.core.common.utils.SmartRecord;
import pps.core.system.entity.SysUserImportLogDetailEnt;
import pps.core.system.mapper.SysUserImportLogDetailMapper;
import pps.core.system.service.data.sys_user_import_log.GetSysUserImportLogDetailInput;
import xstartup.annotation.XService;
import xstartup.base.exception.XServiceException;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.*;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.stream.Collectors;

@XService
@XText("scada采集模型管理")
public class ScadaGatherBoxService {
    private HashMap<Integer,Boolean> isNeedNotEmptyMap=new HashMap<>();

    @XText("根据场站查询模版和配置状态")
    @XApiPost
    public XPageResult<QueryScadaGatherBoxViewOutput> queryScadaGatherConfPage(XContext context, QueryScadaGatherBoxViewInput input){

        ScadaGatherBoxViewMapper boxViewMapper = context.getBean(ScadaGatherBoxViewMapper.class);

        ScadaGatherBoxView boxView = new ScadaGatherBoxView();
        boxView.setStationId(input.getStationId());
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<ScadaGatherBoxView> boxViews = boxViewMapper.selectBoxInfo(boxView);
        PageInfo<ScadaGatherBoxView> pageInfo = new PageInfo<>(boxViews);
        List<QueryScadaGatherBoxViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryScadaGatherBoxViewOutput.class);

        if (!CollectionUtils.isEmpty(outputs)){

            SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
            //调用微服务查询字典
            CreateSysDictionaryInput appServiceConf = new CreateSysDictionaryInput();
            appServiceConf.setAlias("APP_SERVICE_CONF");
            XListResult<QuerySysDictionaryViewOutput> appServiceConfList = sysDictionaryService.querySysDictionarysByAlias(context, appServiceConf);

            CreateSysDictionaryInput scadaInParam = new CreateSysDictionaryInput();
            scadaInParam.setAlias("SCADA_IN_PARAM");
            XListResult<QuerySysDictionaryViewOutput> scadaInParamList = sysDictionaryService.querySysDictionarysByAlias(context, scadaInParam);

            CreateSysDictionaryInput mediaType = new CreateSysDictionaryInput();
            mediaType.setAlias("MEDIUM_TYPE");
            XListResult<QuerySysDictionaryViewOutput> mediumTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediaType);

            for (QueryScadaGatherBoxViewOutput box : outputs) {
                //获取模型关系
                ScadaGatherBoxRelMapper relMapper = context.getBean(ScadaGatherBoxRelMapper.class);
                QueryWrapper<ScadaGatherBoxRelEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(ScadaGatherBoxRelEnt::getGatherBoxId,box.getId());
                List<ScadaGatherBoxRelEnt> boxRelEnts = relMapper.selectList(wrapper);
                List<String> relIds = boxRelEnts.stream().map(ScadaGatherBoxRelEnt::getId).collect(Collectors.toList());
                long finish=0;
                long count=0;
                //出参
                if (!CollectionUtils.isEmpty(relIds)){
                    ScadaGatherBoxRelDetailMapper mapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
                    QueryWrapper<ScadaGatherBoxRelDetailEnt> queryWrapper = new QueryWrapper<>();
                    queryWrapper.lambda().in(ScadaGatherBoxRelDetailEnt::getScadaGatherBoxRelId, relIds);
                    List<ScadaGatherBoxRelDetailEnt> detailEnts = mapper.selectList(queryWrapper);
                    //入参
                    ScadaGatherConfInMapper confInMapper = context.getBean(ScadaGatherConfInMapper.class);
                    QueryWrapper<ScadaGatherConfInEnt> inWrapper = new QueryWrapper<>();
                    inWrapper.lambda().in(ScadaGatherConfInEnt::getScadaGatherBoxRelId,relIds);
                    List<ScadaGatherConfInEnt> confInEnts = confInMapper.selectList(inWrapper);



                List<ScadaGatherConfContent> list = new ArrayList<>();

                for (ScadaGatherBoxRelEnt boxRelEnt : boxRelEnts) {
                    ScadaGatherConfContent contentEntity = new ScadaGatherConfContent();
                    contentEntity.setTitle(boxRelEnt.getTitle());
                    if (boxRelEnt.getSort()!= null){
                        contentEntity.setSort(boxRelEnt.getSort());
                    }else{
                        contentEntity.setSort(999);
                    }
                    contentEntity.setScadaGatherBoxRelId(boxRelEnt.getId());
                    StringBuilder sql = new StringBuilder();
                    sql.append("select ");
                    sql.append("id,");
                    for (ScadaGatherBoxRelDetailEnt detailEnt : detailEnts) {
                        if (boxRelEnt.getId().equals(detailEnt.getScadaGatherBoxRelId())){
                            sql.append("(case when ");
                            sql.append(detailEnt.getTableColumn());
                            sql.append(" is not null ");
                            sql.append(" then ");
                            sql.append(detailEnt.getTableColumn());
                            sql.append(" else 'null' ");
                            sql.append(" end) as ");
                            sql.append(detailEnt.getTableColumn());
                            sql.append(",");
                        }
                    }

                    //查找内容
                    sql = new StringBuilder(sql.substring(0, sql.length() - 1));
                    sql.append(" from ");
                    for (QuerySysDictionaryViewOutput dict : appServiceConfList.getResult()) {
                        if (dict.getDicKey().equals(boxRelEnt.getBusinessModule())) {
                            sql.append(dict.getShortName());
                            break;
                        }
                    }
                    sql.append(".");
                    sql.append(boxRelEnt.getTableName());
                    for (ScadaGatherConfInEnt confInEnt : confInEnts) {
                        if (boxRelEnt.getId().equals(confInEnt.getScadaGatherBoxRelId())){
                            sql.append(" where ");
                            sql.append(confInEnt.getParamCode());
                            sql.append(" = '");
                            if (ScadaConstant.IN_PARAM_TYPE_FIXED.equals(confInEnt.getParamType())){
                                sql.append(confInEnt.getConfInParam());
                                sql.append("'");
                            }else{
                                if ("station".equals(confInEnt.getConfInParam())){
                                    sql.append(input.getStationId());
                                }else{
                                    sql.append(input.getOuId());
                                }
                                sql.append("'");
                            }
                            break;
                        }
                    }
                    sql.append(" order by sort ");
                    List<SmartRecord> smartRecords = boxViewMapper.selectBySql(sql.toString());
                    contentEntity.setContent(smartRecords);
                    //只展示有数据的
                    if (!CollectionUtils.isEmpty(smartRecords) && smartRecords.size()>0){
                        list.add(contentEntity);
                    }
                }


                StringBuilder scadaGatherConfSql = new StringBuilder();
                for (ScadaGatherBoxRelDetailEnt detailEnt : detailEnts) {
                    if (ScadaConstant.SCADA_DATA.equals(detailEnt.getBoxDetailType())) {
                        scadaGatherConfSql.append(" union select id,scada_gather_box_id,scada_gather_box_rel_id,tag_name,config_column_id " +
                                "from pps_base_info.scada_gather_conf where ");
                        scadaGatherConfSql.append("  scada_gather_box_id = '");
                        scadaGatherConfSql.append(box.getId());
                        scadaGatherConfSql.append("'");
                        scadaGatherConfSql.append(" and scada_gather_box_rel_id = '");
                        scadaGatherConfSql.append(detailEnt.getScadaGatherBoxRelId());
                        scadaGatherConfSql.append("'");
                        scadaGatherConfSql.append(" and tag_name = '");
                        scadaGatherConfSql.append(detailEnt.getTableColumn());
                        scadaGatherConfSql.append("'");

                        for (ScadaGatherConfContent content : list) {
                            if (content.getScadaGatherBoxRelId().equals(detailEnt.getScadaGatherBoxRelId())) {
                                List<SmartRecord> records = content.getContent();
                                if (!CollectionUtils.isEmpty(records)){
                                    scadaGatherConfSql.append(" and config_column_id in ( ");
                                    for (SmartRecord record : records) {
                                        scadaGatherConfSql.append("'");
                                        scadaGatherConfSql.append(record.get("id").toString());
                                        scadaGatherConfSql.append("'");
                                        scadaGatherConfSql.append(",");
                                        //计算需要配置的scada总数
                                        Set<String> keys = record.keySet();
                                        for (String key : keys) {
                                            if (key.equals(detailEnt.getTableColumn())){
                                                count+=1;
                                                break;
                                            }
                                        }

                                    }
                                    scadaGatherConfSql= new StringBuilder(scadaGatherConfSql.substring(0, scadaGatherConfSql.length() - 1));
                                    scadaGatherConfSql.append(" ) ");
                                }

                            }
                        }
                    }
                }


                if (XStringUtils.isNotEmpty(scadaGatherConfSql.toString())){
                    String confSql = scadaGatherConfSql.substring(6, scadaGatherConfSql.length());
                    List<SmartRecord> confInfo = boxViewMapper.selectBySql(confSql);
                    //循环当前模型id下所有模型关系
                    for (ScadaGatherConfContent content : list) {
                        //循环当前模型id下所有scada需要配置的列
                        for (SmartRecord smartRecord : confInfo) {
                            if (content.getScadaGatherBoxRelId().equals(smartRecord.get("scada_gather_box_rel_id").toString())) {
                                //循环当前模型关系下所有数据
                                List<SmartRecord> content1 = content.getContent();
                                for (SmartRecord record : content1) {
                                    if (smartRecord.get("config_column_id").toString().equals(record.get("id").toString())){
                                        //找到已配置的scada列
                                        Set<String> keys = record.keySet();
                                        for (String key : keys) {
                                            if (key.equals(String.valueOf(smartRecord.get("tag_name")))){
                                                record.put(key+"already","已配置");
                                                //计算scada已配置的数量
                                                finish+=1;
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }



                }
                box.setCount(count);
                box.setFinish(finish);
                mediumTypeList.getResult().forEach(querySysDictionaryViewOutput -> {
                    if (box.getMediaType().equals(querySysDictionaryViewOutput.getDicKey())) {
                        box.setMediaTypeName(querySysDictionaryViewOutput.getDicName());
                    }
                });

            }
        }

        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }



    @XText("模型列表")
    @XApiGet
    public XPageResult<QueryScadaGatherBoxViewOutput> queryScadaGatherBoxView(XContext context, QueryScadaGatherBoxViewInput input){
        ScadaGatherBoxViewMapper mapper = context.getBean(ScadaGatherBoxViewMapper.class);
        ScadaGatherBoxView record = new ScadaGatherBoxView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<ScadaGatherBoxView> list = mapper.selectList(record);
        PageInfo<ScadaGatherBoxView> pageInfo = new PageInfo<>(list);
        List<QueryScadaGatherBoxViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryScadaGatherBoxViewOutput.class);
        if (!CollectionUtils.isEmpty(outputs)){

            List<String> ids = outputs.stream().map(QueryScadaGatherBoxViewOutput::getId).collect(Collectors.toList());

            ScadaGatherBoxPropertyMapper boxPropertyMapper = context.getBean(ScadaGatherBoxPropertyMapper.class);
            QueryWrapper<ScadaGatherBoxPropertyEnt> boxPropertyWrapper = new QueryWrapper<>();
            boxPropertyWrapper.lambda().in(ScadaGatherBoxPropertyEnt::getScadaGatherBoxId,ids);
            List<ScadaGatherBoxPropertyEnt> boxPropertyEnts = boxPropertyMapper.selectList(boxPropertyWrapper);


            ScadaGatherBoxMOuMapper boxOuMapper = context.getBean(ScadaGatherBoxMOuMapper.class);
            QueryWrapper<ScadaGatherBoxMOuEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().in(ScadaGatherBoxMOuEnt::getGatherBoxId,ids);
            List<ScadaGatherBoxMOuEnt> boxMOuEnts = boxOuMapper.selectList(wrapper);
            SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
            //调用微服务查询字典
            CreateSysDictionaryInput mediumType = new CreateSysDictionaryInput();
            mediumType.setAlias("MEDIUM_TYPE");
            XListResult<QuerySysDictionaryViewOutput> mediumTypeList = sysDictionaryService.querySysDictionarysByAlias(context, mediumType);

            //调用微服务查询字典
            CreateSysDictionaryInput businessType = new CreateSysDictionaryInput();
            businessType.setAlias("business_type");
            XListResult<QuerySysDictionaryViewOutput> businessTypeList = sysDictionaryService.querySysDictionarysByAlias(context, businessType);
            outputs.forEach(queryScadaGatherBoxViewOutput -> {
                StringBuffer buffer = new StringBuffer();
                List<String> businessTypes = new ArrayList<>();
                boxPropertyEnts.forEach(boxPropertyEnt -> {
                    if (queryScadaGatherBoxViewOutput.getId().equals(boxPropertyEnt.getScadaGatherBoxId())){
                        businessTypes.add(boxPropertyEnt.getBusinessType());
                        buffer.append(",");
                        businessTypeList.getResult().forEach(querySysDictionaryViewOutput -> {
                            if (boxPropertyEnt.getBusinessType().equals(querySysDictionaryViewOutput.getDicKey())){
                                buffer.append(querySysDictionaryViewOutput.getDicName());
                            }
                        });


                    }
                });
                if (buffer.length()>1){
                    String businessTypeName = buffer.substring(1, buffer.length());
                    queryScadaGatherBoxViewOutput.setBusinessTypesName(businessTypeName);
                }
                queryScadaGatherBoxViewOutput.setBusinessTypes(businessTypes);

                boxMOuEnts.forEach(scadaGatherBoxMOuEnt -> {
                    if (queryScadaGatherBoxViewOutput.getId().equals(scadaGatherBoxMOuEnt.getGatherBoxId())){
                        if (queryScadaGatherBoxViewOutput.getOuInputs() ==null){
                            queryScadaGatherBoxViewOutput.setOuInputs(new ArrayList<>());
                        }
                        GetScadaGatherBoxMOuOutput mOuOutput = new GetScadaGatherBoxMOuOutput();
                        mOuOutput.setOuId(scadaGatherBoxMOuEnt.getOuId());
                        mOuOutput.setOuName(scadaGatherBoxMOuEnt.getOuName());
                        queryScadaGatherBoxViewOutput.getOuInputs().add(mOuOutput);
                    }
                });
                mediumTypeList.getResult().forEach(querySysDictionaryViewOutput -> {
                    if (queryScadaGatherBoxViewOutput.getMediaType().equals(querySysDictionaryViewOutput.getDicKey())){
                        queryScadaGatherBoxViewOutput.setMediaTypeName(querySysDictionaryViewOutput.getDicName());
                    }
                });


            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("新增模型")
    @XApiPost
    public XServiceResult createScadaGatherBox(XContext context, CreateScadaGatherBoxInput input)throws XServiceException {
        return XTransactionHelper.begin(context,()-> {
            ScadaGatherBoxMapper mapper = context.getBean(ScadaGatherBoxMapper.class);
            ScadaGatherBoxEnt entity = new ScadaGatherBoxEnt();
            XCopyUtils.copyObject(input, entity);
            String boxId = UUID.randomUUID().toString();
            entity.setId(boxId);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setCreateById(session.getId());
            entity.setCreateByName(session.getUserName());
            entity.setCreateTime(new Date());
            mapper.insert(entity);


            if (!CollectionUtils.isEmpty(input.getBusinessTypes())){
                ScadaGatherBoxPropertyMapper propertyMapper = context.getBean(ScadaGatherBoxPropertyMapper.class);
                for (String businessType : input.getBusinessTypes()) {
                    ScadaGatherBoxPropertyEnt propertyEnt = new ScadaGatherBoxPropertyEnt();
                    propertyEnt.setId(UUID.randomUUID().toString());
                    propertyEnt.setScadaGatherBoxId(boxId);
                    propertyEnt.setBusinessType(businessType);
                    propertyEnt.setCreateById(session.getId());
                    propertyEnt.setCreateByName(session.getUserName());
                    propertyEnt.setCreateTime(new Date());
                    propertyMapper.insert(propertyEnt);
                }
            }

            if (!CollectionUtils.isEmpty(input.getOuInputs())){
                ScadaGatherBoxMOuMapper boxOuMapper = context.getBean(ScadaGatherBoxMOuMapper.class);
                List<ScadaGatherBoxMOuEnt> list = new ArrayList<>();
                for (CreateScadaGatherBoxMOuInput ouInput : input.getOuInputs()) {
                    ScadaGatherBoxMOuEnt ouEnt = new ScadaGatherBoxMOuEnt();
                    XCopyUtils.copyObject(ouInput,ouEnt);
                    ouEnt.setId(UUID.randomUUID().toString());
                    ouEnt.setGatherBoxId(boxId);
                    ouEnt.setIsDelete(ScadaConstant.IS_DELETE_FLASE);
                    ouEnt.setCreateById(session.getId());
                    ouEnt.setCreateByName(session.getUserName());
                    ouEnt.setCreateTime(new Date());
                    boxOuMapper.insert(ouEnt);
                    list.add(ouEnt);
                }

                //XMapperHelper.saveBatch(context,ScadaGatherBoxMOuMapper.class,list);
            }
            return XServiceResult.OK;
        });

    }

    @XText("修改模型")
    @XApiPost
    public XServiceResult updateScadaGatherBox(XContext context, UpdateScadaGatherBoxInput input)throws XServiceException{
        return XTransactionHelper.begin(context,()-> {
            ScadaGatherBoxMapper mapper = context.getBean(ScadaGatherBoxMapper.class);
            QueryWrapper<ScadaGatherBoxEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ScadaGatherBoxEnt::getId, input.getId());
            ScadaGatherBoxEnt entity = mapper.selectOne(queryWrapper);
            if(entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            entity.setModifyTime(new Date());
            mapper.updateById(entity);

            ScadaGatherBoxPropertyMapper propertyMapper = context.getBean(ScadaGatherBoxPropertyMapper.class);
            QueryWrapper<ScadaGatherBoxPropertyEnt> queryPropertyWrapper = new QueryWrapper<>();
            queryPropertyWrapper.lambda().eq(ScadaGatherBoxPropertyEnt::getScadaGatherBoxId, input.getId());
            List<ScadaGatherBoxPropertyEnt> propertyEnts = propertyMapper.selectList(queryPropertyWrapper);
            if (!CollectionUtils.isEmpty(propertyEnts)){
                List<String> ids = propertyEnts.stream().map(ScadaGatherBoxPropertyEnt::getId).collect(Collectors.toList());
                propertyMapper.deleteBatchIds(ids);
            }

            if (!CollectionUtils.isEmpty(input.getBusinessTypes())){
                for (String businessType : input.getBusinessTypes()) {
                    ScadaGatherBoxPropertyEnt propertyEnt = new ScadaGatherBoxPropertyEnt();
                    propertyEnt.setId(UUID.randomUUID().toString());
                    propertyEnt.setScadaGatherBoxId(input.getId());
                    propertyEnt.setBusinessType(businessType);
                    propertyEnt.setCreateById(session.getId());
                    propertyEnt.setCreateByName(session.getUserName());
                    propertyEnt.setCreateTime(new Date());
                    propertyMapper.insert(propertyEnt);
                }
            }

            ScadaGatherBoxMOuMapper boxOuMapper = context.getBean(ScadaGatherBoxMOuMapper.class);
            QueryWrapper<ScadaGatherBoxMOuEnt> queryOuWrapper = new QueryWrapper<>();
            queryOuWrapper.lambda().eq(ScadaGatherBoxMOuEnt::getGatherBoxId, input.getId());
            List<ScadaGatherBoxMOuEnt> ouEnts = boxOuMapper.selectList(queryOuWrapper);
            if (!CollectionUtils.isEmpty(ouEnts)){
                List<String> ids = ouEnts.stream().map(ScadaGatherBoxMOuEnt::getId).collect(Collectors.toList());
                boxOuMapper.deleteBatchIds(ids);
            }

            if (!CollectionUtils.isEmpty(input.getOuInputs())){
                List<ScadaGatherBoxMOuEnt> list = new ArrayList<>();
                for (UpdateScadaGatherBoxMOuInput ouInput : input.getOuInputs()) {
                    ScadaGatherBoxMOuEnt ouEnt = new ScadaGatherBoxMOuEnt();
                    XCopyUtils.copyObject(ouInput,ouEnt);
                    ouEnt.setId(UUID.randomUUID().toString());
                    ouEnt.setGatherBoxId(input.getId());
                    ouEnt.setIsDelete(0);
                    ouEnt.setCreateById(session.getId());
                    ouEnt.setCreateByName(session.getUserName());
                    ouEnt.setCreateTime(new Date());
                    boxOuMapper.insert(ouEnt);
                    list.add(ouEnt);
                }
                //XMapperHelper.saveBatch(context,ScadaGatherBoxMOuMapper.class,list);
            }
            return XServiceResult.OK;
        });

    }

    @XText("删除模型")
    @XApiPost
    public XServiceResult deleteScadaGatherBox(XContext context, DeleteScadaGatherBoxInput input)throws XServiceException{
        return XTransactionHelper.begin(context,()-> {
            ScadaGatherBoxMapper mapper = context.getBean(ScadaGatherBoxMapper.class);
            ScadaGatherBoxMOuMapper boxOuMapper = context.getBean(ScadaGatherBoxMOuMapper.class);
            ScadaGatherBoxMStationMapper boxStationMapper = context.getBean(ScadaGatherBoxMStationMapper.class);
            QueryWrapper<ScadaGatherBoxEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ScadaGatherBoxEnt::getId, input.getId());
            ScadaGatherBoxEnt entity = mapper.selectOne(queryWrapper);
            if(entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }


            //删除模型关系
            ScadaGatherBoxRelMapper relMapper = context.getBean(ScadaGatherBoxRelMapper.class);
            QueryWrapper<ScadaGatherBoxRelEnt> relWrapper = new QueryWrapper<>();
            relWrapper.lambda().eq(ScadaGatherBoxRelEnt::getGatherBoxId,input.getId());
            List<ScadaGatherBoxRelEnt> relEnts = relMapper.selectList(relWrapper);
            if (!CollectionUtils.isEmpty(relEnts)){
                List<String> ids = relEnts.stream().map(ScadaGatherBoxRelEnt::getId).collect(Collectors.toList());
                //删除模型具体配置列
                ScadaGatherBoxRelDetailMapper relDetailMapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
                QueryWrapper<ScadaGatherBoxRelDetailEnt> relDetailWrapper = new QueryWrapper<>();
                relDetailWrapper.lambda().in(ScadaGatherBoxRelDetailEnt::getScadaGatherBoxRelId,ids);
                List<ScadaGatherBoxRelDetailEnt> relDetailEnts = relDetailMapper.selectList(relDetailWrapper);
                if (!CollectionUtils.isEmpty(relDetailEnts)){
                    List<String> detailIds = relDetailEnts.stream().map(ScadaGatherBoxRelDetailEnt::getId).collect(Collectors.toList());
                    relDetailMapper.deleteBatchIds(detailIds);
                }
                relMapper.deleteBatchIds(ids);
            }
            //删除scada采集配置
            ScadaGatherConfMapper confMapper = context.getBean(ScadaGatherConfMapper.class);
            QueryWrapper<ScadaGatherConfEnt> confWrapper = new QueryWrapper<>();
            confWrapper.lambda().eq(ScadaGatherConfEnt::getScadaGatherBoxId,input.getId());
            List<ScadaGatherConfEnt> confEnts = confMapper.selectList(confWrapper);
            if (!CollectionUtils.isEmpty(confEnts)){
                List<String> ids = confEnts.stream().map(ScadaGatherConfEnt::getId).collect(Collectors.toList());
                confMapper.deleteBatchIds(ids);
            }
            //删除scada属性
            ScadaGatherBoxPropertyMapper propertyMapper = context.getBean(ScadaGatherBoxPropertyMapper.class);
            QueryWrapper<ScadaGatherBoxPropertyEnt> queryPropertyWrapper = new QueryWrapper<>();
            queryPropertyWrapper.lambda().eq(ScadaGatherBoxPropertyEnt::getScadaGatherBoxId, input.getId());
            List<ScadaGatherBoxPropertyEnt> propertyEnts = propertyMapper.selectList(queryPropertyWrapper);
            if (!CollectionUtils.isEmpty(propertyEnts)){
                List<String> ids = propertyEnts.stream().map(ScadaGatherBoxPropertyEnt::getId).collect(Collectors.toList());
                propertyMapper.deleteBatchIds(ids);
            }
            //删除模型授权的组织机构
            QueryWrapper<ScadaGatherBoxMOuEnt> queryOuWrapper = new QueryWrapper<>();
            queryOuWrapper.lambda().eq(ScadaGatherBoxMOuEnt::getGatherBoxId, input.getId());
            List<ScadaGatherBoxMOuEnt> ouEnts = boxOuMapper.selectList(queryOuWrapper);
            if (!CollectionUtils.isEmpty(ouEnts)){
                boxOuMapper.deleteBatchIds(ouEnts.stream().map(ScadaGatherBoxMOuEnt::getId).collect(Collectors.toList()));
            }

            //删除模型授权的场站
            QueryWrapper<ScadaGatherBoxMStationEnt> queryStationWrapper = new QueryWrapper<>();
            queryStationWrapper.lambda().eq(ScadaGatherBoxMStationEnt::getGatherBoxId, input.getId());
            List<ScadaGatherBoxMStationEnt> stationEnts = boxStationMapper.selectList(queryStationWrapper);
            if (!CollectionUtils.isEmpty(stationEnts)){
                boxStationMapper.deleteBatchIds(stationEnts.stream().map(ScadaGatherBoxMStationEnt::getId).collect(Collectors.toList()));
            }

            mapper.deleteById(entity);
            return XServiceResult.OK;
        });

    }


    @XText("导出小时参数采集配置数据")
    @XApiGet
    public XFileResult exportScadaGatherBoxData(XContext context, GetScadaGatherBoxViewInput input) {
        try {
            if(XStringUtils.isNotEmpty(input.getStationId())){
                ScadaGatherBoxViewMapper boxViewMapper = context.getBean(ScadaGatherBoxViewMapper.class);
                ScadaGatherBoxView boxView = new ScadaGatherBoxView();
                boxView.setStationId(input.getStationId());
                if(!CollectionUtils.isEmpty(input.getIds()))
                    boxView.setIds(input.getIds());
                List<ScadaGatherBoxView> boxViews = boxViewMapper.selectBoxInfoByIdsOrStationId(boxView);

                //调用微服务查询字典
                SystemDictionaryService sysDictionaryService = context.getBean(SystemDictionaryService.class);
                CreateSysDictionaryInput appServiceConf = new CreateSysDictionaryInput();
                appServiceConf.setAlias("APP_SERVICE_CONF");
                XListResult<QuerySysDictionaryViewOutput> appServiceConfList = sysDictionaryService.querySysDictionarysByAlias(context, appServiceConf);
                List<DownloadScadaGatherBoxContent> excelList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(boxViews)){
                    for (ScadaGatherBoxView box : boxViews) {
                        //获取模型关系
                        List<ScadaGatherConfContent> list = new ArrayList<>();
                        ScadaGatherBoxRelMapper relMapper = context.getBean(ScadaGatherBoxRelMapper.class);
                        QueryWrapper<ScadaGatherBoxRelEnt> wrapper = new QueryWrapper<>();
                        wrapper.lambda().eq(ScadaGatherBoxRelEnt::getGatherBoxId,box.getId());
                        List<ScadaGatherBoxRelEnt> boxRelEnts = relMapper.selectList(wrapper);

                        List<String> relIds = boxRelEnts.stream().map(ScadaGatherBoxRelEnt::getId).collect(Collectors.toList());
                        if (CollectionUtils.isEmpty(relIds)){
                            continue;
                        }
                        //出参
                        ScadaGatherBoxRelDetailMapper mapper = context.getBean(ScadaGatherBoxRelDetailMapper.class);
                        QueryWrapper<ScadaGatherBoxRelDetailEnt> queryWrapper = new QueryWrapper<>();
                        queryWrapper.lambda().in(ScadaGatherBoxRelDetailEnt::getScadaGatherBoxRelId, relIds);
                        List<ScadaGatherBoxRelDetailEnt> detailEnts = mapper.selectList(queryWrapper);

                        ScadaGatherConfInMapper confInMapper = context.getBean(ScadaGatherConfInMapper.class);
                        QueryWrapper<ScadaGatherConfInEnt> inWrapper = new QueryWrapper<>();
                        inWrapper.lambda().in(ScadaGatherConfInEnt::getScadaGatherBoxRelId,relIds);
                        List<ScadaGatherConfInEnt> confInEnts = confInMapper.selectList(inWrapper);

                        for (ScadaGatherBoxRelEnt boxRelEnt : boxRelEnts) {
                            ScadaGatherConfContent contentEntity = new ScadaGatherConfContent();
                            contentEntity.setTitle(boxRelEnt.getTitle());
                            contentEntity.setTimerGather(boxRelEnt.getTimerGather());
                            if (boxRelEnt.getSort()!= null){
                                contentEntity.setSort(boxRelEnt.getSort());
                            }else{
                                contentEntity.setSort(999);
                            }
                            List<HeadValue> columns = new ArrayList<>();
                            contentEntity.setScadaGatherBoxRelId(boxRelEnt.getId());

                            StringBuilder sql = new StringBuilder();
                            sql.append("select ");
                            sql.append("id,");
                            for (ScadaGatherBoxRelDetailEnt detailEnt : detailEnts) {
                                if (boxRelEnt.getId().equals(detailEnt.getScadaGatherBoxRelId())){
                                    if (detailEnt.getCheckShow() != null && detailEnt.getCheckShow()){
                                        HeadValue headValue = new HeadValue();
                                        headValue.setName(detailEnt.getTableColumnDescription());
                                        headValue.setCode(detailEnt.getTableColumn());
                                        headValue.setBoxDetailType(detailEnt.getBoxDetailType());
                                        headValue.setCodeAlready(detailEnt.getTableColumn()+"already");
                                        headValue.setColumnWidth(detailEnt.getColumnWidth());
                                        if (detailEnt.getSort() != null){
                                            headValue.setSort(detailEnt.getSort());
                                        }else{
                                            headValue.setSort(999);
                                        }
                                        columns.add(headValue);
                                    }
                                    sql.append("(case when ");
                                    sql.append(detailEnt.getTableColumn());
                                    sql.append(" is not null ");
                                    sql.append(" then ");
                                    sql.append(detailEnt.getTableColumn());
                                    sql.append(" else 'null' ");
                                    sql.append(" end) as ");
                                    sql.append(detailEnt.getTableColumn());
                                    sql.append(",");

                                }
                            }
                            //列名
                            columns=columns.stream().sorted(Comparator.comparing(HeadValue::getSort)).collect(Collectors.toList());
                            contentEntity.setColumns(columns);
                            //查找内容
                            sql = new StringBuilder(sql.substring(0, sql.length() - 1));
                            sql.append(" from ");
                            for (QuerySysDictionaryViewOutput dict : appServiceConfList.getResult()) {
                                if (dict.getDicKey().equals(boxRelEnt.getBusinessModule())) {
                                    sql.append(dict.getShortName());
                                    break;
                                }
                            }
                            sql.append(".");
                            sql.append(boxRelEnt.getTableName());
                            for (ScadaGatherConfInEnt confInEnt : confInEnts) {
                                if (boxRelEnt.getId().equals(confInEnt.getScadaGatherBoxRelId())){
                                    sql.append(" where ");
                                    sql.append(confInEnt.getParamCode());
                                    sql.append(" = '");
                                    if (ScadaConstant.IN_PARAM_TYPE_FIXED.equals(confInEnt.getParamType())){
                                        sql.append(confInEnt.getConfInParam());
                                        sql.append("'");
                                    }else{
                                        if ("station".equals(confInEnt.getConfInParam())){
                                            sql.append(input.getStationId());
                                        }else{
                                            sql.append(input.getOuId());
                                        }
                                        sql.append("'");
                                    }
                                    break;
                                }
                            }
                            sql.append(" order by sort ");
                            List<SmartRecord> smartRecords = boxViewMapper.selectBySql(sql.toString());
                            contentEntity.setContent(smartRecords);
                            //只展示有数据的
                            if (!CollectionUtils.isEmpty(smartRecords) && smartRecords.size()>0){
                                list.add(contentEntity);
                            }
                        }
                        //关系排序
                        list=list.stream().sorted(Comparator.comparing(ScadaGatherConfContent::getSort)).collect(Collectors.toList());

                        StringBuilder sql = new StringBuilder();

                        for (ScadaGatherBoxRelDetailEnt detailEnt : detailEnts) {
                            if (ScadaConstant.SCADA_DATA.equals(detailEnt.getBoxDetailType())) {
                                sql.append(" union select id, scada_gather_box_id, scada_gather_box_rel_id, scada_environment_info_id, environment_info_name, scada_inventory_info_id, scada_tag, config_column_id, tag_name, collect_start_time, collect_interval, alarm_min, alarm_max, data_unit, move_towards ,comput_coefficient,gather_type,gather_offset, description " +
                                        "from pps_base_info.scada_gather_conf where ");
                                sql.append("  scada_gather_box_id = '");
                                sql.append(box.getId());
                                sql.append("'");
                                sql.append(" and scada_gather_box_rel_id = '");
                                sql.append(detailEnt.getScadaGatherBoxRelId());
                                sql.append("'");
                                sql.append(" and tag_name = '");
                                sql.append(detailEnt.getTableColumn());
                                sql.append("'");

                                for (ScadaGatherConfContent content : list) {
                                    if (content.getScadaGatherBoxRelId().equals(detailEnt.getScadaGatherBoxRelId())) {
                                        List<SmartRecord> records = content.getContent();
                                        if (!CollectionUtils.isEmpty(records)){
                                            sql.append(" and config_column_id in ( ");
                                            for (SmartRecord record : records) {
                                                sql.append("'");
                                                sql.append(record.get("id").toString());
                                                sql.append("'");
                                                sql.append(",");
                                            }
                                            sql= new StringBuilder(sql.substring(0, sql.length() - 1));
                                            sql.append(" ) ");
                                        }
                                    }
                                }
                            }
                        }

                        String substring = sql.substring(6, sql.length());
                        List<SmartRecord> smartRecords = boxViewMapper.selectBySql(substring);
                        //循环当前模型id下所有模型关系
                        for (ScadaGatherConfContent confContent : list) {
                            //循环当前模型id下所有scada需要配置的列
                            List<HeadValue> columns =  confContent.getColumns().stream().filter(item -> item.getBoxDetailType() == 1).collect(Collectors.toList());
                            List<HeadValue> columnData =  confContent.getColumns().stream().filter(item -> item.getBoxDetailType() == 2).collect(Collectors.toList());
                            List<SmartRecord> contents = confContent.getContent();
                           for(SmartRecord record : contents){
                               DownloadScadaGatherBoxContent boxContentTemplate = XCopyUtils.copyNewObject(confContent , DownloadScadaGatherBoxContent.class);
                               int i = 1 ;
                                for(HeadValue columnTitle : columns){
                                    if(record.containsKey(columnTitle.getCode()) && i <= 6){
                                        setContentKeyValue( boxContentTemplate , "otherInfo" + i , record.get(columnTitle.getCode()));
                                        i ++ ;
                                    }
                                }
                                for(HeadValue column : columnData){
                                    DownloadScadaGatherBoxContent boxContent = XCopyUtils.copyNewObject(boxContentTemplate , DownloadScadaGatherBoxContent.class);
                                    setContentKeyValue( boxContent , "tagTitle" , column.getName());
                                    if(!CollectionUtils.isEmpty(smartRecords)){
                                        SmartRecord smartRecord = smartRecords.stream().filter(item-> item.get("scada_gather_box_rel_id").toString().equals(confContent.getScadaGatherBoxRelId())
                                                && item.get("config_column_id").toString().equals(record.get("id").toString())
                                                && item.get("tag_name") != null && item.get("tag_name").toString().equals(column.getCode())).findFirst().orElse(null);
                                        if(null != smartRecord  ){
                                            ScadaGatherConfEnt confEnt = JSON.parseObject(JSON.toJSONString(smartRecord), ScadaGatherConfEnt.class);
                                            XCopyUtils.copyObject(confEnt , boxContent);
                                        }
                                    }
                                    boxContent.setTagName(column.getCode());
                                    boxContent.setScadaGatherBoxId(box.getId());
                                    boxContent.setConfigColumnId(record.get("id").toString());
                                    boxContent.setTemplateName(box.getGatherBoxName());
                                    boxContent.setStationId(box.getStationId());
                                    boxContent.setStationName(box.getStationName());
                                    if(XStringUtils.isNotEmpty(boxContent.getCollectStartTime()))
                                        boxContent.setCollectStartTime(XDateUtils.getDateTimeString(new Date(Long.parseLong(boxContent.getCollectStartTime()))));
                                    excelList.add(boxContent);
                                }
                            }
                        }
                    }
                }
                String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/小时参数采集配置.xlsx");
                String templateFilePath = "template/exportScadaGatherBoxTemplate.xlsx";
                if (!CollectionUtils.isEmpty(excelList)) {
                    Map<String, Object> params = new HashMap<>();
                    params.put("list", excelList);
                    InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
                    byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, params);
                    ExportExcelUtils.outputFileData(excelContent, fileSavePath);
                    return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
                } else {
                    return XFileResult.error(200, "暂无数据导出");
                }
            }else
                return XFileResult.error(200, "请选择场站");
        } catch (Exception ex) {
            context.getLogger().error("导出出错", ex);
            return XFileResult.error(500, "导出失败");
        }
    }

    @XText("导入小时参数采集配置数据")
    @XApiUpload
    public XServiceResult importExcelData(XContext context, ImportScadaGatherBoxInput input) {
        try {
            if (input.getFile() == null)
                return XServiceResult.error(500, "未获取上传文件");
            PpsUserSession session = context.getSession(PpsUserSession.class);
            List<ScadaGatherConfEnt> successEntList =  this.dealExcelData(context, input );
            ScadaGatherConfMapper mapper = context.getBean(ScadaGatherConfMapper.class);
            XThreadHelper.async(new Runnable() {
                @Override
                public void run() {
                    for (int i = 0; i < successEntList.size(); i++) {
                        Date currentDate = new Date();
                        ScadaGatherConfEnt record = successEntList.get(i);
                        QueryWrapper<ScadaGatherConfEnt> queryWrapper = new QueryWrapper<>();
                        queryWrapper.lambda().eq(ScadaGatherConfEnt::getScadaGatherBoxId, record.getScadaGatherBoxId())
                                .eq(ScadaGatherConfEnt::getScadaGatherBoxRelId,record.getScadaGatherBoxRelId())
                                .eq(ScadaGatherConfEnt::getConfigColumnId,record.getConfigColumnId())
                                .eq(ScadaGatherConfEnt::getTagName,record.getTagName());
                        ScadaGatherConfEnt entity = mapper.selectOne(queryWrapper);
                        if(null != entity){
                            ScadaGatherConfEnt updateEnt = XCopyUtils.copyNewObject(record , ScadaGatherConfEnt.class);
                            updateEnt.setId(entity.getId());
                            updateEnt.setModifyById(session.getId());
                            updateEnt.setModifyByName(session.getUserName());
                            updateEnt.setModifyTime(currentDate);
                            mapper.updateById(updateEnt);
                        }else {
                            record.setId(UUID.randomUUID().toString());
                            record.setCreateById(session.getId());
                            record.setCreateByName(session.getUserName());
                            record.setCreateTime(currentDate);
                            record.setComputCoefficient("1");
                            record.setMoveTowards("1");
                            record.setGatherType("1");
                            record.setGatherOffset("0");
                            mapper.insert(record);
                        }
                    }
                }
            });
            return XServiceResult.OK;
        } catch (Exception ex) {
            context.getLogger().error("导入失败", ex);
            return XServiceResult.error(500, "导入失败：" + ex.toString().replace("java.lang.Exception:", ""));
        }
    }

    private List<ScadaGatherConfEnt> dealExcelData(XContext context, ImportScadaGatherBoxInput param) throws Exception {
        List<ScadaGatherConfEnt> result = new ArrayList<>();
        String fileName = param.getFile().getFileName();
        InputStream input = param.getFile().getInputStream();
        this.checkError(!fileName.endsWith(".xlsx"),"导入模板文件格式不正确，请核对(文档格式应为.xlsx)");
        Workbook wb = new XSSFWorkbook(input);
        Sheet sheet = wb.getSheetAt(0);
        Iterator<Row> rows = sheet.rowIterator();
        this.checkError(rows==null || !rows.hasNext(),"未导入任何数据");
        DataFormatter formatter = new DataFormatter();
        List<String> tempRowTitle= this.createTitles();
        Row titleRowFromExcel = rows.next();
        String titleErrorMessage="";
        isNeedNotEmptyMap.clear();
        for (int i = 0; i < tempRowTitle.size(); i++) {
            String titleFormExcel = formatter.formatCellValue(titleRowFromExcel.getCell(i)).trim();
            isNeedNotEmptyMap.put(i,titleFormExcel.startsWith("*"));
            titleFormExcel=titleFormExcel.replace("*","");

            if(!tempRowTitle.get(i).equals(titleFormExcel)){
                titleErrorMessage = titleErrorMessage.concat("第"+ (i+1)+"列表头不正确应为“"+tempRowTitle.get(i)+"”,");
            }
        }
        this.checkError(StringUtils.isNotBlank(titleErrorMessage),titleErrorMessage+"请根据模板修改；");

        StringBuilder errorMessageBuilder=new StringBuilder();
        while (rows.hasNext()) {
            Row row = rows.next();
            int rowIndex=row.getRowNum();

            if (ImportExcelUtils.isRowEmpty(row))
            {
                errorMessageBuilder.append("第"+rowIndex+"行数据不能为空；");
                continue;
            }

            String dateErrorMessage = this.checkEmptyData(row , tempRowTitle);
            if(StringUtils.isNotBlank(dateErrorMessage))
            {
                errorMessageBuilder.append(dateErrorMessage);
                continue;
            }
            ScadaGatherConfEnt resultItem=new ScadaGatherConfEnt();
            for (int i = 0; i < tempRowTitle.size(); i++) {
                Cell cell = row.getCell(i);
                String cellText=formatter.formatCellValue(cell).trim();
                String titleName=tempRowTitle.get(i);

                if(StringUtils.isBlank(cellText))
                    continue;
                switch (i) {
                    case 2: //scada采集模型关系配置id
                        resultItem.setScadaGatherBoxId(cellText);
                        break;
                    case 4: //scada采集模型关系配置主键
                        resultItem.setScadaGatherBoxRelId(cellText);
                        break;
                    case 6: //config行数据主键
                        resultItem.setConfigColumnId(cellText);
                        break;
                    case 7: //Scada对应采集标签名称
                        resultItem.setTagName(cellText);
                        break;
                    case 9: //scada环境标识
                        resultItem.setScadaEnvironmentInfoId(cellText);
                        break;
                    case 10: //scada环境名称
                        resultItem.setEnvironmentInfoName(cellText);
                        break;
                    case 11: //采集频率
                        resultItem.setCollectInterval(cellText);
                        break;
                    case 12: //Scada点号
                        resultItem.setScadaTag(cellText);
                        break;
//                    case 11: //计算系数
//                        resultItem.setComputCoefficient(cellText);
//                        break;
//                    case 12: //向前向后（1-向前；2-向后）
//                        resultItem.setMoveTowards(cellText);
//                        break;
//                    case 13: //采集类型（1-常规；2-自定义）
//                        resultItem.setGatherType(cellText);
//                        break;
//                    case 14: //偏移量
//                        resultItem.setGatherOffset(cellText);
//                        break;
                    case 13: //开始采集时间
                        resultItem.setCollectStartTime(cellText);
                        break;
                    case 14: //描述
                        resultItem.setDescription(cellText);
                        break;
                    default:
                        break;
                }
            }
            if(resultItem!=null)
                result.add(resultItem);
        }
        this.checkError(errorMessageBuilder.length()>0,errorMessageBuilder.toString());
        return result;
    }

    private List<String> createTitles() {
        List<String> titles = new ArrayList<String>();
        titles.add("场站ID（勿动）");
        titles.add("场站（勿动）");
        titles.add("业务模版ID（勿动）");
        titles.add("业务模版名称（勿动）");
        titles.add("模版配置标题ID（勿动）");
        titles.add("模版配置标题名称（勿动）");
        titles.add("配置行id（勿动）");
        titles.add("位号did（勿动）");
        titles.add("位号名称（勿动）");
        titles.add("scada环境标识Id");

        titles.add("scada环境标识名称");
        titles.add("采集频率");
        titles.add("scada点号");
//        titles.add("计算系数");
//        titles.add("向前向后采集（1-向前；2-向后）");
//        titles.add("采集类型（1-常规；2-自定义）");
//        titles.add("偏移量");
        titles.add("开始采集时间");
        titles.add("描述");
        return titles;
    }

    private void checkError(boolean isError, String errorMessage) throws Exception {
        if (isError)
            throw new Exception(errorMessage);
    }

    public void setContentKeyValue(Object object, String column, Object fieldValue) {
        try {
            if (XStringUtils.isNotEmpty(column)) {
                Field field = object.getClass().getDeclaredField(column);
                field.setAccessible(true);
                field.set(object, fieldValue);
            }
        } catch (Exception e) {

        }
    }

    private String checkEmptyData(Row row , List<String> tempRowTitle){
        String result="";
        DataFormatter formatter = new DataFormatter();
        for (int i = 0; i < isNeedNotEmptyMap.size(); i++) {
            Cell cell = row.getCell(i);
            Boolean isNeedNotEmpty=isNeedNotEmptyMap.get(i);
            Boolean isEmptyValue=StringUtils.isBlank(formatter.formatCellValue(cell));

            if(isNeedNotEmpty && isEmptyValue){
                result = result.concat(String.format( "“%s”",tempRowTitle.get(i).replace("*","")));
            }
        }
        if(StringUtils.isNotBlank(result))
        {
            int rowNum=row.getRowNum();
            result="第"+rowNum+"行"+result+"不能为空；";
        }
        return result;
    }
}
