package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.ScadaGatherConfCloudService;
import pps.cloud.base.service.data.*;
import pps.cloud.scadapi.service.ScadaPiService;
import pps.cloud.scadapi.service.data.GetValueByTimeOrderInput;
import pps.cloud.scadapi.service.data.PiPointValue;
import pps.core.base.entity.ScadaEnvironmentInfoEnt;
import pps.core.base.entity.ScadaGatherConfEnt;
import pps.core.base.mapper.ScadaEnvironmentInfoMapper;
import pps.core.base.mapper.ScadaGatherConfMapper;
import pps.core.base.service.data.scada_environment_info.GetScadaEnvironmentInfoInput;
import pps.core.base.service.data.scada_environment_info.GetScadaEnvironmentInfoOutput;
import pps.core.base.service.data.scada_gather_conf.GetScadaParamOutput;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

@XService
public class ScadaGatherConfCloudServiceImpl implements ScadaGatherConfCloudService {
    @Override
    public XSingleResult<ScadaConfigOut> getScadaConfigData(XContext context, ScadaConfigInput input) {
        ScadaConfigOut out = new ScadaConfigOut();
        ScadaGatherConfMapper mapper = context.getBean(ScadaGatherConfMapper.class);
        QueryWrapper<ScadaGatherConfEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaGatherConfEnt::getConfigColumnId, input.getConfigCloumnId())
                .eq(ScadaGatherConfEnt::getTagName,input.getTagName());
        ScadaGatherConfEnt entity = mapper.selectOne(queryWrapper);
        if(ObjectUtils.isNotEmpty(entity)){
            out.setScadaTag(entity.getScadaTag());
            out.setScadaEnvironmentInfoId(entity.getScadaEnvironmentInfoId());
            ScadaEnvironmentInfoMapper environmentMapper = context.getBean(ScadaEnvironmentInfoMapper.class);
            QueryWrapper<ScadaEnvironmentInfoEnt> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.lambda().eq(ScadaEnvironmentInfoEnt::getId, entity.getScadaEnvironmentInfoId());
            ScadaEnvironmentInfoEnt environmentInfoEnt = environmentMapper.selectOne(queryWrapper1);
            if(ObjectUtils.isNotEmpty(environmentInfoEnt)){
                String type = environmentInfoEnt.getType();
                out.setType(type);
            }

        }

        return XSingleResult.success(out);
    }



    @Override
    public XListResult<GetDateScadaParamOutput> getPiScadaTagDateValue(XContext context , GetScadaParamDateInput input) throws Exception {
        ScadaPiService piService = context.getBean(ScadaPiService.class);
        ScadaEnvironmentInfoMapper scadaEnvironmentInfoMapper = context.getBean(ScadaEnvironmentInfoMapper.class);
        QueryWrapper<ScadaEnvironmentInfoEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(ScadaEnvironmentInfoEnt::getId,input.getScadaEnvironmentInfoId());
        ScadaEnvironmentInfoEnt environmentInfoEnt = scadaEnvironmentInfoMapper.selectOne(wrapper);
        if (environmentInfoEnt == null){
            return XListResult.error(200,"未找到scada环境数据");
        }
        List<GetDateScadaParamOutput> paramOutputs = new ArrayList<>();

        GetValueByTimeOrderInput tagParam = new GetValueByTimeOrderInput();
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar cl=Calendar.getInstance();
        cl.setTime(input.getDataDate());
        cl.add(Calendar.DATE,1);
        tagParam.setPiClientIP(environmentInfoEnt.getClientIp());
        tagParam.setPiClientPort(environmentInfoEnt.getClientPort());
        tagParam.setPiServerIP(environmentInfoEnt.getIpAddress());
        tagParam.setTags(input.getTags());
        tagParam.setStartTime(sdf.format(input.getDataDate()));
        tagParam.setEndTime(sdf.format(cl.getTime()));
        tagParam.setOrderWord("desc");


        XListResult<PiPointValue> currentValue = piService.getValueByTimeOrder(context, tagParam);
        List<PiPointValue> result = currentValue.getResult();
        if (!CollectionUtils.isEmpty(result)){
            for (PiPointValue piPointValue : result) {
                GetDateScadaParamOutput output = new GetDateScadaParamOutput();
                output.setScadaTag(piPointValue.getTag());
                output.setTagValueStr(piPointValue.getValue());
                output.setTagTimeStr(piPointValue.getTime());
                output.setCount(result.size());
                paramOutputs.add(output);
            }
        }

        return XListResult.success(paramOutputs);
    }

    @Override
    public XSingleResult<GetScadaEnvironmentInfo> getScadaEnvironmentInfo(XContext context, GetScadaEnvironmentInfo input){
        ScadaEnvironmentInfoMapper mapper = context.getBean(ScadaEnvironmentInfoMapper.class);
        QueryWrapper<ScadaEnvironmentInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaEnvironmentInfoEnt::getEnvironmentInfoName, input.getEnvironmentInfoName());
        ScadaEnvironmentInfoEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetScadaEnvironmentInfo output = new GetScadaEnvironmentInfo();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @Override
    public XSingleResult<GetScadaEnvironmentInfo> getScadaEnvironmentInfoById(XContext context, GetScadaEnvironmentInfo input){
        ScadaEnvironmentInfoMapper mapper = context.getBean(ScadaEnvironmentInfoMapper.class);
        QueryWrapper<ScadaEnvironmentInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaEnvironmentInfoEnt::getId, input.getId());
        ScadaEnvironmentInfoEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetScadaEnvironmentInfo output = new GetScadaEnvironmentInfo();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

}
