package pps.core.base.service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.logging.log4j.util.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.data.GetDateScadaParamOutput;
import pps.cloud.base.service.data.GetScadaParamDateInput;
import pps.cloud.ihd.service.IhdService;
import pps.cloud.ihd.service.data.*;
import pps.cloud.scadapi.service.ScadaPiService;
import pps.cloud.scadapi.service.data.*;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.scada_gather_conf.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XRawResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.helper.XTransactionHelper;

@XService
@XText("scada采集配置管理")
public class ScadaGatherConfService {
    private static final Logger logger = LoggerFactory.getLogger(ScadaGatherConfService.class);

    @XText("scada补采")
    @XApiPost
    public XServiceResult scadaRemedy(XContext context ,GetScadaParamInput input){
        if (XStringUtils.isEmpty(input.getScadaEnvironmentInfoId())){
            return XServiceResult.error(200,"请选择需要补采的环境");
        }
        ScadaEnvironmentInfoMapper environmentInfoMapper = context.getBean(ScadaEnvironmentInfoMapper.class);
        QueryWrapper<ScadaEnvironmentInfoEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(ScadaEnvironmentInfoEnt::getId,input.getScadaEnvironmentInfoId());
        ScadaEnvironmentInfoEnt infoEnt = environmentInfoMapper.selectOne(wrapper);
        if (infoEnt == null){
            return XServiceResult.error(200,"选择的环境有误，请联系管理员排查！");
        }
        XServiceResult xServiceResult = null;
        if ("PI".equals(infoEnt.getType())){
            ScadaPiService piService = context.getBean(ScadaPiService.class);
            SyncScadaDataParamsInput paramsInput = new SyncScadaDataParamsInput();
            paramsInput.setSyncType(1);
            paramsInput.setScadaEnvironmentInfoId(Integer.parseInt(input.getScadaEnvironmentInfoId()));
            paramsInput.setCollectInterval(input.getCollectInterval());
            paramsInput.setGatherType(input.getGatherType());
            paramsInput.setGatherOffset(input.getGatherOffset());
            paramsInput.setCurrentTime(input.getRemedyDate());
            paramsInput.setMoveTowards(input.getMoveTowards());
            xServiceResult = piService.SyncScadaDataParamsbak(context, paramsInput);
            xServiceResult.throwIfFail();

        }else if ("IHD".equals(infoEnt.getType())){
            //等于斌写完scada补采再接
            System.out.println();
        }
        return xServiceResult;
    }

    /**
     * 支持到分钟
     * */
    @XText("IHD根据时间补采")
    @XApiPost
    public XServiceResult scadaIHDRemedy(XContext context ,GetScadaParamInput input){

        IhdService ihdService = context.getBean(IhdService.class);
        RemedyCollectionParam tagParam = new RemedyCollectionParam();
        tagParam.setDate(input.getRemedyDate());
        XServiceResult xServiceResult = ihdService.remedyCollection(context, tagParam);
        xServiceResult.throwIfFail();
        return xServiceResult;
    }

    /**
     * 支持到小时
     * */
    @XText("PI根据时间补采")
    @XApiPost
    public XServiceResult scadaPIRemedy(XContext context ,GetScadaParamInput input){
        ScadaPiService piService = context.getBean(ScadaPiService.class);
        SyncScadaDataParamsInput paramsInput = new SyncScadaDataParamsInput();
        paramsInput.setSyncType(1);
        paramsInput.setCurrentTime(input.getRemedyDate());
        XServiceResult xServiceResult = piService.SyncScadaDataParamsbak(context, paramsInput);
        xServiceResult.throwIfFail();
        return xServiceResult;
    }


    @XText("ihd模糊查询scada点号")
    @XApiPost
    public XListResult<GetScadaParamOutput> getIhdLikeScadaTag(XContext context ,GetScadaParamInput input){
        IhdService ihdService = context.getBean(IhdService.class);
        QueryBasicTagParam tagParam = new QueryBasicTagParam();
        tagParam.setEnvironmentId(input.getScadaEnvironmentInfoId());
        tagParam.setTagName(input.getScadaTag());
        tagParam.setTagDesc(input.getScadaDesc());
        if (input.getCount()==null){
            tagParam.setCount(50);
        }else{
            tagParam.setCount(input.getCount());
        }
        logger.info("ihd模糊查询点号参数: ={}", tagParam);

        XListResult<TagResult> listResult = ihdService.queryBasicTagsSimple(context, tagParam);
        List<TagResult> result = listResult.getResult();
        logger.info("ihd模糊查询返回值: ={}", result);
        List<GetScadaParamOutput> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(result)){
            for (TagResult tagResult : result) {
                GetScadaParamOutput output = new GetScadaParamOutput();
                output.setScadaTag(tagResult.getTagName());
                output.setScadaDesc(tagResult.getTagDesc());
                output.setCount(result.size());
                output.setScadaEnvironmentInfoId(input.getScadaEnvironmentInfoId());
                list.add(output);
            }
        }
        return XListResult.success(list);
    }


    @XText("ihd根据时间范围和tag号查询scada值")
    @XApiPost
    public XListResult<GetScadaParamOutput> getIhdScadaTagValue(XContext context ,GetScadaParamInput input){
        IhdService ihdService = context.getBean(IhdService.class);
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SearchIhdParam tagParam = new SearchIhdParam();
        tagParam.setEnvironmentId(input.getScadaEnvironmentInfoId());
        tagParam.setTagName(input.getScadaTag());
        try {
            tagParam.setStartDate(format.parse(input.getStartDate()));
            tagParam.setEndDate(format.parse(input.getEndDate()));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        if (input.getTimeSort() == null){
            tagParam.setTimeSort(false);
        }else{
            tagParam.setTimeSort(input.getTimeSort());
        }
        if (input.getCount()==null){
            tagParam.setCount(50);
        }else{
            if (input.getCount()>100){
                tagParam.setCount(100);
            }else{
                tagParam.setCount(input.getCount());
            }
        }
        logger.info("ihd根据时间范围查询参数: ={}", tagParam);
        XListResult<ScadaValueResult> listResult = ihdService.queryTagHisRawRecords(context, tagParam);
        List<ScadaValueResult> result = listResult.getResult();
        logger.info("ihd根据时间范围返回值: ={}", result);
        List<GetScadaParamOutput> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(result)){
            for (ScadaValueResult tagResult : result) {
                GetScadaParamOutput output = new GetScadaParamOutput();
                output.setScadaTag(input.getScadaTag());
                output.setCount(result.size());
                output.setScadaEnvironmentInfoId(input.getScadaEnvironmentInfoId());
                output.setTagTimeStr(tagResult.getTagTimeStr());
                output.setTagValueStr(tagResult.getTagValueStr());
                list.add(output);
            }
        }
        return XListResult.success(list);
    }

    @XText("pi查询scada点号 调用本地数据库")
    @XApiAnonymous
    @XApiPost
    public XListResult<GetScadaParamOutput> getPiScadaTag(XContext context ,GetScadaParamInput input) throws Exception {

        /***
         * 修改为查询 本地库中的 scada 点号
         3	东部储运PI数据库-jdbc
         8	东部储运PI数据库-sdk
         4	北方管道PI数据库-jdbc
         7	北方管道PI数据库-sdk
         10	华南PI数据库-sdk
         2	油气调控中心PI数据库-jdbc-北京
         6	油气调控中心PI数据库-sdk-北京
         9	油气调控中心PI数据库-sdk-廊坊
         */

        logger.info("pi模糊查询参数: ={}", input);

        CommonDataMapper commonDataMapper = context.getBean(CommonDataMapper.class);

        String sql = "select distinct scada_tag ,description  from pps_base_info.scada_inventory_info where scada_environment_info_id  in( #ids# ) ";

        if(input.getScadaEnvironmentInfoId().equals("2")||input.getScadaEnvironmentInfoId().equals("6")||input.getScadaEnvironmentInfoId().equals("9")){
            sql = sql.replace("#ids#","'2','6','9'");
        }else if(input.getScadaEnvironmentInfoId().equals("3")||input.getScadaEnvironmentInfoId().equals("8")){
            sql = sql.replace("#ids#","'8','3'");
        }else if(input.getScadaEnvironmentInfoId().equals("4")||input.getScadaEnvironmentInfoId().equals("7")){
            sql = sql.replace("#ids#","'4','7'");
        }else{
            sql = sql.replace("#ids#",input.getScadaEnvironmentInfoId());
        }

        Map<String,Object> map = new HashMap<>();

        if(!input.getScadaTag().isEmpty()){
            map.put("scada_tag",input.getScadaTag());
            sql += " and scada_tag like concat('%',#{param.scada_tag},'%') ";

        }

        if(!input.getScadaDesc().isEmpty()){
            map.put("description",input.getScadaDesc());
            sql+=" and description  like concat('%',#{param.description},'%') ";
        }

        if(map.size()<1){
            return XListResult.error(200,"请输入查询条件");
        }

        sql+=" limit 1000 ";

        List<Map<String, Object>> l =commonDataMapper.getData(sql,map);
        List<GetScadaParamOutput> paramOutputs = new ArrayList<>();
        if(l!=null&&l.size()>0){
            l.forEach(tag->{
                GetScadaParamOutput output = new GetScadaParamOutput();
                if(tag!=null){
                    if(tag.get("scada_tag")!=null){
                        output.setScadaTag(tag.get("scada_tag").toString());
                    }

                    if(tag.get("description")!=null){
                        output.setScadaDesc(tag.get("description").toString());
                    }

                    output.setCount(100);
                    paramOutputs.add(output);
                }
            });
        }

        return XListResult.success(paramOutputs);
    }

    @XText("pi查询scada点号 调用本地数据库 根据管线场站 精确查找")
    @XApiPost
    @XApiAnonymous
    public XListResult<GetScadaParamOutput> getPiScadaTagByPipelineStationCode(XContext context ,getPiScadaTagByPipelineStationCodeInput input) throws Exception {

        /***
         * 修改为查询 本地库中的 scada 点号
         3	东部储运PI数据库-jdbc
         8	东部储运PI数据库-sdk
         4	北方管道PI数据库-jdbc
         7	北方管道PI数据库-sdk
         10	华南PI数据库-sdk
         2	油气调控中心PI数据库-jdbc-北京
         6	油气调控中心PI数据库-sdk-北京
         9	油气调控中心PI数据库-sdk-廊坊
         */

        logger.info("pi模糊查询参数: ={}", input);

        CommonDataMapper commonDataMapper = context.getBean(CommonDataMapper.class);

        String sql = "select distinct scada_tag ,description  from pps_base_info.scada_inventory_info where scada_environment_info_id  in( #ids# ) ";

        if(input.getScadaEnvironmentInfoId().equals("2")||input.getScadaEnvironmentInfoId().equals("6")||input.getScadaEnvironmentInfoId().equals("9")){
            sql = sql.replace("#ids#","'2','6','9'");
        }else if(input.getScadaEnvironmentInfoId().equals("3")||input.getScadaEnvironmentInfoId().equals("8")){
            sql = sql.replace("#ids#","'8','3'");
        }else if(input.getScadaEnvironmentInfoId().equals("4")||input.getScadaEnvironmentInfoId().equals("7")){
            sql = sql.replace("#ids#","'4','7'");
        }else{
            sql = sql.replace("#ids#",input.getScadaEnvironmentInfoId());
        }

        Map<String,Object> map = new HashMap<>();

        if(!input.getGroupCode().isEmpty()){
            map.put("scada_tag",input.getGroupCode());
            sql += " and scada_tag like concat( #{param.scada_tag},'%') ";

        }

        if(map.size()<1){
            return XListResult.error(200,"请输入查询条件");
        }

        sql+=" limit 1000 ";

        List<Map<String, Object>> l =commonDataMapper.getData(sql,map);
        List<GetScadaParamOutput> paramOutputs = new ArrayList<>();
        if(l!=null&&l.size()>0){
            l.forEach(tag->{
                GetScadaParamOutput output = new GetScadaParamOutput();
                if(tag!=null){
                    if(tag.get("scada_tag")!=null){
                        output.setScadaTag(tag.get("scada_tag").toString());
                    }

                    if(tag.get("description")!=null){
                        output.setScadaDesc(tag.get("description").toString());
                    }

                    output.setCount(100);
                    paramOutputs.add(output);
                }
            });
        }

        return XListResult.success(paramOutputs);
    }


    @XText("pi查询scada点号 调用远程中间数据库")
    @XApiPost
    public XListResult<GetScadaParamOutput> getPiScadaTagbak(XContext context ,GetScadaParamInput input) throws Exception {
        ScadaPiService piService = context.getBean(ScadaPiService.class);
        ScadaEnvironmentInfoMapper scadaEnvironmentInfoMapper = context.getBean(ScadaEnvironmentInfoMapper.class);
        QueryWrapper<ScadaEnvironmentInfoEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(ScadaEnvironmentInfoEnt::getId,input.getScadaEnvironmentInfoId());
        ScadaEnvironmentInfoEnt environmentInfoEnt = scadaEnvironmentInfoMapper.selectOne(wrapper);
        if (environmentInfoEnt == null){
            return XListResult.error(200,"未找到scada环境数据");
        }
        List<GetScadaParamOutput> paramOutputs = new ArrayList<>();

        FindByTagOrDesInput tagParam = new FindByTagOrDesInput();
        tagParam.setDescription(input.getScadaDesc());
        tagParam.setTag(input.getScadaTag());
        tagParam.setPiServerIP(environmentInfoEnt.getIpAddress());
        tagParam.setPiClientIP(environmentInfoEnt.getClientIp());
        tagParam.setPiClientPort(environmentInfoEnt.getClientPort());

        logger.info("pi模糊查询参数: ={}", tagParam);

        XListResult<PiPointInfo> byTagOrDes = piService.findByTagOrDes(context, tagParam);
        List<PiPointInfo> result = byTagOrDes.getResult();
        logger.info("pi模糊查询返回值: ={}", result);
        if (!CollectionUtils.isEmpty(result)){
            for (PiPointInfo pointInfo : result) {
                GetScadaParamOutput output = new GetScadaParamOutput();
                output.setScadaTag(pointInfo.getTag());
                output.setScadaDesc(pointInfo.getRemarks());
                output.setCount(result.size());
                paramOutputs.add(output);
            }
        }

        return XListResult.success(paramOutputs);
    }

    @XText("pi查询scada点号 获取管线场站 列表")
    @XApiAnonymous
    @XApiPost
    public XListResult<GetPipelineStationsOutput> getPipelineStations(XContext context, GetPipelineStationsInput input)throws Exception{
/***
 * 修改为查询 本地库中的 scada 点号
 3	东部储运PI数据库-jdbc
 8	东部储运PI数据库-sdk
 4	北方管道PI数据库-jdbc
 7	北方管道PI数据库-sdk
 10	华南PI数据库-sdk
 2	油气调控中心PI数据库-jdbc-北京
 6	油气调控中心PI数据库-sdk-北京
 9	油气调控中心PI数据库-sdk-廊坊
 */

        logger.info("pi模糊查询参数: ={}", input);

        CommonDataMapper commonDataMapper = context.getBean(CommonDataMapper.class);

        String sql = "select distinct group_code ,pipeline_station from pps_base_info.scada_inventory_group\n" +
                "where scada_environment_info_id  in( #ids# ) ";

        if(input.getScadaEnvironmentInfoId().equals("2")||input.getScadaEnvironmentInfoId().equals("6")||input.getScadaEnvironmentInfoId().equals("9")){
            sql = sql.replace("#ids#","'2','6','9'");
        }else if(input.getScadaEnvironmentInfoId().equals("3")||input.getScadaEnvironmentInfoId().equals("8")){
            sql = sql.replace("#ids#","'8','3'");
        }else if(input.getScadaEnvironmentInfoId().equals("4")||input.getScadaEnvironmentInfoId().equals("7")){
            sql = sql.replace("#ids#","'4','7'");
        }else{
            sql = sql.replace("#ids#",input.getScadaEnvironmentInfoId());
        }

        Map<String,Object> map = new HashMap<>();

        if(input.getKeyValue()!=null&&!input.getKeyValue().trim().isEmpty()){
            sql +=" and (group_code like concat('%',#{param.keyValue},'%') or pipeline_station like concat('%',#{param.keyValue},'%'))  ";

            map.put("keyValue",input.getKeyValue());
        }

        sql+=" order by group_code";

        List<Map<String, Object>> l =commonDataMapper.getData(sql,map);
        List<GetPipelineStationsOutput> list = new ArrayList<>();
        if(l!=null&&l.size()>0){
            l.forEach(p->{

                if(p!=null){
                    GetPipelineStationsOutput out = new GetPipelineStationsOutput();
                    if(p.get("group_code")!=null){
                        out.setGroup_code(p.get("group_code").toString());
                    }
                    if(p.get("pipeline_station")!=null){
                        out.setPipeline_station(p.get("pipeline_station").toString());
                    }

                    list.add(out);
                }

            });
        }

        return XListResult.success(list);

    }



    @XText("pi根据时刻和tag号查询scada值")
    @XApiAnonymous
    @XApiPost
    public XListResult<GetDateScadaParamOutput> getPiScadaTagDateValue(XContext context , GetScadaParamDateInput input) throws Exception {
        ScadaPiService piService = context.getBean(ScadaPiService.class);
        ScadaEnvironmentInfoMapper scadaEnvironmentInfoMapper = context.getBean(ScadaEnvironmentInfoMapper.class);
        QueryWrapper<ScadaEnvironmentInfoEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(ScadaEnvironmentInfoEnt::getId,input.getScadaEnvironmentInfoId());
        ScadaEnvironmentInfoEnt environmentInfoEnt = scadaEnvironmentInfoMapper.selectOne(wrapper);
        if (environmentInfoEnt == null){
            return XListResult.error(200,"未找到scada环境数据");
        }
        List<GetDateScadaParamOutput> paramOutputs = new ArrayList<>();

        GetValueByTimeOrderInput tagParam = new GetValueByTimeOrderInput();
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar cl=Calendar.getInstance();
        cl.setTime(input.getDataDate());
        cl.add(Calendar.DATE,1);
        tagParam.setPiClientIP(environmentInfoEnt.getClientIp());
        tagParam.setPiClientPort(environmentInfoEnt.getClientPort());
        tagParam.setPiServerIP(environmentInfoEnt.getIpAddress());
        tagParam.setTags(input.getTags());
        tagParam.setStartTime(sdf.format(input.getDataDate()));
        tagParam.setEndTime(sdf.format(cl.getTime()));
        tagParam.setOrderWord("desc");


        XListResult<PiPointValue> currentValue = piService.getValueByTimeOrder(context, tagParam);
        List<PiPointValue> result = currentValue.getResult();
        if (!CollectionUtils.isEmpty(result)){
            for (PiPointValue piPointValue : result) {
                GetDateScadaParamOutput output = new GetDateScadaParamOutput();
                output.setScadaTag(piPointValue.getTag());
                output.setTagValueStr(piPointValue.getValue());
                output.setTagTimeStr(piPointValue.getTime());
                output.setCount(result.size());
                paramOutputs.add(output);
            }
        }

        return XListResult.success(paramOutputs);
    }


    @XText("pi根据时间范围和tag号查询scada值")
    @XApiPost
    public XListResult<GetScadaParamOutput> getPiScadaTagValue(XContext context ,GetScadaParamInput input) throws Exception {
        ScadaPiService piService = context.getBean(ScadaPiService.class);
        ScadaEnvironmentInfoMapper scadaEnvironmentInfoMapper = context.getBean(ScadaEnvironmentInfoMapper.class);
        QueryWrapper<ScadaEnvironmentInfoEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(ScadaEnvironmentInfoEnt::getId,input.getScadaEnvironmentInfoId());
        ScadaEnvironmentInfoEnt environmentInfoEnt = scadaEnvironmentInfoMapper.selectOne(wrapper);
        if (environmentInfoEnt == null){
            return XListResult.error(200,"未找到scada环境数据");
        }
        List<GetScadaParamOutput> paramOutputs = new ArrayList<>();

        GetHisValuesInput tagParam = new GetHisValuesInput();
        tagParam.setPiClientIP(environmentInfoEnt.getClientIp());
        tagParam.setPiClientPort(environmentInfoEnt.getClientPort());
        tagParam.setPiServerIP(environmentInfoEnt.getIpAddress());
        tagParam.setTag(input.getScadaTag());
        tagParam.setStartTime(input.getStartDate());
        tagParam.setEndTime(input.getEndDate());
        if (input.getTimeSort() == null){
            tagParam.setTimeSort(false);
        }else{
            tagParam.setTimeSort(input.getTimeSort());
        }
        if (input.getCount()==null){
            tagParam.setCount(50);
        }else{
            if (input.getCount()>100){
                tagParam.setCount(100);
            }else{
                tagParam.setCount(input.getCount());
            }
        }
        logger.info("pi根据时间范围查询参数: ={}", tagParam);

        XListResult<PiPointValue> currentValue = piService.getHisValues(context, tagParam);
        List<PiPointValue> result = currentValue.getResult();
        logger.info("pi根据时间范围查询返回值: ={}", result);
        if (!CollectionUtils.isEmpty(result)){
            for (PiPointValue piPointValue : result) {
                GetScadaParamOutput output = new GetScadaParamOutput();
                output.setScadaTag(piPointValue.getTag());
                output.setTagValueStr(piPointValue.getValue());
                output.setTagTimeStr(piPointValue.getTime());
                output.setCount(result.size());
                paramOutputs.add(output);
            }
        }

        return XListResult.success(paramOutputs);
    }


    @XText("编辑用于回显")
    @XApiPost
    public XSingleResult<GetScadaGatherConfOutput> getScadaGatherConf(XContext context, GetScadaGatherConfInput input){
        ScadaGatherConfMapper mapper = context.getBean(ScadaGatherConfMapper.class);
        QueryWrapper<ScadaGatherConfEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaGatherConfEnt::getScadaGatherBoxId, input.getScadaGatherBoxId())
                .eq(ScadaGatherConfEnt::getScadaGatherBoxRelId,input.getScadaGatherBoxRelId())
                .eq(ScadaGatherConfEnt::getConfigColumnId,input.getConfigColumnId())
                .eq(ScadaGatherConfEnt::getTagName,input.getTagName());
        ScadaGatherConfEnt entity = mapper.selectOne(queryWrapper);

        GetScadaGatherConfOutput output = new GetScadaGatherConfOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("查看")
    @XApiPost
    public XSingleResult<GetScadaGatherConfOutput> getScadaGatherConfById(XContext context, GetScadaGatherConfByIdInput input){
        if(Strings.isBlank(input.getScadaConfigId())) return XSingleResult.success(new GetScadaGatherConfOutput());
        ScadaGatherConfMapper mapper = context.getBean(ScadaGatherConfMapper.class);
        ScadaGatherConfEnt entity = mapper.selectById(input.getScadaConfigId());
        GetScadaGatherConfOutput output = new GetScadaGatherConfOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("创建scada配置")
    @XApiPost
    public XServiceResult createScadaGatherConf(XContext context, CreateScadaGatherConfInput input)throws XServiceException{
        XServiceResult result = XTransactionHelper.begin(context,()-> {
            ScadaGatherConfLogMapper confLogMapper = context.getBean(ScadaGatherConfLogMapper.class);
            if (XStringUtils.isEmpty(input.getId())){
                ScadaGatherConfMapper mapper = context.getBean(ScadaGatherConfMapper.class);
                ScadaGatherConfEnt entity = new ScadaGatherConfEnt();
                XCopyUtils.copyObject(input, entity);

                if (XStringUtils.isEmpty(input.getGatherExpiryDate())) {
                    entity.setGatherExpiryDate(null);
                }

                String confId = UUID.randomUUID().toString();
                entity.setId(confId);
                input.setId(confId);

                if (XStringUtils.isEmpty(entity.getGatherOffset())){
                    entity.setGatherOffset("0");
                }
                if (XStringUtils.isEmpty(entity.getComputWay())){
                    entity.setComputWay("MULTIPLY");
                }
                if (XStringUtils.isEmpty(entity.getComputCoefficient())){
                    entity.setComputCoefficient("1");
                }
                PpsUserSession session = context.getSession(PpsUserSession.class);
                entity.setCreateById(session.getId());
                entity.setCreateTime(new Date());
                entity.setCreateByName(session.getUserName());
                mapper.insert(entity);
                //保存操作日志
                ScadaGatherConfLogEnt logEnt = new ScadaGatherConfLogEnt();
                logEnt.setId(UUID.randomUUID().toString());
                logEnt.setScadaGatherConfId(confId);
                logEnt.setScadaEnvironmentInfoId(input.getScadaEnvironmentInfoId());
                logEnt.setScadaTag(input.getScadaTag());
                logEnt.setCollectInterval(input.getCollectInterval());
                logEnt.setCollectStartTime(input.getCollectStartTime());
                logEnt.setGatherExpiryDate(input.getGatherExpiryDate());
                logEnt.setCreateById(session.getId());
                logEnt.setCreateTime(new Date());
                logEnt.setCreateByName(session.getUserName());
                confLogMapper.insert(logEnt);

                return XServiceResult.OK;
            }else{
                ScadaGatherConfMapper mapper = context.getBean(ScadaGatherConfMapper.class);
                QueryWrapper<ScadaGatherConfEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(ScadaGatherConfEnt::getId, input.getId());
                ScadaGatherConfEnt entity = mapper.selectOne(queryWrapper);
                if(entity == null) {
                    return XServiceResult.error(context, XError.NotFound);
                }
                XCopyUtils.copyObject(input, entity);
                if (XStringUtils.isEmpty(entity.getGatherExpiryDate())) {
                    entity.setGatherExpiryDate(null);
                }

                if (XStringUtils.isEmpty(input.getCollectStartTime())) {
                    entity.setCollectStartTime("9999-12-30 00:00:00");
                }
                if (XStringUtils.isEmpty(entity.getGatherOffset())){
                    entity.setGatherOffset("0");
                }
                if (XStringUtils.isEmpty(entity.getComputWay())){
                    entity.setComputWay("MULTIPLY");
                }
                if (XStringUtils.isEmpty(entity.getComputCoefficient())){
                    entity.setComputCoefficient("1");
                }
                PpsUserSession session = context.getSession(PpsUserSession.class);
                entity.setModifyById(session.getId());
                entity.setModifyTime(new Date());
                entity.setModifyByName(session.getUserName());
                mapper.updateById(entity);
                //保存操作日志
                ScadaGatherConfLogEnt logEnt = new ScadaGatherConfLogEnt();
                logEnt.setId(UUID.randomUUID().toString());
                logEnt.setScadaGatherConfId(entity.getId());
                logEnt.setScadaEnvironmentInfoId(input.getScadaEnvironmentInfoId());
                logEnt.setScadaTag(input.getScadaTag());
                logEnt.setCollectInterval(input.getCollectInterval());
                logEnt.setCollectStartTime(input.getCollectStartTime());
                logEnt.setGatherExpiryDate(input.getGatherExpiryDate());
                logEnt.setCreateById(session.getId());
                logEnt.setCreateTime(new Date());
                logEnt.setCreateByName(session.getUserName());
                confLogMapper.insert(logEnt);
                return XServiceResult.OK;
            }
        });

        if(result.isSuccess()){
            ScadaPiService scadaPiService = context.getBean(ScadaPiService.class);
            //配置同步预处理
            XServiceResult a = scadaPiService.syncDatapreproccessByScadagatherconfid(context,new SyncDataPreproccessInput(input.getId()));
        }
        return result;
    }


    @XText("scada数据默认值sql预处理全部")
    @XApiPost
    public XServiceResult syncDataSqlPreproccess(XContext context ){
        try
        {
            ScadaPiService scadaPiService = context.getBean(ScadaPiService.class);
            scadaPiService.syncDataPreproccess(context);
            return XServiceResult.OK;
        }
        catch (Exception ex){

            return XServiceResult.error(500,ex.getMessage());
        }

    }

}
