package pps.core.base.service;

import java.util.Date;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import pps.core.base.constant.ScadaConstant;
import pps.core.base.mapper.*;
import pps.core.base.entity.*;
import pps.core.base.service.data.scada_inventory_info.*;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import xstartup.annotation.XService;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import xstartup.base.data.CustomQueryInput;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XService
@XText("scada清单管理")
public class ScadaInventoryInfoService {

    @XText("查询某一个scada环境下的scada点号")
    @XApiPost
    public XListResult<GetScadaInventoryInfoOutput> queryScadaInventoryInfos(XContext context, GetScadaInventoryInfoInput input){
        ScadaInventoryInfoMapper mapper = context.getBean(ScadaInventoryInfoMapper.class);
        QueryWrapper<ScadaInventoryInfoEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(ScadaInventoryInfoEnt::getScadaEnvironmentInfoId,input.getScadaEnvironmentInfoId())
                .eq(ScadaInventoryInfoEnt::getIsValid, ScadaConstant.IS_VALID_TRUE);
        List<ScadaInventoryInfoEnt> scadaInventoryInfoEnts = mapper.selectList(wrapper);
        List<GetScadaInventoryInfoOutput> outputs = XCopyUtils.copyNewList(scadaInventoryInfoEnts, GetScadaInventoryInfoOutput.class);
        return XListResult.success(outputs);
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult createScadaInventoryInfo(XContext context, CreateScadaInventoryInfoInput input){
        ScadaInventoryInfoMapper mapper = context.getBean(ScadaInventoryInfoMapper.class);
        ScadaInventoryInfoEnt entity = new ScadaInventoryInfoEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateScadaInventoryInfo(XContext context, UpdateScadaInventoryInfoInput input){
        ScadaInventoryInfoMapper mapper = context.getBean(ScadaInventoryInfoMapper.class);
        QueryWrapper<ScadaInventoryInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaInventoryInfoEnt::getId, input.getId());
        ScadaInventoryInfoEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteScadaInventoryInfo(XContext context, DeleteScadaInventoryInfoInput input){
        ScadaInventoryInfoMapper mapper = context.getBean(ScadaInventoryInfoMapper.class);
        QueryWrapper<ScadaInventoryInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaInventoryInfoEnt::getId, input.getId());
        ScadaInventoryInfoEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetScadaInventoryInfoOutput> getScadaInventoryInfo(XContext context, GetScadaInventoryInfoInput input){
        ScadaInventoryInfoMapper mapper = context.getBean(ScadaInventoryInfoMapper.class);
        QueryWrapper<ScadaInventoryInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ScadaInventoryInfoEnt::getId, input.getId());
        ScadaInventoryInfoEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetScadaInventoryInfoOutput output = new GetScadaInventoryInfoOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryScadaInventoryInfoOutput> queryScadaInventoryInfo(XContext context, QueryScadaInventoryInfoInput input){
        ScadaInventoryInfoMapper mapper = context.getBean(ScadaInventoryInfoMapper.class);
        QueryWrapper<ScadaInventoryInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(ScadaInventoryInfoEnt::getId, input.getId())
            .like(ScadaInventoryInfoEnt::getScadaEnvironmentInfoId, input.getScadaEnvironmentInfoId())
            .like(ScadaInventoryInfoEnt::getScadaTag, input.getScadaTag())
            .like(ScadaInventoryInfoEnt::getDescription, input.getDescription())
            .like(ScadaInventoryInfoEnt::getNumber, input.getNumber())
            .like(ScadaInventoryInfoEnt::getIsValid, input.getIsValid())
            .eq(ScadaInventoryInfoEnt::getCreateTime, input.getCreateTime())
            .like(ScadaInventoryInfoEnt::getCreateById, input.getCreateById())
            .like(ScadaInventoryInfoEnt::getCreateByName, input.getCreateByName())
            .eq(ScadaInventoryInfoEnt::getModifyTime, input.getModifyTime())
            .like(ScadaInventoryInfoEnt::getModifyById, input.getModifyById())
            .like(ScadaInventoryInfoEnt::getModifyByName, input.getModifyByName());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryScadaInventoryInfoOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryScadaInventoryInfoOutput> dynamicQueryScadaInventoryInfo(XContext context, CustomQueryInput input){
        ScadaInventoryInfoMapper mapper = context.getBean(ScadaInventoryInfoMapper.class);
        return XMapperHelper.query(mapper, input, ScadaInventoryInfoEnt.class, QueryScadaInventoryInfoOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetScadaInventoryInfoViewOutput> getScadaInventoryInfoView(XContext context, GetScadaInventoryInfoViewInput input){
        ScadaInventoryInfoViewMapper mapper = context.getBean(ScadaInventoryInfoViewMapper.class);
        ScadaInventoryInfoView record = new ScadaInventoryInfoView();
        XCopyUtils.copyObject(input, record);
        ScadaInventoryInfoView view = mapper.selectOne(record);
        if(view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetScadaInventoryInfoViewOutput output = new GetScadaInventoryInfoViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryScadaInventoryInfoViewOutput> queryScadaInventoryInfoView(XContext context, QueryScadaInventoryInfoViewInput input){
        ScadaInventoryInfoViewMapper mapper = context.getBean(ScadaInventoryInfoViewMapper.class);
        ScadaInventoryInfoView record = new ScadaInventoryInfoView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<ScadaInventoryInfoView> list = mapper.selectList(record);
        PageInfo<ScadaInventoryInfoView> pageInfo = new PageInfo<>(list);
        List<QueryScadaInventoryInfoViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryScadaInventoryInfoViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
